      subroutine mod06ct_set_qa( line, pixel, nmissing, ncloudy,
     &  qual_flag, conf_flag, hc_flag, ci_flag )

c-----------------------------------------------------------------------
c
c!F77
c
c!Description:
c    Set the appropriate MOD06 QA bits for a box of pixels for the
c    cloud top properties.
c
c!Input Parameters:
c    line        Line number at corner of this box
c    pixel       Pixel number at corner of this box
c    nmissing    Number of missing (bad radiance) pixels in this box
c    ncloudy     Number of cloudy pixels in this box
c    qual_flag   quality of product flag
c    conf_flag   confidence in product flag
c    ci_flag     cirrus cloud found flag
c    hc_flag     high cloud found flag
c
c
c!Output Parameters: None
c
c!Revision History:
c $Id: mod06ct_set_qa.f,v 1.1 2006/02/07 22:45:29 gumley Exp $
c
c!Team-unique Header:
c
c!End
c
c-----------------------------------------------------------------------
      
      implicit none

      
      include 'mod06uw_data.inc'
      include 'mod06uw_debug.inc'

c ... arguments

      integer line, pixel, nmissing, ncloudy, 
     +        qual_flag, conf_flag, hc_flag, ci_flag

c ... local variables

      integer iout, jout, nclear
      
c whuang 05/07/01: Added save statement for common / mod06_data / and
c                  / mod06_debug /
      save / mod06_data /, / mod06_debug /

c ... Pixel and line coordinates in output 5x5 sampled array
      
      iout = ( pixel / isamp ) + 1
      jout = ( line / isamp ) + 1

c ... If retrieval was good, set product run time QA flags      

      if ( qual_flag .eq. 1 ) then
      
c ...   Set the quality bits first
c ...   Cloud Top Pressure 
        call set_qa_bit( product_qa( 1, iout, jout ), 0 )

c ...   Cloud Top Temperature
        call set_qa_bit( product_qa( 1, iout, jout ), 4 )

c ...   Cloud Fraction 
        call set_qa_bit( product_qa( 1, iout, jout ), 8 )

c ...   Cloud Effective Emissivity
        call set_qa_bit( product_qa( 1, iout, jout ), 12 )

c ...   Now the confidence bits

        if (conf_flag .eq. 1) then

c ...     Cloud Top Pressure 
          call set_qa_bit( product_qa( 1, iout, jout ), 1 )

c ...     Cloud Top Temperature
          call set_qa_bit( product_qa( 1, iout, jout ), 5 )

c ...     Cloud Fraction 
          call set_qa_bit( product_qa( 1, iout, jout ), 9 )

c ...     Cloud Effective Emissivity
          call set_qa_bit( product_qa( 1, iout, jout ), 13 )
 
        else if (conf_flag .eq. 2) then

c ...     Cloud Top Pressure 
          call set_qa_bit( product_qa( 1, iout, jout ), 2 )

c ...     Cloud Top Temperature
          call set_qa_bit( product_qa( 1, iout, jout ), 6 )

c ...     Cloud Fraction 
          call set_qa_bit( product_qa( 1, iout, jout ), 10 )

c ...     Cloud Effective Emissivity
          call set_qa_bit( product_qa( 1, iout, jout ), 14 )

        else if (conf_flag .eq. 3) then

c ...     Cloud Top Pressure 
          call set_qa_bit( product_qa( 1, iout, jout ), 1 )
          call set_qa_bit( product_qa( 1, iout, jout ), 2 )

c ...     Cloud Top Temperature
          call set_qa_bit( product_qa( 1, iout, jout ), 5 )
          call set_qa_bit( product_qa( 1, iout, jout ), 6 )

c ...     Cloud Fraction 
          call set_qa_bit( product_qa( 1, iout, jout ), 9 )
          call set_qa_bit( product_qa( 1, iout, jout ), 10 )

c ...     Cloud Effective Emissivity
          call set_qa_bit( product_qa( 1, iout, jout ), 13 )
          call set_qa_bit( product_qa( 1, iout, jout ), 14 )

        endif
      endif

c ... Don't forget to set the cirrus and high cloud flags
      if (ci_flag .eq. 2 ) then
        call set_qa_bit( product_qa( 1, iout, jout ), 21 )
      else if  (ci_flag .eq. 1 ) then
        call set_qa_bit( product_qa( 1, iout, jout ), 20 )
      endif
      if (hc_flag .eq. 2 ) then
        call set_qa_bit( product_qa( 1, iout, jout ), 23 )
      else if (hc_flag .eq. 1 ) then
        call set_qa_bit( product_qa( 1, iout, jout ), 22 )
      endif


c ... Set optional run time QA flags

      product_qa( 4, iout, jout ) = ncloudy
      nclear = isamp*isamp - nmissing - ncloudy
      product_qa( 5, iout, jout ) = nclear
      product_qa( 6, iout, jout ) = nmissing

c ... Finally, set the input data resource flags - Setting
c ...  here because I really don't know where else to
c ...  set them.
c ...  Clear radiance origin
       if (mod06ct_proc_opt .eq. 1 ) then
         call set_qa_bit( product_qa( 1, iout, jout ), 54 )
       else if (mod06ct_proc_opt .eq. 3 ) then
         call set_qa_bit( product_qa( 1, iout, jout ), 55 )
       endif

c ...  Moisture profile, Temp profile, surface temp over land
       if (tem1(pixel,line) .lt. 1.0) then
          call set_qa_bit( product_qa( 1, iout, jout ), 56 )
          call set_qa_bit( product_qa( 1, iout, jout ), 57 )
          call set_qa_bit( product_qa( 1, iout, jout ), 58 )
          call set_qa_bit( product_qa( 1, iout, jout ), 59 )
          call set_qa_bit( product_qa( 1, iout, jout ), 60 )
          call set_qa_bit( product_qa( 1, iout, jout ), 61 )
          call set_qa_bit( product_qa( 1, iout, jout ), 62 )
          call set_qa_bit( product_qa( 1, iout, jout ), 63 )
       endif

c ...  surface pressure
       if (pre1(pixel,line) .lt. 1.0) then
         call set_qa_bit( product_qa( 1, iout, jout ), 64 )
         call set_qa_bit( product_qa( 1, iout, jout ), 65 )
       endif
        
c ...  Surface emissivity currently not used
       call set_qa_bit( product_qa( 1, iout, jout ), 68 )
       call set_qa_bit( product_qa( 1, iout, jout ), 69 )

c ...  Ecosystem type
       if (eco(pixel,line) .lt. 0) then
         call set_qa_bit( product_qa( 1, iout, jout ), 70 )
         call set_qa_bit( product_qa( 1, iout, jout ), 71 )
       endif

c ... debug statement ............................................
      if (debug .gt. 2) then
         write(h_output,'(1x,3x,'' CT QA Bits set '')')
         write(h_output,'(''qual_flag,conf_flag,hc_flag,ci_flag '',
     &    4i5)') qual_flag,conf_flag,hc_flag,ci_flag
      endif
c ................................................................



      
      return
      end
