      program Driver_MOD_PR06OD

c-------------------------------------------------------------------
c
c!F77
c
c!Description:
c Driver program for MODIS cloud property algorithm.
c
c!Input Parameters:
c None
c
c!Output Parameters:
c None
c
c!Revision History:
c Revision 3.0  2004/08/24 mag
c the new driver
c
c!Team-unique Header:
c Cloud Retrieval Group, NASA/GSFC, Code 913, Greenbelt, Maryland, USA
c
c!Credit and Reference:
c Programmed by:
c Mark Gray (mag)
c L-3 GSI
c NASA/GSFC, code 913
c Greenbelt, Maryland, USA
c
c!End
c---------------------------------------------------------------------
      implicit none
 
      include 'mod06.inc'
      include 'Atmos_AncData.inc'
      include 'PGS_SMF.f'
      include 'PGS_PC.f'
      include 'PGS_MODIS_39500.f'
      include 'mapi.inc'

c.....Parameter Declarations
      character*(*) FUNCNAME
      PARAMETER    (FUNCNAME = 'Driver_MOD_PR06OD')

      integer       FAIL,    SUCCEED
      parameter    (FAIL=-1, SUCCEED=0)

      integer       INVENTORYMETADATA,     ARCHIVEMETADATA
      parameter    (INVENTORYMETADATA = 2, ARCHIVEMETADATA=3)

      integer       MAX_NUM_MEAS_PARM
      parameter    (MAX_NUM_MEAS_PARM = 10)

      integer       MAX_NUM_PSA
      parameter    (MAX_NUM_PSA = 20)

      integer       MAX_NUM_INPUT_PNTRS
      parameter    (MAX_NUM_INPUT_PNTRS = 50)

      integer       MAX_NUM_ArchivePSA_SC
      parameter    (MAX_NUM_ArchivePSA_SC = 8)

      real          MIN_SZA_THRESHOLD
      parameter    (MIN_SZA_THRESHOLD = 81.4)


c*****************************************************
c     Variable Declarations
c*****************************************************

c.....Function declarations
      integer  Set_InvMet_MOD06,
     1         Set_ArchMet_MOD06,
     2         Set_MinECSMet,
     3         chk_input_L2,string_loc,
     4         pgs_met_getpcattr_s,pgs_pc_getreference

c.....ECS archive metadata variable declarations
      character*100 Name_ArchivePSA_SC(MAX_NUM_ArchivePSA_SC)

      integer NUM_ArchivePSA_SC

      real Value_ArchivePSA_SC(MAX_NUM_ArchivePSA_SC)


c.....ECS inventory metadata variables
      real     statistics(7)

      character*30   Auto_QA_Flag(MAX_NUM_MEAS_PARM) / MAX_NUM_MEAS_PARM*' ' /
      character*30   Auto_QA_Expl(MAX_NUM_MEAS_PARM) / MAX_NUM_MEAS_PARM*' ' /
      character*30   PSAName(MAX_NUM_PSA) / 'SuccessCloudOptPropRtrPct_VIS',
     1                                      'CloudCoverFractionPct_VIS',
     2                                      'WaterCloudDetectedPct_VIS', 
     3                                      'IceCloudDetectedPct_VIS',
     4                                       16*' ' /
      character*60   Name_MeasParm(MAX_NUM_MEAS_PARM) / MAX_NUM_MEAS_PARM*' ' /
      character*2048                    msgbuf
      character*(PGSd_MET_GROUP_NAME_L) Met_Handles(PGSd_MET_NUM_OF_GROUPS)
      character*(MAX_ECS_NAME_L-1)      HDFAttNms(PGSd_MET_NUM_OF_GROUPS)

      integer QA_Miss(MAX_NUM_MEAS_PARM)     / MAX_NUM_MEAS_PARM*0 /
      integer NUM_Input_LRN, 
     1        NUM_MODIS_InputFiles, 
     2        NUM_MeasParam, 
     3        NUM_PSA 
      integer Input_LRN     (MAX_NUM_INPUT_PNTRS)  /MAX_NUM_INPUT_PNTRS*-99999/,
     1        Input_Vrsn    (MAX_NUM_INPUT_PNTRS)  /MAX_NUM_INPUT_PNTRS*1/,
     2        ANC_Input_LRN (MAX_NUM_INPUT_PNTRS)  /MAX_NUM_INPUT_PNTRS*-99999/

      logical ExtGeoPntr_Flag 

      real    PSAValue(MAX_NUM_PSA) 


c.....Declare other internal variables
      character*8                       msg8a,msg8b
      character*256                     HDFAttrName,ECSParmName
      character*(PGSd_PC_FILE_PATH_MAX) FileName_MOD06

      integer NUM_of_HDFAttrNms
      integer fbytea,fbyteb,lbytea,lbyteb
      integer i,ErrorFlag
      integer rtn,rtn_3,rtn_GEO,rtn_L1B,rtn_CldMsk
      integer Index1,Index2,Vrsn_No,modfil_out(MODFILLEN)

      logical error_flag, error_flag1  

      real    MinSolarZenithAngle


c--------------------------------------------------------------------------------
c Begin executable code by writing "Begin" message to LogStatus file 
c--------------------------------------------------------------------------------

      msgbuf = 
     1   char(10)
     2   // '--------------------------------------------------'
     3   // char(10)
     4   // 'Begin Cloud Optical Properties Retrieval Algorithm'
     5   // char(10)
     6   // '--------------------------------------------------'

      call MODIS_SMF_SETDYNAMICMSG(MODIS_M_GENERIC,msgbuf,FUNCNAME)


c Initialize variables and arrays
      error_flag  = .FALSE.
      error_flag1 = .FALSE.


c--------------------------------------------------------------------------------
c Perform Cloud Optical Properties Retrieval 
c--------------------------------------------------------------------------------

c.....do input file time checks
      Vrsn_No    = 1
      rtn_GEO    = chk_input_L2(LRN_Geo,   Vrsn_No)
      rtn_L1B    = chk_input_L2(LRN_L1B,   Vrsn_No)
      rtn_CldMsk = chk_input_L2(LRN_CldMsk,Vrsn_No)
      rtn_3 = rtn_L1B + rtn_GEO + rtn_CldMsk

      If (rtn_3 .ne. 0) Then
         error_flag = .TRUE.

         If (rtn_GEO .ne. SUCCEED) Then
            msgbuf = 'MODIS Geolocation input granule failed time check'
            call MODIS_SMF_SETDYNAMICMSG(MODIS_E_GENERIC,msgbuf,FUNCNAME)
         End If

         If (rtn_L1B .ne. SUCCEED) Then
            msgbuf = 'MODIS L1B 1km input granule failed time check'
            call MODIS_SMF_SETDYNAMICMSG(MODIS_E_GENERIC,msgbuf,FUNCNAME)
         End If

         If (rtn_CldMsk .ne. SUCCEED) Then
            msgbuf = 'MODIS CldMsk input granule failed time check'
            call MODIS_SMF_SETDYNAMICMSG(MODIS_E_GENERIC,msgbuf,FUNCNAME)
         End If


c.....Read granule Minimum Solar Zenith Angle from Cloud Mask product 
      Else
         msgbuf = 'Temporal coverage of MODIS L1 and L2 input products matches '
     1            // 'current collection period'
         call MODIS_SMF_SETDYNAMICMSG(MODIS_S_GENERIC,msgbuf,FUNCNAME)

         HDFAttrName = MECS_CORE
         ECSParmName = 'MinSolarZenithAngle'

         rtn = pgs_met_getpcattr_s(LRN_CldMsk,Vrsn_No,HDFAttrName,ECSParmName,msg8a)

         If (rtn .ne. PGS_S_SUCCESS) Then
            error_flag = .TRUE.
            msgbuf = 'pgs_met_getpcattr_s unable to retrieve MinSolarZenithAngle '
     1               // 'from Cloud Mask product.' // char(10) // 'Halt processing.'
            call MODIS_SMF_SetDynamicMSG(MODIS_E_GENERIC,msgbuf,FUNCNAME)


c........check that MinSolarZenithAngle criterion are met to enable retrieval
         Else
            read(msg8a,'(f8.2)') MinSolarZenithAngle

            If (MinSolarZenithAngle .ge. MIN_SZA_THRESHOLD) Then
               write(msg8a,'(f8.2)') MinSolarZenithAngle
               write(msg8b,'(f8.2)') MIN_SZA_THRESHOLD
               rtn = string_loc(msg8a,fbytea,lbytea)
               rtn = string_loc(msg8b,fbyteb,lbyteb)

               msgbuf = 'Minimum Solar zenith angle in scene, ' // msg8a(fbytea:lbytea) 
     1                  // ', is greater than ' // msg8b(fbyteb:lbyteb) // '-degrees ' 
     2                  // char(10) // 'processing threshold.  The Cloud Optical Properties '
     3                  // 'code will not be run on this scene.'
     4                  // char(10) // 'Preparing to exit normally. '

               call MODIS_SMF_SETDYNAMICMSG(MODIS_M_GENERIC,msgbuf,FUNCNAME)


c...........Perform cloud optical properties retrieval
            Else
               call MOD_PR06OD(statistics,ErrorFlag)
               if (ErrorFlag .eq. 0 ) then
                 msgbuf = 'MOD_PR06OD Science call completed with no errors.'
                 call MODIS_SMF_SETDYNAMICMSG(MODIS_S_GENERIC,msgbuf,FUNCNAME)
               else
                 msgbuf = 'MOD_PR06OD Science call reported a fatal exception. '//
     1                    'See earlier messages for details.'
                 call MODIS_SMF_SETDYNAMICMSG(MODIS_E_GENERIC,msgbuf,FUNCNAME)
               endif

            End If   !Solar zenith angle check

         End If   !Retrieve MinSolarZenithAngle PSA from Cloud Mask

      End If   !MODIS input time checks


c--------------------------------------------------------------------------------
c Define parameters to be passed to ECS metadata writers - separately for 
c "Day" (MinSolarZenithAngle <  MIN_SZA_THRESHOLD) and "Night".
c--------------------------------------------------------------------------------

c.....Nighttime 
      If (MinSolarZenithAngle .ge. MIN_SZA_THRESHOLD) Then 

c........Inventory
         Input_LRN(1)    = LRN_L1B         ! Input Pointers
         ExtGeoPntr_Flag = .FALSE.         ! External Geolocation Pointer

         NUM_PSA       = 0
         NUM_MeasParam = 0
         NUM_Input_LRN = 1

c.....Day
      Else 

c........add MODIS products to list of inputs
         NUM_Input_LRN = 18 

         Input_LRN(1)  = LRN_Geo 
         Input_LRN(2)  = LRN_L1B 
         Input_LRN(3)  = LRN_CldMsk 

c rhucek 08/26/02: revise list of library files  
         Input_LRN(4)  = 417501
         Input_LRN(5)  = 417502
         Input_LRN(6)  = 417521
         Input_LRN(7)  = 417522
         Input_LRN(8)  = 417523
         Input_LRN(9)  = 417524
         Input_LRN(10) = 417525
         Input_LRN(11) = 417526
         Input_LRN(12) = 417527
         Input_LRN(13) = 417540 
        
c........add ancillary products to list of inputs
         Input_LRN(14) = LUN_GDAS_0ZF
         Input_LRN(15) = LUN_OZ_DAILY
         Input_LRN(16) = LUN_REYNSST
         Input_LRN(17) = LUN_SEA_ICE
         Input_LRN(18) = LUN_NISE

         NUM_PSA     = 4 
         PSAValue(1) = statistics(1)               ! MOD_PR06OD PSAs
         PSAValue(2) = Statistics(2)
         PSAValue(3) = Statistics(3)
         PSAValue(4) = Statistics(4)

         ExtGeoPntr_Flag = .TRUE.                  ! External Geolocation Pointer

         NUM_MeasParam = 0
         
c........No Archive metadata set by MOD_PR06OD 
         NUM_ArchivePSA_SC       = 0

      EndIf


c-------------------------------------------------------------------------------
c Write ECS metadata to MOD06_L2 HDF product, but for now only if Day.   
c-------------------------------------------------------------------------------

      If (MinSolarZenithAngle .lt. MIN_SZA_THRESHOLD) Then 

         Vrsn_No = 1

c........retrieve name of hdf product from pcf
         rtn = pgs_pc_getreference(LRN_MOD06,Vrsn_No,FileName_MOD06)

         If (rtn .ne. PGS_S_SUCCESS) Then
            error_flag = .TRUE.

            msgbuf = 
     1      'pgs_pc_getreference detected error retrieving Cloud Product file name from PCF. ' 
     3      // char(10) // 'Cloud Optical Properties Retrieval code to exit.' 
     4      // char(10) // 'Operator Action:  Notify SDST. '

            Call modis_smf_setdynamicmsg(MODIS_E_GENERIC,msgbuf,FUNCNAME)

c........Reopen Cloud Product HDF file for MAPI access
         Else
            rtn = opmfil(FileName_MOD06, 'a', Modfil_Out)

            If (rtn .ne. MAPIOK) Then
               error_flag = .TRUE.

               msgbuf = 
     1         'OPMFIL detected error reopening Cloud Product.  Halt Processing.'
     2         // char(10) // 'Operator Action:  Notify SDST. '

               Call modis_smf_setdynamicmsg(MODIS_E_GENERIC,msgbuf,FUNCNAME)
            Else


c..............write ECS inventory metadata
               rtn = Set_InvMet_MOD06(ExtGeoPntr_Flag,
     1                                NUM_Input_LRN,Input_LRN,Input_Vrsn,
     2                                NUM_MeasParam,Name_MeasParm,QA_Miss,
     3                                Auto_QA_Flag,Auto_QA_Expl,
     4                                NUM_PSA,PSAName,PSAValue,
     5                                Met_Handles)
               If (rtn .eq. FAIL) Then
                  error_flag = .TRUE.

                  msgbuf = 
     1            'Set_InvMet_MOD06 detected error attempting to '
     2            // char(10) // 'update ECS inventory metadata. '
     3            // char(10) // 'Operator Action:  Notify SDST. '

                  Call MODIS_SMF_SETDYNAMICMSG(MODIS_E_GENERIC,msgbuf,FUNCNAME)
               Else
                  msgbuf = 
     1            'Set_InvMet_MOD06 successfully updated ECS ' 
     2            // char(10) // 'inventory metadata. '

                  Call MODIS_SMF_SETDYNAMICMSG(MODIS_S_GENERIC,msgbuf,FUNCNAME)
               End If
 

C---------------------------------------------------------------------
C      Set Archive Metadata
C
C Note:  Set_ArchMet_MOD06 uses the same three arrays (No_Input_LRN,
C        Input_LRN and Input_Vrsn) to identify input file entries in 
C        the PCF as does Set_InvMet_MOD06.  Unlike Set_InvMet_MOD06, 
C        it retrieves and saves the file name, rather than the "UR", 
C        (Universal Reference) to the HDF product metadata. 
C
C---------------------------------------------------------------------

c rhucek 08/26/02:  Replaced NUM_MODIS_InputFiles, LRN_MODIS_InputFiles
c                   and VRSN_MODIS_InputFiles with NUM_Input_LRN, 
c                   Input_LRN and Input_Vrsn.  
c
c                   That is, we now track all input filess in the 
c                   Archive metadata, not just MODIS products.  
c                   Declarations and definitions of the former 
c                   variables were removed. 

               rtn = Set_ArchMet_MOD06( Met_Handles,
     1                                  NUM_Input_LRN, 
     2                                  Input_LRN, 
     3                                  Input_Vrsn, 
     4                                  NUM_ArchivePSA_SC,
     5                                  Name_ArchivePSA_SC,
     6                                  Value_ArchivePSA_SC)

               If (rtn .eq. FAIL) Then
                  error_flag = .TRUE.

                  msgbuf = 
     1            'Set_ArchMet_MOD06 detected error attempting to '
     2            // char(10) // 'set ECS archive metadata. '
     3            // char(10) // 'Operator Action:  Notify SDST. '

                  Call MODIS_SMF_SETDYNAMICMSG(MODIS_E_GENERIC,msgbuf,FUNCNAME)
               Else
                  msgbuf = 'Set_ArchMet_MOD06 successfully set ECS archive metadata. '

                  Call MODIS_SMF_SETDYNAMICMSG(MODIS_S_GENERIC,msgbuf,FUNCNAME)
               End If
 

c..............write ECS metadata to cloud product and close file
               Num_of_HDfAttrNms = 2      
               HDFAttNms(INVENTORYMETADATA) = MECS_CORE
               HDFAttNms(ARCHIVEMETADATA)   = MECS_ARCHIVE

               rtn = CPMFIL(Modfil_Out,Met_Handles,HDFAttNms,Num_Of_HDFAttrNms)

               If (rtn .ne. MAPIOK) Then 
                  error_flag = .TRUE.

                  msgbuf = 
     1            'CPMFIL detected error closing Cloud Product HDF file. '
     2            // char(10) // 'Operator Action:  Notify SDST. '

                  Call MODIS_SMF_SETDYNAMICMSG(MODIS_E_GENERIC,msgbuf,FUNCNAME)
               Else
                  msgbuf = 'CPMFIL successfully closed Cloud Product HDF file. '

                  Call MODIS_SMF_SETDYNAMICMSG(MODIS_S_GENERIC,msgbuf,FUNCNAME)
               End If

            End If   !Re-open cloud product HDF file 

         End If   !Retrieve cloud product file name from PCF 

      End If   !Solar zenith angle check



c--------------------------------------------------------------------------------
c Exit Cloud Optical Properties Retrieval code 0 (Success) of 1 (Fail) 
c--------------------------------------------------------------------------------

      If (error_flag) Then
         msgbuf = 'MODIS Cloud Optical Properties Retrieval code failed, exiting code 1.'

         call MODIS_SMF_SetDynamicMSG(MODIS_E_GENERIC,msgbuf,FUNCNAME)
         call Exit(1)
      Else
         msgbuf = 'MODIS Cloud Optical Properties Retrieval code completed successfully,'
     1   // char(10) // 'exiting code 0.'

         call MODIS_SMF_SetDynamicMSG(MODIS_S_GENERIC,msgbuf,FUNCNAME)
         call Exit(0)
      End If

      End
