module ancillary_module

 implicit none
 private

 public :: set_ancillary

contains

subroutine set_ancillary( &
                         gdas_name,                 &
                         ncepice_name,              &
                         sst_name,                  &
                         nise_name,                 &
                         ozone_name,                &
                         mod06input_name,           &
                         mod06input_filedata,       &
                         surfacealbedo_lib_659,      &
                         surfacealbedo_lib_858,      &
                         surfacealbedo_lib_124,      &
                         surfacealbedo_lib_164,      &
                         surfacealbedo_lib_21,       &
                         ecosystem_data_name,        &
                         snowicealbedo_data_name,    &
                         mod06_start, mod06_stride, mod06_edge, &
                         debug,                     &
                         status)
   use nonscience_parameters
   use GeneralAuxType
   use core_arrays

   implicit none
   logical,     intent(in)  :: debug
   integer,     intent(in),dimension(:)  :: mod06_start, mod06_stride, mod06_edge,mod06input_filedata
   character(*),intent(in)  :: gdas_name, ncepice_name, sst_name, &
                               nise_name,   &
                               ozone_name, mod06input_name,  &
                               surfacealbedo_lib_659,      &
                               surfacealbedo_lib_858,      &
                               surfacealbedo_lib_124,      &
                               surfacealbedo_lib_164,      &
                               surfacealbedo_lib_21,       &
                               ecosystem_data_name,        &
                               snowicealbedo_data_name

   integer,     intent(out) :: status

   real(single),allocatable,dimension(:,:)   ::  land_flag, prmsl, pwat, ugrd, vgrd, icec, sst, surfacetemp,ozone
   real(single),allocatable,dimension(:,:,:) ::  pressurelevels, temp_profiles, water_profiles
   integer                                   :: xsize, ysize, levels, i,j
   integer, allocatable, dimension(:)         :: mod06_edge_5km, mod06_stride_5km, mod06_start_5km

   status = 0
   levels = 16

   allocate(mod06_edge_5km(size(mod06_edge)))
   allocate(mod06_stride_5km(size(mod06_stride)))
   allocate(mod06_start_5km(size(mod06_start)))


   xsize = size(latitude,1)
   ysize = size(latitude,2)
   allocate(land_flag(xsize, ysize))
   allocate(surfacetemp(xsize, ysize))
   allocate(prmsl(xsize, ysize))
   allocate(pwat(xsize, ysize))
   allocate(ugrd(xsize, ysize))
   allocate(vgrd(xsize, ysize))
   allocate(icec(xsize, ysize))
   allocate(sst(xsize, ysize))
   allocate(ozone(xsize, ysize))
   allocate(pressurelevels(xsize,ysize,0:levels-1))
   allocate(temp_profiles(xsize,ysize,0:levels-1))
   allocate(water_profiles(xsize,ysize,0:levels-1))
   status = 0
 
   call read_ancillary_grids(  &
                  latitude,      &
                  longitude,     &
                  gdas_name,     &
                  ncepice_name,  &
                  sst_name,      &
                  ozone_name,    &
                  pressurelevels,&
                  temp_profiles, &
                  water_profiles,&
                  land_flag,     &
                  surfacetemp,   &
                  prmsl,         &
                  pwat,          &
                  ugrd,          &
                  vgrd,          &
                  ozone,         &
                  icec,          &
                  sst)
     
    surface_temperature = surfacetemp

    do i= 1, xsize
      do j  = 1, ysize
         if (cloudsummary(i,j)%ocean_surface) then !  .and. land_flag(i,j) == 0.) then
           !this is a ocean surface pixel
           surface_temperature(i,j) = sst(i,j)
         endif      
      enddo
    enddo
   mod06_start_5km  = floor(real(mod06_start)/5.)
   mod06_stride_5km = floor(real(mod06_stride)/5.)
   mod06_edge_5km   = floor(real(mod06_edge)/5.)

  call get_cloud_top_properties(mod06input_filedata,   &
                                mod06input_name,       &
                                mod06_start_5km,       &
                                mod06_stride_5km,      &
                                mod06_edge_5km,        &
                                cloud_top_pressure,    &
                                cloud_top_temperature, &
                                status)
  call get_above_cloud_properties(temp_profiles, &
                                   water_profiles,   &
                                   pressurelevels,  &
                                   cloud_top_pressure,   &
                                   abovecloud_watervapor,&
                                   above_cloud_temperature, status)

  call get_surface_albedo(latitude, &
                          longitude,&
                          nise_name, &
                          surfacealbedo_lib_659,      &
                          surfacealbedo_lib_858,      &
                          surfacealbedo_lib_124,      &
                          surfacealbedo_lib_164,      &
                          surfacealbedo_lib_21,       &
                          ecosystem_data_name,        &
                          snowicealbedo_data_name,    &
                          debug, &
                          icec, &
                          surface_albedo, &
                          status)
   
  model_temp_profile(:,:,1:16)    = temp_profiles(:,:,0:15)
  model_water_profile(:,:,1:16)   = water_profiles(:,:,0:15)
  model_pressure_levels(:,:,1:16) = pressurelevels(:,:,0:15)
  column_ozone = ozone 
  deallocate( land_flag,surfacetemp,prmsl,pwat,ugrd,vgrd,icec,sst,ozone)
  deallocate( pressurelevels, temp_profiles, water_profiles )
  deallocate( mod06_start_5km, mod06_stride_5km, mod06_edge_5km)
end subroutine set_ancillary

subroutine read_ancillary_grids(  lat, lon, ncepgdas_name, ice_name,sst_name,  &
                                 ozone_name, pres, &
                                 temp, mixr, land, sfctmp,      &
                                 prmsl, pwat, ugrd, vgrd, ozone,  icec, sst )
  use GeneralAuxType
  use nonscience_parameters
  use global_model_grids
  implicit none

! rhucek 01/12/05:  added local_read_reynsst interface
    interface
subroutine local_read_reynsst ( filename,  &
                          year, month, day, hour, sst, &
                          file_format, sst_success )

  implicit none
      character*(*) , intent(in)   ::  filename
      integer,        intent(out)  ::  year, month, day, hour
      logical,        intent(out)  ::  sst_success
      real,           intent(out)  ::  sst(0:, 0:)
      character*(*),  intent(out)  ::  file_format
end subroutine local_read_reynsst
     end interface

  include 'Atmos_AncData.f90.inc'

  !-----------------------------------------------------------------------
  ! !f90
  !
  ! !description:
  !      retrieve ancillary data items for a given set of 
  !      latitude and longitude pairs.
  !
  ! !input parameters:
  !      lat       latitude (degrees, -90s to +90.0n)
  !      lon       longitude (degrees, -180w to +180e, greenwich=0)
  !
  ! !output parameters:
  !      pres      array of pressure levels (hpa)
  !      temp      array of atmospheric temperatures (k) at pres(0:15)
  !      mixr      array of water vapor mixing ratios (g/kg) at pres(0:15)
  !      land      land mask (0=water, 1=land)
  !      sfctmp    surface temperature (k)
  !      prmsl     pressure (hpa) at mean sea level
  !      pwat      precipitable water (g/cm**2)
  !      ugrd      surface wind u component (m/s)
  !      vgrd      surface wind v component (m/s)
  !      Ozone     total column Ozone (dobson units)
  !      icec      ice concentration (fraction)
  !
  ! !notes
  !      Modified from the MOD_PR06OD Collection 4 ReadNCEP.f algorithm written
  !      by Liam Gumley and Jason Li.

  real(single), intent(in)     :: lat(:,:),lon(:,:)
  character(*), intent(in)     :: ncepgdas_name, ice_name, sst_name,ozone_name

  real(single), intent(out)     ::  pres(:,:,0:), temp(:,:,0:), mixr(:,:,0:), &
                                    land(:,:), sfctmp(:,:), prmsl(:,:),  &
                                    pwat(:,:), ugrd(:,:), vgrd(:,:),   &
                                    icec(:,:), sst(:,:), ozone(:,:)

  ! ... parameter definitions
  integer     npoints_x,        npoints_y  
  parameter ( npoints_x = 360,  npoints_y = 180 )

  real        missing
  parameter ( missing = -999.0 )

  ! ... declaration of local variables and  arrays
  character*3    sst_file_fmt
  character*8    esdt_name_ncep, esdt_name_ozone, esdt_name
  character*7    esdt_name_ice
  character*160  errmsg
  character*13   ncepgdastemp_name, ozonetemp_name
  character*12   icetemp_name

  integer header( 0:7 ), i, ios, j, k, level, lun, pcfnum, reclen, status

  integer data_xsize, data_ysize, grid_i, grid_j

  real(single)  ::   p( 0:15 ), satmix, x, x0, xlon, dx, y, y0, dy

  ! ... declaration of external functions
  integer modis_grib_driver , openr_temp
  external modis_grib_driver , openr_temp

  real     svpwat
  external svpwat

 ! ... define temperature and moisture profile pressure levels (hpa)
  data p / 1000.0, 925.0, 850.0, 700.0, 500.0, 400.0, 300.0, 250.0, &
          200.0, 150.0, 100.0,  70.0,  50.0,  30.0,  20.0,  10.0 /

  !-----------------------------------------------------------------------
  !      begin executable code 
  !-----------------------------------------------------------------------
  ! ... read input data files if this is the first call

  data_xsize = size(lat,1)
  data_ysize = size(lat,2)

  if(.not. grids_are_read) then
  grids_are_read = .true. 
  ! ..... set data ingest success/fail flags
  met_success  = .false.
  ice_success  = .false.
  sst_success  = .false.
  ozn_success  = .false.

  do i = 1, 4
    met_date( i )  = int( missing )
    ice_date( i )  = int( missing )
    sst_date( i ) =  int( missing )
    ozn_date( i )  = int( missing )
  end do

!-----------------------------------------------------------------------
! Get TOVS Ozone Data
!-----------------------------------------------------------------------
! Unpack grib ozone file and write to binary file
  errmsg    = ' '
  ESDT_name = 'OZ_DAILY'
  status    = modis_grib_driver( LUN_OZ_DAILY, LUN_TEMP_OZ_DAILY, &
                                 ESDT_name, errmsg, &
                                 ozn_year, ozn_month, &
                                 ozn_day, ozn_hour )

  if ( status /= success ) then
    status = failure
    call local_message_handler ('error reported from grib driver, ozone read, Check PCF file', &
                                status, &
                                'read_ancillary_grids')
  else
    ! Open unpacked ozone file
    pcfnum = LUN_TEMP_OZ_DAILY
    reclen = 360*181*4
    status = openr_temp( pcfnum, reclen, lun )

    if ( status /= success ) then
      status = failure
      call local_message_handler ('error reported from openr, ozone read, Check PCF file', &
                                status, &
                                'read_ancillary_grids')
    else
      ! Read the unpacked ozone file
      if ( status == success ) then
        read( lun, rec = 1, iostat = ios ) ozn_grid
          if ( ios /= 0 ) then
            level = 2
            call local_message_handler ('error reported ozone temp file read, Check PCF file', &
                                status, &
                                'read_ancillary_grids')
          else
            ozn_date( 1 ) = ozn_year
            ozn_date( 2 ) = ozn_month
            ozn_date( 3 ) = ozn_day
            ozn_date( 4 ) = ozn_hour
          endif
        call modis_io_gen_closef( pcfnum, lun )
      endif
    endif
  endif

  !-----------------------------------------------------------------------
  !      get ncep meteorological data
  !-----------------------------------------------------------------------
  ! ...   unpack grib met file and write to binary file
  errmsg    = ' '
  ESDT_name = 'GDAS_0ZF'
  status    = modis_grib_driver( LUN_GDAS_0ZF, LUN_TEMP_GDAS_0ZF, &
                                 ESDT_name, errmsg, &
                                 met_year, met_month, &
                                 met_day, met_hour )
 
  if ( status /= success ) then
    status = failure
    call local_message_handler ('error reported from grib driver, ncep read, Check PCF file', &
                                status, &
                                'read_ancillary_grids')

    ! ....  open unpacked met file
  else
    pcfnum = LUN_TEMP_GDAS_0ZF
    reclen = 360*181*30*4
    status = openr_temp( pcfnum, reclen, lun )

    if ( status /= 0 ) then
      status = 2
      call local_message_handler ('error reported from openr, ncep read, Check PCF file', &
                                status, &
                                'read_ancillary_grids')
    else 
      !  read the unpacked met file
      if ( status == 0 ) then
        ios= 0
        read( lun, rec = 1, iostat = ios ) met_grid

        if ( ios /= 0 ) then
          level = 2
        else
          met_success   = .true.
          met_date( 1 ) = met_year
          met_date( 2 ) = met_month
          met_date( 3 ) = met_day
          met_date( 4 ) = met_hour
        endif

        call modis_io_gen_closef( pcfnum, lun )
      endif
    endif
  endif

!-----------------------------------------------------------------------
!     Get SSM/I sea ice concentration
!-----------------------------------------------------------------------
! ...   Unpack grib sea ice file and write to binary file
  errmsg    = ' '
  ESDT_name = 'SEA_ICE'
  status    = modis_grib_driver( LUN_SEA_ICE, LUN_TEMP_SEA_ICE, &
                                 ESDT_name, errmsg, &
                                 ice_year, ice_month, &
                                 ice_day, ice_hour )
  if ( status /= success ) then
    status = failure
    call local_message_handler ('error reported from grib driver, NCEP sea ice read, Check PCF file', &
                                status, &
                                'read_ancillary_grids')
  else

!   Open unpacked ice file
    pcfnum = LUN_TEMP_SEA_ICE
    reclen = 720*360*4
    status = openr_temp( pcfnum, reclen, lun )

    if ( status /= success ) then
      status = failure
      call local_message_handler ('error reported opening temp sea ice, Check PCF file', &
                                  status, &
                                  'read_ancillary_grids')
    else
!     Read the unpacked ice file
      if ( status == 0 ) then
        ios  = 0
        read( lun, rec = 1, iostat = ios )ice_grid
        if ( ios /= success ) then
          status = success
          call local_message_handler ('error reported reading temp sea ice file, Check PCF file', &
                                status, &
                                'read_ancillary_grids')
        else
          ice_success   = .true.
          ice_date( 1 ) = ice_year
          ice_date( 2 ) = ice_month
          ice_date( 3 ) = ice_day
          ice_date( 4 ) = ice_hour
        endif
        call modis_io_gen_closef( pcfnum, lun )
      endif
    endif
  endif


!----------------------------------------------------------------------
! Get Reynolds sea surface temperature
!----------------------------------------------------------------------
  call local_read_reynsst ( sst_name, sst_year, sst_month, &
                      sst_day, sst_hour, sst_grid, sst_file_fmt, &
                      sst_success)
  if( (.not. sst_success) ) then

    status = failure
    call local_message_handler ('error reported from sst read, Check PCF file', &
                                status, &
                                'read_ancillary_grids')
  else
    sst_date( 1 ) = sst_year
    sst_date( 2 ) = sst_month
    sst_date( 3 ) = sst_day
    sst_date( 4 ) = sst_hour

!   correct Y2K problem with SST year
    if ( sst_date( 1 ) .le. 99 ) then
      if ( sst_date( 1 ) .ge. 70 ) then 
        sst_date( 1 ) = sst_date( 1 ) + 1900
      else
        sst_date( 1 ) = sst_date( 1 ) + 2000
      endif
    endif
  end if 
  end if
! get lat/long data from grids

  do grid_i = 1, data_xsize
    do grid_j = 1, data_ysize
      ! save output pressure levels
      do k = 0, 15
        pres(grid_i,grid_j, k ) = p(k )
      end do

      if (lon(grid_i,grid_j) == -999. .or. lat(grid_i,grid_j) == -999.0) then
      land(grid_i, grid_j)   = -999.
      sfctmp(grid_i, grid_j) =  -999.
      prmsl(grid_i, grid_j)  =  -999.
      pwat(grid_i, grid_j)   =  -999.
      ugrd(grid_i, grid_j)   =  -999.
      vgrd(grid_i, grid_j)   =  -999.
      icec(grid_i, grid_j)   = -999.
      sst(grid_i, grid_j)    = -999.
      ozone(grid_i, grid_j)  = -999.
      mixr(grid_i, grid_j,: )  = -999.
      temp(grid_i, grid_j, : ) = -999.
      else
!     compute cell coordinates in met grid
      x = min( max( lon(grid_i,grid_j),  -179.99 ), 179.99 )
      if( x > -999. .and. x < 0.0 ) x = lon(grid_i,grid_j)+ 360.0
      x0 = 0.0
      dx = 1.0
      i = int( ( x - x0 + 0.5*dx ) / dx )

      if( i .eq. 360 ) i = 0

      y = min( max( lat(grid_i,grid_j), -89.99 ), 89.99 )
      y0 = 90.0
      dy = -1.0
      j = int( ( y - y0 + 0.5*dy ) / dy )


!     save output met data
!     (note that water vapor profile is relative humidity (%))

      do k = 0, 15
        temp(grid_i, grid_j, k ) = met_grid( i, j, k )
      end do

      do k = 0, 6
        mixr(grid_i, grid_j,k ) = met_grid( i, j, k + 16 )
      end do

      land(grid_i, grid_j)   = met_grid( i, j, 23 )
      sfctmp(grid_i, grid_j) = met_grid( i, j, 24 )
      prmsl(grid_i, grid_j)  = met_grid( i, j, 25 ) * 0.01
      pwat(grid_i, grid_j)   = met_grid( i, j, 26 )
      ugrd(grid_i, grid_j)   = met_grid( i, j, 27 )
      vgrd(grid_i, grid_j)   = met_grid( i, j, 28 )

!     convert relative humidity profile (%) to mixing ratio (g/kg)
      do k = 0, 6
!       compute mixing ratio at 100% relative humidity
        satmix = 622.0 * svpwat( temp(grid_i, grid_j, k ) ) / pres(grid_i, grid_j,k )

!       convert relative humidity to mixing ratio
         mixr(grid_i, grid_j, k ) = satmix * 0.01 * mixr(grid_i, grid_j, k )
      end do

!     extrapolate mixing ratio profile from 300 hPa to 10 hPa
      do k = 7, 15
        mixr(grid_i, grid_j, k ) = max( mixr(grid_i, grid_j, 6 ), 0.003 ) * ( pres(grid_i, grid_j, k ) / 300.0 )**3
        mixr(grid_i, grid_j, k ) = max( mixr(grid_i, grid_j, k ), 0.003 )
      end do

!     get ozone data
      ozone(grid_i, grid_j)  = ozn_grid( i, j )

!     compute cell coordinates in ice grid and save ice data
      x = min( max( lon(grid_i,grid_j), -179.99 ), 179.99 )
      if( x .lt. 0.0 ) x = lon(grid_i,grid_j) + 360.0
      x0 = 0.25
      dx = 0.5
      i = int( ( x - x0 + 0.5*dx ) / dx )
      if( i .eq. 720 ) i = 0

      y = min( max( lat(grid_i,grid_j), -89.99 ), 89.99 )
      y0 = 89.75
      dy = -0.5
      j = int( ( y - y0 + 0.5*dy ) / dy )

      icec(grid_i, grid_j) = ice_grid( i, j )

!     Compute cell coordinates in sst grid.  Binary SST data is
!     shifted 180 degrees relative to ASCII data.
      if (sst_file_fmt .eq. 'fmt') then
        x = min( max( lon(grid_i,grid_j), -179.99 ),  179.99 )
        x0 = -179.5
      else if (sst_file_fmt .eq. 'unf') then
        if(lon(grid_i,grid_j) .lt. 0.0) then
          xlon = lon(grid_i,grid_j) + 360.0
        else
          xlon = lon(grid_i,grid_j)
        end if
        x = min( max( xlon, 0.00 ),  359.99 )
        x0 = 0.5
      end if

      dx = 1.0
      i = int( ( x - x0 + 0.5*dx ) / dx )

      y = min( max( lat(grid_i,grid_j), -89.99 ), 89.99 )
      y0 = -89.5
      dy = 1.0
      j = int( ( y - y0 + 0.5*dy ) / dy )

!     Save output sst data
      
      sst(grid_i, grid_j) = sst_grid( i, j ) + 273.15
      endif
    enddo
  enddo

end subroutine read_ancillary_grids

subroutine get_surface_albedo(latitude, &
                              longitude,&
                              nise_filename, &
                              surfacealbedo_lib_659,      &
                              surfacealbedo_lib_858,      &
                              surfacealbedo_lib_124,      &
                              surfacealbedo_lib_164,      &
                              surfacealbedo_lib_21,       &
                              IGBPfilename,        &
                              snowicealbedo_data_name,    &
                              debug, &
                              icec, &
                              surface_albedo, &
                              status)
!-----------------------------------------------------------------------
!f90 modisretrieval
!
!Description:
!
! get surface albedos for all required bands
!
!input parameters:
!
!output parameters:
!
!revision history:
!
!team-unique header:
!
! Cloud Retrieval Group, NASA GSFC, Greenbelt, Maryland, USA
!
!references and credits:
!
! Mark Gray
! gray@climate.gsfc.nasa.gov
! EmergentIT
! Code 913, NASA GSFC
! Greenbelt, MD 20771
!
!
!design note:
!
!end
!----------------------------------------------------------------------

! collection 4 albedo determination
  use GeneralAuxType
  use nonscience_parameters
  use MOD06AlbedoEcoModule, only : getAlbedoEco
  use core_arrays, only: cloudsummary
  implicit none

  include 'PGS_PC.f'
  include 'PGS_TD.f'
  include 'PGS_SMF.f'
  
  real, dimension(:,:), intent(in)    :: latitude, longitude
  character(*),intent(in)             :: IGBPfilename,  &
                                         nise_filename, &
                                         surfacealbedo_lib_659,      &
                                         surfacealbedo_lib_858,      &
                                         surfacealbedo_lib_124,      &
                                         surfacealbedo_lib_164,      &
                                         surfacealbedo_lib_21,       &
                                         snowicealbedo_data_name
  logical, intent(in)                 :: debug 
  real(single), dimension(:,:), intent(in) ::  icec
  real, dimension(:,:,:), intent(out) :: surface_albedo
  integer, intent(out)                :: status 

  integer   (kind = integer_fourbyte),  parameter       :: MaxFileNameLength  = 2025
  integer   (integer_fourbyte),       &
              allocatable, dimension(:,:)               :: ecosystem
  integer   (integer_fourbyte)                          :: julianday,i,j,NumAlbWavelengths
  character (len = MaxFileNameLength),&
              allocatable, dimension(:)                 :: AlbedoWavelengthFN
  character (len = MaxFileNameLength)                   :: SnowAlbedoFN
  character (len = 10), allocatable, dimension(:)       :: WavelengthText
  real      (kind = single), allocatable, dimension(:)  :: Wavelength
  logical,    allocatable, dimension(:)                   :: ProcessLandAlbedo

  !Timestamp variables:
  integer (kind = integer_fourbyte), parameter          :: LUN_TimeStamp = 10258
  character (len = 40)                                  :: dateTime_a, dateTime_b
  integer (kind = integer_fourbyte)                     :: PGS_PC_GetConfigData, &
                                                           PGS_TD_asciitime_atob
  character (len = 3)                                   :: DayOfYearString


  allocate(ecosystem(size(latitude,1), size(latitude,2)))


  NumAlbWavelengths = 10
  !Allocate the wavelength arrays:
  allocate ( WavelengthText    (1:NumAlbWavelengths) )
  allocate ( Wavelength        (1:NumAlbWavelengths) )
  allocate ( AlbedoWavelengthFN(1:NumAlbWavelengths) )
  allocate ( ProcessLandAlbedo (1:NumAlbWavelengths) )

  !Define the wavelengths:
  WavelengthText(1) = "0.659"
  WavelengthText(2) = "0.858"
  WavelengthText(3) = "1.24"
  WavelengthText(4) = "1.64"
  WavelengthText(5) = "2.13"
  WavelengthText(6) = "3.7"
  WavelengthText(7) = ".94"
  WavelengthText(8) = "1.38"
  WavelengthText(9) = "8.5"
  WavelengthText(10) = "11.0"

  !Read in the HDF filenames containing the specified wavelengths from PCF file:
  ! Note to mark, these are read in from PCF file.
  AlbedoWavelengthFN(1) = surfacealbedo_lib_659 
  AlbedoWavelengthFN(2) = surfacealbedo_lib_858
  AlbedoWavelengthFN(3) = surfacealbedo_lib_124
  AlbedoWavelengthFN(4) = surfacealbedo_lib_164
  AlbedoWavelengthFN(5) = surfacealbedo_lib_21
  AlbedoWavelengthFN(6) = "notUsed"
  AlbedoWavelengthFN(7) = "notUsed"
  AlbedoWavelengthFN(8) = "notUsed"
  AlbedoWavelengthFN(9) = "notUsed"
  AlbedoWavelengthFN(10) = "notUsed"
  SnowAlbedoFN = snowicealbedo_data_name


  !Determine the Julian Day by reading in the time stamp and converting:
  !Get the Time Stamp from the pcf file:
  Status = PGS_PC_GetConfigData(LUN_TimeStamp,dateTime_a)
  if (status /= PGS_S_SUCCESS) then
    call local_message_handler('Problem Reading in the PCF Time Stamp',status,'get_surface_albedo') 
  endif 
  !Convert to day of year string:
  Status = PGS_TD_asciitime_atob(dateTime_a,dateTime_b)
  if (status /= PGS_S_SUCCESS) then
    call local_message_handler('Problem converting time stamp to DOY',status,'get_surface_albedo') 
  endif 
  !Extract the Day of Year:
  DayOfYearString = dateTime_b(6:8)
  !Convert to integer:
  read(DayOfYearString, *) JulianDay
  !If not successful, set JulianDay to 1 so that it does not crash:
  if (JulianDay < 1 .or. JulianDay > 366) JulianDay = 1

  ProcessLandAlbedo (1) = .true.
  ProcessLandAlbedo (2) = .true.
  ProcessLandAlbedo (3) = .true.
  ProcessLandAlbedo (4) = .true.
  ProcessLandAlbedo (5) = .true.
  ProcessLandAlbedo (6) = .true.
  ProcessLandAlbedo (7) = .false.
  ProcessLandAlbedo (8) = .false.
  ProcessLandAlbedo (9) = .false.
  ProcessLandAlbedo (10) = .false.
 
  call getAlbedoEco (latitude,          &
                     longitude,         &
                     julianday,         &
                     AlbedoWavelengthFN,&
                     IGBPfilename,      &
                     SnowAlbedoFN,      &
                     nise_filename,     &
                     Debug,             &
                     WavelengthText,    &
                     ProcessLandAlbedo, &
                     icec,              &
                     Wavelength,        &
                     surface_albedo,    &
                     Ecosystem,         &
                     Status             )


  do i = 1, size(Ecosystem,1)
   do j = 1, size(Ecosystem,2)
     if (Ecosystem(i,j) == 0 )then
       cloudsummary(i,j)%land_surface  = .false.
       cloudsummary(i,j)%ocean_surface = .true.
       cloudsummary(i,j)%coastal_surface = .false.
       cloudsummary(i,j)%desert_surface = .false.
       cloudsummary(i,j)%snowice_surface = .false.
     elseif (Ecosystem(i,j) == 15) then
       cloudsummary(i,j)%land_surface  = .false.
       cloudsummary(i,j)%ocean_surface = .false.
       cloudsummary(i,j)%coastal_surface = .false.
       cloudsummary(i,j)%desert_surface = .false.
       cloudsummary(i,j)%snowice_surface = .true.
     else 
       cloudsummary(i,j)%land_surface  = .true.
       cloudsummary(i,j)%ocean_surface = .false.
       cloudsummary(i,j)%coastal_surface = .false.
       cloudsummary(i,j)%desert_surface = .false.
       cloudsummary(i,j)%snowice_surface = .false.
     endif
   enddo
  enddo
  deallocate(ecosystem,WavelengthText,Wavelength,AlbedoWavelengthFN,ProcessLandAlbedo)
  if (status /= success) then
    call local_message_handler('Problem reported in get_surface_albedo, see earlier message/s',status,'get_surface_albedo') 
  endif 
end subroutine get_surface_albedo

!===============================================================================
!
! Subroutine local_read_reynsst below was commented out by R. Hucek on 4/13/05 
! and relocated to a separate file under the MOD_PR06OD.  The newer Reynolds  
! products are written in binary format and require this f90 module to be 
! compiled with a byte swapping flag to run correctly under Linux.  The Reynolds
! requirement though conflicts with the portable NCEP grib format files that 
! are also opened and read by this module.
!
!===============================================================================
!  
!subroutine local_read_reynsst ( filename, &
!                                year, month, day, hour, sst, & 
!                                file_format, sst_success ) 
!
!  implicit none
!
!!-----------------------------------------------------------------------
!! !F77
!!
!! !DESCRIPTION:  Retrieves an array of global sea surface temperature  
!!                (sst) data from the NCEP Reynolds sst data product.
!!                Both formatted (oi.mean.bias) and unformatted (oisst)
!!                products are supported.  
!!
!!
!! !INPUT PARAMETERS:
!!  Integer       npoints_x   - number of points around a latitude circle 
!!  Integer       npoints_y   - number of grid points along a meridian  
!!
!! !OUTPUT PARAMETERS:
!!  character*(*) file_format - one of either 'unk', 'fmt' or 'unf'
!!
!!  Integer       year        - data year
!!  Integer       month       - data month
!!  Integer       day         - data day
!!  Intger        hour        - set to 0
!!
!!  Logical       sst_success - .true. if able to retrieve sst data;
!!                              otherwise .false.
!!
!!  Real          sst         - global array of REYNSST sea surface
!!                              temperature data in degree Celsius
!!
!! !REVISION HISTORY:
!!
!!
!! !TEAM-UNIQUE HEADER:
!!
!!    This software wasdeveloped by the MODIS Science Data Support Team
!!    together with MODIS Group, CIMSS/SSEC, UW-Madison for the National
!!    Aeronautics and Space Administration, Goddard Space Flight Center,
!!    under contract NAS5-32373.
!!
!! !REFERENCES AND CREDITS:
!!
!!    Written by Rich Frey, MODIS Group, CIMSS/SSEC, UW-Madison
!!        and
!!    Rich Hucek, Decision Systems Technologies, INC
!!    SAIC/GSC MODIS Science Data Support Office
!!    7501 Forbes Blvd,  Seabrook MD 20706
!!    rhucek@ltpmail.gsfc.nasa.gov
!!
!!
!! !DESIGN NOTES: 
!!
!!    The NCEP Reynolds sst file format (formatted or unformatted) is
!!    identified by comparing the size of the product staged for input
!!    against known, unique and fixed values for the two format types.
!!    If a match is made, processing continues.  Otherwise, read_reynsst
!!    aborts with a fatal error.
!!
!!    If a read error occurs, read_reynsst returns control to the
!!    calling program and sets the subroutine logical status flag,
!!    sst_success, to .false. 
!!
!!
!!  Externals:
!!
!!    Functions and Subroutines:
!!        modis_io_gen_closef       (MODIS_IO_GEN_CLOSE.f in src_L2)
!!        pgs_io_gen_openf          (libPGSTK.a)
!!        pgs_pc_getfilesize        (libPGSTK.a)
!!
!!    Named Constant:
!!        LUN_REYNSST               (Atmos_AncData.inc in src_L2)
!!        SIZE_OF_REYNSST_FMT       (Atmos_AncData.inc in src_L2)
!!        SIZE_OF_REYNSST_UNF       (Atmos_AncData.inc in src_L2)
!!        PGS_S_SUCCESS             (PGS_SMF.f)
!!
!! !END
!!-----------------------------------------------------------------------
!
!!-----Declaration of function arguments
!      character*(*) , intent(in)   ::  filename
!      integer,        intent(out)  ::  year, month, day, hour
!      logical,        intent(out)  ::  sst_success
!      real,           intent(out)  ::  sst(0:, 0:)
!      character*(*),  intent(out)  ::  file_format
!
!
!!-----Declaration of local variables
!      integer       npoints_x, npoints_y
!      character*160 errmsg
!      integer       filesize, header(8), i, ios, j, level, lun,  &
!                   reclen, status, version , values,blah
!
!!-----------------------------------------------------------------------
!!     INITIALIZATION
!!-----------------------------------------------------------------------
!  npoints_x = size(sst,1)-1
!  npoints_y = size(sst,2)-1
!  sst_success = .false.
!  file_format = 'unk'
!
!!-----------------------------------------------------------------------
!!     DETERMINE FILE FORMAT
!!-----------------------------------------------------------------------
!  reclen  = 1
!  version = 1
!  status = 0
!  lun=40
!
!  if ( index(filename,'oi.sst') /= 0 .or. index(filename,'oi.mean') /= 0 ) then
!    file_format = 'fmt'
!    open( unit = lun, &
!          file = filename,  &
!          form = 'formatted', &
!          access = 'sequential',    &
!          recl = reclen, &
!          status = 'old')
!
!  elseif( index(filename,'oisst') /= 0) then
!    file_format = 'unf'
!    open( unit = lun, &
!          file = filename,  &
!          form = 'unformatted', &
!          access = 'sequential',    &
!!          recl = reclen, &
!          status = 'old')
!  else
!    level   =  2
!    status  = -1
!  endif
!
!!-----------------------------------------------------------------------
!!     READ SST DATA
!!-----------------------------------------------------------------------
!
!  if ( status .eq. 0) then
!
!  if (file_format .eq. 'fmt') then
!    read( lun, '( 8i5 )', iostat = ios ) header
!  else if(file_format .eq. 'unf') then
!    read( lun, iostat = ios ) header
!  end if
!
!  if ( ios .ne. 0 ) then
!    level = 1
!  else
!    year  = header( 1 )
!    month = header( 2 )
!    day   = header( 3 )
!    hour  = 0
!  endif
!
!  if (file_format .eq. 'fmt') then
!    read( lun, '( 20f4.2 )', iostat = ios ) sst
!  else if (file_format .eq. 'unf') then
!    read( lun, iostat = ios ) ( (sst(i,j), i=0,359), j=0,179)
!  end if
!  if ( ios .ne. 0 ) then
!    level = 1
!  else
!    sst_success = .true.
!  endif
!  close(lun)
!endif
!end subroutine local_read_reynsst

subroutine get_cloud_top_properties(mapi_filedata, &
                                    filename, &
                                    start,    &
                                    stride,   &
                                    edge,     &
                                    cloud_top_pressure, &
                                    cloud_top_temperature, &
                                    status)

  use hdf
  use GeneralAuxType
   use nonscience_parameters
  use modis_io_module,only: getintegerarray_toolkit
  implicit none

  character(*), intent(in)                  :: filename
  integer,      intent(in)                  :: mapi_filedata(:)
  integer,      intent(inout), dimension(:) :: start, stride, edge
  real(single), dimension(:,:), intent(out) :: cloud_top_pressure, cloud_top_temperature
  integer,      intent(out)                 :: status

  integer(integer_twobyte)                  :: fill_value
  integer                                   :: i, j, local_i, local_j, dimsizes(2)
  real(single), dimension(:,:), allocatable :: ctp_temp_array, ctt_temp_array
  logical :: offset

  cloud_top_pressure = 0.
  cloud_top_temperature = 0.

  allocate(ctp_temp_array(edge(1), edge(2)))
  allocate(ctt_temp_array(edge(1), edge(2)))

  offset = .true.
  call getintegerarray_toolkit(mapi_filedata,        &
                               'Cloud_Top_Pressure',  &
                               start, stride, edge,   &
                              offset, &
                               ctp_temp_array,            &
                               status)
  call getintegerarray_toolkit(mapi_filedata,        &
                               'Cloud_Top_Temperature',  &
                               start, stride, edge,   &
                              offset, &
                               ctt_temp_array,            &
                               status)
  dimsizes(1) = size(ctp_temp_array,1)
  dimsizes(2) = size(ctp_temp_array,2)
   
  do i = 1, size(cloud_top_pressure,1)
    local_i = (i - 1) / 5 + 1
    do j = 1, size(cloud_top_pressure,2)
       local_j = (j - 1) / 5 + 1
       if (local_i > dimsizes(1) ) local_i = dimsizes(1)
       cloud_top_pressure(i,j) =  ctp_temp_array(local_i, local_j)
       cloud_top_temperature(i,j) = ctt_temp_array(local_i, local_j)
    enddo
  enddo
  deallocate( ctt_temp_array, ctp_temp_array )

  if (status /= success) then
    call local_message_handler('Problem reported, see earlier message/s',status,'get_cloud_top_properties') 
  endif 

end subroutine get_cloud_top_properties

subroutine get_above_cloud_properties(temp_profiles,        &
                                   water_profiles,       &
                                   pressurelevels,       &
                                   cloud_top_pressure,   &
                                   abovecloud_watervapor,&
                                   abovecloud_temperature, &
                                   status)

  use GeneralAuxType
   use nonscience_parameters
  implicit none

  real(single), intent(in), dimension(:,:,:) :: temp_profiles, water_profiles, pressurelevels
  real(single), intent(in), dimension(:,:)   :: cloud_top_pressure

  real(single), intent(out),dimension(:,:)   :: abovecloud_watervapor, abovecloud_temperature 
  integer, intent(out)                       :: status

  integer    :: i,j, level_index, k
  real       :: pw_layer, t_layer, total_pw, sum_t, slope
  real       :: t_upper, t_lower, watervapor_lower, watervapor_upper

  abovecloud_watervapor = -999.
  abovecloud_temperature = -999.

  do i = 1, size(cloud_top_pressure,1)
    do j = 1, size(cloud_top_pressure,2)
     if (cloud_top_pressure(i,j) /= -999.) then    
      k = size(pressurelevels,3) - 1
      total_pw = 0.
      sum_t = 0.
      do while ( pressurelevels(i,j,k) < cloud_top_pressure(i,j) .and. k > 1)
        pw_layer = ( pressurelevels(i,j,k) - pressurelevels(i,j,k+1) ) *  &
                   ( water_profiles(i,j,k+1) + water_profiles(i,j,k) ) * 0.5/ 980.616

        t_layer = (temp_profiles(i,j,k) + temp_profiles(i,j,k+1))*.5
        total_pw = total_pw + pw_layer
        sum_t = sum_t + pw_layer*t_layer
        k = k - 1
      enddo

      slope = ( water_profiles(i,j,k) -  water_profiles(i,j,k+1) ) / &
              ( pressurelevels(i,j,k) - pressurelevels(i,j,k+1))  

      watervapor_upper = water_profiles(i,j,k+1 )
      watervapor_lower = slope * (cloud_top_pressure(i,j) - pressurelevels(i,j,k+1)) + &
                         watervapor_upper
      pw_layer = (cloud_top_pressure(i,j) - pressurelevels(i,j,k+1)) * &
                 (watervapor_upper + watervapor_lower) * 0.5/980.616
      abovecloud_watervapor(i,j) = total_pw + pw_layer

      slope = (temp_profiles(i,j,k) - temp_profiles(i,j,k+1)) / &
              ( pressurelevels(i,j,k) - pressurelevels(i,j,k+1))

      t_upper = temp_profiles(i,j,k+1)
      t_lower = slope * (cloud_top_pressure(i,j) - pressurelevels(i,j,k+1)) + t_upper

      t_layer = (t_upper + t_lower ) *.5
      abovecloud_temperature(i,j) = t_layer     
     endif
    enddo
  enddo

  if (status /= success) then
    call local_message_handler('Problem reported, see earlier message/s',status,'get_above_cloud_properties')
  endif

end subroutine get_above_cloud_properties

end module ancillary_module
