module core_arrays

   use GeneralAuxType
   use modis_cloudstructure
   use modis_sciencestructure

   implicit none

   ! core algorithm arrays
   real(single),   allocatable, dimension (:,:)   :: optical_thickness
   real(single),   allocatable, dimension (:,:)   :: optical_thickness_1621
   real(single),   allocatable, dimension (:,:)   :: effective_radius_16
   real(single),   allocatable, dimension (:,:)   :: effective_radius_21
   real(single),   allocatable, dimension (:,:)   :: effective_radius_37
   real(single),   allocatable, dimension (:,:)   :: effective_radius_1621
   real(single),   allocatable, dimension (:,:)   :: liquid_water_path
   real(single),   allocatable, dimension (:,:)   :: liquid_water_path_1621
   real(single),   allocatable, dimension (:,:)   :: optical_thickness_error
   real(single),   allocatable, dimension (:,:)   :: effective_radius_21_error
   real(single),   allocatable, dimension (:,:)   :: liquid_water_path_error
   real(single),   allocatable, dimension (:,:)   :: optical_thickness_1621_error
   real(single),   allocatable, dimension (:,:)   :: effective_radius_1621_error
   real(single),   allocatable, dimension (:,:)   :: liquid_water_path_1621_error
   integer(integer_onebyte),      allocatable, dimension (:,:)   :: cloud_layer_flag
   integer(integer_onebyte),      allocatable, dimension (:,:)   :: CSR_flag_array
   type(processflag)  , allocatable,dimension (:,:)   :: cloudsummary
   type(cloudphase)   ,allocatable,dimension (:,:)   :: ir_cloudphase

   !input data arrays (measurement)
   real(single),   allocatable, dimension (:,:)   :: latitude
   real(single),   allocatable, dimension (:,:)   :: longitude
   real(single),   allocatable, dimension (:,:)   :: solar_zenith_angle
   real(single),   allocatable, dimension (:,:)   :: sensor_zenith_angle
   real(single),   allocatable, dimension (:,:)   :: solar_azimuth_angle
   real(single),   allocatable, dimension (:,:)   :: sensor_azimuth_angle
   real(single),   allocatable, dimension (:,:)   :: relative_azimuth_angle
   real(single),   allocatable, dimension (:,:)   :: aircraft_pitch
   real(single),   allocatable, dimension (:)     :: central_wavelength
   real(single),   allocatable, dimension (:)     :: solar_spectral_irradiance
   integer(single),allocatable, dimension (:)     :: greenwichmeantime
   real(single),   allocatable, dimension (:,:,:) :: band_measurements
   type(cloudmask_type),allocatable,dimension (:,:)   :: cloudmask

   !atmopsheric correction, thermal
   real(single), allocatable, dimension(:,:,:)    :: thermal_correction_twoway
   real(single), allocatable, dimension(:,:,:)    :: thermal_correction_oneway
   real(single), allocatable, dimension(:)        :: thermal_correction_bands

   ! atmospheric correction, for uncertainty computation
   real(single), allocatable, dimension(:,:,:)    :: meandelta_trans
   real(single), allocatable, dimension(:,:,:)    :: transmittance_twoway
   real(single), allocatable, dimension(:,:,:)    :: transmittance_stddev

   ! ancillary arrays
   real(single),   allocatable, dimension (:,:,:) :: surface_albedo
   real(single),   allocatable, dimension (:,:)   :: surface_temperature
   real(single),   allocatable, dimension (:,:)   :: cloud_top_temperature
   real(single),   allocatable, dimension (:,:)   :: above_cloud_temperature
   real(single),   allocatable, dimension (:,:)   :: cloud_top_pressure
   real(single),   allocatable, dimension (:,:)   :: abovecloud_watervapor
   real(single),   allocatable, dimension (:,:)   :: column_ozone
   real(single),   allocatable, dimension (:,:)   :: ozone_absorption

   !Other
   character*5                                    :: platform_name
   real(single), allocatable, dimension(:,:,:)    :: model_temp_profile, model_water_profile
   real(single), allocatable, dimension(:,:,:)    :: model_pressure_levels 
   real(single), allocatable, dimension(:,:)    :: nonabs_obs_corrected, absorbing_obs_corrected 
   integer(integer_onebyte),  allocatable, dimension(:,:)    :: nonabsorbing_band
   type(qualityanalysis), allocatable, dimension(:,:) :: processing_information
   integer(integer_fourbyte),allocatable, dimension (:)     :: bands
   
end module core_arrays

