module corescience_module

 implicit none
 private

 public :: corescience

 contains


subroutine corescience(xpoint,ypoint,process,debug,status)
   
   use GeneralAuxType
   use core_arrays
   use libraryarrays
   use libraryinterpolates
   use modis_cloudstructure, only: processflag
   use modis_numerical_module, only: linearinterpolation
   use multi_layer_clouds
   use get_retrieval_uncertainty
   use science_parameters
   use nonscience_parameters
   use mod06_run_settings

   implicit none
 
   logical,           intent(in)       :: debug
   type(processflag), intent(in)       :: process
   integer,           intent(in)       :: xpoint,ypoint
   integer,           intent(out)      :: status
 
   integer                             :: quality_indexes, index_upper, index_lower, &
                                          nonabsorbingband_index, absorbingband_index, maxradii
                            
   real                                :: thermal_trans_1way, thermal_trans_2way, &
                                          cosine_solarzenith, cosine_sensorzenith

   real, allocatable                   :: optical_thickness_vector(:), residual(:)
   real        ::  retrievalopticalthickness,    &
                   retrievalopticalthickness16,  &
                   retrievalopticalthickness37,  &
                   retrievalopticalthickness1621,&
                   retrievalradius21,            &
                   retrievalradius16,            &
                   retrievalradius37,            &
                   retrievalradius1621,          &
                   water_path,                   &
                   water_path1621,               &
                   local_cloud_ot21_error,       & 
                   local_cloud_re21_error,       & 
                   local_wp21_error,             &
                   local_cloud_ot1621_error,     &
                   local_cloud_re1621_error,     &
                   local_wp1621_error
  
   real    :: cloud_reflectance(2),delta_reflectance(2), albedo_holder(2) , &
              delta_transmittance(2), tauRegimeThreshold(2)
   real    :: asymmetry, blah
   integer :: uncertainty_nonabsorbing, uncertainty_nonabsorbing_1621, R1R2wavelengthIdx(2)
   integer(integer_onebyte)  :: retrieval_qa, local_cloud_layer_flag
   character(10) :: phase

   status = 0

   thermal_trans_1way   = thermal_correction_oneway(xpoint,ypoint,1)   ! only the 3.7um is used
   thermal_trans_2way   = thermal_correction_twoway(xpoint,ypoint,1)   ! only the 3.7um is used
 
   nonabsorbingband_index = band_0065
   processing_information(xpoint, ypoint)%band_used_for_optical_thickness =  0

   if (process%cloudobserved .and. (process%ocean_surface .or. process%coastal_surface)) then
      if( process%snowice_surface) then
          ! 1.2 um
          nonabsorbingband_index = band_0124
      else
          ! .86um
          nonabsorbingband_index = band_0086
      endif
      processing_information(xpoint, ypoint)%band_used_for_optical_thickness = nonabsorbingband_index
   endif
 
   if( process%cloudobserved .and. (process%land_surface .or. process%desert_surface) ) then      
      if( process%snowice_surface) then
         ! 1.2 um  
          nonabsorbingband_index = band_0124
      else
         ! .68um 
          nonabsorbingband_index = band_0065
      endif 
      processing_information(xpoint, ypoint)%band_used_for_optical_thickness = nonabsorbingband_index
   endif

   if( process%cloudobserved .and.process%snowice_surface) then
      ! 1.2 um
      nonabsorbingband_index = band_0124
      processing_information(xpoint, ypoint)%band_used_for_optical_thickness = nonabsorbingband_index
   endif
 
 
   uncertainty_nonabsorbing = nonabsorbingband_index
   uncertainty_nonabsorbing_1621 = band_0163

!  set surface QA

!  if the measured surface reflectance in the non absorbing bands is greater than
!  the surface albedo , and we have a cloudy pixel then we can try a retrieval

   if (process%cloudobserved ) then

!    check for .86um saturation
!    if the .86um saturates, try the other .65um band (and change the QA flag)
     if (nonabsorbingband_index == band_0086 .and. band_measurements(xpoint, nonabsorbingband_index, ypoint) < 0.) then
       if (band_measurements(xpoint,band_0065,ypoint) > 0.) nonabsorbingband_index = band_0065
       processing_information(xpoint, ypoint)%band_used_for_optical_thickness = nonabsorbingband_index
     endif



!     if the shortwave saturates, try the other shortwave band 

      if  (process%watercloud .or. process%unknowncloud) then

        allocate(optical_thickness_vector(size(water_radii)))
        allocate(residual(size(water_radii)))
        call vis_nonabsorbing_science(band_measurements(xpoint, nonabsorbingband_index, ypoint), &
                              nonabsorbingband_index,                 &
                              surface_albedo(xpoint, ypoint,nonabsorbingband_index), &
                              library_wavelengths,                    &
                              library_taus,                           &
                              water_radii,                            &
                              asymmetry_water,                        &
                              singlescattering_water,                 &
                              extinction_water,                       &
                              ak_water,                               &
                              el_water,                               &
                              em_water,                               &
                              en_water,                               &
                              spherical_albedo_water,                 &
                              int_escapefuncwater_solar,              &
                              int_fluxdownwater_sensor,               &
                              int_fluxdownwater_solar,                & 
                              int_escapefuncwater_sensor,             &
                              int_reflectance_water,                  &
                              sensor_zenith_angle(xpoint,ypoint),     &
                              solar_zenith_angle(xpoint,ypoint),      &
                              relative_azimuth_angle(xpoint,ypoint),  &
                              cloud_top_pressure(xpoint,ypoint),      &
                              process,                                &
                              optical_thickness_vector)

        absorbingband_index = band_0163 ! 1.6um band                      
        if(band_measurements(xpoint, absorbingband_index, ypoint) > 0. ) then 
           
        call vis_absorbing_science(optical_thickness_vector,          &
                              band_measurements(xpoint, absorbingband_index, ypoint), &
                              absorbingband_index,                    &
                              surface_albedo(xpoint, ypoint,absorbingband_index), &
                              library_wavelengths,                    &
                              library_taus,                           &
                              water_radii,                            &
                              asymmetry_water,                        &
                              singlescattering_water,                 &
                              extinction_water,                       &
                              ak_water,                               &
                              el_water,                               &
                              em_water,                               &
                              en_water,                               &
                              spherical_albedo_water,                 &
                              int_escapefuncwater_solar,              &
                              int_fluxdownwater_sensor,               &
                              int_fluxdownwater_solar,                &
                              int_escapefuncwater_sensor,             &
                              int_reflectance_water,                  &
                              residual,maxradii, debug)

        if (maxradii > 2) then
          call solveretrieval(residual(1:maxradii),                   &
                              optical_thickness_vector(1:maxradii),   &
                              water_radii(1:maxradii),                &
                              retrievalradius16,                      &
                              retrievalopticalthickness16, debug)
       else
          retrievalradius16 = fillvalue_real
          retrievalopticalthickness16 = fillvalue_real
        endif
       else
          retrievalradius16 = fillvalue_real
          retrievalopticalthickness16 = fillvalue_real
       endif

        absorbingband_index = band_0213 ! 2.1um
        call vis_absorbing_science(optical_thickness_vector,          &
                              band_measurements(xpoint, absorbingband_index, ypoint), &
                              absorbingband_index,                    &
                              surface_albedo(xpoint, ypoint,absorbingband_index), &
                              library_wavelengths,                    &
                              library_taus,                           &
                              water_radii,                            &
                              asymmetry_water,                        &
                              singlescattering_water,                 &
                              extinction_water,                       &
                              ak_water,                               &
                              el_water,                               &
                              em_water,                               &
                              en_water,                               &
                              spherical_albedo_water,                 &
                              int_escapefuncwater_solar,              &
                              int_fluxdownwater_sensor,               &
                              int_fluxdownwater_solar,                &
                              int_escapefuncwater_sensor,             &
                              int_reflectance_water,                  &
                              residual, maxradii, debug)

        if (maxradii > 2) then  
          call solveretrieval(residual(1:maxradii), optical_thickness_vector(1:maxradii), &
                             water_radii(1:maxradii), &
                             retrievalradius21, &
                             retrievalopticalthickness,debug)

        else
          retrievalradius21 = fillvalue_real
          retrievalopticalthickness = fillvalue_real
        endif

        absorbingband_index = band_0370 ! 3.7um

! sep, 4 May: absorbingband_index - 1 being passed in for all libarary indices because 
!             band_0370 = 7 and library index corresponding to this band is 6. In the 
!             libraries, band_0935 is index 7.
!
! wind, 7 Dec: absorbingband_index - 1 has to be passed in for albedo as well. 
!

        call nir_absorbing_science(platform_name,&
                              optical_thickness_vector,               &
                              band_measurements(xpoint, absorbingband_index, ypoint), &
                              absorbingband_index - 1,                &
                              surface_albedo(xpoint, ypoint,absorbingband_index-1), &
                              cloud_top_temperature(xpoint, ypoint),  &
                              surface_temperature(xpoint, ypoint),    &
                              thermal_trans_1way,                     &
                              thermal_trans_2way,                     &
                              solar_zenith_angle(xpoint,ypoint),      &
                              library_wavelengths,                    &
                              library_taus,                           &
                              water_radii,                            &
                              asymmetry_water,                        &
                              singlescattering_water,                 &
                              extinction_water,                       &
                              ak_water,                               &
                              el_water,                               &
                              em_water,                               &
                              en_water,                               &
                              spherical_albedo_water,                 &
                              int_escapefuncwater_solar,              &
                              int_fluxdownwater_sensor,               &
                              int_fluxdownwater_solar,                &
                              int_fluxupwater_sensor,                 &
                              int_escapefuncwater_sensor,             &
                              int_reflectance_water,                  &
                              residual, maxradii, debug)

        if ( maxradii > 2 ) then
          call solveretrieval(residual(1:maxradii), optical_thickness_vector(1:maxradii), &
                             water_radii(1:maxradii), &
                             retrievalradius37, &
                             retrievalopticalthickness37,debug)

        else
          retrievalradius37 = fillvalue_real
          retrievalopticalthickness37 = fillvalue_real
        endif

        retrievalradius1621 = fillvalue_real
        retrievalopticalthickness1621 = fillvalue_real
        if( (process%ocean_surface .or. process%snowice_surface) .and. &
            band_measurements(xpoint, nonabsorbingband_index, ypoint) > 0. ) then
        nonabsorbingband_index = band_0163
        absorbingband_index = band_0213
        call vis_nonabsorbing_science(band_measurements(xpoint, nonabsorbingband_index, ypoint), &
                              nonabsorbingband_index,                 &
                              surface_albedo(xpoint, ypoint,nonabsorbingband_index), &
                              library_wavelengths,                    &
                              library_taus,                           &
                              water_radii,                            &
                              asymmetry_water,                        &
                              singlescattering_water,                 &
                              extinction_water,                       &
                              ak_water,                               &
                              el_water,                               &
                              em_water,                               &
                              en_water,                               &
                              spherical_albedo_water,                 &
                              int_escapefuncwater_solar,              &
                              int_fluxdownwater_sensor,               &
                              int_fluxdownwater_solar,                &
                              int_escapefuncwater_sensor,             &
                              int_reflectance_water,                  &
                              sensor_zenith_angle(xpoint,ypoint),     &
                              solar_zenith_angle(xpoint,ypoint),      &
                              relative_azimuth_angle(xpoint,ypoint),  &
                              cloud_top_pressure(xpoint,ypoint),      &
                              process,                                &
                              optical_thickness_vector)

        call vis_absorbing_science(optical_thickness_vector,          &
                              band_measurements(xpoint, absorbingband_index, ypoint), &
                              absorbingband_index,                    &
                              surface_albedo(xpoint, ypoint,absorbingband_index), &
                              library_wavelengths,                    &
                              library_taus,                           &
                              water_radii,                            &
                              asymmetry_water,                        &
                              singlescattering_water,                 &
                              extinction_water,                       &
                              ak_water,                               &
                              el_water,                               &
                              em_water,                               &
                              en_water,                               &
                              spherical_albedo_water,                 &
                              int_escapefuncwater_solar,              &
                              int_fluxdownwater_sensor,               &
                              int_fluxdownwater_solar,                &
                              int_escapefuncwater_sensor,             &
                              int_reflectance_water,                  &
                              residual,maxradii, debug)
        if (maxradii > 2) then
          call solveretrieval(residual(1:maxradii), optical_thickness_vector(1:maxradii), &
                             water_radii(1:maxradii), &
                             retrievalradius1621, &
                             retrievalopticalthickness1621, debug)
        endif

        endif

        if (retrievalopticalthickness == fillvalue_real .or. retrievalradius21 == fillvalue_real )then
          water_path = fillvalue_real
        else
           if (retrievalopticalthickness < 0.01) then 
              retrievalopticalthickness = 0.01
           endif
           if (retrievalopticalthickness > 100.) then 
              retrievalopticalthickness = 100.
           endif
!          water_path = (2./3.)* retrievalopticalthickness * retrievalradius21 * liquid_water_density
! added by G. Wind, new formula for water path in a subroutine
           call compute_water_path(retrievalopticalthickness, &
                                   retrievalradius21, &
                                   liquid_water_density, &
                                   water_radii, &
                                   extinction_water(1, :), &
                                   water_path)

        endif

        if (retrievalopticalthickness1621 == fillvalue_real .or. retrievalradius1621 == fillvalue_real )then
          water_path1621 = fillvalue_real
        else
           if (retrievalopticalthickness1621 < 0.01) then 
              retrievalopticalthickness1621 = 0.01
           endif

            if (retrievalopticalthickness1621 > 100.) then 
              retrievalopticalthickness1621 = 100.
           endif
!         water_path1621 = (2./3.)* retrievalopticalthickness1621 * retrievalradius1621 * liquid_water_density
! added by G. Wind, new formula for water path in a subroutine
           call compute_water_path(retrievalopticalthickness1621, &
                                   retrievalradius1621, &
                                   liquid_water_density, &
                                   water_radii, &
                                   extinction_water(1, :), &
                                   water_path1621)
           

       endif

        deallocate(optical_thickness_vector, residual)

     elseif (process%icecloud) then

       absorbingband_index = band_0163
        allocate(optical_thickness_vector(size(ice_radii)))
        allocate(residual(size(ice_radii)))
        call vis_nonabsorbing_science(band_measurements(xpoint, nonabsorbingband_index, ypoint), &
                              nonabsorbingband_index,                &
                              surface_albedo(xpoint, ypoint,nonabsorbingband_index), &
                              library_wavelengths,                   &
                              library_taus,                          &
                              ice_radii,                             &
                              asymmetry_ice,                         &
                              singlescattering_ice,                  &
                              extinction_ice,                        &
                              ak_ice,                                &
                              el_ice,                                &
                              em_ice,                                &
                              en_ice,                                &
                              spherical_albedo_ice,                  &
                              int_escapefuncice_solar,               &
                              int_fluxdownice_sensor,                &
                              int_fluxdownice_solar,                 &
                              int_escapefuncice_sensor,              &
                              int_reflectance_ice,                   &
                              sensor_zenith_angle(xpoint,ypoint),    &
                              solar_zenith_angle(xpoint,ypoint),     &
                              relative_azimuth_angle(xpoint,ypoint), &
                              cloud_top_pressure(xpoint,ypoint),     &
                              process,                               &
                              optical_thickness_vector)

        if (band_measurements(xpoint, absorbingband_index, ypoint) > 0.) then 
        call vis_absorbing_science(optical_thickness_vector,         &
                              band_measurements(xpoint, absorbingband_index, ypoint), &
                              absorbingband_index,                   &
                              surface_albedo(xpoint, ypoint,absorbingband_index), &
                              library_wavelengths,                   &
                              library_taus,                          &
                              ice_radii,                             &
                              asymmetry_ice,                         &
                              singlescattering_ice,                  &
                              extinction_ice,                        &
                              ak_ice,                                &
                              el_ice,                                &
                              em_ice,                                &
                              en_ice,                                &
                              spherical_albedo_ice,                  &
                              int_escapefuncice_solar,               &
                              int_fluxdownice_sensor,                &
                              int_fluxdownice_solar,                 &
                              int_escapefuncice_sensor,              &
                              int_reflectance_ice,                   &
                              residual,maxradii, debug)
        if ( maxradii > 2) then
          call solveretrieval(residual(1:maxradii), optical_thickness_vector(1:maxradii), &
                             ice_radii(1:maxradii), &
                             retrievalradius16, &
                             retrievalopticalthickness16, debug)
        else
          retrievalradius16 = fillvalue_real
          retrievalopticalthickness = fillvalue_real
        endif

        else
          retrievalradius16 = fillvalue_real
          retrievalopticalthickness = fillvalue_real
        endif



        absorbingband_index = band_0213
        call vis_absorbing_science(optical_thickness_vector,         &
                              band_measurements(xpoint, absorbingband_index, ypoint), &
                              absorbingband_index,                   &
                              surface_albedo(xpoint, ypoint,absorbingband_index), &
                              library_wavelengths,                   &
                              library_taus,                          &
                              ice_radii,                             &
                              asymmetry_ice,                         &
                              singlescattering_ice,                  &
                              extinction_ice,                        &
                              ak_ice,                                &
                              el_ice,                                &
                              em_ice,                                &
                              en_ice,                                &
                              spherical_albedo_ice,                  &
                              int_escapefuncice_solar,               &
                              int_fluxdownice_sensor,                &
                              int_fluxdownice_solar,                 &
                              int_escapefuncice_sensor,              &
                              int_reflectance_ice,                   &
                              residual,maxradii, debug)


        if ( maxradii > 2) then
          call solveretrieval(residual(1:maxradii), optical_thickness_vector(1:maxradii), &
                             ice_radii(1:maxradii), &
                             retrievalradius21, &
                             retrievalopticalthickness, debug)
        else
          retrievalradius21 = fillvalue_real
          retrievalopticalthickness = fillvalue_real
        endif


        absorbingband_index = band_0370

! sep, 4 May: absorbingband_index - 1 being passed in for all libarary indices because 
!             band_0370 = 7 and library index corresponding to this band is 6. In the 
!             libraries, band_0935 is index 7.
!
! wind, 7 Dec: absorbingband_index - 1 has to be passed in for albedo as well. 
!


        call nir_absorbing_science(platform_name, &
                              optical_thickness_vector,              &
                              band_measurements(xpoint, absorbingband_index, ypoint), &
                              absorbingband_index - 1,               &
                              surface_albedo(xpoint, ypoint,absorbingband_index-1), &
                              cloud_top_temperature(xpoint, ypoint), &
                              surface_temperature(xpoint, ypoint),   &
                              thermal_trans_1way,                    &
                              thermal_trans_2way,                    &
                              solar_zenith_angle(xpoint,ypoint),     &
                              library_wavelengths,                   &
                              library_taus,                          &
                              ice_radii,                             &
                              asymmetry_ice,                         &
                              singlescattering_ice,                  &
                              extinction_ice,                        &
                              ak_ice,                                &
                              el_ice,                                &
                              em_ice,                                &
                              en_ice,                                &
                              spherical_albedo_ice,                  &
                              int_escapefuncice_solar,               &
                              int_fluxdownice_sensor,                &
                              int_fluxdownice_solar,                 &
                              int_fluxupice_sensor,                  &
                              int_escapefuncice_sensor,              &
                              int_reflectance_ice,                   &
                              residual, maxradii, debug)

        if ( maxradii > 2 ) then
          call solveretrieval(residual(1:maxradii), optical_thickness_vector(1:maxradii), &
                             ice_radii(1:maxradii), &
                             retrievalradius37, &
                             retrievalopticalthickness37,debug)


        else
          retrievalradius37 = fillvalue_real
          retrievalopticalthickness37 = fillvalue_real
        endif
                           
        retrievalradius1621 = fillvalue_real
        retrievalopticalthickness1621 = fillvalue_real
        if( (process%ocean_surface .or. process%snowice_surface) .and. &
            band_measurements(xpoint, nonabsorbingband_index, ypoint) > 0. ) then
        nonabsorbingband_index = band_0163
        absorbingband_index = band_0213
        call vis_nonabsorbing_science(band_measurements(xpoint, nonabsorbingband_index, ypoint), &
                              nonabsorbingband_index,                &
                              surface_albedo(xpoint, ypoint,nonabsorbingband_index), &
                              library_wavelengths,                   &
                              library_taus,                          &
                              ice_radii,                             &
                              asymmetry_ice,                         &
                              singlescattering_ice,                  &
                              extinction_ice,                        &
                              ak_ice,                                &
                              el_ice,                                &
                              em_ice,                                &
                              en_ice,                                &
                              spherical_albedo_ice,                  &
                              int_escapefuncice_solar,               &
                              int_fluxdownice_sensor,                &
                              int_fluxdownice_solar,                 &
                              int_escapefuncice_sensor,              &
                              int_reflectance_ice,                   &
                              sensor_zenith_angle(xpoint,ypoint),    &
                              solar_zenith_angle(xpoint,ypoint),     &
                              relative_azimuth_angle(xpoint,ypoint), &
                              cloud_top_pressure(xpoint,ypoint),     &
                              process,                               &
                              optical_thickness_vector)

        call vis_absorbing_science(optical_thickness_vector,         &
                              band_measurements(xpoint, absorbingband_index, ypoint), &
                              absorbingband_index,                   &
                              surface_albedo(xpoint, ypoint,absorbingband_index), &
                              library_wavelengths,                   &
                              library_taus,                          &
                              ice_radii,                             &
                              asymmetry_ice,                         &
                              singlescattering_ice,                  &
                              extinction_ice,                        &
                              ak_ice,                                &
                              el_ice,                                &
                              em_ice,                                &
                              en_ice,                                &
                              spherical_albedo_ice,                  &
                              int_escapefuncice_solar,               &
                              int_fluxdownice_sensor,                &
                              int_fluxdownice_solar,                 &
                              int_escapefuncice_sensor,              &
                              int_reflectance_ice,                   &
                              residual,maxradii,debug)

        if (maxradii > 2 ) then
          call solveretrieval(residual(1:maxradii), optical_thickness_vector(1:maxradii), &
                             ice_radii(1:maxradii), &
                             retrievalradius1621, &
                             retrievalopticalthickness1621,debug)
        endif
        endif
        if (retrievalopticalthickness ==  fillvalue_real .or. retrievalradius21 == fillvalue_real )then
          water_path = fillvalue_real
        else
           if (retrievalopticalthickness < 0.01) then 
              retrievalopticalthickness = 0.01
           endif
           if (retrievalopticalthickness > 100.) then 
              retrievalopticalthickness = 100.
           endif
!          water_path = (2./3.)* retrievalopticalthickness * retrievalradius21 * ice_water_density
! added by G. Wind, new formula for water path in a subroutine
           call compute_water_path(retrievalopticalthickness, &
                                   retrievalradius21, &
                                   ice_water_density, &
                                   ice_radii, &
                                   extinction_ice(1, :), &
                                   water_path)

        endif

        if (retrievalopticalthickness1621 == fillvalue_real .or. retrievalradius1621 == fillvalue_real )then
          water_path1621 = fillvalue_real
        else
            if (retrievalopticalthickness1621 < 0.01) then 
              retrievalopticalthickness1621 = 0.01
           endif

           if (retrievalopticalthickness1621 > 100.) then 
              retrievalopticalthickness1621 = 100.
           endif
!           water_path1621 = (2./3.)* retrievalopticalthickness1621 * retrievalradius1621 * ice_water_density
! added by G. Wind, new formula for water path in a subroutine
           call compute_water_path(retrievalopticalthickness1621, &
                                   retrievalradius1621, &
                                   ice_water_density, &
                                   ice_radii, &
                                   extinction_ice(1, :), &
                                   water_path1621)

        endif


        deallocate(optical_thickness_vector, residual)
     endif ! end of ice/water/unknown logic

   else
      ! there is no cloud
      water_path = fillvalue_real
      retrievalopticalthickness = fillvalue_real
      retrievalopticalthickness1621 = fillvalue_real
      retrievalopticalthickness16 = fillvalue_real
      retrievalopticalthickness37  = fillvalue_real
      retrievalradius16 = fillvalue_real
      retrievalradius21 = fillvalue_real
      retrievalradius1621 = fillvalue_real
      retrievalradius37 = fillvalue_real
      cloud_layer_flag(xpoint, ypoint) = 0
      status = 1
   endif


   !QA for large optical thickness
   if (retrievalopticalthickness > 150.) then
     processing_information(xpoint, ypoint)%optical_thickness_outofbounds = 2
   elseif (retrievalopticalthickness > 100. .and. retrievalopticalthickness <= 150.) then
     processing_information(xpoint, ypoint)%optical_thickness_outofbounds = 1
   else
     processing_information(xpoint, ypoint)%optical_thickness_outofbounds = 0
   endif

   if ( retrievalopticalthickness > 100.) then
     retrievalopticalthickness = 100.
   endif

!   post retrieval QA on final answers: removing clearly non physical solutions

!   if ( retrievalradius21 == 30.0 .and.  process%watercloud) then
!     retrievalopticalthickness = fillvalue_real
!     retrievalradius16 = fillvalue_real
!   retrievalradius21 = fillvalue_real
!    retrievalradius37 = fillvalue_real
!     water_path = fillvalue_real
!     status = 1
!   endif 
!   if (retrievalopticalthickness < 0.) then
!     retrievalopticalthickness = fillvalue_real
!     retrievalradius16 = fillvalue_real
!     retrievalradius21 = fillvalue_real
!     retrievalradius37 = fillvalue_real
!     water_path = fillvalue_real
!     status = 1
!   endif
!   if (retrievalradius21 < 0. .or. retrievalradius21 > 100. ) then
!     retrievalopticalthickness = fillvalue_real
!     retrievalradius16 = fillvalue_real
!     retrievalradius21 = fillvalue_real
!     retrievalradius37 = fillvalue_real
!   endif

!  post-retrieval QA on final answers: removing clearly non physical solutions
!  sep: 4-28-05. New logic for standard retrievals
! wind: 10.05.05. New logic for standard and aux retrievals.

   if (process%watercloud .or. process%unknowncloud) then 
      if (retrievalradius21 == 30.0 .or. retrievalradius21 <= 4.0 ) then 
           retrievalradius21 = fillvalue_real
           water_path = fillvalue_real
           status = 1
        endif
      if (retrievalradius16 == 30.0 .or. retrievalradius16 <= 4.0 ) &
           retrievalradius16 = fillvalue_real
      if (retrievalradius37 == 30.0 .or. retrievalradius37 <= 4.0 ) &
           retrievalradius37 = fillvalue_real

   endif


   if( retrievalopticalthickness < 0.) then
     retrievalopticalthickness = fillvalue_real
     retrievalradius16 = fillvalue_real
     retrievalradius21 = fillvalue_real
     retrievalradius37 = fillvalue_real
     water_path = fillvalue_real
     status = 1
   endif

   if (retrievalradius21 < 0.) then
      retrievalradius21 = fillvalue_real
      water_path = fillvalue_real
      status = 1
   endif
   if (retrievalradius16 < 0.) retrievalradius16 = fillvalue_real
   if (retrievalradius37 < 0.) retrievalradius37 = fillvalue_real
   




!  sep: 4-28-05. Same logic as above but for 1621 retrievals
   if ( ( (retrievalradius1621 == 30.0 .or. retrievalradius1621 <= 4.0) .and. (process%watercloud .or. process%unknowncloud)) &
         .OR. (retrievalopticalthickness1621 < 0.) &
         .OR. (retrievalradius1621 < 0. .or. retrievalradius1621 > 100. ) ) then
!  bpw: 5-9-05. The following line was not in sep's commit, but probably should be?
     retrievalopticalthickness1621 = fillvalue_real
     retrievalradius1621 = fillvalue_real
     water_path1621 = fillvalue_real
     status = 1
   endif
 
!  get visible band overlapping cloud indicator
   local_cloud_layer_flag = 0
   if (retrievalopticalthickness > 0.) then
     cosine_solarzenith = cos(d2r * solar_zenith_angle(xpoint,ypoint))
     cosine_sensorzenith = cos(d2r * sensor_zenith_angle(xpoint,ypoint))
!    must have uncorrected reflectances!!!! G. Wind. 4.18.05
     call Compute_Multilayer_Map(platform_name, &
                                 transmit_correction_table,             &   
                                 band_measurements(xpoint, band_0065, ypoint)*transmittance_twoway(xpoint,ypoint,band_0065),  &   
                                 band_measurements(xpoint, band_0086, ypoint)*transmittance_twoway(xpoint,ypoint,band_0086),  &   
                                 band_measurements(xpoint, band_0935, ypoint)*transmittance_twoway(xpoint,ypoint,band_0935),  &   
                                 band_measurements(xpoint, band_0124, ypoint)*transmittance_twoway(xpoint,ypoint,band_0124),  &   
                                 band_measurements(xpoint, band_1100, ypoint),  &   
                                 cloudsummary(xpoint,ypoint),           &   
                                 ir_cloudphase(xpoint,ypoint),          &   
                                 model_pressure_levels(xpoint,ypoint,:),&   
                                 model_water_profile(xpoint,ypoint,:),  &   
                                 model_temp_profile(xpoint,ypoint,:),   &
                                 cloud_top_pressure(xpoint,ypoint),     &   
                                 abovecloud_watervapor(xpoint,ypoint),  &   
                                 cosine_sensorzenith,                   &   
                                 cosine_solarzenith,                    &  
                                 retrievalopticalthickness,             &   
                                 local_cloud_layer_flag)              
   else
     local_cloud_layer_flag = 0
   endif


!  get retrieval uncertainty estimate
   if ((retrievalopticalthickness .ge. 0.01) .and. (retrievalradius21 .ge. 0.01) .and. process%cloudobserved .and. &
      (process%icecloud .or. process%watercloud .or. process%unknowncloud)) then
 
     absorbingband_index = band_0213

     if ( process%icecloud ) then
        phase = 'ice'
        call getradiibounds(size(ice_radii), ice_radii, retrievalradius21, index_upper, index_lower)
        asymmetry = linearinterpolation(ice_radii(index_lower:index_upper), &
                                        asymmetry_ice(uncertainty_nonabsorbing,index_lower:index_upper), &
                                        retrievalradius21)
        tauRegimeThreshold(1) = scaledTauRegimeThreshold/(1.-asymmetry)

        asymmetry = linearinterpolation(ice_radii(index_lower:index_upper), &
                                        asymmetry_ice(absorbingband_index,index_lower:index_upper), & 
                                        retrievalradius21)
        tauRegimeThreshold(2) = scaledTauRegimeThreshold/(1.-asymmetry)

     else
     !if (process%watercloud .or. process%unknowncloud) then
        phase = 'water'
        call getradiibounds(size(water_radii), water_radii, retrievalradius21, index_upper, index_lower)
        asymmetry = linearinterpolation(water_radii(index_lower:index_upper),  &
                                        asymmetry_water(uncertainty_nonabsorbing,index_lower:index_upper),&
                                        retrievalradius21)
        tauRegimeThreshold(1) = scaledTauRegimeThreshold/(1.-asymmetry)
        asymmetry = linearinterpolation(water_radii(index_lower:index_upper), &
                                        asymmetry_water(absorbingband_index,index_lower:index_upper), &
                                        retrievalradius21)
        tauRegimeThreshold(2) = scaledTauRegimeThreshold/(1.-asymmetry)
     endif

     albedo_holder =  (/surface_albedo(xpoint, ypoint,uncertainty_nonabsorbing), &
                        surface_albedo(xpoint, ypoint,absorbingband_index)/)
     cloud_reflectance = (/band_measurements(xpoint, uncertainty_nonabsorbing, ypoint), &
                           band_measurements(xpoint, absorbingband_index, ypoint)/) 
     R1R2wavelengthIdx = (/uncertainty_nonabsorbing, absorbingband_index/)

     call getuncertainties(retrievalopticalthickness, &
                           retrievalradius21,         &
                           water_path, &
                           phase,                     & 
                           tauRegimeThreshold,        &
                           R1R2wavelengthIdx,         &
                           albedo_holder,             &
                          transmittance_twoway(xpoint,ypoint,uncertainty_nonabsorbing), &
                          transmittance_twoway(xpoint,ypoint,absorbingband_index), &
                          meandelta_trans(xpoint,ypoint,uncertainty_nonabsorbing), &
                          meandelta_trans(xpoint,ypoint,absorbingband_index), &
                          transmittance_stddev(xpoint,ypoint,uncertainty_nonabsorbing), &
                          transmittance_stddev(xpoint,ypoint,absorbingband_index), &
                           local_cloud_ot21_error,    &
                           local_cloud_re21_error,   &
                           local_wp21_error)

   else
     local_cloud_ot21_error = fillvalue_real
     local_cloud_re21_error = fillvalue_real
     local_wp21_error       = fillvalue_real
   endif

   if ( local_cloud_ot21_error .lt. epsilon(local_cloud_ot21_error) .or.  &
        local_cloud_re21_error .lt. epsilon(local_cloud_re21_error)  .or. &
        local_wp21_error .lt. epsilon(local_wp21_error)) then 
     local_cloud_ot21_error = fillvalue_real
     local_cloud_re21_error = fillvalue_real
     local_wp21_error       = fillvalue_real
   endif


!  get 1621 retrieval uncertainty estimate
   if ((process%ocean_surface .or. process%snowice_surface) .and. & ! this first check may be redundant, but harmless if it is
      (retrievalopticalthickness1621 .ge. 0.01) .and. (retrievalradius1621 .ge. 0.01) .and. process%cloudobserved .and. &
      (process%icecloud .or. process%watercloud .or. process%unknowncloud)) then
 
     absorbingband_index = band_0213

     if ( process%icecloud ) then
        phase = 'ice'
        call getradiibounds(size(ice_radii), ice_radii, retrievalradius1621, index_upper, index_lower)
        asymmetry = linearinterpolation(ice_radii(index_lower:index_upper), &
                                        asymmetry_ice(uncertainty_nonabsorbing_1621,index_lower:index_upper), &
                                        retrievalradius1621)
        tauRegimeThreshold(1) = scaledTauRegimeThreshold/(1.-asymmetry)

        asymmetry = linearinterpolation(ice_radii(index_lower:index_upper), &
                                        asymmetry_ice(absorbingband_index,index_lower:index_upper), & 
                                        retrievalradius1621)
        tauRegimeThreshold(2) = scaledTauRegimeThreshold/(1.-asymmetry)

     else
     !if (process%watercloud .or. process%unknowncloud) then
        phase = 'water'
        call getradiibounds(size(water_radii), water_radii, retrievalradius1621, index_upper, index_lower)
        asymmetry = linearinterpolation(water_radii(index_lower:index_upper),  &
                                        asymmetry_water(uncertainty_nonabsorbing_1621,index_lower:index_upper),&
                                        retrievalradius1621)
        tauRegimeThreshold(1) = scaledTauRegimeThreshold/(1.-asymmetry)
        asymmetry = linearinterpolation(water_radii(index_lower:index_upper), &
                                        asymmetry_water(absorbingband_index,index_lower:index_upper), &
                                        retrievalradius1621)
        tauRegimeThreshold(2) = scaledTauRegimeThreshold/(1.-asymmetry)
     endif

     albedo_holder =  (/surface_albedo(xpoint, ypoint,uncertainty_nonabsorbing_1621), &
                        surface_albedo(xpoint, ypoint,absorbingband_index)/)
     cloud_reflectance = (/band_measurements(xpoint, uncertainty_nonabsorbing_1621, ypoint), &
                           band_measurements(xpoint, absorbingband_index, ypoint)/) 
     R1R2wavelengthIdx = (/uncertainty_nonabsorbing_1621, absorbingband_index/)

     call getuncertainties(retrievalopticalthickness1621, &
                           retrievalradius1621,         &
                           water_path1621, &
                           phase,                     & 
                           tauRegimeThreshold,        &
                           R1R2wavelengthIdx,         &
                           albedo_holder,             &
                          transmittance_twoway(xpoint,ypoint,uncertainty_nonabsorbing_1621), &
                          transmittance_twoway(xpoint,ypoint,absorbingband_index), &
                          meandelta_trans(xpoint,ypoint,uncertainty_nonabsorbing_1621), &
                          meandelta_trans(xpoint,ypoint,absorbingband_index), &
                          transmittance_stddev(xpoint,ypoint,uncertainty_nonabsorbing_1621), &
                          transmittance_stddev(xpoint,ypoint,absorbingband_index), &
                           local_cloud_ot1621_error,    &
                           local_cloud_re1621_error,   &
                           local_wp1621_error)

   else
     local_cloud_ot1621_error = fillvalue_real
     local_cloud_re1621_error = fillvalue_real
     local_wp1621_error       = fillvalue_real
   endif

   if ( local_cloud_ot1621_error .lt. epsilon(local_cloud_ot1621_error) .or.  &
        local_cloud_re1621_error .lt. epsilon(local_cloud_re1621_error)  .or. &
        local_wp1621_error .lt. epsilon(local_wp1621_error)) then 
     local_cloud_ot1621_error = fillvalue_real
     local_cloud_re1621_error = fillvalue_real
     local_wp1621_error       = fillvalue_real
   endif


   absorbingband_index = band_0213

   !assign the retrievals to the relevant arrays
   nonabsorbing_band(xpoint, ypoint)         = uncertainty_nonabsorbing
   if (uncertainty_nonabsorbing > 0 ) &
   nonabs_obs_corrected(xpoint, ypoint)      = band_measurements(xpoint, uncertainty_nonabsorbing, ypoint)
   if (absorbingband_index >0) &
   absorbing_obs_corrected(xpoint, ypoint)   = band_measurements(xpoint, absorbingband_index, ypoint)
   liquid_water_path(xpoint, ypoint)         = water_path
   liquid_water_path_1621(xpoint, ypoint)    = water_path1621
   optical_thickness(xpoint, ypoint)         = retrievalopticalthickness
   optical_thickness_1621(xpoint, ypoint)    = retrievalopticalthickness1621
   effective_radius_16(xpoint, ypoint)       = retrievalradius16
   effective_radius_21(xpoint, ypoint)       = retrievalradius21
   effective_radius_1621(xpoint, ypoint)     = retrievalradius1621
   effective_radius_37(xpoint, ypoint)       = retrievalradius37
   cloud_layer_flag(xpoint, ypoint)          = local_cloud_layer_flag
   optical_thickness_error(xpoint, ypoint)   = local_cloud_ot21_error
   effective_radius_21_error(xpoint, ypoint) = local_cloud_re21_error
   liquid_water_path_error(xpoint,ypoint)    = local_wp21_error
   optical_thickness_1621_error(xpoint, ypoint) = local_cloud_ot1621_error
   effective_radius_1621_error(xpoint, ypoint)  = local_cloud_re1621_error
   liquid_water_path_1621_error(xpoint,ypoint)  = local_wp1621_error

end subroutine corescience

subroutine compute_water_path(tau, re, density, library_re, extinction_efficiency, water_path)
! added by G. Wind 11.07.05 to compute the water path properly

  use modis_numerical_module
  implicit none

  real, intent(in) :: tau, re, density
  real, dimension(:), intent(in) :: extinction_efficiency, library_re
  real, intent(out) :: water_path

  integer :: ilo, ihi
  real :: x(2), y(2), Qe

  ilo = 0 
  ihi = 0 

! determine the bounds for interpolation
  call bisectionsearch(library_re, re, ilo, ihi)
  x(1) = library_re(ilo)
  x(2) = library_re(ihi)
  y(1) = extinction_efficiency(ilo)
  y(2) = extinction_efficiency(ihi)
  
  Qe = linearinterpolation(x,y,re)
  
  water_path = ( 4. * tau * re * density ) / (3. * Qe)


end subroutine compute_water_path




subroutine vis_nonabsorbing_science(reflectance_nonabsorbing,      &
                            nonabsorbing_index,                    &
                            nonabsorbing_albedo,                   &
                            library_wavelengths,                   &
                            library_taus,                          &
                            library_radii,                         &
                            g,w,ce,ak,el,em,en,sfr,fki0,fti1,fti0, &
                            fki1, rfi,                             &
                            theta, theta0, phi,                    &
                            cloudtop_pressure, process,            &
                            optical_thickness_vector)

! sep modification w/Rayleigh scattering, 24 May 2005, etc.

  use GeneralAuxType
  use modis_cloudstructure, only: processflag
  use modis_numerical_module

  implicit none

  type(processflag), intent(in) :: process
  integer, intent(in)  ::  nonabsorbing_index 

  real, intent(in)     ::  reflectance_nonabsorbing, nonabsorbing_albedo,          &
                           theta, theta0, phi, cloudtop_pressure
  real(single), dimension (:),     intent(in)  ::   library_wavelengths, library_taus, library_radii
  real(single), dimension (:,:),   intent(in)  ::   g, w, ce, ak, el, em, en, fki1, fki0
  real(single), dimension (:,:,:), intent(in)  ::   sfr,  fti1, fti0, rfi                       

  real, intent(out)    ::   optical_thickness_vector(:)

  !local variables
  integer :: radii, nx, i, total_taus, wavelengths, ii, quality, number_local_minima, ilo, ihi, checkvariable
  integer :: correction_iteration, maximum_correction_iterations
  integer, dimension(:), allocatable :: local_minima, isw

  real(single)        :: log_argument 
  real                :: ga1, reflectance_corrected,  rfx(6), tsmx1 , tsmx2, local_reflectance_nonabsorbing, q1, &
                         library_opticalthickness, rf1, optical_thickness
  real, dimension(20)                :: tcmx1, tcmx2
  real, dimension(:), allocatable    :: ga2, c0, c1, c2, rfi1, ekk, z,  visibleref_tc, b11
  real, dimension(:,:), allocatable  :: taux

   q1= 0.712
   nx = 6
   tsmx1 = 1.8
   tsmx2 = 2.0

   total_taus = size(library_taus)
   radii = size(library_radii)
   wavelengths = size(library_wavelengths)

   allocate( visibleref_tc(radii), ga2(radii), c0(radii), c1(radii), c2(radii), rfi1(radii), ekk(radii), z(radii) )
   allocate( taux(6,radii), b11(radii), isw(radii) )

!  *****************************************************************
!  get cloud optical thickness from the so-called non-absorbing band
!  *****************************************************************
 
!  1-g 
   b11(1:radii) = 1. - g(nonabsorbing_index,1:radii) 

                            
!  get a set of unscaled optical thicknesses 
!  (with a range covering library and bridging
!  into the asymptotic region).
   
!  for equation (27) conservative
   ga1 = 1. - nonabsorbing_albedo
!  for use with equation (28) non-conservative, sfr is spherical albedo
   ga2(1:radii) = 1. - nonabsorbing_albedo*sfr(total_taus,nonabsorbing_index,1:radii)

   c0 = 0.
   c1 = 0.
   c2 = 0.
   
   do i=1, radii
     if (w(nonabsorbing_index,i) > 0.99999) then  
!       conservative case   
        isw(i) = 1          
     else                
!       non-conservative case: for use in eq.(28) in king's 1987 paper
        isw(i) = 0
!        sep: pull ga2 out of the do loop?
!        ga2(1:radii) = 1. - nonabsorbing_albedo*sfr(total_taus,nonabsorbing_index,1:radii)          
        c0(i) = nonabsorbing_albedo*em(nonabsorbing_index,i)*en(nonabsorbing_index,i)**2
        c1(i) = 0.5/ak(nonabsorbing_index,i)
        c2(i) = el(nonabsorbing_index,i) - c0(i)/ga2(i)
     end if
   enddo

!  convert max tc from scaled to actual optical thickness:
!  tsmx1 = 1.8 scaled optical thickness for asymptotic theory
!  tsmx2 = 2.0 scaled optical thickness for asymptotic theory
        
   tcmx1(1:radii) = tsmx1/b11(1:radii)
   tcmx2(1:radii) = tsmx2/b11(1:radii)
   taux(1,1:radii) = 0

   do i = 1, total_taus-1
     taux(i+1,1:radii) = library_taus(i)/b11(1:radii)
   enddo
   taux(total_taus+1,1:radii) = tcmx1(1:radii)
   taux(total_taus+2,1:radii) = tcmx2(1:radii)

!  taux are now the actual optical thicknesses 
!  corresponding to scaled tau of
!  0, .5, 1.0, 1.5, 1.8, 2.0

!  calculated refl value for tauc=inf at non absorbing wavelength for all radii
   rfi1(1:radii) = rfi(total_taus,nonabsorbing_index,1:radii)  

!  the product of escape functions: K(u)*K(u0)
   ekk(1:radii) = fki1(nonabsorbing_index,1:radii)*fki0(nonabsorbing_index,1:radii) 

   DO i = 1, radii   

!  sep, 25 May: this has to be inside the re loop, otherwise local_reflectance_nonabsorbing decreases
!  with each subsequent re, in addition to each subsequent iterations
   local_reflectance_nonabsorbing = reflectance_nonabsorbing

!  Perform Rayleigh correction iteration loop only for 0.65 um band
   if (nonabsorbing_index==1) then
       maximum_correction_iterations = 3
   else
       maximum_correction_iterations = 1   
   end if

!!! to turn Rayleigh off START
!if(debugPRN) then
!     write (*,'(/a/)')          '### IN SUBROUTINE vis_nonabsorbing_science: Rayleigh Correction TURNED OFF!'
!endif
!maximum_correction_iterations = 1   
!!! to turn Rayleigh off END   
     
   do correction_iteration=1, maximum_correction_iterations

! wind fix 6.24.05 correct for divide-by-zero and also indexing changed to (i) from (1:radii)
         if ( (rfi1(i) - local_reflectance_nonabsorbing) < 0.0001) then
            optical_thickness_vector(i) = 100.
            exit
         endif

         z(i) =  ekk(i)/ (rfi1(i) - local_reflectance_nonabsorbing)

!    (1) Try asymptotic theory to find the tau corresponding 
!        to the measurement for all radii. Then test to see if applicable.

         if (z(i) < 0. )then
            visibleref_tc(i) = -1.
         else
            if (isw(i) == 1 ) then
!              conservative case Eq. 27 from King(1987)
               visibleref_tc(i) = (4.0 * (z(i) -  nonabsorbing_albedo/ga1)/3.0 - 2.*q1)/b11(i)
            else
!              non-conservative case Eq. 28 from King (1987)
               visibleref_tc(i) = -1.
               log_argument = c2(i)  *(em(nonabsorbing_index,i)*z(i) + el(nonabsorbing_index,i)) 
               if (log_argument > 0. ) visibleref_tc(i) = c1(i) * log(log_argument)
            endif
         endif

!    (2) Test whether asymptotic calculation is appropriate. If not use library values and scaled tau = 1.8, 2.0
!        and interpolate for non-asymptotic tau solution.

!        sep: comparing to collect 4, mag added >0. in following line for some reason
!        if (visibleref_tc(i) < tcmx1(i) .and. visibleref_tc(i) > 0. ) then

         if (visibleref_tc(i) < tcmx1(i) ) then   ! sep fix: 4-27-05

!           sep, 3 May: Note that tcmx1 corresponds to a tau_scaled=1.8, not 1.5. Why isn't 1.5 chosen? 
!           I guess 1.5-1.8 is considered an overlap region and treated as such.

!           rfx index=1 is set to the surface albedo
            rfx(1) = nonabsorbing_albedo

!           rfx index=2,3,4 are library values
            rfx(2:4) = rfi(1:3, nonabsorbing_index, i) +  &
                    fti1(1:3, nonabsorbing_index, i) *    &
                    fti0(1:3, nonabsorbing_index, i) *    &
                    nonabsorbing_albedo  /                &
                    (1. -nonabsorbing_albedo*sfr(1:3,nonabsorbing_index,i)) 


!           rfx index=5,6 are from asymptotic theory
            call asymptotic_calcu(taux(5, i),g(nonabsorbing_index,i),              &
                               w(nonabsorbing_index,i),                            &
                               ak(nonabsorbing_index,i),el(nonabsorbing_index,i),  &
                               em(nonabsorbing_index,i),en(nonabsorbing_index,i),  &
                               sfr(total_taus,nonabsorbing_index,i),               &
                               rfi1(i),fki1(nonabsorbing_index,i),                 &
                               fki0(nonabsorbing_index,i),nonabsorbing_albedo,rfx(5))

            call asymptotic_calcu(taux(6,i),g(nonabsorbing_index,i),               &
                               w(nonabsorbing_index,i),                            &
                               ak(nonabsorbing_index,i),el(nonabsorbing_index,i),  &
                               em(nonabsorbing_index,i),en(nonabsorbing_index,i),  &
                               sfr(total_taus,nonabsorbing_index,i),               &
                               rfi1(i),fki1(nonabsorbing_index,i),                 &
                               fki0(nonabsorbing_index,i),nonabsorbing_albedo,rfx(6))

!           Now we need to interpolate to get the optical thickness. In previous versions of the algorithm,
!           optical thickness was calculated from a cubic spline interpolation.  This is not permissible here
!           since the reflectance vector rfx may not be of the type x(0) < x(1) ... < x(n).  Multiple solutions
!           of the optical thickness for a given reflectance are uncommon, but possible.  The following routine
!           must contain a path for this case.  For now, when this happens we will take the larger of the two
!           optical thickness retrievals.

!            if (debugPRN) print*, "RFX at: ", nonabsorbing_index, " is: ", rfx


            call interpolate_refl_cot (local_reflectance_nonabsorbing, rfx, taux(:,i), optical_thickness) 

         else
            optical_thickness = visibleref_tc(i) 
            if (visibleref_tc(i) < 0. )  optical_thickness = -1.
         endif
   
         optical_thickness_vector(i) = optical_thickness
         if (nonabsorbing_index == 1 .and. optical_thickness > 0. .and. maximum_correction_iterations /= 1) then

            call rayleighcorrection (local_reflectance_nonabsorbing,                                              &
                          cloudtop_pressure, process,                                                             &
                          optical_thickness, nonabsorbing_albedo,                                                 &
                          g(nonabsorbing_index,i), w(nonabsorbing_index,i), ce(nonabsorbing_index,i),             &
                          ak(nonabsorbing_index,i), el(nonabsorbing_index,i),                                     &
                          em(nonabsorbing_index,i), en(nonabsorbing_index,i),                                     &
                          fki1(nonabsorbing_index,i), fki0(nonabsorbing_index,i),                                 &
                          fti1(:,nonabsorbing_index,i),fti0(:,nonabsorbing_index,i), sfr(:,nonabsorbing_index,i), &
                          nonabsorbing_index,i,theta0,theta,phi,                                                  &
                          reflectance_corrected)

            local_reflectance_nonabsorbing = reflectance_corrected


            if (local_reflectance_nonabsorbing > rfi1(i)) then 
               optical_thickness_vector(i) = 100.
               exit
            endif


         else
            exit
         endif

      end do   ! Rayleigh correction loop
   END DO      ! radii loop for nonabsorbing band optical thickness

!  convert optical thickness at arbitrary wavelength to optical thickness at 0.65 micron 
!  (meaningful only if non absorbing wavelength is not .65 um, i.e., tau_wl = Qe_0.65/Qe_wl * tau_0.65)

   where (optical_thickness_vector > 0.)
      optical_thickness_vector = optical_thickness_vector * ce(1,:)/ce(nonabsorbing_index,:)
   end where 

   do i =1 , size(optical_thickness_vector)
   !  if (optical_thickness_vector(i) > 100.) optical_thickness_vector(i) = 100.
      if(rfi1(i) < reflectance_nonabsorbing )  optical_thickness_vector(i) = 100.
      
   end do      

   deallocate( ga2, c0, c1, c2, rfi1, ekk, z, visibleref_tc, b11, taux, isw)

end subroutine vis_nonabsorbing_science


subroutine vis_absorbing_science(optical_thickness_vector,  &
                                 reflectance_absorbing, &
                                 absorbing_index,   &
                                 absorbing_albedo,       &
                                 library_wavelengths,       &
                                 library_taus,              &
                                 library_radii,             &
                                 g,w,ce,ak,el,em,en,sfr,fki0,fti1,fti0,&
                                 fki1, rfi, &
                                 residual, maxradii, debug )

! sep, 4 May: example call
!
!        call vis_absorbing_science(optical_thickness_vector,          &
!                              band_measurements(xpoint, absorbingband_index, ypoint), &
!                              absorbingband_index,                 &
!                              surface_albedo(xpoint, ypoint,absorbingband_index), &
!                              library_wavelengths,                  &
!                              library_taus,                         &
!                              ice_radii,                            &
!                              asymmetry_ice,                        &
!                              singlescattering_ice,                 &
!                              extinction_ice,                       &
!                              ak_ice,                               &
!                              el_ice,                               &
!                              em_ice,                               &
!                              en_ice,                               &
!                              spherical_albedo_ice,                 &
!                              int_escapefuncice_solar,              &
!                              int_fluxdownice_sensor,               &
!                              int_fluxdownice_solar,                &
!                              int_escapefuncice_sensor,             &
!                              int_reflectance_ice,                  &
!                              residual,maxradii,debug)


! with 
!   1) a known vector of optical thicknesses 
!   2) absorbing band reflectance for the scene of interest
!   3) library values for the scene of interest 
!
! cloud top effective particle radius is calculated

  use GeneralAuxType
  use spline_module

  implicit none

  real, intent(in)     ::   optical_thickness_vector(:), reflectance_absorbing,  &
                            absorbing_albedo       

  integer, intent(in)   ::  absorbing_index
 

  real(single), intent(in)     ::   library_wavelengths(:),    &
                            library_taus(:),           &
                            library_radii(:),          &
                            g(:,:),                    &
                            w(:,:),                    &
                            ak(:,:),                   &
                            ce(:,:),                   &
                            el(:,:),                   &
                            em(:,:),                   &
                            en(:,:),          &
                            sfr(:,:,:),               &
                            fki1(:,:),                 &
                            fki0(:,:),                 &
                            fti1(:,:,:),               &
                            fti0(:,:,:),               &
                            rfi(:,:,:)

  real, intent(out)    ::   residual(:)
  logical, intent(in)  ::   debug
  integer, intent(out) ::   maxradii

  !local variables
  real     :: ga1, reflectance_corrected, yy2(6), rfx(6), tsmx1 , tsmx2, &
              q1, value, &
              library_opticalthickness, rf1

  real, allocatable    :: rfi1(:)
  real, allocatable    :: taux(:,:), localoptical_thickness_vector(:), b11(:), tcmx1(:), tcmx2(:)

  integer  :: isw, radii, nx, i, total_taus, wavelengths, ii, quality,  ilo, ihi, checkvariable, points

  q1= 0.712
  nx = 6
  tsmx1 = 1.8
  tsmx2 = 2.0

  total_taus = size(library_taus)
  radii = size(library_radii)
  wavelengths = size(library_wavelengths)

  allocate(rfi1(radii))
  allocate(taux(6,radii))
  allocate(localoptical_thickness_vector(radii))
  allocate(b11(radii))
  allocate(tcmx1(radii))
  allocate(tcmx2(radii))

  b11(1:radii) = 1. -g(absorbing_index,1:radii)
  tcmx1(1:radii) = tsmx1/b11(1:radii)
  tcmx2(1:radii) = tsmx2/b11(1:radii)


  taux(1,1:radii) = 0.


! science fixer, for when the semi-infinite layer library reflectance goes wiggly
  do i = radii, 2, -1
    maxradii = i
    if (optical_thickness_vector(i) > 0. ) exit
  enddo

  do i = 1, total_taus - 1
     taux(i+1,1:radii) = library_taus(i)/b11(1:radii)
  enddo

   taux(total_taus+1,1:radii) = tcmx1
   taux(total_taus+2,1:radii) = tcmx2

!  get reflectance residual (library - measurement)  for the absorbing band

   localoptical_thickness_vector = optical_thickness_vector * ce(absorbing_index,:)/ce(1,:)

!  calculated refl value for tauc=inf at absorbing wavelength for all radii
   rfi1(1:radii) = rfi(total_taus,absorbing_index,1:radii)
!  reflection functions for channel 2 and 3 with fixed tc
!  all variables must be redefined because this part is used
!  after line-scanning for ch-1.

!  optical thickness  for the appropriate wavelength = qext(wavelength)/qext(0.65 um) * tauc(0.65 um)
!  determine the reflectance at calculated optical thickness

   do i = 1,maxradii
   if (localoptical_thickness_vector(i)  >= tcmx1(i)) then

      call asymptotic_calcu(localoptical_thickness_vector(i) ,g(absorbing_index,i), &
                     w(absorbing_index,i),ak(absorbing_index,i),el(absorbing_index,i), &
                     em(absorbing_index,i),en(absorbing_index,i), &
                     sfr(total_taus,absorbing_index,i), &
                     rfi1(i),fki1(absorbing_index,i), &
                     fki0(absorbing_index,i),absorbing_albedo,rf1)
   else
      rfx(1) =  absorbing_albedo

      rfx(2:4) = rfi(1:3,absorbing_index,i)  + &
                 fti1(1:3,absorbing_index,i) * &
                 fti0(1:3,absorbing_index,i) * &
                 absorbing_albedo /            &
                 ( 1.-absorbing_albedo*sfr(1:3,absorbing_index,i))

      call asymptotic_calcu(taux(5,i),  &
                 g(absorbing_index,i),w(absorbing_index,i),      &
                 ak(absorbing_index,i),el(absorbing_index,i),    &
                 em(absorbing_index,i),en(absorbing_index,i),    &
                 sfr(total_taus,absorbing_index,i),  &
                 rfi1(i),fki1(absorbing_index,i),       &
                 fki0(absorbing_index,i),absorbing_albedo,rfx(5))

      call asymptotic_calcu(taux(6,i),  &
                 g(absorbing_index,i),w(absorbing_index,i),      &
                 ak(absorbing_index,i),el(absorbing_index,i),    &
                 em(absorbing_index,i),en(absorbing_index,i),    &
                 sfr(total_taus,absorbing_index,i),  &
                 rfi1(i),fki1(absorbing_index,i),       &
                 fki0(absorbing_index,i),absorbing_albedo,rfx(6))
!      if(debugPRN) print*, "RFX at: ", absorbing_index, " is: ", rfx


!     computing reflectance function at
!     tau(lambda) using spline fitting.
      points = 6
      call spline(points, taux(:,i),rfx,yy2)
      rf1 = splint(points,localoptical_thickness_vector(i) ,taux(:,i),rfx,yy2)
   endif

   residual(i) = rf1/reflectance_absorbing- 1.0
   enddo
   deallocate(rfi1,   taux,localoptical_thickness_vector, tcmx1, b11, tcmx2)
end subroutine vis_absorbing_science
subroutine nir_absorbing_science(platform_name,&
                                 optical_thickness_vector,  &
                                 reflectance_absorbing,     &
                                 absorbing_index,           &
                                 absorbing_albedo,          &
                                 cloud_top_temperature,     &
                                 surface_temperature,       &
                                 thermal_trans_1way,        &
                                 thermal_trans_2way,        &
                                 solar_zenith,              &
                                 library_wavelengths,       &
                                 library_taus,              &
                                 library_radii,             &
                                 g,w,ce,ak,el,em,en,        &
                                 sfr,fki0,fti1,fti0,fri1,   &
                                 fki1, rfi,                 &
                                 residual, maxradii, debug)

! sep, 4 May: example call
!
!        call nir_absorbing_science(platform_name, &
!                              optical_thickness_vector,          &
!                              band_measurements(xpoint, absorbingband_index, ypoint), &
!                              absorbingband_index - 1,                 &
!                              surface_albedo(xpoint, ypoint,absorbingband_index), &
!                              cloud_top_temperature(xpoint, ypoint), &
!                              surface_temperature(xpoint, ypoint), &
!                              thermal_trans_1way,            &
!                              thermal_trans_2way,            &
!                              solar_zenith_angle(xpoint,ypoint),    &
!                              library_wavelengths,                  &
!                              library_taus,                         &
!                              ice_radii,                            &
!                              asymmetry_ice,                        &
!                              singlescattering_ice,                 &
!                              extinction_ice,                       &
!                              ak_ice,                               &
!                              el_ice,                               &
!                              em_ice,                               &
!                              en_ice,                               &
!                              spherical_albedo_ice,                 &
!                              int_escapefuncice_solar,              &
!                              int_fluxdownice_sensor,               &
!                              int_fluxdownice_solar,                &
!                              int_fluxupice_solar,                  &
!                              int_escapefuncice_sensor,             &
!                              int_reflectance_ice,                  &
!                              residual, maxradii, debug)


  use GeneralAuxType

  implicit none

  character*(*),intent(in)::  platform_name

  real, intent(in)     ::   optical_thickness_vector(:), reflectance_absorbing,  &
                            absorbing_albedo,solar_zenith

  integer, intent(in)   ::  absorbing_index

  real, intent(in)      :: cloud_top_temperature, surface_temperature, thermal_trans_2way,thermal_trans_1way

  real(single), intent(in)     ::   library_wavelengths(:),    &
                            library_taus(:),           &
                            library_radii(:),          &
                            g(:,:),                    &
                            w(:,:),                    &
                            ak(:,:),                   &
                            ce(:,:),                   &
                            el(:,:),                   &
                            em(:,:),                   &
                            en(:,:),          &
                            sfr(:,:,:),               &
                            fki1(:,:),                 &
                            fki0(:,:),                 &
                            fti1(:,:,:),               &
                            fti0(:,:,:),               &
                            fri1(:,:,:),               &
                            rfi(:,:,:)
  logical, intent(in)  ::  debug
  real, intent(out)    ::   residual(:)
  integer, intent(out) ::   maxradii

  integer              :: total_taus, radii, wavelengths,i
  real                 :: ReflectanceSolarMeasured,rf1, rtherm37
  real                 :: tc, tsmx1, tsmx2, local_trans_1way, local_trans_2way
  real, allocatable    :: localoptical_thickness_vector(:), rfi1(:), localtaux(:,:),taux(:,:), b11(:),tcmx1(:), tcmx2(:)

  tsmx1 = 1.8
  tsmx2 = 2.0

  total_taus = size(library_taus)
  radii = size(library_radii)
  wavelengths = size(library_wavelengths)


! Early exit criteria.  If we have no thermal information a retrieval is
! impossible

  if (cloud_top_temperature < 0. ) then
    residual  =-999.
    return
  endif    

  allocate(localoptical_thickness_vector(radii))
  allocate(rfi1(radii))
  allocate(taux(6,radii))
  allocate(localtaux(5,radii))
  allocate(b11(radii))
  allocate(tcmx1(radii))
  allocate(tcmx2(radii))

  b11(1:radii) = 1. -g(absorbing_index,1:radii)
  tcmx1(1:radii) = tsmx1/b11(1:radii)
  tcmx2(1:radii) = tsmx2/b11(1:radii)

  taux(1,1:radii) = 0.
  do i = 1, total_taus - 1
     taux(i+1,1:radii) = library_taus(i)/b11(1:radii)
  enddo
 
  localtaux = 0. 
  do i = 1, 4
     localtaux(i+1,1:radii) = library_taus(i)/b11(1:radii)
  enddo

  taux(total_taus+1,1:radii) = tcmx1
  taux(total_taus+2,1:radii) = tcmx2


! science fixer, for when the semi-infinite layer library reflectance goes wiggly
  do i = radii, 2, -1
    maxradii = i
    if (optical_thickness_vector(i) > 0. ) exit
  enddo

  rfi1(1:radii) = rfi(total_taus,absorbing_index,1:radii)

  localoptical_thickness_vector = optical_thickness_vector * ce(absorbing_index,:)/ce(1,:)
  do i = 1, maxradii
    call toa_radiance37 (platform_name, &
                         library_taus, total_taus,optical_thickness_vector(i),&
                         tcmx1(i),taux(:,i),localtaux(:,i), &
                         g(absorbing_index,i),      w(absorbing_index,i),       &
                         ak(absorbing_index,i),     el(absorbing_index,i),      &
                         em(absorbing_index,i),     en(absorbing_index,i),      &
                         sfr(:,absorbing_index,i),  rfi1(i),                    &
                         fki1(absorbing_index,i),   fki0(absorbing_index,i),    &
                         fti0(:,absorbing_index,i), fti1(:,absorbing_index,i),  &
                         fri1(:,absorbing_index,i),rfi(:,absorbing_index,i), &
                         absorbing_albedo,                   &
                         surface_temperature,cloud_top_temperature, &
                         rf1,                                        &
                         rtherm37)

!   thermal component of the top of the atmosphere measurement in radiance units
    
    if (thermal_trans_1way < 0. .or. thermal_trans_1way > 1.) then
      local_trans_1way = 1.
    else
      local_trans_1way = thermal_trans_1way
    endif

    if (thermal_trans_2way < 0. .or. thermal_trans_2way > 1.) then
      local_trans_2way = 1.
    else
      local_trans_2way = thermal_trans_2way
    endif

    rtherm37 = rtherm37 * local_trans_1way

!   solar component of the top of the atmosphere measurement in reflectance units
    ReflectanceSolarMeasured = pi * (reflectance_absorbing-rtherm37) / (cos(solar_zenith*d2r)*11.739)

!   Fri Sep 27, 2002: 3.7 um solar iradiance changed to 11.297 from 10.715 (sep)
!   Th, June 2, 2005: 3.7 um solar iradiance  changed from 11.297 to 11.739 W/m2-um based on Juan Fontenla (LASP, CU) solor model

!   above cloud atmosphere corrected  solar component
!   of the top of the atmosphere measurement in reflectance units
    ReflectanceSolarMeasured = ReflectanceSolarMeasured / local_trans_2way

!if(debugPRN) then
!     write (*,'(/a)')              '=== IN SUBROUTINE nir_absorbing_science'
!     write (*,'( a, 2(2x,f6.2))')  'local_trans_1way, v: ', local_trans_1way, local_trans_2way 
!     write (*,'(3(a, (2x,f6.2)))') 'rtherm37 * local_trans_1way = ',  rtherm37, ' Measured radiance=', reflectance_absorbing, &
!     ' ReflectanceSolarMeasured/local_trans_2way =', ReflectanceSolarMeasured
!endif

!   Compare this with the library value rf1
    residual(i) = ( rf1 / ReflectanceSolarMeasured ) - 1.

!   rtherm37 > reflectance_absorbing is not physical. Strictly speaking
!   the answer here would be infinity.

    if (rtherm37 .gt. reflectance_absorbing) then
      residual(i) = 10000.
    endif
  enddo
  deallocate(localtaux, rfi1, localoptical_thickness_vector,taux,b11,tcmx1,tcmx2)
end subroutine nir_absorbing_science
subroutine asymptotic_calcu(tc,g,w,ak,el,em,en, &
                            sfr, rfi, fk1,fk0,ag,rfg)
  use GeneralAuxType
  implicit none

  real(single), intent(in)    :: tc,ag, rfi
  real(single), intent(in)    :: g,w,ak,el,em,en,sfr, fk1, fk0
  real(single), intent(out)   :: rfg

  real(single)                :: q1, rf0, ft1, sfr1, x, fkk, exc, ft0
  parameter (q1=0.712)

  fkk = fk1*fk0

  if (w > 0.999991) then

!    conservative case
     x = 4.0/3.0/ ((1.-g)*tc+2.0*q1)      
     ft1 = fk1*x                          !this is eq.(17)
     ft0 = fk0*x                          !this is eq.(17)
     sfr1 = 1.0- x                        !this is eq.(19)
     rf0 = rfi - fkk*x                    !this is eq.(4)
  else

!    non-conservative case
     exc = ak*tc
     if (exc <= 50.0) then
        exc = exp(-exc)
     else
        exc = 0.
     end if

     x = em*exc/ (1.0 - (el*exc)**2)      
     ft1 = en*fk1*x                       !this is eq.(20)
     ft0 = en*fk0*x                       !this is eq.(20)
     sfr1 = sfr - el*en**2*exc*x          !this is eq.(22)
     rf0 = rfi - el*fkk*exc*x             !this is eq.(6)
  end if

  rfg = rf0 + ft1*ft0*ag/ (1.-ag*sfr1)    !this is eq.(11)
end subroutine asymptotic_calcu
subroutine interpolate_refl_cot(reflectance, &
                                reflectance_vector, &
                                optical_thickness_vector, &
                                optical_thickness)
!
!  We need to interpolate the reflectance vector to get the optical thickness that
!  corresponds to the reflectance given.  We cannot assume however that the reflectance
!  is uniformly increasing as a function of the optical thickness.  There are cases
!  where there maybe multiple solutions of the optical thickness for a given reflectance
!  In this case we take the larger solution and move on.


   use GeneralAuxType
   use spline_module
          

   implicit none

   real, intent(in) :: reflectance, reflectance_vector(:), optical_thickness_vector(:)
   real, intent(out):: optical_thickness

   real, dimension(20) :: differences, derivatives
   integer  :: i, number_solutions, refl_size
   real     :: mindiff
   
   derivatives = 0.
   derivatives = 0.

   refl_size = size(reflectance_vector)

   if (reflectance < reflectance_vector(1) .or.  &
       reflectance > reflectance_vector(refl_size) ) then
!     reflectance is out of range. no interpolation is possible
      optical_thickness = - 1.
      return
   endif
      
!  is the reflectance vector increasing?

   differences(1:refl_size-1) = reflectance_vector(1:refl_size-1) - reflectance_vector(2:refl_size)
   mindiff = minval(differences(1:refl_size-1))
  
   if (mindiff > 0.) then 
!     we can do a plain cubic spline

      call spline(refl_size, reflectance_vector,optical_thickness_vector, derivatives(1:refl_size) )
      optical_thickness = splint(refl_size, reflectance, reflectance_vector,optical_thickness_vector,derivatives(1:refl_size))

   endif

   if (mindiff <= 0. ) then
     if (reflectance >=  reflectance_vector(1) .and. &
        reflectance < reflectance_vector(2)) then
        optical_thickness = optical_thickness_vector(1) + &
                            (( reflectance - reflectance_vector(1)) /                &
                             ( reflectance_vector(2) - reflectance_vector(1)) *    &
                             ( optical_thickness_vector(2) - optical_thickness_vector(1)) )
     else
        call spline(refl_size-1, &
                    reflectance_vector(2:refl_size),        &
                    optical_thickness_vector(2:refl_size),  &
                    derivatives(1:refl_size-1) )
        optical_thickness = splint(refl_size-1, &
                                   reflectance,                            &
                                   reflectance_vector(2:refl_size),        &
                                   optical_thickness_vector(2:refl_size),  &
                                   derivatives(1:refl_size-1))

     endif
   endif     

end subroutine interpolate_refl_cot

subroutine rayleighcorrection (reflectance, cloudtoppressure, process,  &
                               optical_thickness, nonabsorbing_galbedo, &
                               g, w, ce, ak, el, em, en,                &
                               fki1, fki0, fti1, fti0, sfr, iw, ir,     &
                               solarzenith, sensorzenith, azimuth,      &
                               reflectance_corrected)

   ! Versioning history: 
   !
   !  Initial, but incomplete, portions written by M. Gray
   !  Modified by B. Wind, G. Wind to add cloud albedo
   !  5-16-05, etc.: Modified by S. Platnick to include non-zero surface albedo and reflectances in the asymptotic regime,
   !   as well as changes to loop logic/structure in vis_nonabsorbing_science

   ! This algorithm is based on the Rayleigh correction algorithm described in Wang and King (1997). The 
   ! Rayleigh correction algorithm described in the paper is derived for a black surface and describes
   ! results for water clouds only.  Here the algorithm has been extended to a non-zero surface albedo 
   ! and uses forward library water and ice cloud fluxes based on the decision tree cloud phase.

   ! VARIABLE TRANSLATION
   !
   ! nonabsorbing_galbedo = sfc albedo in nonabsorbing band; scalar
   ! g, w, ce       = asymmetry parameter, single scattering albedo, extinction efficiency; indexing
   ! ak, el, em, en = asymptotic parameters
   ! fki0, fki1     = escape functions interpolated to mu0, mu
   ! fti0, fti1     = transmitted fluxes interpolated to mu0, mu; indexing: (tau index)
   ! sfr            = spherical albedo; indexing: (tau index)
   ! iw, ir         = band and effective radius indices, respectively
 
   use GeneralAuxType
   use libraryarrays
   use modis_cloudstructure, only: processflag

   implicit none

  !intent in/out variables
   type(processflag), intent(in)  :: process
   integer, intent(in) :: iw, ir
   real, intent(in)    ::  reflectance, cloudtoppressure, optical_thickness, nonabsorbing_galbedo, &
                           solarzenith, sensorzenith, azimuth
   real, intent(in)    :: g, w, ce, ak, el, em, en, fki1, fki0, sfr(:), fti0(:), fti1(:)

   real, intent(out)   :: reflectance_corrected

  !local variables
   real  :: taur0, taur, Cm, rd, surfacepressure, cloud_albedo
   real  :: mu0, mu, flux0, flux1, pray, x, refl_s
   real  :: local_int_fluxup_solar,local_int_fluxup_sensor
   parameter (surfacepressure=1013., taur0=0.044, Cm=0.84)

   local_int_fluxup_solar=0.; local_int_fluxup_sensor=0.; reflectance_corrected=0.

!  Rayleigh optical depth from TOA to cloud-top
   taur = taur0*cloudtoppressure/surfacepressure

   if(process%icecloud) then
      call interp_lib_reflFlux_cloudAlbedo              &
                       (solarzenith, sensorzenith,      &
                        optical_thickness,              &
                        nonabsorbing_galbedo, sfr,      &
                        g, w, ce, ak, el, em, en,       &
                        fki1, fki0, fti1, fti0, iw, ir, &
                        library_fluxsolarzenith,        &
                        library_fluxsensorzenith,       &
                        flux_up_ice_solar,              &
                        flux_up_ice_sensor,             &
                        local_int_fluxup_solar,         &
                        local_int_fluxup_sensor)
   else
      call interp_lib_reflFlux_cloudAlbedo              &
                       (solarzenith, sensorzenith,      &
                        optical_thickness,              &
                        nonabsorbing_galbedo, sfr,      &
                        g, w, ce, ak, el, em, en,       &
                        fki1, fki0, fti1, fti0, iw, ir, &
                        library_fluxsolarzenith,        &
                        library_fluxsensorzenith,       &
                        flux_up_water_solar,            &
                        flux_up_water_sensor,           &
                        local_int_fluxup_solar,         &
                        local_int_fluxup_sensor)
   endif

   mu0 = cos(d2r*solarzenith)
   mu  = cos(d2r*sensorzenith)

!  Rayleigh scattering phase function, where x=scattering angle (Eq. 3, Wang and King)
!  sep, 13 May: confirmed that pi is not needed in pray formula since it is for reflectance, not intensity.
!  sep, 16 May: w/KIng, confirmed that this formula is correct for the usual definition of relative azimuth
!  and mu, mo0 defined to be always positive.

   x = -mu0*mu + sqrt((1.-mu0**2)*(1.-mu**2))*cos(d2r*azimuth)
   pray = 3.*(1.+x**2)/4.0

!  compute single scattering reflectance from molecules bounded by cloud layer (portion of Eq. 11, Wang and King)

   refl_s = taur*pray/(4.*mu0*mu) + &
            0.5*taur* local_int_fluxup_sensor * exp(-taur/mu )/mu0 + &
            0.5*taur* local_int_fluxup_solar  * exp(-taur/mu0)/mu

!  remove rayleigh scattering effects (Eq. 11, Wang and King)

   reflectance_corrected = (reflectance - refl_s)*exp(Cm*taur*(1./mu0 + 1./mu))

end subroutine rayleighcorrection

subroutine interp_lib_reflFlux_cloudAlbedo              &
                       (solarzenith, sensorzenith,      &
                        optical_thickness,              &
                        nonabsorbing_galbedo, sfr,      &
                        g, w, ce, ak, el, em, en,       &
                        fki1, fki0, fti1, fti0, iw, ir, &
                        fluxsolarzenithangles,          &
                        fluxsensorzenithangles,         &
                        fluxup_solar, fluxup_sensor,    &
                        interp_fluxup_solar, interp_fluxup_sensor)

   use GeneralAuxType
   use modis_numerical_module
   use libraryarrays
   implicit none

  !intent in/out variables
   integer, intent(in) :: iw, ir
   real, intent(in)    ::  solarzenith, sensorzenith, optical_thickness, nonabsorbing_galbedo
   real, intent(in)    ::  g, w, ce, ak, el, em, en, fki1, fki0, sfr(:), fti0(:), fti1(:)
   real, dimension(:), intent(in)       :: fluxsolarzenithangles, fluxsensorzenithangles
   real, dimension(:,:,:,:), intent(in) :: fluxup_solar, fluxup_sensor

   real, intent(out)  :: interp_fluxup_solar, interp_fluxup_sensor

  !local variables
   integer  :: lowbound, highbound, its, nts, taubounds(2)
   real     :: scaledTau, iftaus(2), angles(2), functionvalues(2)
   real     :: q1, interp_fti0, interp_fti1, interp_sfr
   real, dimension(:), allocatable :: fluxup_solar_tau,fluxup_sensor_tau

!  -------------------------------------------------------------------------------------
!  Subroutine to interpolate .65 micron band cloud albedo library in angle and tau space
!  -------------------------------------------------------------------------------------

!  sep NOTES: 
!
! 'bisectionsearch' subroutine (looks like numerical recipe code) and function 'linearinterpolation'
!   both exist in modis_numerical_module.f90
!       
!  Get the cloud-top albedo, including the effect of a non-zero surface albedo
!
!   For non-asymptotic regime:
!
!    With a non-zero surface albedo, the cloud-top albedo is:
!
!      A(tau, mu, galb) = A(tau, mu, galb=0) + T(tau, mu, galb=0)*galb*Tsph(tau)/(1 - Rsph(tau)*galb)
!
!       where,
!
!       A(tau, mu0, galb=0) = local_int_fluxup_solar
!       A(tau, mu,  galb=0) = local_int_fluxup_sensor
!       T(tau, mu,  galb=0) = transmitted flux = fti1/0 = interp_fluxdown_sensor/solar
!       Rsph(tau)           = "spherical" albedo for galb=0 
!       Tsph(tau)           = "spherical" transmittance for galb=0 = 1 - Rsph for conservative scattering
!
!   For asymptotic regime, the cloud-surface albedo for conservative scattering can be simply written as (from M. King notes):
!
!     A(tau, mu, galb) = 1 - 4*(1-galb)*K(mu0)/[3*(1-galb)*(1-g)*(tau+2q0) + 4*galb]

!    NOTE: from modis_frontend_module.f90:1522:  
!      allocate(flux_up_ice_solar (number_fluxsolarzenith, number_taus, number_wavelengths, number_iceradii))

!  optical thickness needs to be scaled in order to interpolate libraries
   scaledTau = optical_thickness * (1. - g)


!  ----------------------------------------------------------
!  ASYMPTOTIC REGIME: CLOUD-TOP ALBEDO w /SURFACE REFLECTANCE
!  ----------------------------------------------------------

   if (scaledTau >= 1.5) then

!      A(tau, mu, galb) = 1 - 4*(1-galb)*K(mu0)/[3*(1-galb)*(1-g)*(tau+2q0) + 4*galb]
!      Escape function interpolated to the solar angles = fki0, for sensor angle = fki1
       q1 = 0.712
       interp_fluxup_solar  = 1. - 4*(1-nonabsorbing_galbedo)*fki0 / &
          (3*(1-nonabsorbing_galbedo)*(1-g)*(optical_thickness + 2*q1) + 4*nonabsorbing_galbedo)
       interp_fluxup_sensor = 1. - 4*(1-nonabsorbing_galbedo)*fki1 / &
          (3*(1-nonabsorbing_galbedo)*(1-g)*(optical_thickness + 2*q1) + 4*nonabsorbing_galbedo)

       RETURN
 
   end if
   
   interp_fluxup_solar=0.; interp_fluxup_sensor=0.

!  allocations and initializations
   nts = size(library_taus)
   allocate(fluxup_solar_tau(nts),fluxup_sensor_tau(nts))
   interp_fluxup_solar = 0.; interp_fluxup_sensor = 0.; taubounds = 0; lowbound=0; highbound=0; iftaus=0. 

!  -------------------------------------------------------
!  NON-ASYMPTOTIC REGIME: CLOUD-TOP ALBEDO w/BLACK SURFACE
!  -------------------------------------------------------

!  sep, 24May: scaledTau<0.5 was not being explicitly counted for in non-asymptotic calculations. 
!  Fortunately (or by design) the values returned for library_taus(0) and fluxup_solar_tau(0) = 0,
!  despite the arrays not being declared for a zero index. Following modified to treat scaledTau<0.5 explicitly.

!
!  Tau index bounds for the scaled optical thickness
!
   if (scaledTau < library_taus(1)) then
      taubounds(1) = 1
      iftaus(1)    = 0.
      iftaus(2)    = library_taus(taubounds(1))
   else
      call bisectionsearch(library_taus, scaledTau, lowbound, highbound)
      taubounds(1) = lowbound
      taubounds(2) = highbound
      iftaus(1)    = library_taus(taubounds(1))
      iftaus(2)    = library_taus(taubounds(2))
   end if

!
!  Interpolation of fti0, fti1, and srf to the scaled optical thickness (for incorporating surface reflectance)
!
   
   if (scaledTau < library_taus(1)) then
      functionvalues(1) = 1.
      functionvalues(2) = fti0(1)
      interp_fti0 = linearinterpolation(iftaus,functionvalues,scaledTau)
      functionvalues(1) = 1.
      functionvalues(2) = fti1(1)
      interp_fti1 = linearinterpolation(iftaus,functionvalues,scaledTau)
      functionvalues(1) = 0.
      functionvalues(2) = sfr(1)
      interp_sfr  = linearinterpolation(iftaus,functionvalues,scaledTau)
   else
      functionvalues(1) = fti0(lowbound)
      functionvalues(2) = fti0(highbound)
      interp_fti0 = linearinterpolation(iftaus,functionvalues,scaledTau)
      functionvalues(1) = fti1(lowbound)
      functionvalues(2) = fti1(highbound)
      interp_fti1 = linearinterpolation(iftaus,functionvalues,scaledTau)
      functionvalues(1) = sfr(lowbound)
      functionvalues(2) = sfr(highbound)
      interp_sfr  = linearinterpolation(iftaus,functionvalues,scaledTau)
   end if

!
!  Interpolation for solar zenith angles and scaled optical thickness
!
   call bisectionsearch(library_fluxsolarzenith, solarzenith, lowbound, highbound)
   angles(1) = fluxsolarzenithangles(lowbound)
   angles(2) = fluxsolarzenithangles(highbound)

   do its = 1,nts
      functionvalues(1) = fluxup_solar(lowbound,its, iw,ir)
      functionvalues(2) = fluxup_solar(highbound,its,iw,ir)
      fluxup_solar_tau(its) = linearinterpolation(angles,functionvalues,solarzenith)
   enddo

   if (scaledTau < library_taus(1)) then
     functionvalues(1) = 0.
     functionvalues(2) = fluxup_solar_tau(taubounds(1))
   else
     functionvalues(1) = fluxup_solar_tau(taubounds(1))
     functionvalues(2) = fluxup_solar_tau(taubounds(2))
   end if
   interp_fluxup_solar = linearinterpolation(iftaus,functionvalues,scaledTau)

!
!  Interpolation for sensor zenith angles and scaled optical thickness
!
   call bisectionsearch(library_fluxsensorzenith, sensorzenith, lowbound, highbound)
   angles(1) = fluxsensorzenithangles(lowbound)
   angles(2) = fluxsensorzenithangles(highbound)

   do its = 1,nts
      functionvalues(1) = fluxup_sensor(lowbound, its,iw,ir)
      functionvalues(2) = fluxup_sensor(highbound,its,iw,ir)
      fluxup_sensor_tau(its) = linearinterpolation(angles,functionvalues,solarzenith)
   enddo

   if (scaledTau < library_taus(1)) then
     functionvalues(1) = 0.
     functionvalues(2) = fluxup_sensor_tau(taubounds(1))
   else
     functionvalues(1) = fluxup_sensor_tau(taubounds(1))
     functionvalues(2) = fluxup_sensor_tau(taubounds(2))
   end if
   interp_fluxup_sensor = linearinterpolation(iftaus,functionvalues,scaledTau)


!  --------------------------------
!  NON-ASYMPTOTIC REGIME: w/SURFACE
!  --------------------------------

!  A(tau, mu, galb) = A(tau, mu, galb=0) + T(tau, mu, galb=0)*galb*Tsph(tau)/(1 - Rsph(tau)*galb)
   interp_fluxup_solar  = interp_fluxup_solar + &
                          interp_fti0*nonabsorbing_galbedo*(1-interp_sfr)/(1 - interp_sfr*nonabsorbing_galbedo)
   interp_fluxup_sensor = interp_fluxup_sensor + &
                          interp_fti1*nonabsorbing_galbedo*(1-interp_sfr)/(1 - interp_sfr*nonabsorbing_galbedo)

   deallocate(fluxup_solar_tau,fluxup_sensor_tau)

end subroutine interp_lib_reflFlux_cloudAlbedo

subroutine toa_radiance37(platform_name, &
                          library_taus, &
                          nts,tc,       &
                          tcmax,taux,localtaux,g,w,ak,      &
                          el,em,en,sfr,rfi1,      &
                          fki1,fki0,fti0,fti1,    &
                          fri1,rfi,galbedo,gtemp, &
                          ttop,rf1,  &
                          rtherm37)

!      call toa_radiance37 (platform_name, &
!                         library_taus, &
!                         total_taus, optical_thickness_vector(i), &
!                         tcmx1(i),taux(:,i),localtaux(:,i), &
!                         g(absorbing_index,i),      w(absorbing_index,i),       &
!                         ak(absorbing_index,i),     el(absorbing_index,i),      &
!                         em(absorbing_index,i),     en(absorbing_index,i),      &
!                         sfr(:,absorbing_index,i),  rfi1(i),                    &
!                         fki1(absorbing_index,i),   fki0(absorbing_index,i),    &
!                         fti0(:,absorbing_index,i), fti1(:,absorbing_index,i),  &
!                         fri1(:,absorbing_index,i),rfi(:,absorbing_index,i), &
!                         absorbing_albedo,                   &
!                         surface_temperature,cloud_top_temperature, &
!                         rf1,                                       &
!                         rtherm37)

  use GeneralAuxType
  use spline_module
  implicit none

  character*(*),intent(in)        ::  platform_name

  integer, intent(in) :: nts
  real, intent(in) :: taux(:),library_taus(:), localtaux(:)

  real(single), intent(in) :: g,w,ak,el,em,en, &
                              fki1,fki0, sfr(:),   &
                              fti1(:), fri1(:), fti0(:),rfi(:)
  real, intent(in)  :: tc,tcmax,      &
                       galbedo, rfi1, &
                       gtemp,ttop

  real, intent(out) :: rtherm37, rf1
  
  integer   ::  i,num, points
  parameter (num=6)
  real      :: sfr1,rthermc, rthermg, tc1 ,frefl1,  trans, localsfr(5)
  real      :: temp(num),tranx(num),frefl(num), rfx(num)
  real      :: derivatives_5(5), derivatives_6(6)
  
  real mu, mu0, remission, intensity
  real rd
  real MODIS_PLANCK
  external MODIS_PLANCK
  parameter (rd=pi/180.)
  
  
  !note: previous developer suggests finding a 
  !better way to pass these variables. no reason was given
  
  !compute reflectance, transmittance, cloud albedo, 
  !and spherical albedo of cloud layer bounded by 
  !lambertian sea surface at the top for band 3.7 micron.
  derivatives_5 = 0.
  derivatives_6 = 0.
  localsfr(1) = 0.
  localsfr(2:5) = sfr

!if(debugPRN) then
!     write (*,'(/a)')                   '+++ IN SUBROUTINE toa_radiance37'
!     write (*,'( a)')                   'INTENT(IN):'
!     write (*,'( a, 3(2x,f6.2))')       'ttop, gtemp, galbedo: ', ttop, gtemp, galbedo
!     write (*,'( a, 2(2x,f6.2),2x,i3)') 'tc, tcmax, nts: ', tc, tcmax, nts
!     write (*,'( a,20(1x,f6.2)))')      'library_taus (scaled)  :   ', library_taus(1:3)
!     write (*,'( a,20(1x,f6.2)))')      'library_taus (unscaled):   ', library_taus(1:3)/(1-g)
!     write (*,'( a, 2(2x,f7.5))')       'g, w:  ',g, w
!     write (*,'( a, 4(2x,f7.5))')       'el, em, en, ak:  ',el,em,en,ak  
!     write (*,'( a, 2(2x,f6.2))')       'fki1,fki0:  ',fki1,fki0 
!     write (*,'( a,20(1x,f6.2))')       'fri1: ', fri1(:)
!     write (*,'( a,20(1x,f6.2))')       'fti0: ', fti0(:)
!     write (*,'( a,20(1x,f6.2))')       'fti1: ', fti1(:)
!     write (*,'( a,20(1x,f6.2))')       'rfi:  ', rfi(:)
!endif

! sep, 3 May: note that tcmax is the actual tau corresponding to a tau_scaled=1.8, not 1.5
  if (tc .ge. tcmax) then                      

!   use asymptotic theory 
 
!   upward reflectance
!   sep, 3 May: bidirection reflectance at tc, used in residual calculation by calling routine?
    call asymptotic_calcu(tc,g,w,ak,el,em,en,sfr(nts),rfi1, fki1,fki0,galbedo,rf1)
  
!   compute downward flux
    call asymptotic_trans(tc,ak,el,em,en,fki1,trans) 
  
!   upward flux
    frefl1 = fri1(nts) - el*em*en*fki1*exp(-2.*ak*tc) / ( 1.-el*el*exp(-2.*ak*tc) )
  
!   spherical albedo
    sfr1 = sfr(nts) - el*em*en*en*exp(-2.*ak*tc)   / ( 1.-el*el*exp(-2.*ak*tc) )

!if(debugPRN) then
!     write (*,'(a)') '    in asymptotic regime ...'
!endif
  
  else 

!   set reflectance, transmissivity and upward flux for optical thickness == 0.

!   sep, 3 May: this rfx zero tau value is for a black surface, but rfx(2),... include the surface albedo as they should
!   rfx(1)   = 0.
    rfx(1)   = galbedo

    tranx(1) = 1.0 
    frefl(1) = 0.
 
!   then for the 3 library optical thicknesses 
    do i = 1, nts - 1
      rfx(i+1) = rfi(i)+fti1(i)*fti0(i)*galbedo/ (1.-galbedo*sfr(i))
  
!     downward flux
      tranx(i+1) = fti1(i)                  
  
!     upward flux
      frefl(i+1) = fri1(i)                  
  
    enddo

! *** sep, 2 May: check indexing, is tau=0 included as i=1? Yes. Indexing is OK. 
    do i = nts + 1, nts + 2
      tc1 = taux(i)
      call asymptotic_calcu(tc1,g,w,ak,el,em,en,sfr(nts),rfi1,fki1,fki0,galbedo,rf1)
      call asymptotic_trans(tc1,ak,el,em,en,fki1,trans)
  
!     reflectance
      rfx(i) = rf1
  
!     transmission
      tranx(i) = trans
  
!     upward flux
      frefl(i) = fri1(nts) - el*em*en*fki1*exp(-2.*ak*tc1) / ( 1.-el*el*exp(-2.*ak*tc1) ) 
!if(debugPRN) then
!     write (*,'( a, 2(2x,f10.6))')       'tc1, exp(-2*ak*tc1) :  ',tc1, exp(-2*ak*tc1)
!endif

    enddo 
  
!   computing reflectance function at tau(lambda) using spline fitting.
!   sep, 3 May: bidirection reflectance at tc, used in residual calculation by calling routine?

    points = 6
    call spline(points,  taux,rfx,derivatives_6)
    rf1 = splint(points, tc,taux,rfx,derivatives_6) 
  
!   computing the cloud albedo at tau(lambda) using spline fitting.
    call spline(points, taux,frefl,derivatives_6)
    frefl1 = splint(points, tc,taux,frefl,derivatives_6)
  
!   computing transmission function at tau(lambda) using spline fitting.
    call spline(points, taux,tranx,derivatives_6)
    trans = splint(points, tc,taux,tranx,derivatives_6)
 
!   if(trans .le. 1.e-9) trans = 0.
    if(trans .le. 1.e-3) trans = 0.
    points = 5 
!   computing spherical albedo at tau(lambda) using spline fitting.
    call spline(points, localtaux,localsfr,derivatives_5)
    sfr1 = splint(points, tc,localtaux,localsfr,derivatives_5)

!if(debugPRN) then
!     write (*,'(a)') '    in non-asymptotic regime ...'
!     write (*,'( a,20(1x,f6.2))') 'rfx:   ',   rfx
!     write (*,'( a,20(1x,f6.2))') 'trans: ', tranx
!     write (*,'( a,20(1x,f6.2))') 'frefl: ', frefl
!endif

  end if

! emission from the cloud (radiance units) at 3.7um
  intensity = modis_planck(platform_name, ttop, 20, 1)
  rthermc = (1.- trans - frefl1) * intensity

! surface emission from the ground (radiance units) at 3.7um
!  intensity = modis_planck(platform_name, ttop, 20, 1)
! *** sep, 2 May: should use gtemp for sfc instead of ttop. Fixed on 3 May.
  intensity = modis_planck(platform_name, gtemp, 20, 1)
  rthermg = trans*(1.-galbedo) * intensity / (1.-sfr1*galbedo)
  
! total thermal radiance contribution at band 3.7 micron
  rtherm37 = rthermc + rthermg

! if(debugPRN) then
!     write (*,'( a, 5(2x,f6.2))') 'cloud: rthermc, intensity, trans, frefl1, emissivity = ', &
!                                         rthermc, intensity, trans, frefl1, (1.- trans - frefl1)
!     write (*,'( a, 5(2x,f6.2))') 'surfc: rthermg, intensity, trans, sfr1,   emissivity = ', &
!                                         rthermg, intensity, trans, sfr1, (1.- galbedo)
!     write (*,'( a, 5(2x,f6.2))') 'total: rtherm37 = ',  rtherm37
!endif

end subroutine toa_radiance37
subroutine asymptotic_trans(tc,ak,el,em,en,fk1,trans)

  implicit none

  real, intent(in)    :: tc,ak,el,em,en,fk1
  real, intent(out)   :: trans

  real                :: exc

  exc = ak*tc
  if (exc.le.50.) then
     exc = exp(-exc)
  else
     exc = 0.
  end if

  trans = en*fk1*em*exc/(1.-(el*exc)**2)   ! this is eq.(20)
  
end subroutine asymptotic_trans

! ssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss
  subroutine find_zero_crossings (residual, crossing_vector, crossing_num)
! ssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss

!   *** S. Platnick, 17-Oct-2005
!   *** G. Wind 20-Oct-2005 -- removed the dynamic memory allocation to speed things up a bit
    implicit none

    REAL, intent(in)     :: residual(:)
    INTEGER, intent(out) :: crossing_num, crossing_vector(:)

!   local variables
!	INTEGER, allocatable, dimension(:) :: k
    integer :: k(15)

    INTEGER :: i,length

    length = size(residual)
!	allocate (k(length))

! G.Wind 12.19.05: Replaced the where statement below with a do loop.  This 
! eliminates ambiguities arising from the use of nonconformable arrays. 
    k =   1
!   where(residual<0) k=-1

    do i=1, length
       if ( residual(i) < 0 ) k(i) = -1
    end do  


!   crossing index in vector is defined for the smaller index between which the zero lies
    crossing_num=0; crossing_vector=0.
    Do i=1,length-1
       if( abs(k(i+1)-k(i)) > 1) then
          crossing_num = crossing_num + 1
          crossing_vector(crossing_num) = i
       end if
    End do
    
    !	deallocate (k)
    
  end subroutine find_zero_crossings

! ssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss
  subroutine find_local_minima (residual, local_min_vector, local_min_num)
! ssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss

!   *** S. Platnick, 17-Oct-2005
!   This give the local minima for the absolute residual, not the residual**2.
!   10-18-05: modify so that minima at end points are not considered local minima
!    to the extent that they do not indicate a change in curvature.

    implicit none

	REAL, intent(in)     :: residual(:)
	INTEGER, intent(out) :: local_min_num, local_min_vector(:)

!   local variables
	INTEGER i,length

	length = size(residual)

	local_min_num=0; local_min_vector=0.

	if (length > 2) then
		Do i=2,length-1
			if( residual(i) < residual(i-1) .AND. residual(i) < residual(i+1) ) then
				local_min_num = local_min_num + 1
				local_min_vector(local_min_num) = i
			end if
		End do
	end if

  end subroutine find_local_minima

! ssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss
  subroutine find_local_maxima (residual, local_max_vector, local_max_num)
! ssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss

!   *** S. Platnick, 17-Oct-2005
!   This give the local maxima for the absolute residual, not the residual**2.
!   10-18-05: modify so that minima at end points are not considered local maxima
!    to the extent that they do not indicate a change in curvature.

    implicit none

	REAL, intent(in)     :: residual(:)
	INTEGER, intent(out) :: local_max_num, local_max_vector(:)

!   local variables
	INTEGER i,length

	length = size(residual)

	local_max_num=0; local_max_vector=0.

	if (length > 2) then
		Do i=2,length-1
			if( residual(i) > residual(i-1) .AND. residual(i) > residual(i+1) ) then
				local_max_num = local_max_num + 1
				local_max_vector(local_max_num) = i
			end if
		End do
	end if

  end subroutine find_local_maxima

! sssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss
  subroutine solution_re (re, residual, crossing_vector, crossing_num,    &
  		local_min_vector, local_min_num, local_max_vector, local_max_num, &
  		retrieval, quality )
! sssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss

!   *** S. Platnick, 17-Oct-2005
!       Using input from sep routines 'find_zero_crossings', 'find_local_minima', and 'find_local_maxima'

  implicit none

  integer, intent(in)    :: crossing_vector(:), crossing_num, &
  							local_min_vector(:), local_min_num, local_max_vector(:), local_max_num
  real, intent(in)       :: residual(:), re(:)

  real, intent(out)      :: retrieval
  integer*1, intent(out)   :: quality

  integer                :: residual_crossing_index, nearest_residual_index, index1(1)
  real                   :: fillval_r4
  logical                :: process_zero_crossing_residual

  fillval_r4 = -1.; residual_crossing_index = 0

! .TRUE. corresponds to the logic that had apparently been in place since the beginning of the c5 deliveries
  process_zero_crossing_residual = .TRUE.   

! Default retrieval output:
  retrieval = fillval_r4; quality = 0

! If there IS NOT a zero crossing (i.e., change of sign) for the residual vector:

  IF (crossing_num == 0) THEN
    if (process_zero_crossing_residual) then
    	index1 = MINLOC(abs(residual))
    	retrieval = re(index1(1))
    	quality = 1
	else
		return
	end if
  END IF

! print*, "line 345"
 
  IF (crossing_num > 0) THEN

    ! If there IS MORE THAN 2 zero crossings for the residual vector, choose the larger re:
	  residual_crossing_index = crossing_vector(crossing_num)
	  
	! Determine nearest_residual_index, where residual_crossing_index is defined from 'find_zero_crossings' 
!       as being smaller index between which the zero lies:

	  If ( abs(residual(residual_crossing_index)) > abs(residual(residual_crossing_index+1))) then
		nearest_residual_index = residual_crossing_index + 1
	  else
		nearest_residual_index = residual_crossing_index
	  end if
!	print*, "line 360"

   ! Solve for the re corresponding to the chosen zero crossing:
	  call solve_for_zero_crossing (re, residual, residual_crossing_index, nearest_residual_index, fillval_r4, &
			local_min_vector, local_min_num, local_max_vector, local_max_num, retrieval, quality)

!	print*, "line 366"

  END IF    

  end subroutine solution_re
! ssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss
  subroutine solve_for_zero_crossing (re, residual, residual_crossing_index, nearest_residual_index, fillval_r4, &
  	local_min_vector, local_min_num, local_max_vector, local_max_num, retrieval, quality)
! ssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssssss

!   *** S. Platnick, 17-Oct-2005

  implicit none

  real,    intent (in)   :: re(:), residual(:), fillval_r4
  integer, intent (in)   :: residual_crossing_index, nearest_residual_index,  &
  							local_min_vector(:), local_min_num, local_max_vector(:), local_max_num

  real,    intent(inout) :: retrieval
  integer*1, intent(inout) :: quality
  
  integer i, status, sign_change
  real x(4), y(4), retrieval_lower_quad, retrieval_upper_quad
  character*15 numerical_type

!  Choice of interpolation scheme for solution of zero crossing based on c5 logic:
!
!    For change in curvature at residual_crossing_index or residual_crossing_index+1: linear
!    For change in curvature at residual_crossing_index-1 or residual_crossing_index+2 AND 
!     nearest_residual_index = residual_crossing: avg of 2 quadratics
!    Otherwise: quadratic
!
!  Note, by definition:
!    1. Both residual_crossing_index and residual_crossing_index+1 must exist.
!    2. The residual_crossing_index is  defined as being the smaller of the two indices between which the zero lies.
!    3. local minima and maxima can not exist at the endpoints of the residual vector.


!  -------------------------------
!  Determine interpolation scheme:
!  -------------------------------

!  Note: 
!    The orig logic for quadavg was independent of the location of the nearest crossing, i.e.,:
!      if (residual_crossing_index-1 == local_max_vector(i)) numerical_type = 'quadavg'
!    Added the if-else-endif to recreate c5 logic for quadavg where use of quadavg depends on the location of nearest crossing.

   numerical_type = 'quad'
   Do i=1,local_max_num
     if (residual_crossing_index == local_max_vector(i) .OR. residual_crossing_index+1 == local_max_vector(i)) then
     	numerical_type = 'linear'
     	exit
     end if
     if (residual_crossing_index-1 >= 1) then
     	if (residual_crossing_index-1 == local_max_vector(i) .AND. nearest_residual_index == residual_crossing_index) then
     		numerical_type = 'quadavg'
     		exit
     	end if
	 end if
     if (residual_crossing_index+2 <= size(residual)) then
     	if (residual_crossing_index+2 == local_max_vector(i) .AND. nearest_residual_index == residual_crossing_index+1) then
     		numerical_type = 'quadavg'
     		exit
     	end if
	 end if
   End do

   If (numerical_type == 'quad') then
   Do i=1,local_min_num
     if (residual_crossing_index == local_min_vector(i) .OR. residual_crossing_index+1 == local_min_vector(i)) then
     	numerical_type = 'linear'
     	exit
     end if
     if (residual_crossing_index-1 >= 1) then
     	if (residual_crossing_index-1 == local_min_vector(i) .AND. nearest_residual_index == residual_crossing_index) then
     		numerical_type = 'quadavg'
     		exit
     	end if
	 end if
     if (residual_crossing_index+2 <= size(residual)) then	
	     if (residual_crossing_index+2 == local_min_vector(i) .AND. nearest_residual_index == residual_crossing_index+1) then
	     	numerical_type = 'quadavg'
	     	exit
	     end if
	 end if
   End do
   End If

!   print *,'residual_crossing_index= ', residual_crossing_index
!   print *,'nearest_residual_index= ', nearest_residual_index
!   print *,'numerical_type= ', numerical_type

!  ------------------------------------------------
!  Implement interpolation scheme and solve for re:
!  ------------------------------------------------

   x=0; y=0; retrieval=fillval_r4; quality=0
   
   IF (numerical_type == 'linear') THEN
        y(1) = residual(residual_crossing_index); y(2) = residual(residual_crossing_index+1)
        if (sign(1.,y(1))*sign(1.,y(2)) <= 0) then
	        call linear_interpolate_for_root (y(1:2), re(residual_crossing_index:residual_crossing_index+1), retrieval)
	        quality = 2
	     end if
   END IF

   IF (numerical_type == 'quad') THEN
        If (nearest_residual_index-1 >=1 .AND. nearest_residual_index+1 <= size(residual)) Then
	        x(1) = re(nearest_residual_index-1);       x(2) = re(nearest_residual_index);         x(3) = re(nearest_residual_index+1)
	        y(1) = residual(nearest_residual_index-1); y(2) = residual(nearest_residual_index)
			y(3) = residual(nearest_residual_index+1)
	    Else If (nearest_residual_index-2 >=1) Then
	        x(1) = re(nearest_residual_index-2);       x(2) = re(nearest_residual_index-1);       x(3) = re(nearest_residual_index)
	        y(1) = residual(nearest_residual_index-2); y(2) = residual(nearest_residual_index-1)
			y(3) = residual(nearest_residual_index)
	    Else If (nearest_residual_index+2 <= size(residual)) Then
	        x(1) = re(nearest_residual_index);         x(2) = re(nearest_residual_index+1);       x(3) = re(nearest_residual_index+2)
	        y(1) = residual(nearest_residual_index);   y(2) = residual(nearest_residual_index+1)
			y(3) = residual(nearest_residual_index+2)
	    Else
	    	y = -1.
	    End If
        call check_for_signchange(y(1:3), sign_change)
        if (sign_change>0) then
        	call quad_interpolate_for_root(x(1:3),y(1:3),retrieval,status)
	        quality = 3
        end if

!       print *, 'In quad interpolate: x = ',x
!       print *, 'In quad interpolate: y = ',y
!        print *, 'Quad interpolate solution status= ', status
   END IF

   IF (numerical_type == 'quadavg') THEN

        x(2) = re(residual_crossing_index);       x(3) = re(residual_crossing_index+1)
        y(2) = residual(residual_crossing_index); y(3) = residual(residual_crossing_index+1)

        If (residual_crossing_index-1 >= 1) Then
			x(1) = re(residual_crossing_index-1); y(1) = residual(residual_crossing_index-1)
        	call check_for_signchange(y(1:3), sign_change)
        	if (sign_change>0) then
	        	call quad_interpolate_for_root(x(1:3),y(1:3),retrieval_lower_quad,status)
			else
         		retrieval_lower_quad = fillval_r4
			end if
		Else
       	 		retrieval_lower_quad = fillval_r4
		End if

        If (residual_crossing_index+2 <= size(residual)) Then
			y(4) = residual(residual_crossing_index+2); x(4) = re(residual_crossing_index+2)
	        call check_for_signchange(y(2:4), sign_change)
    	    if (sign_change>0) then
    		    call quad_interpolate_for_root(x(2:4),y(2:4),retrieval_upper_quad,status)
    		else
       	 		retrieval_upper_quad = fillval_r4
			end if
		Else
       	 		retrieval_upper_quad = fillval_r4
		End if
		
        if (retrieval_lower_quad>0 .AND. retrieval_upper_quad>0) then
        	retrieval = 0.5*(retrieval_lower_quad + retrieval_upper_quad)
        	quality = 3
        end if
        if (retrieval_lower_quad<0 .AND. retrieval_upper_quad>0) then
        	retrieval = retrieval_upper_quad
        	quality = 2
        end if
        if (retrieval_lower_quad>0 .AND. retrieval_upper_quad<0) then
        	retrieval = retrieval_lower_quad
        	quality = 2
        end if
!        print *,'retrieval_lower_quad, retrieval_upper_quad:',retrieval_lower_quad,retrieval_upper_quad
        
   END IF

  end subroutine solve_for_zero_crossing


subroutine solveretrieval( residual,  &
                           optical_thickness_vector, &
                           library_radii, &
                           effective_radius, &
                           optical_thickness,debug)

  use GeneralAuxType
  real, intent(in)   :: residual(:), optical_thickness_vector(:)
  real(single), intent(in) :: library_radii(:)
  logical, intent(in)  :: debug
  real, intent(out)  :: effective_radius, optical_thickness

  integer              :: number_local_minima, ilo, ihi, zero_count
  integer  :: local_minima(8)


   INTEGER, parameter   :: nre=15
   INTEGER :: crossing_num, crossing_vector(nre), local_max_num, &
        local_max_vector(nre), local_min_num, local_min_vector(nre)
   integer*1 :: quality


  logical :: re_altered

  local_minima(:) = 0


	call find_zero_crossings (residual, crossing_vector,  crossing_num)
	call find_local_minima   (residual, local_min_vector, local_min_num)
	call find_local_maxima   (residual, local_max_vector, local_max_num)


	call solution_re (library_radii, residual, crossing_vector, crossing_num, &
			local_min_vector, local_min_num, local_max_vector, local_max_num, &
                        effective_radius, quality )




! wind 10.20.05 this is the old solution logic. We will just kind of keep it to the side for now
!  call find_minimum_points(residual,     &
!                           local_minima,     &
!                           number_local_minima)

!if (number_local_minima == 3 ) number_local_minima = 2

 
! call solution_location(real(library_radii),              &
!                         residual,                   &
!                         local_minima,               &
!                         number_local_minima,        &
!                         effective_radius,  &
!                         quality)
 
! get optical thickness solution

  call findinterpolationrange(3,   &
                          effective_radius,    &
                          size(library_radii), &
                          real(library_radii), &
                          ilo,  &
                          ihi)
  re_altered = .false.
  if (size(library_radii) < 15 .and. effective_radius == 30.) then
       effective_radius = 10.
       re_altered = .true.
  endif


  if (effective_radius >= 0.) then
  optical_thickness = lagrangeinterp(effective_radius, &
                                      real(library_radii(ilo:ilo+2)), &
                                      optical_thickness_vector(ilo:ilo+2) )



  if( optical_thickness < optical_thickness_vector(ilo)) then
    call findinterpolationrange(2,   &
                          effective_radius,    &
                          size(library_radii), &
                          real(library_radii), &
                          ilo,  &
                          ihi)
    optical_thickness = optical_thickness_vector(ilo) + &
                        ( ( effective_radius - real(library_radii(ilo))) / (real(library_radii(ilo+1)-library_radii(ilo))) ) * &
                          (optical_thickness_vector(ilo+1) - optical_thickness_vector(ilo) )
  endif

  if (re_altered) effective_radius = 30.

  else
    optical_thickness = -1.
  endif
   
end subroutine solveretrieval

subroutine linear_interpolate_for_root(residual, rad, re)
  implicit none

  real, intent(in)     :: residual(2), rad(2)
  real, intent(out)    :: re

  re = -1.0*( rad(1)*residual(2)/(residual(1)-residual(2)) + rad(2)*residual(1)/(residual(2)-residual(1)) )

end subroutine linear_interpolate_for_root
subroutine findinterpolationrange( n1, xx, n,x, k1, k2)
  implicit none

  integer, intent(in)   :: n1, n
  real, intent(in)      :: xx, x(n)

  integer,intent(out)   :: k1, k2
  
  integer               :: i, n2


  if (n > n1) then
     n2 = 0.5*n1
     i = 1
     do while (xx > x(i) .and. i < n)
        i = i + 1
     enddo
     k1 = max(1,i-n2)
     k2 = k1 + n1 - 1
     if (k2 > n) then
        k2 = n
        k1 = k2 - n1 + 1
     end if
  else

!    if # of exist data set n <= n1, i
!    use n1 = n for interp_mas_geoolation.
     k1 = 1
     k2 = n
  end if


end subroutine findinterpolationrange
real function lagrangeinterp( xx, x, y)

! in the previous interation of the COT retrieval code this was known as 'BINTPB'
  implicit none
  real, intent(in)  :: x(3), y(3), xx 
  lagrangeinterp = (xx-x(2))* (xx-x(3))/ (x(1)-x(2))/ (x(1)-x(3))*y(1) + &
                   (xx-x(3))* (xx-x(1))/ (x(2)-x(3))/ (x(2)-x(1))*y(2) + &
                   (xx-x(1))* (xx-x(2))/ (x(3)-x(1))/ (x(3)-x(2))*y(3)
end function lagrangeinterp




! sssssssssssssssssssssssssssssssssssssssssssssss
  subroutine check_for_signchange (x, signchange)
! sssssssssssssssssssssssssssssssssssssssssssssss

!  *** Copied from collect 5 core_science code

! this routine reports on sign change for a set of 3 values
! signchnage == 0 -> no sign change
! signchange == 1 -> sign changes between x1 and x2
! signchange == 2 -> sign changes bweteen x2 and x3

  implicit none

  real, intent(in)      :: x(3)
  integer, intent(out)  :: signchange

  if ( (x(1)==0 .and. x(2)==0) .or. (x(1)==0 .and. x(3)==0) .or.(x(2)==0 .and. x(3)==0) ) Then
     signchange = 0
  else
     if ( sign(1.,x(1))*sign(1.,x(2))*sign(1.,x(3)) /= -1) Then

        signchange =1

        if ( x(1) > 0 .AND. x(2) > 0 .AND. x(3) > 0) then
           signchange =0
        end if

        if ( x(1) < 0 .AND. x(2) < 0 .AND. x(3) < 0) then
           signchange =0
        end if

     else
         signchange =1
     endif
  endif

  if (signchange == 1) then
     if ( sign(1.,x(1))*sign(1.,x(2)) == -1 ) signchange = 1
     if ( sign(1.,x(2))*sign(1.,x(3)) == -1 ) signchange = 2
  endif

  end subroutine check_for_signchange



subroutine quad_interpolate_for_root ( radii, residual, radius_solution, status)

  use GeneralAuxType
  implicit none

  real(single), intent(in)  :: radii(:), residual(:)

  real(single), intent(out) :: radius_solution
  integer,      intent(out) :: status

  real(double) :: d1, d2, d3, a0, a1, a2, root1, root2, z

  d1 = residual(1) / ((radii(1)-radii(2))*(radii(1)-radii(3)))
  d2 = residual(2) / ((radii(2)-radii(1))*(radii(2)-radii(3)))
  d3 = residual(3) / ((radii(3)-radii(1))*(radii(3)-radii(2)))

  a0 = d1*radii(2)*radii(3) + d2*radii(1)*radii(3) + d3*radii(1)*radii(2)
  a1 = -d1*(radii(2)+radii(3)) - d2*(radii(1)+radii(3)) - d3*(radii(1)+radii(2))
  a2 = d1 + d2 + d3


  z = a1**2 - 4*a0*a2

  if(z >= 0. .and. a2 /= 0.) then
    root1 = (-a1 + sqrt(z))/(2*a2)
    root2 = (-a1 - sqrt(z))/(2*a2)
  else
    root1 = -999; root2 = -999
    status = 1
    return
  end if
  if (root1 < radii(1) .and. root2 < radii(1)) then
     radius_solution =-999.
     status = 2
  elseif (root1 > radii(3) .and. root2 > radii(3)) then
     radius_solution =-999.
     status = 3
  elseif (root1 >= radii(1) .and. root1 <= radii(3)) then
     radius_solution = root1
     status = 4
  elseif(root2 >= radii(1) .and. root2 <= radii(3)) then
     radius_solution = root2
     status = 5
  else
     radius_solution = -999.
     status = 1
  endif

end subroutine quad_interpolate_for_root

end module corescience_module
