module libraryarrays

  use GeneralAuxType
  !library descriptor parameters
  integer                   :: number_wavelengths, number_waterradii, number_iceradii
  integer                   :: number_solarzenith, number_relazimuth, number_sensorazimuth
  integer                   :: number_taus, number_sensorzenith
  integer                   :: number_fluxsolarzenith, number_fluxsensorzenith
  
  ! library bidirectional reflectance
  real(single), allocatable, dimension(:,:,:,:,:,:)   :: library_reflectance_water
  real(single), allocatable, dimension(:,:,:,:,:,:)   :: library_reflectance_ice

  ! flux parameters

  !  extinction  		ce             extinction coefficient
  !  singlescattering		w              single scattering albedo
  !  asymmetry 			g              asymetry factor
  !  spherical_albedo		sfr            spherical albedo
  !  ak, el, em, en		ak, el, em, en Asymptotic formula parameters
  !  escapefunction         	 	fk             escape-function
  !  flux_up			fr             upwards flux
  !  flux_down			ft             downwards flux
  ! 

  real                                          :: library_max_solar_zenith

  real, allocatable,dimension(:)                :: trans_pressureheights(:), trans_precipitablewater(:), &
                                                   trans_anglevalues(:), trans_wavelengths(:) 
  real, allocatable,dimension(:,:,:,:)          :: transmit_correction_table, transmit_stddev_table

  real(single), allocatable, dimension(:)       :: library_wavelengths, library_taus, library_relative_zenith
  real(single), allocatable, dimension(:)       :: library_solar_zenith, library_relative_azimuth
  real(single), allocatable, dimension(:)       :: library_sensor_azimuth, library_sensor_zenith
  real(single), allocatable, dimension(:)       :: library_fluxsolarzenith, library_fluxsensorzenith
  
  real(single), allocatable, dimension(:)       :: water_radii
  real(single), allocatable, dimension(:,:)     :: extinction_water, singlescattering_water
  real(single), allocatable, dimension(:,:)     :: asymmetry_water, ak_water, el_water, em_water, en_water
  real(single), allocatable, dimension(:,:,:)   :: spherical_albedo_water 
  real(single), allocatable, dimension(:,:,:)   :: escapefunction_water_solar,escapefunction_water_sensor
  real(single), allocatable, dimension(:,:,:,:) :: flux_up_water_solar, flux_down_water_solar
  real(single), allocatable, dimension(:,:,:,:) :: flux_up_water_sensor, flux_down_water_sensor

  
  real(single), allocatable, dimension(:)       :: ice_radii
  real(single), allocatable, dimension(:,:)     :: extinction_ice, singlescattering_ice
  real(single), allocatable, dimension(:,:)     :: asymmetry_ice, ak_ice, el_ice, em_ice, en_ice
  real(single), allocatable, dimension(:,:,:)   :: spherical_albedo_ice
  real(single), allocatable, dimension(:,:,:)   :: escapefunction_ice_solar,escapefunction_ice_sensor
  real(single), allocatable, dimension(:,:,:,:) :: flux_up_ice_solar, flux_down_ice_solar  
  real(single), allocatable, dimension(:,:,:,:) :: flux_up_ice_sensor, flux_down_ice_sensor
end module libraryarrays
