C-----------------------------------------------------------------------
C !F77-INC
C
C !DESCRIPTION: SDS array and dimension metadata (i.e. attributes) for 
C               MODIS product 6.  
C
C !Input Parameters: None
C
C !Output Parameters: None
C
C !REVISION HISTORY:
c
c Revision 2.6  2004/07/19 mag
c Added new Collection 5 SDS name entries
c Removed entries for Collection 4 temporary Ancillary data file
c Added LRN for new static datafile (albedo dataset)
c
c 08/14/98 fhliang fixed prolog.
c
c Revision 2.5  1998/06/19  06:26:54  rsong
c added one more LRN for DAO processing
c
c Revision 2.4  1998/04/17  22:38:42  rsong
c Changed LRNs of DAO files and L1B file.
c
c Revision 2.3  1998/04/07  01:27:32  rsong
c changed the LRNs of MOD06 output and MCF files.
c
c Revision 2.2  1998/02/06  00:00:21  rsong
c Added SDS name for water_path .
c
c Revision 2.1  1998/01/24  01:24:31  rsong
c Initial revision
c renamed after mod06_inc.f (required by forcheck)
c
c Revision 3.1  1997/07/17  21:27:36  rsong
c added a few new parameters
c
c Revision 3.0  1997/02/04  15:57:52  rsong
c Initial Revision.
c
c Revision 1.4  1996/11/21  16:41:30  vlin
c Updated logical reference numbers
c
c Revision 1.3  1996/07/12  19:25:40  vlin
c added parameter "Lines_Per_Write"
c
c Revision 1.2  1996/06/26  12:35:38  vlin
c Updated logical reference number for V1 processes
c
c Revision 1.1  1996/05/10  13:54:31  vlin
c Initial revision
c
C !TEAM-UNIQUE HEADER:
C
C    This software is developed by the MODIS Science Data Support
C    Team for the National Aeronautics and Space Administration,
C    Goddard Space Flight Center, under contract NAS5-32373.
C
C !REFERENCES and CREDITS:
C
C    Written by:
C    Vicky Lin                       08/08/95
C    Research and Data systems Corporation
C    SAIC/GSC MODIS Science Data Support Office
C    7501 Forbes Blvd, Seabrook MD 20706
C
C    vlin@ltpmail.gsfc.nasa.gov
C
C !DESIGN NOTES:
C
C    Internals Variables:
C         MODFIL(1)  File identifier used by HDF SD interface.
C         MODFIL(2)  File identifier used by HDF VS interface.
C         MODFIL(3)  All file opening and writing functions.
C         RANK1      The number of dimensions in the array is 1.
C         RANK2      The number of dimensions in the array is 2.
C         RANK3      The number of dimensions in the array is 3.
C         RTN        Return value from multiple functions.
C
C !END
C-----------------------------------------------------------------------
C  Names of the scientific data sets

      CHARACTER*(*)  SDS1_N, SDS2_N, SDS3_N, SDS4_N, SDS5_N, SDS6_N
      CHARACTER*(*)  SDS7_N, SDS8_N, SDS9_N,SDS10_N,SDS11_N,SDS12_N
      CHARACTER*(*) SDS13_N,SDS14_N,SDS15_N,SDS16_N,SDS17_N,SDS18_N
      CHARACTER*(*) SDS19_N,SDS20_N,SDS21_N,SDS22_N,SDS23_N,SDS24_N
      CHARACTER*(*) SDS25_N,SDS26_N,SDS27_N,SDS28_N,SDS29_N,SDS30_N

      PARAMETER (SDS1_N ='Day', SDS2_N ='Local_Time')
      PARAMETER (SDS3_N ='Lat', SDS4_N ='Lon')
      PARAMETER (SDS5_N ='Scanline_Num')
      PARAMETER (SDS6_N ='Scanline_Element')
      PARAMETER (SDS7_N ='Sat_Zenith_Angle')
      PARAMETER (SDS8_N ='Solar_Zenith_Angle')
      PARAMETER (SDS9_N ='Brightness_Temp')
      PARAMETER (SDS10_N='Land_Sea_Flag')
      PARAMETER (SDS11_N='Processing_Flag')
      PARAMETER (SDS12_N='Spec_Cloud_Forcing')
      PARAMETER (SDS13_N='Cloud_H_Method')
      PARAMETER (SDS14_N='Cloudtop_Eff_Emi')
      PARAMETER (SDS15_N='Cloudtop_Pres')
      PARAMETER (SDS16_N='Cloudtop_Temp')
      PARAMETER (SDS17_N='Cloud_Fraction')
      PARAMETER (SDS18_N='Sfc_Temp')
      PARAMETER (SDS19_N='Sfc_Pres')
      PARAMETER (SDS20_N='Cloudtop_Pres_Ratio')
      PARAMETER (SDS21_N='Cloudtop_Pres_IR')
      PARAMETER (SDS22_N='Tropopause_Height')
      PARAMETER (SDS23_N='Sfc_type')
      PARAMETER (SDS24_N='Radiance_Var')
      PARAMETER (SDS25_N='Brightness_Temp_Diff')
      PARAMETER (SDS26_N='Cloud_Phase_IR')
      PARAMETER (SDS27_N='Eff_Particle_Rad')
      PARAMETER (SDS28_N='Cloud_Opt_Thickness')
      PARAMETER (SDS29_N='Cloud_Phase_VIS')
      PARAMETER (SDS30_N='Statistics')

      CHARACTER*(*) SDS_NAME01, SDS_NAME02, SDS_NAME03 
      CHARACTER*(*) SDS_NAME04, SDS_NAME05, SDS_NAME06
      CHARACTER*(*) SDS_NAME07, SDS_NAME08, SDS_NAME09
      CHARACTER*(*) SDS_NAME10, SDS_NAME11, SDS_NAME12
      CHARACTER*(*) SDS_NAME13, SDS_NAME14, SDS_NAME15
      PARAMETER (SDS_NAME01 = 'Cloud_Effective_Radius')
      PARAMETER (SDS_NAME02 = 'Cloud_Optical_Thickness')
      PARAMETER (SDS_NAME03 = 'Cloud_Effective_Radius_1621')
      PARAMETER (SDS_NAME04 = 'Cloud_Optical_Thickness_1621')
      PARAMETER (SDS_NAME05 = 'Effective_Radius_Difference')
      PARAMETER (SDS_NAME06 = 'Cloud_Mask_1km')
      PARAMETER (SDS_NAME07 = 'Quality_Assurance_1km')
      PARAMETER (SDS_NAME08 = 'Statistics_1km')
      PARAMETER (SDS_NAME09 = 'Cloud_Water_Path')
      PARAMETER (SDS_NAME10 = 'Cloud_Water_Path_1621')
      PARAMETER (SDS_NAME11 = 'Cloud_Effective_Radius_Uncertainty')
      PARAMETER (SDS_NAME12 = 'Cloud_Optical_Thickness_Uncertainty')
      PARAMETER (SDS_NAME13 = 'Cloud_Water_Path_Uncertainty')
      PARAMETER (SDS_NAME14 = 'Cloud_Phase_Optical_Properties')
      PARAMETER (SDS_NAME15 = 'Cloud_Multi_Layer_Flag')





C  Data types of the scientific data sets
      CHARACTER*(*)  TYPE1, TYPE2, TYPE3, TYPE4, TYPE5, TYPE6
      PARAMETER (TYPE1='INTEGER*2', TYPE2='INTEGER*2') 
      PARAMETER (TYPE3='INTEGER*2', TYPE4='INTEGER*1') 
      PARAMETER (TYPE5='INTEGER*1', TYPE6='REAL*4') 

      CHARACTER*(*)  DTYPE1, DTYPE2, DTYPE3, DTYPE4, DTYPE5, DTYPE6
      CHARACTER*(*)  DTYPE7, DTYPE8, DTYPE9,DTYPE10,DTYPE11,DTYPE12
      CHARACTER*(*) DTYPE13,DTYPE14,DTYPE15,DTYPE16,DTYPE17,DTYPE18
      CHARACTER*(*) DTYPE19,DTYPE20,DTYPE21,DTYPE22,DTYPE23,DTYPE24
      CHARACTER*(*) DTYPE25,DTYPE26,DTYPE27,DTYPE28,DTYPE29,DTYPE30

      PARAMETER (DTYPE1='INTEGER*4',DTYPE2='INTEGER*4')
      PARAMETER (DTYPE3='INTEGER*4',DTYPE4='INTEGER*4')
      PARAMETER (DTYPE5='INTEGER*2',DTYPE6='INTEGER*2')
      PARAMETER (DTYPE7='INTEGER*2',DTYPE8='INTEGER*2')
      PARAMETER (DTYPE9='INTEGER*2',DTYPE10='INTEGER*1')
      PARAMETER (DTYPE11='INTEGER*1',DTYPE12='INTEGER*2')
      PARAMETER (DTYPE13='INTEGER*1',DTYPE14='INTEGER*1')
      PARAMETER (DTYPE15='INTEGER*2',DTYPE16='INTEGER*2')
      PARAMETER (DTYPE17='INTEGER*1',DTYPE18='INTEGER*2')
      PARAMETER (DTYPE19='INTEGER*2',DTYPE20='INTEGER*2')
      PARAMETER (DTYPE21='INTEGER*2',DTYPE22='INTEGER*2')
      PARAMETER (DTYPE23='INTEGER*2',DTYPE24='INTEGER*2')
      PARAMETER (DTYPE25='INTEGER*2',DTYPE26='INTEGER*1')
      PARAMETER (DTYPE27='INTEGER*2',DTYPE28='INTEGER*2')
      PARAMETER (DTYPE29='INTEGER*1',DTYPE30='INTEGER*2')

      INTEGER RANK1, RANK2,RANK3
      PARAMETER (RANK1=1, RANK2=2,RANK3=3)

C  Scale factors of the scientific data sets
 
      BYTE     SCALE10,SCALE11,SCALE26,SCALE29
      DOUBLE PRECISION SCALE27,SCALE28,SCALE30
      INTEGER  SCALE1, SCALE2, SCALE5, SCALE6,SCALE13,SCALE23
      REAL      SCALE3, SCALE4, SCALE7, SCALE8, SCALE9,SCALE12,SCALE14
      REAL     SCALE15,SCALE16,SCALE17,SCALE18,SCALE19,SCALE20,SCALE21
      REAL     SCALE22,SCALE24,SCALE25

      PARAMETER (SCALE1=1,SCALE2=1,SCALE3=10000.0,SCALE4=10000.0) 
      PARAMETER (SCALE5=1,SCALE6=1,SCALE7=10.0,SCALE8=10.0)
      PARAMETER (SCALE9=100.0,SCALE10=1,SCALE11=1,SCALE12=100.0)
      PARAMETER (SCALE13=1,SCALE14=100.0,SCALE15 =1.0,SCALE16=100.0)
      PARAMETER (SCALE17=100.0,SCALE18=100.0,SCALE19=1.0,SCALE20=1.0)
      PARAMETER (SCALE21=1.0,SCALE22=1.0,SCALE23=1,SCALE24=100.0)
      PARAMETER (SCALE25=100.0,SCALE26=1)
      PARAMETER (SCALE27=10.0,SCALE28=10.0)
      PARAMETER (SCALE29=1,SCALE30=10.0)

C  Add offsets of the scientific data sets

      BYTE       OFFSET10,OFFSET11,OFFSET26,OFFSET29
      DOUBLE PRECISION OFFSET27,OFFSET28,OFFSET30
      INTEGER     OFFSET1,OFFSET2,OFFSET5, OFFSET6,OFFSET13,OFFSET23
      REAL OFFSET3, OFFSET4, OFFSET7, OFFSET8, OFFSET9,OFFSET12
      REAL OFFSET14,OFFSET15,OFFSET16,OFFSET17,OFFSET18,OFFSET19
      REAL OFFSET20,OFFSET21,OFFSET22,OFFSET24,OFFSET25

      PARAMETER (OFFSET1=0,OFFSET2=0,OFFSET3=0.0,OFFSET4=0.0)
      PARAMETER (OFFSET5=0,OFFSET6=0,OFFSET7=0.0,OFFSET8=0.0)
      PARAMETER (OFFSET9=-15000.0,OFFSET10=0,OFFSET11=0,OFFSET12=0.0)
      PARAMETER (OFFSET13=0,OFFSET14=0.0,OFFSET15=0.0,
     2            OFFSET16=-15000.0)
      PARAMETER (OFFSET17=0.0,OFFSET18=-15000.0,OFFSET19=0.0,
     2            OFFSET20=0.0)
      PARAMETER (OFFSET21=0.0,OFFSET22=0.0,OFFSET23=0,OFFSET24=0.0)
      PARAMETER (OFFSET25=0.0,OFFSET26=0)
      PARAMETER (OFFSET27=0.0,OFFSET28=0.0)
      PARAMETER (OFFSET29=0,OFFSET30=0.0)

C VARIABLE DECLARATION FOR INPUT FILE
      integer LRN_HDF,LRN_L1B,LRN_Geo,LRN_CldMsk, LRN_MCF,
     &        LRN_AtmoCorr1, LRN_AtmoCorr2, LRN_AtmoCorr3

      parameter(LRN_AtmoCorr1 = 417521,LRN_AtmoCorr2 = 417522,
     &          LRN_AtmoCorr3 = 417523,
     &          LRN_HDF = 417510, LRN_L1B = 700002, 
     &          LRN_Geo = 600000, LRN_CldMsk = 422500, LRN_MCF=412501)

C VARIABLE DECLARATION FOR OUTPUT FILE
      INTEGER    LRN_MOD06
      PARAMETER (LRN_MOD06 = 412500 )

      INTEGER    LRN_MOD07
      PARAMETER (LRN_MOD07 = 420000)

c 07/07/1998 fhliang: added following 2 lines:
      INTEGER LUN_Of_QCFile_MCF, LUN_Of_QCFile_RP
      PARAMETER (LUN_Of_QCFile_MCF = 417500, LUN_Of_QCFile_RP = 417531)

C VARIABLE DECLARATION FOR ANCILLARY FILE
      INTEGER    LRN_ANCIL
      PARAMETER (LRN_ANCIL = 417530)

C VARIABLE DECLARATION FOR FILL VALUES 
      INTEGER FV_Lon,FV_lat,FV_SolAz,FV_SenAz,FV_SolZen,FV_SenZen,
     &        FV_Range,FV_Height
      PARAMETER (FV_Lon = -1000,   FV_lat = -1000,    FV_SolAz = -1000,
     &           FV_SenAz = -1000, FV_SolZen = -1000, FV_SenZen = -1000,
     &           FV_Range = -1000, FV_Height = -1000)

C Declaration of PARAMETERS to hold Fill Values of L1B and
C Geolocation parameter data in science code.
      REAL FV_L1B, FV_GEO
      PARAMETER ( FV_L1B = -999.0, FV_GEO = -999.0 )

C Other PARAMETERS
      INTEGER Lines_Per_Write
      PARAMETER    (Lines_Per_Write = 10)
