module modis_frontend_module

 implicit none
 private

 public :: initialize_run, check_datasources, check_datasize, readlibraries, &
           allocate_libraryarrays, allocate_arrays, init_qualitydata

contains

subroutine initialize_run( start, edge, stride,        &
                           tilesize,                   &
                           level1b_name,               &
                           cloudmask_name,             &
                           mod06_name,            &
                           geolocation_name,           &
                           gdas_name,                  &
                           ncepice_name,               &
                           sst_name,                   &
                           nise_name,                  &
                           ozone_name,                 &
                           water_library,              &
                           ice_library,                &
                           transmittance_library,      &
                           surfacealbedo_lib_659,      &
                           surfacealbedo_lib_858,      &
                           surfacealbedo_lib_124,      &
                           surfacealbedo_lib_164,      &
                           surfacealbedo_lib_21,       &
                           ecosystem_data_name,        &
                           snowicealbedo_data_name,    &
                           threshold_solar_zenith,     &
                           threshold_sensor_zenith,    &
                           threshold_relative_azimuth, &
                           debug,                      &
                           status )
   
   use core_arrays
   use mod06_run_settings
   use nonscience_parameters

   implicit none

   include 'mapi.f90.inc'
   include 'PGS_SMF.f'
   include 'PGS_IO.f'
   include 'PGS_MODIS_39500.f'

   integer,          intent (out)         :: status, tilesize 

   character(*),     intent (out)         :: level1b_name,  cloudmask_name,&
                                             water_library, ice_library, &
                                             transmittance_library, &
                                             geolocation_name, mod06_name,   &
                                             gdas_name, ncepice_name, sst_name, &
                                             nise_name, ozone_name,surfacealbedo_lib_659,      &
                                             surfacealbedo_lib_858,      &
                                             surfacealbedo_lib_124,      &
                                             surfacealbedo_lib_164,      &
                                             surfacealbedo_lib_21,       &
                                             ecosystem_data_name,        &         
                                             snowicealbedo_data_name


   real,                     intent(out)  :: threshold_solar_zenith,      &
                                             threshold_sensor_zenith,     &
                                             threshold_relative_azimuth

   integer,  dimension (2),  intent (out) :: start,edge,stride 
   logical,                  intent(out)  :: debug

   integer                                :: number_of_bands, checkvariable,lun, readstatus,&
                                             i,index, file_version, localstatus 

   character*(20) , parameter :: functionname = 'initialize_run'
   integer fileid, version, gpn_status

!  function definitions
   integer pgs_pc_getreference, get_platform_name
   external pgs_pc_getreference, get_platform_name
 
   file_version = 1
   localstatus = pgs_pc_getreference(level1b_lrn,file_version,level1b_name)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for L1B file',functionname)
     status = failure
   endif 

   file_version = 1
   gpn_status = 0
   localstatus = get_platform_name(level1b_lrn, file_version, gpn_status, platform_name)
 
   file_version = 1
   localstatus = pgs_pc_getreference(cloudmask_lrn,file_version,cloudmask_name)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for cloudmask file',functionname)
     status = failure
   endif

   file_version = 1
   localstatus = pgs_pc_getreference(water_library_lrn,file_version,water_library)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for water library file',functionname)
     status = failure
   endif

   file_version = 1
   localstatus = pgs_pc_getreference(ice_library_lrn,file_version,ice_library)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for ice library file',functionname)
     status = failure
   endif

   file_version = 1
   localstatus = pgs_pc_getreference(transmittance_library_lrn,file_version,transmittance_library)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for transmittance file',functionname)
     status = failure
   endif

   file_version = 1
   localstatus = pgs_pc_getreference(mod06_lrn,file_version,mod06_name)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for mod06 file',functionname)
     status = failure
   endif

   file_version = 1
   localstatus = pgs_pc_getreference(geolocation_lrn,file_version,geolocation_name)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for gelocation file',functionname)
     status = failure
   endif

   file_version = 1
   localstatus = pgs_pc_getreference(gdas_lrn,file_version,gdas_name)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for GDAS file',functionname)
     status = failure
   endif

   file_version = 1
   localstatus = pgs_pc_getreference(ncepice_lrn,file_version,ncepice_name)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for NCEP file',functionname)
     status = failure
   endif

   file_version = 1
   localstatus = pgs_pc_getreference(nise_lrn ,file_version,nise_name)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for NISE file',functionname)
     status = failure
   endif

   file_version = 1
   localstatus = pgs_pc_getreference(sst_lrn,file_version,sst_name)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for SST file',functionname)
     status = failure
   endif

   file_version = 1
   localstatus = pgs_pc_getreference(ozone_lrn,file_version,ozone_name)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for ozone file',functionname)
     status = failure
   endif

   file_version = 1
   localstatus = pgs_pc_getreference(surfacealbedo_lib_659_lrn,file_version,surfacealbedo_lib_659)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for albedo file 0.659',functionname)
     status = failure
   endif

   file_version = 1
   localstatus = pgs_pc_getreference(surfacealbedo_lib_858_lrn,file_version,surfacealbedo_lib_858)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for albedo file 0.858',functionname)
     status = failure
   endif

   file_version = 1
   localstatus = pgs_pc_getreference(surfacealbedo_lib_124_lrn,file_version,surfacealbedo_lib_124)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for albedo file 1.24',functionname)
     status = failure
   endif

   file_version = 1
   localstatus = pgs_pc_getreference(surfacealbedo_lib_164_lrn,file_version,surfacealbedo_lib_164)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for albedo file 1.64',functionname)
     status = failure
   endif

   file_version = 1
   localstatus = pgs_pc_getreference(surfacealbedo_lib_21_lrn,file_version,surfacealbedo_lib_21)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for albedo file 2.1',functionname)
     status = failure
   endif

   file_version = 1
   localstatus = pgs_pc_getreference(ecosystem_data_lrn,file_version,ecosystem_data_name)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for ecosystem file',functionname)
     status = failure
   endif

   file_version = 1
   localstatus = pgs_pc_getreference(snowicealbedo_data_lrn,file_version,snowicealbedo_data_name)
   if (localstatus /= PGS_S_SUCCESS)  then
     call modis_smf_setdynamicmsg(MODIS_E_GENERIC,'pgs_pc_getreference failure for snow/ice albedo file',functionname)
     status = failure
   endif

   start = set_start
   stride = set_stride
   edge = set_edge

   threshold_solar_zenith = set_threshold_solar_zenith
   threshold_sensor_zenith = set_threshold_sensor_zenith
   threshold_relative_azimuth = set_threshold_relative_azimuth

   number_of_bands = size(set_bands)
   allocate(bands(number_of_bands))
   bands(:) = set_bands(:)
  
   tilesize = set_tilesize

end subroutine initialize_run
subroutine check_datasources(l1bname,                   &
                          cloudmask_name,            &
                          geolocation_name,          &
                          gdas_name,                 &
                          ncepice_name,              &
                          sst_name,                  &
                          nise_name,                 &
                          ozone_name,                &
                          mod06_name,           &
                          water_library,             &
                          ice_library,               &
                          transmittance_library,     &
                          surfacealbedo_lib_659,     &
                          surfacealbedo_lib_858,     &
                          surfacealbedo_lib_124,     &
                          surfacealbedo_lib_164,     &
                          surfacealbedo_lib_21,      &
                          ecosystem_data_name,       &
                          snowicealbedo_data_name,   & 
                          status)
    use nonscience_parameters
    implicit none

    character*(*), intent(in)  ::l1bname, cloudmask_name, geolocation_name, gdas_name, ncepice_name,  &
                                 sst_name, nise_name, mod06_name, water_library, ice_library,    &
                                 transmittance_library,  ozone_name, surfacealbedo_lib_659, &
                                 surfacealbedo_lib_858, surfacealbedo_lib_124, surfacealbedo_lib_164, &
                                 surfacealbedo_lib_21, ecosystem_data_name, snowicealbedo_data_name
    
    integer,dimension(20)      :: allstatus
    integer, intent(out)       :: status
    !integer, external          :: checkfile  

    status = 0
    allstatus = 0


    allstatus(1) = checkfile(l1bname)
    if (allstatus(1) /= success) then
      status = failure
      call local_message_handler ('Error noted: level 1B file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif
    allstatus(2) = checkfile(cloudmask_name)
    if (allstatus(2) /= success) then
      status = failure
      call local_message_handler ('Error noted: cloud mask file. Check PCF and input files.', &
                                   status, &
                                   'check_datasources')
    endif
    allstatus(3) = checkfile(geolocation_name)
    if (allstatus(3) /= success) then
      status = failure
      call local_message_handler ('Error noted: geolocation file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif
    allstatus(4) = checkfile(gdas_name)    
    if (allstatus(4) /= success) then
      status = failure
      call local_message_handler ('Error noted: GDAS file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif
    allstatus(5) = checkfile(ncepice_name)    
    if (allstatus(5) /= success) then
      status = failure
      call local_message_handler ('Error noted: NCEP Ice file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif
    allstatus(6) = checkfile(sst_name)    
    if (allstatus(6) /= success) then
      status = failure
      call local_message_handler ('Error noted: SST file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif
    allstatus(7) = checkfile(nise_name)    
    if (allstatus(7) /= success) then
      status = failure
      call local_message_handler ('Error noted: NISE  file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif
    allstatus(8) = checkfile(ozone_name)    
    if (allstatus(8) /= success) then
      status = failure
      call local_message_handler ('Error noted: TOVS ozone file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif
    allstatus(9) = checkfile(mod06_name)    
    if (allstatus(9) /= success) then
      status = failure
      call local_message_handler ('Error noted: MOD06_L2 file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif
    allstatus(10) = checkfile(water_library)    
    if (allstatus(10) /= success) then
      status = failure
      call local_message_handler ('Error noted: water library file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif
    allstatus(11) = checkfile(ice_library)    
    if (allstatus(11) /= success) then
      status = failure
      call local_message_handler ('Error noted: ice library file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif
    allstatus(12) = checkfile(transmittance_library)    
    if (allstatus(12) /= success) then
      status = failure
      call local_message_handler ('Error noted: transmittance file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif
    allstatus(13) = checkfile(surfacealbedo_lib_659)     
    if (allstatus(13) /= success) then
      status = failure
      call local_message_handler ('Error noted: albedo (.659um) file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif
    allstatus(14) = checkfile(surfacealbedo_lib_858)    
    if (allstatus(14) /= success) then
      status = failure
      call local_message_handler ('Error noted: albedo (.858um) file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif
    allstatus(15) = checkfile(surfacealbedo_lib_124)    
    if (allstatus(15) /= success) then
      status = failure
      call local_message_handler ('Error noted: albedo (1.24um) file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif
    allstatus(16) = checkfile(surfacealbedo_lib_164)    
    if (allstatus(16) /= success) then
      status = failure
      call local_message_handler ('Error noted: albedo (1.64um) file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif
    allstatus(17) = checkfile(surfacealbedo_lib_21)    
    if (allstatus(17) /= success) then
      status = failure
      call local_message_handler ('Error noted: albedo (2.1um) file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif
    allstatus(18) = checkfile(ecosystem_data_name)    
    if (allstatus(18) /= success) then
      status = failure
      call local_message_handler ('Error noted: IGBP Ecosystem file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif
    allstatus(19) = checkfile(snowicealbedo_data_name)    
    if (allstatus(19) /= success) then
      status = failure
      call local_message_handler ('Error noted: Static SnowIce Albedo file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif

    status = maxval(allstatus)
   
    if (status /= success) then
      status = failure 
      call local_message_handler ('Error noted for a required input file. Check PCF and input files.' ,&
                                   status, &
                                   'check_datasources')
    endif
end subroutine check_datasources
integer function checkfile(name)

  implicit none
  
  character(*), intent(in)  :: name
  logical                   :: exist
  character(len=11)         :: readability

  checkfile = 0

  inquire( file = name,      &
           exist= exist,   &
           read = readability)
  if (.not. exist) then
         checkfile = 1
  endif
  if (readability == 'NO') then
         checkfile = 2
  endif
end function checkfile


subroutine check_datasize(l1b_filedata, start, stride, edge, status)

   use GeneralAuxType
   use nonscience_parameters
   use hdf

   implicit none

   include 'PGS_MODIS_39500.f'
   include 'mapi.f90.inc'

   integer, intent(in)                   :: l1b_filedata(modfillen)
   integer, dimension (2), intent(inout) :: start, edge, stride
   integer, intent(out)                  :: status

   integer      :: Scans_Per_Granule, Max_EV_Frames, Dead_Detectors(490), &
                   No_EV_Frames(Max_Scan_Per_Granule)
   character*4  :: Scan_Type(Max_Scan_Per_Granule)
   character*40 :: att_N, dtype
   integer      :: bsize, handle_QC, nms, ii, RTN

   status = success

!  get number of scans
   att_N = 'Number of Scans'
   dtype = 'INTEGER*4'
   nms = 1
  
   RTN = GMFIN(l1b_filedata,att_N,dtype,nms,Scans_Per_Granule)
   if (rtn /= MAPIOK) then
     call MODIS_SMF_SETDYNAMICMSG(MODIS_W_GENERIC, &
                                  'MAPI function GMFIN for Number of Scans  failed', &
                                  'check_datasize')
     status = failure
   endif

   edge(2) = Scans_Per_Granule * 10

end subroutine check_datasize
!-----------------------------------------------------------------------
!f90 readlibraries
!
!Description:
!
! Read bidirectional reflectance and flux library values for all
! angles, as defined in a library description file.  Library arrays
! are allocated as required to read all library data.
!
!input parameters:
!
!output parameters:
!
!revision history:
!
! v2.1  July 2002 arrays are now allocated to only include the angles
!               needed.
!
! v2.0  June 2002 Changed the read routines to work with the HDF
!         version of the libraries. -- Gala. wind@climate.gsfc.nasa.gov
!
! v1.0  November 2001 Initial work mag gray@climate.gsfc.nasa.gov
!
!team-unique header:
!
! Cloud Retrieval Group, NASA GSFC, Greenbelt, Maryland, USA 
!
!references and credits:
!
! Mark Gray
! gray@climate.gsfc.nasa.gov
! EmergentIT 
! Code 913, NASA GSFC 
! Greenbelt, MD 20771
!
! Gala Wind
! wind@climate.gsfc.nasa.gov
! L-3 Comm Analytics
! all same, all same
!
!design note:
!
!end
!----------------------------------------------------------------------

subroutine readlibraries(trans_correct_lib, hdf_water_lib, hdf_ice_lib,debug,status)
   
   use GeneralAuxType
   use libraryarrays
   use nonscience_parameters
   use hdf
   implicit none  

   character(*), intent(in)                   :: hdf_water_lib, hdf_ice_lib,trans_correct_lib
   logical, intent(in)                        :: debug
   integer,intent (out)                       :: status

   integer                                    :: file_id, var_id, var_index,i, j
   integer, dimension(:), allocatable         :: start, stride, edge
   character(MAX_SDS_NAME_LEN)                              :: dummy_name
   integer                                    :: dummy_type, dummy_num_attr, dummy_rank
   integer                                    :: dim_sizes(6),dim_sizes4d(4)
   real(single), dimension(:), allocatable    :: solarzenith_all, sensorzenith_all, &
                                                 relativeazimuth_all, solarzenith_flux_all
   integer                                    :: index_solarzenith, index_sensorzenith,&
                                                index_relativeazimuth,index_solarzenith_flux
   real(single), dimension(:,:,:,:,:,:), allocatable:: temp_reflectance

   status = 0

!  read transmittance correction library
   allocate (start(4), stride(4), edge(4))
   start = 0
   stride = 1
   edge(1) = size(transmit_correction_table,1)
   edge(2) = size(transmit_correction_table,2)
   edge(3) = size(transmit_correction_table,3)
   edge(4) = size(transmit_correction_table,4)

   file_id = sfstart(trans_correct_lib, DFACC_READ)
   var_index = sfn2index(file_id, "Transmittance")
   var_id = sfselect(file_id, var_index)
   status = sfginfo(var_id, dummy_name, dummy_rank, &
                      dim_sizes, dummy_type, dummy_num_attr)
   status = sfrdata(var_id, start,stride,edge, transmit_correction_table)
   status = sfendacc(var_id)

   var_index = sfn2index(file_id, "Standard_Deviation")
   var_id = sfselect(file_id, var_index)
   status = sfginfo(var_id, dummy_name, dummy_rank, &
                      dim_sizes, dummy_type, dummy_num_attr)
   status = sfrdata(var_id, start,stride,edge, transmit_stddev_table)
   status = sfendacc(var_id)


   status = sfend(file_id)

   deallocate(start, stride, edge)

   number_waterradii = size(water_radii)
   number_iceradii = size(ice_radii) 

!  Read all the ice stuff there is to read.
   file_id = sfstart(hdf_ice_lib, DFACC_READ)

!  this is all that there is to read for ice flux:

!  extinction_ice, singlescattering_ice, asymmetry_ice, AK_ice, EL_ice,
!  EM_ice, EN_ice, spherical_albedo_ice, escapefunction_ice, flux_up_ice,
!  flux_down_ice
 
!  arrays that are functions of angles are not read in their entirety, 
!  but only for the angles previous detemined as required.  This
!  minimises the memory usage of the arrays (which is ~410mb for all arrays
!  at real*8)

!  first we real the angles as stored in the libraries to 
!  compare against the angles we are using as our limits

   allocate (start(1), stride(1), edge(1))

   var_index = sfn2index(file_id, "ReflectanceRelativeAzimuth")
   var_id = sfselect(file_id, var_index)

   status = sfginfo(var_id, dummy_name, dummy_rank, &
                      dim_sizes, dummy_type, dummy_num_attr)

   index_relativeazimuth = dim_sizes(1)
   allocate (relativeazimuth_all(index_relativeazimuth))

!  hdf data read setup
   start = 0
   edge = index_relativeazimuth
   stride = 1

!  end setup

   status = sfrdata(var_id, start, stride, edge, relativeazimuth_all)
   status = sfendacc(var_id)


   var_index = sfn2index(file_id, "ReflectanceSolarZenith")
   var_id = sfselect(file_id, var_index)

   status = sfginfo(var_id, dummy_name, dummy_rank, &
                      dim_sizes, dummy_type, dummy_num_attr)

   index_solarzenith = dim_sizes(1)

   allocate (solarzenith_all(index_solarzenith))

   start = 0
   edge =index_solarzenith
   stride = 1

   status = sfrdata(var_id, start, stride, edge, solarzenith_all)
   status = sfendacc(var_id)

   library_max_solar_zenith = maxval(solarzenith_all)

   var_index = sfn2index(file_id, "ReflectanceSensorZenith")
   var_id = sfselect(file_id, var_index)

   status = sfginfo(var_id, dummy_name, dummy_rank, &
                      dim_sizes, dummy_type, dummy_num_attr)

   index_sensorzenith = dim_sizes(1)
   allocate (sensorzenith_all(index_sensorzenith))
   ! hdf data read setup
   edge = index_sensorzenith
   ! end setup

   status = sfrdata(var_id, start, stride, edge, sensorzenith_all)
   status = sfendacc(var_id)


   var_index = sfn2index(file_id, "FluxSolarZenith")
   var_id = sfselect(file_id, var_index)

   status = sfginfo(var_id, dummy_name, dummy_rank, &
                      dim_sizes, dummy_type, dummy_num_attr)
   index_solarzenith_flux= dim_sizes(1)

   allocate (solarzenith_flux_all(index_solarzenith_flux))
   ! hdf data read setup
   edge =index_solarzenith_flux
   ! end setup

   status = sfrdata(var_id, start, stride, edge, solarzenith_flux_all)
   status = sfendacc(var_id)
  
   deallocate(start, stride, edge)


!  Start reading the 2-dimensional arrays for ice

   var_index = sfn2index(file_id, "ExtinctionCoefficient")
   var_id = sfselect(file_id, var_index)
   allocate(start(0:1), stride(0:1), edge(0:1))

!  set up hdf read bounds
   start = 0
   stride = 1
   edge(0) = number_wavelengths
   edge(1) = number_iceradii


   status = sfrdata(var_id, start,stride,edge, extinction_ice)
   status = sfendacc(var_id)
   var_index = sfn2index(file_id, "SingleScatterAlbedo")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, singlescattering_ice)
   status = sfendacc(var_id)

   var_index = sfn2index(file_id, "Phase: AsymmetryParameter")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, asymmetry_ice)
   status = sfendacc(var_id)
   var_index = sfn2index(file_id, "MinimumEigenvalue")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, AK_ice)
   status = sfendacc(var_id)

   var_index = sfn2index(file_id, "NegInternalReflectionCoefficient")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, EL_ice)
   status = sfendacc(var_id)

   var_index = sfn2index(file_id, "DiffusionPatternIntegral")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, EM_ice)
   status = sfendacc(var_id)

   var_index = sfn2index(file_id, "WeightedMeanOfEscapeFunction")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, EN_ice)
   status = sfendacc(var_id)

   deallocate(start, stride, edge)


!  Reading the 3-dimensional arrays for ice

   allocate(start(0:2), stride(0:2), edge(0:2))
   ! set up hdf read bounds
   start = 0
   stride = 1
   edge(0) = size(spherical_albedo_ice,1)
   edge(1) = size(spherical_albedo_ice,2)
   edge(2) = size(spherical_albedo_ice,3)

   var_index = sfn2index(file_id, "SphericalAlbedo")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, spherical_albedo_ice)
   status = sfendacc(var_id)

   start(0) = findpoint(solarzenith_flux_all,minval(library_fluxsolarzenith))-1

   edge(0) = size(escapefunction_ice_solar,1)
   var_index = sfn2index(file_id, "EscapeFunction")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, escapefunction_ice_solar)
   status = sfendacc(var_id)
   start(0) = findpoint(solarzenith_flux_all,minval(library_fluxsensorzenith))-1

   edge(0) = size(escapefunction_ice_sensor,1)
   var_index = sfn2index(file_id, "EscapeFunction")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, escapefunction_ice_sensor)
   status = sfendacc(var_id)

   deallocate(start, stride, edge)

!  Reading the 4-dimensional arrays for ice
   allocate(start(0:3), stride(0:3), edge(0:3))

!  set up hdf read bounds
   start = 0
   start(0) = findpoint(solarzenith_flux_all,minval(library_fluxsolarzenith))-1
   stride = 1
   edge(0) = size(flux_up_ice_solar,1)
   edge(1) = size(flux_up_ice_solar,2)
   edge(2) = size(flux_up_ice_solar,3)
   edge(3) = size(flux_up_ice_solar,4)

   var_index = sfn2index(file_id, "ReflectedFlux")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, flux_up_ice_solar)
   status = sfendacc(var_id)
   
   var_index = sfn2index(file_id, "TransmittedFlux")
   var_id = sfselect(file_id, var_index)

   status = sfginfo(var_id, dummy_name, dummy_rank, &
                      dim_sizes4d, dummy_type, dummy_num_attr)

   status = sfrdata(var_id, start,stride,edge, flux_down_ice_solar)
   status = sfendacc(var_id)

   start = 0
   start(0) = findpoint(solarzenith_flux_all,minval(library_fluxsensorzenith))-1
   stride = 1
   edge(0) = size(flux_up_ice_sensor,1)
   edge(1) = size(flux_up_ice_sensor,2)
   edge(2) = size(flux_up_ice_sensor,3)
   edge(3) = size(flux_up_ice_sensor,4)

   var_index = sfn2index(file_id, "ReflectedFlux")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, flux_up_ice_sensor)
   status = sfendacc(var_id)
  
   var_index = sfn2index(file_id, "TransmittedFlux")
   var_id = sfselect(file_id, var_index)

   status = sfginfo(var_id, dummy_name, dummy_rank, &
                      dim_sizes4d, dummy_type, dummy_num_attr)

   status = sfrdata(var_id, start,stride,edge, flux_down_ice_sensor)
   status = sfendacc(var_id)
   deallocate(start, stride, edge)

!  read reflectance for ice
   allocate(start(0:5), stride(0:5), edge(0:5))

!  set up hdf read bounds, where indexs 0,1,2 are angle indexes 
!  and can sized <= the size of the array in the hdf file.

   start = 0
   start(0) = findpoint(sensorzenith_all,minval(library_sensor_zenith))-1
   start(1) = findpoint(solarzenith_all,minval(library_solar_zenith))-1
   start(2) = findpoint(relativeazimuth_all,minval(library_relative_azimuth))-1
   stride = 1
   edge(0) = size(library_reflectance_ice,1) ! sensor zenith
   edge(1) = size(library_reflectance_ice,2) ! solar zenith
   edge(2) = size(library_reflectance_ice,3) ! relative azimuth
   edge(3) = size(library_reflectance_ice,4)
   edge(4) = size(library_reflectance_ice,5)-1
   edge(5) = size(library_reflectance_ice,6)
   allocate(temp_reflectance(edge(0), edge(1), edge(2), edge(3), &
              edge(4), edge(5)))

!  end setup
   var_index = sfn2index(file_id, "BidirectionalReflectance")
   var_id = sfselect(file_id, var_index)

!  zero-out the reflectance library for the purely thermal band
   library_reflectance_ice(:,:,:,:,7,:)  = 0.0
 
   status = sfrdata(var_id, start,stride,edge,  &
                   temp_reflectance)
   library_reflectance_ice(:,:,:,:,1:6,:) = temp_reflectance(:,:,:,:,:,:)
   status = sfendacc(var_id)
   deallocate(start, stride, edge, temp_reflectance)
 
   status = sfend(file_id)
 

!  Now that we're done with ice, get started with water...

!  Read all the water stuff there is to read.
   file_id = sfstart(hdf_water_lib, DFACC_READ)

! this is all that there is to read for water flux:
!     extinction_water
!     singlescattering_water
!     asymmetry_water
!     AK_water
!     EL_water
!     EM_water
!     EN_water
!     spherical_albedo_water
!     escapefunction_water
!     flux_up_water
!     flux_down_water
  
   var_index = sfn2index(file_id, "ExtinctionCoefficient")
   var_id = sfselect(file_id, var_index)
   allocate(start(0:1), stride(0:1), edge(0:1))

!  set up hdf read bounds
   start = 0
   stride = 1
   edge(0) = number_wavelengths
   edge(1) = number_waterradii
   !end setup
   status = sfrdata(var_id, start,stride,edge, extinction_water)
   status = sfendacc(var_id)

   var_index = sfn2index(file_id, "SingleScatterAlbedo")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, singlescattering_water)
   status = sfendacc(var_id)

   var_index = sfn2index(file_id, "Phase: AsymmetryParameter")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, asymmetry_water)
   status = sfendacc(var_id)

   var_index = sfn2index(file_id, "MinimumEigenvalue")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, AK_water)
   status = sfendacc(var_id)

   var_index = sfn2index(file_id, "NegInternalReflectionCoefficient")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, EL_water)
   status = sfendacc(var_id)
 
   var_index = sfn2index(file_id, "DiffusionPatternIntegral")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, EM_water)
   status = sfendacc(var_id)

   var_index = sfn2index(file_id, "WeightedMeanOfEscapeFunction")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, EN_water)
   status = sfendacc(var_id)
 
   deallocate(start, stride, edge)

   allocate(start(0:2), stride(0:2), edge(0:2))
  ! set up hdf read bounds
   start = 0
   stride = 1
   edge(0) = size(spherical_albedo_water,1)
   edge(1) = size(spherical_albedo_water,2)
   edge(2) = size(spherical_albedo_water,3)
   !end setup

   var_index = sfn2index(file_id, "SphericalAlbedo")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, spherical_albedo_water)
   status = sfendacc(var_id)

   start(0) = findpoint(solarzenith_flux_all,minval(library_fluxsolarzenith)) -1
   edge(0) = size(escapefunction_water_solar,1)
   var_index = sfn2index(file_id, "EscapeFunction")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, escapefunction_water_solar)
   status = sfendacc(var_id)
  
   start(0) = findpoint(solarzenith_flux_all,minval(library_fluxsensorzenith)) -1
   edge(0) = size(escapefunction_water_sensor,1)
   var_index = sfn2index(file_id, "EscapeFunction")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, escapefunction_water_sensor)
   status = sfendacc(var_id)
 
   deallocate(start, stride, edge)
 
   allocate(start(0:3), stride(0:3), edge(0:3))
  ! set up hdf read bounds
   start = 0
   start(0) = findpoint(solarzenith_flux_all,minval(library_fluxsolarzenith))-1
   stride = 1
   edge(0) = size(flux_up_water_solar,1)
   edge(1) = size(flux_up_water_solar,2)
   edge(2) = size(flux_up_water_solar,3)
   edge(3) = size(flux_up_water_solar,4)
   !end setup

   var_index = sfn2index(file_id, "ReflectedFlux")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, flux_up_water_solar)
   status = sfendacc(var_id)
 
   var_index = sfn2index(file_id, "TransmittedFlux")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, flux_down_water_solar)
   status = sfendacc(var_id)

   start = 0
   start(0) = findpoint(solarzenith_flux_all,minval(library_fluxsensorzenith))-1
   stride = 1
   edge(0) = size(flux_up_water_sensor,1)
   edge(1) = size(flux_up_water_sensor,2)
   edge(2) = size(flux_up_water_sensor,3)
   edge(3) = size(flux_up_water_sensor,4)
   !end setup

   var_index = sfn2index(file_id, "ReflectedFlux")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, flux_up_water_sensor)
   status = sfendacc(var_id)

   var_index = sfn2index(file_id, "TransmittedFlux")
   var_id = sfselect(file_id, var_index)
   status = sfrdata(var_id, start,stride,edge, flux_down_water_sensor)
   status = sfendacc(var_id)

   deallocate(start, stride, edge)

!  read reflectance and go on to water
   allocate(start(0:5), stride(0:5), edge(0:5))

!  set up hdf read bounds
   start = 0

   start(0) = findpoint(sensorzenith_all,minval(library_sensor_zenith))-1
   start(1) = findpoint(solarzenith_all,minval(library_solar_zenith))-1
   start(2) = findpoint(relativeazimuth_all,minval(library_relative_azimuth))-1
   stride = 1
   edge(0) = size(library_reflectance_water,1)
   edge(1) = size(library_reflectance_water,2)
   edge(2) = size(library_reflectance_water,3)
   edge(3) = size(library_reflectance_water,4)
   edge(4) = size(library_reflectance_water,5)-1
   edge(5) = size(library_reflectance_water,6)
 
   allocate(temp_reflectance(edge(0), edge(1), edge(2), edge(3), &
              edge(4), edge(5)))

   var_index = sfn2index(file_id, "BidirectionalReflectance")
   var_id = sfselect(file_id, var_index)

!  zero-out the reflectance library for the purelythermal bands
   library_reflectance_water(:,:,:,:,7,:)  = 0.0

   status = sfrdata(var_id, start,stride,edge,  temp_reflectance)
   library_reflectance_water(:,:,:,:,1:6,:) = temp_reflectance(:,:,:,:,:,:)

   status = sfendacc(var_id)
   deallocate(start, stride, edge, temp_reflectance)
 
   status = sfend(file_id)

   deallocate(solarzenith_all, sensorzenith_all, relativeazimuth_all, solarzenith_flux_all )
end subroutine readlibraries
subroutine allocate_libraryarrays (trans_correct_lib,hdf_water_lib, hdf_ice_lib, debug,status ) 

   use GeneralAuxType
   use libraryarrays
   use nonscience_parameters
   use libraryinterpolates
   use hdf
   implicit none

   character(*),          intent (in)    :: hdf_water_lib, hdf_ice_lib, trans_correct_lib
   logical,               intent (in)    :: debug
   integer,               intent (out)   :: status

   integer                               :: checkvariable, filelun, number
   integer                               :: xdimension, ydimension
   integer                               :: process_flags,i,j,k
   
!  HDF library input
   integer                               :: file_id, var_id, var_index,index_solarzenith
   integer, dimension(:), allocatable    :: start, stride, edge
   logical, dimension(:), allocatable    :: mask
   real(single), dimension(100)          :: tempvector
   real(single), dimension(:), allocatable    :: solarzenith_all
   character(MAX_SDS_NAME_LEN)                         :: dummy_name,sdsname
   integer                               :: dummy_type, dummy_num_attr, dummy_rank, &
                                            localmin, localmax
   integer                               :: dim_sizes(6)

   integer                               :: minimum_solar_zenith, maximum_solar_zenith, &
                                            minimum_sensor_zenith, maximum_sensor_zenith,&
                                            minimum_relative_azimuth, maximum_relative_azimuth, &
                                            number_trans_precipitablewater, number_trans_anglevalues, &
                                            number_trans_wavelengths, number_trans_pressureheights
   
   minimum_solar_zenith    = 0
   maximum_solar_zenith    = 82

   minimum_sensor_zenith   = 0
   maximum_sensor_zenith   = 70

   minimum_relative_azimuth = 0
   maximum_relative_azimuth  = 180
   status = 0


!  Transmittance Correction Library array allocation
   allocate (start(1), stride(1), edge(1))
   start = 0
   stride = 1

   if (.not. allocated(trans_pressureheights)) then
      file_id = sfstart(trans_correct_lib, DFACC_READ)
      var_index = sfn2index(file_id, "Cloud_Top_Pressure")
      var_id = sfselect(file_id, var_index)
      status = sfginfo(var_id, dummy_name, dummy_rank, &
                         dim_sizes, dummy_type, dummy_num_attr)
      number_trans_pressureheights = dim_sizes(1)
      allocate(trans_pressureheights(number_trans_pressureheights))
      edge = number_trans_pressureheights
      status = sfrdata(var_id, start, stride, edge, trans_pressureheights)
      status = sfendacc(var_id)
   endif
   number_trans_pressureheights = size(trans_pressureheights)

   if(.not. allocated(trans_precipitablewater)) then 
      var_index = sfn2index(file_id, "Precipitable_Water")
      var_id = sfselect(file_id, var_index)
      status = sfginfo(var_id, dummy_name, dummy_rank, &
                         dim_sizes, dummy_type, dummy_num_attr)
      number_trans_precipitablewater = dim_sizes(1)
      allocate(trans_precipitablewater(number_trans_precipitablewater))
      edge = number_trans_precipitablewater
      status = sfrdata(var_id, start, stride, edge, trans_precipitablewater)
      status = sfendacc(var_id)
   endif
   number_trans_precipitablewater = size(trans_precipitablewater)

   if(.not. allocated(trans_anglevalues)) then
      var_index = sfn2index(file_id, "View_Angle_Cosine")
      var_id = sfselect(file_id, var_index)
      status = sfginfo(var_id, dummy_name, dummy_rank, &
                         dim_sizes, dummy_type, dummy_num_attr)
      number_trans_anglevalues = dim_sizes(1)
      allocate(trans_anglevalues(number_trans_anglevalues))
      edge = number_trans_anglevalues
      status = sfrdata(var_id, start, stride, edge, trans_anglevalues)
      status = sfendacc(var_id)
   endif 
   number_trans_anglevalues = size(trans_anglevalues)

   if(.not. allocated(trans_wavelengths)) then
      var_index = sfn2index(file_id, "Wavelengths")
      var_id = sfselect(file_id, var_index)
      status = sfginfo(var_id, dummy_name, dummy_rank, &
                         dim_sizes, dummy_type, dummy_num_attr)
      number_trans_wavelengths = dim_sizes(1)
      allocate(trans_wavelengths(number_trans_wavelengths))
      edge = number_trans_wavelengths
      status = sfrdata(var_id, start, stride, edge, trans_wavelengths)
      status = sfendacc(var_id)
   endif
   number_trans_wavelengths = size(trans_wavelengths)


   status = sfend(file_id)


   if (.not. allocated(transmit_correction_table)) then

      allocate(transmit_correction_table(number_trans_pressureheights,   &
                                         number_trans_precipitablewater, &
                                         number_trans_anglevalues,       &
                                         number_trans_wavelengths))
   endif
   if(.not. allocated(transmit_stddev_table)) then
      allocate(transmit_stddev_table(number_trans_pressureheights,   &
                                     number_trans_precipitablewater, &
                                     number_trans_anglevalues,       &
                                     number_trans_wavelengths))
   endif

   deallocate(start, stride, edge)
 
!  Library arrays and decription vectors

   if (.not. allocated(solarzenith_all)) then 
     file_id = sfstart(hdf_ice_lib, DFACC_READ)

     var_index = sfn2index(file_id, "ReflectanceSolarZenith")
     var_id = sfselect(file_id, var_index)

     status = sfginfo(var_id, dummy_name, dummy_rank, &
                      dim_sizes, dummy_type, dummy_num_attr)

     index_solarzenith = dim_sizes(1)
     allocate (solarzenith_all(index_solarzenith))
     allocate (start(1), stride(1), edge(1))
     start = 0
     edge =index_solarzenith
     stride = 1

     status = sfrdata(var_id, start, stride, edge, solarzenith_all)
     status = sfendacc(var_id)
   endif
   index_solarzenith = size(solarzenith_all)

!  connect to the wavelengths dataset
   if(.not. allocated(library_wavelengths) ) then
     var_index = sfn2index(file_id, "Wavelengths")
     var_id = sfselect(file_id, var_index)

     status = sfginfo(var_id, dummy_name, dummy_rank, &
                      dim_sizes, dummy_type, dummy_num_attr)
     number_wavelengths = dim_sizes(1)
     allocate (library_wavelengths(number_wavelengths))
     ! hdf data read setup
   
     start = 0
     edge = number_wavelengths
     stride = 1
     ! end setup

     status = sfrdata(var_id, start, stride, edge, library_wavelengths)
     status = sfendacc(var_id)
   endif
   number_wavelengths = size(library_wavelengths)
 
  ! connect to the radii dataset
   if(.not. allocated(ice_radii)) then
     var_index = sfn2index(file_id, "ParticleRadius")
     var_id = sfselect(file_id, var_index)

     status = sfginfo(var_id, dummy_name, dummy_rank, &
                      dim_sizes, dummy_type, dummy_num_attr)

     number_iceradii = dim_sizes(1)

     allocate (ice_radii(number_iceradii))
     ! hdf data read setup
     edge = number_iceradii
     ! end setup

     status = sfrdata(var_id, start, stride, edge, ice_radii)
     status = sfendacc(var_id)
   endif
   number_iceradii = size(ice_radii)

!  connect to the solar zenith angle dataset. We use the one from reflectance
!  according to the setup file the original code used...
   tempvector(:)= 0.
   if ( maximum_solar_zenith > maxval(solarzenith_all) ) then
     minimum_solar_zenith = minimum_solar_zenith - (maximum_solar_zenith -maxval(solarzenith_all)) -2.
     maximum_solar_zenith = maxval(solarzenith_all) 
     number_solarzenith = number_solarzenith + 2
   endif
   sdsname =  "ReflectanceSolarZenith"
   call optimumvector(file_id, &
                      sdsname, &
                      minimum_solar_zenith, &
                      maximum_solar_zenith, &
                      tempvector, &
                      number_solarzenith, &
                      status)

   if (allocated(library_solar_zenith) )then
     if(number_solarzenith .ne. size(library_solar_zenith)) deallocate(library_solar_zenith)
   endif
   if (.not. allocated(library_solar_zenith)) then
     allocate(library_solar_zenith(number_solarzenith))
   endif
   library_solar_zenith= tempvector(1:number_solarzenith)

!  connect to the sensor zenith angle dataset. We use the one from reflectance
!  according to the setup file the original code used...
   tempvector(:)= 0.
   sdsname = "ReflectanceSensorZenith"
   call optimumvector(file_id, &
                      sdsname, &
                      minimum_sensor_zenith, &
                      maximum_sensor_zenith, &
                      tempvector, &
                      number_sensorzenith, &
                      status )
   if (allocated(library_sensor_zenith) )then
     if(number_sensorzenith .ne. size(library_sensor_zenith)) deallocate(library_sensor_zenith)
   endif              
   if (.not. allocated(library_sensor_zenith)) then
     allocate(library_sensor_zenith(number_sensorzenith))
   endif
   library_sensor_zenith = tempvector(1:number_sensorzenith)


!  connect to the relative azimuth angle dataset. We use the one from reflectance
!  according to the setup file the original code used...
   tempvector(:)= 0.
   sdsname = "ReflectanceRelativeAzimuth"
   call optimumvector(file_id, &
                      sdsname, &
                      minimum_relative_azimuth, &
                      maximum_relative_azimuth, &
                      tempvector, &
                      number_relazimuth, &
                      status)

   if (allocated(library_relative_azimuth) )then
     if(number_relazimuth .ne. size(library_relative_azimuth)) deallocate(library_relative_azimuth)
   endif    
   if (.not. allocated(library_relative_azimuth)) then
     allocate(library_relative_azimuth(number_relazimuth))
   endif
   library_relative_azimuth= tempvector(1:number_relazimuth)



!  connect to the flux solar zenith angle dataset. 
   tempvector(:)= 0.
   sdsname = "FluxSolarZenith"
   call optimumvector(file_id, &
                      sdsname, &
                      minimum_solar_zenith, &
                      maximum_solar_zenith, &
                      tempvector, &
                      number_fluxsolarzenith, &
                      status)

   if (allocated(library_fluxsolarzenith) )then
     if(number_fluxsolarzenith .ne. size(library_fluxsolarzenith)) deallocate(library_fluxsolarzenith)
   endif 
   if (.not. allocated(library_fluxsolarzenith)) then
     allocate(library_fluxsolarzenith(number_fluxsolarzenith))
   endif
   library_fluxsolarzenith= tempvector(1:number_fluxsolarzenith)

!  connect to the flux solar zenith angle dataset.
   tempvector(:)= 0.
   sdsname = "FluxSolarZenith"
   call optimumvector(file_id, &
                      sdsname,&
                      minimum_sensor_zenith, &
                      maximum_sensor_zenith, &
                      tempvector, &
                      number_fluxsensorzenith, &
                      status)

   if (allocated(library_fluxsensorzenith) )then
     if(number_fluxsensorzenith .ne. size(library_fluxsensorzenith)) deallocate(library_fluxsensorzenith)
   endif 
   if (.not. allocated(library_fluxsensorzenith)) then
     allocate(library_fluxsensorzenith(number_fluxsensorzenith ))
   endif
   library_fluxsensorzenith= tempvector(1:number_fluxsensorzenith)


   ! get the number of taus from some dataset whose one dimension is a tau.
   ! reflectance makes a good candidate...
   if(.not. allocated(library_taus)) then
     var_index = sfn2index(file_id, "BidirectionalReflectance")
     var_id = sfselect(file_id, var_index)
     status = sfginfo(var_id, dummy_name, dummy_rank, &
                      dim_sizes, dummy_type, dummy_num_attr)
     number_taus = dim_sizes(4)
     allocate (library_taus(number_taus))
     status = sfendacc(var_id)

     ! read the set of taus used. The taus are a global attribute.
     var_id = sffattr(file_id, "OpticalThicknessUsed")
     status = sfgainfo(file_id, var_id, dummy_name, dummy_type, dummy_num_attr)
     status = sfrnatt(file_id, var_id, library_taus)

     status = sfend(file_id)
   endif
   number_taus = size(library_taus)

   ! now the water radius info
 
   if(.not. allocated(water_radii)) then 
     file_id = sfstart(hdf_water_lib, DFACC_READ)
   
     ! get the water radii info
     var_index = sfn2index(file_id, "ParticleRadius")
     var_id = sfselect(file_id, var_index)

     status = sfginfo(var_id, dummy_name, dummy_rank, &
                      dim_sizes, dummy_type, dummy_num_attr)

     number_waterradii = dim_sizes(1)

     allocate (water_radii(number_waterradii))
     ! hdf data read setup
     edge = number_waterradii
     ! end setup

     status = sfrdata(var_id, start, stride, edge, water_radii)
     status = sfendacc(var_id)
   endif
   number_waterradii = size(water_radii)


   status = sfend(file_id)

   deallocate(start, stride, edge, solarzenith_all)


!  finally. Let's allocate the arrays we'll need later 
   if(allocated(extinction_water)) then
     if (size(extinction_water,1) /= number_wavelengths .or. size(extinction_water,2) /= number_waterradii) then
       deallocate(extinction_water, asymmetry_water, ak_water, el_water, em_water, en_water)
     endif
   endif
   if (allocated(spherical_albedo_water)) then
     if (size(spherical_albedo_water,1) /= number_taus        .or. &
         size(spherical_albedo_water,2) /= number_wavelengths .or. &
         size(spherical_albedo_water,3) /= number_waterradii) then
        deallocate(spherical_albedo_water)
     endif
   endif
   
   if (allocated(escapefunction_water_solar)) then
     if (size(escapefunction_water_solar,1) /= number_fluxsolarzenith .or. &
         size(escapefunction_water_solar,2) /= number_wavelengths     .or. &
         size(escapefunction_water_solar,3) /= number_waterradii) then
        deallocate(escapefunction_water_solar, flux_up_water_solar, flux_down_water_solar)
     endif
   endif
   
   if (allocated(escapefunction_water_sensor)) then
     if (size(escapefunction_water_sensor,1) /= number_fluxsensorzenith .or. &
         size(escapefunction_water_sensor,2) /= number_wavelengths      .or. &
         size(escapefunction_water_sensor,3) /= number_waterradii) then
        deallocate(escapefunction_water_sensor,flux_up_water_sensor, flux_down_water_sensor)
     endif
   endif
   
   if (allocated(library_reflectance_water)) then
       if (size(library_reflectance_water,1) /= number_sensorzenith .or. &
           size(library_reflectance_water,2) /= number_solarzenith  .or. &
           size(library_reflectance_water,3) /= number_relazimuth   .or. &
           size(library_reflectance_water,4) /= number_taus         .or. &
           size(library_reflectance_water,5) /= number_wavelengths  .or. &
           size(library_reflectance_water,6) /= number_waterradii ) then
         deallocate(library_reflectance_water)
       endif
   endif
   
   if (.not. allocated(extinction_water)) then
     allocate(extinction_water(number_wavelengths,number_waterradii))
   endif
   if(.not. allocated(singlescattering_water)) then
      allocate(singlescattering_water(number_wavelengths,number_waterradii))
   endif
   if(.not. allocated(asymmetry_water)) then
      allocate(asymmetry_water(number_wavelengths,number_waterradii))
   endif
   if(.not. allocated(ak_water)) then
      allocate(ak_water(number_wavelengths,number_waterradii))
   endif
   if(.not. allocated(el_water)) then
      allocate(el_water(number_wavelengths,number_waterradii))
   endif
   if(.not. allocated(em_water)) then
      allocate(em_water(number_wavelengths,number_waterradii))
   endif
   if(.not. allocated(en_water)) then
      allocate(en_water(number_wavelengths,number_waterradii))
   endif
   
   if(.not. allocated(spherical_albedo_water)) then
      allocate(spherical_albedo_water(number_taus,number_wavelengths,number_waterradii))
   endif
   
   if(.not. allocated(escapefunction_water_solar)) then
   allocate(escapefunction_water_solar(number_fluxsolarzenith,number_wavelengths,number_waterradii))
   endif
   
   if(.not. allocated(flux_up_water_solar)) then
   allocate(flux_up_water_solar (number_fluxsolarzenith, number_taus,number_wavelengths,number_waterradii))
   endif
   
   if(.not. allocated(flux_down_water_solar)) then
   allocate(flux_down_water_solar (number_fluxsolarzenith, number_taus,number_wavelengths,number_waterradii)) 
   endif
   
   if(.not. allocated(escapefunction_water_sensor)) then
   allocate(escapefunction_water_sensor(number_fluxsensorzenith,number_wavelengths,number_waterradii))
   endif
   
   if(.not. allocated(flux_up_water_sensor)) then
   allocate(flux_up_water_sensor(number_fluxsensorzenith, number_taus,number_wavelengths,number_waterradii))
   endif
   
   if(.not. allocated(flux_down_water_sensor)) then
   allocate(flux_down_water_sensor(number_fluxsensorzenith, number_taus,number_wavelengths,number_waterradii)) 
   endif
   
   if(.not. allocated(library_reflectance_water)) then
   allocate(library_reflectance_water(number_sensorzenith, number_solarzenith, number_relazimuth, number_taus, &
                                      number_wavelengths, number_waterradii))
   endif
   
   
!  ice arrays, library, not interpolated
   if(allocated(extinction_ice)) then
     if (size(extinction_ice,1) /= number_wavelengths .or. size(extinction_ice,2) /= number_iceradii) then
       deallocate(extinction_ice, asymmetry_ice, ak_ice, el_ice, em_ice, en_ice)
     endif
     
   endif
   if (allocated(spherical_albedo_ice)) then
     if (size(spherical_albedo_ice,1) /= number_taus        .or. &
         size(spherical_albedo_ice,2) /= number_wavelengths .or. &
         size(spherical_albedo_ice,3) /= number_iceradii) then
        deallocate(spherical_albedo_ice)
     endif
   endif
   
   if (allocated(escapefunction_ice_solar)) then
     if (size(escapefunction_ice_solar,1) /= number_fluxsolarzenith .or. &
         size(escapefunction_ice_solar,2) /= number_wavelengths     .or. &
         size(escapefunction_ice_solar,3) /= number_iceradii) then
        deallocate(escapefunction_ice_solar, flux_up_ice_solar, flux_down_ice_solar)
     endif
   endif
   
   if (allocated(escapefunction_ice_sensor)) then
     if (size(escapefunction_ice_sensor,1) /= number_fluxsensorzenith .or. &
         size(escapefunction_ice_sensor,2) /= number_wavelengths      .or. &
         size(escapefunction_ice_sensor,3) /= number_iceradii) then
        deallocate(escapefunction_ice_sensor,flux_up_ice_sensor, flux_down_ice_sensor)
     endif
   endif
   
   if (allocated(library_reflectance_ice)) then
       if (size(library_reflectance_ice,1) /= number_sensorzenith .or. &
           size(library_reflectance_ice,2) /= number_solarzenith  .or. &
           size(library_reflectance_ice,3) /= number_relazimuth   .or. &
           size(library_reflectance_ice,4) /= number_taus         .or. &
           size(library_reflectance_ice,5) /= number_wavelengths  .or. &
           size(library_reflectance_ice,6) /= number_iceradii ) then
         deallocate(library_reflectance_ice)
       endif
   endif
   if(.not. allocated(extinction_ice)) then
   allocate(extinction_ice(number_wavelengths,number_iceradii))
   endif
   
   if(.not. allocated(singlescattering_ice)) then
   allocate(singlescattering_ice(number_wavelengths,number_iceradii))
   endif
   
   if(.not. allocated(asymmetry_ice)) then
   allocate(asymmetry_ice(number_wavelengths,number_iceradii))
   endif
   
   if(.not. allocated(ak_ice)) then
   allocate(ak_ice(number_wavelengths,number_iceradii))
   endif
   
   if(.not. allocated(el_ice)) then
   allocate(el_ice(number_wavelengths,number_iceradii))
   endif
   
   if(.not. allocated(em_ice)) then
   allocate(em_ice(number_wavelengths,number_iceradii))
   endif
   
   if(.not. allocated(en_ice)) then
   allocate(en_ice(number_wavelengths,number_iceradii))
   endif
   
   if(.not. allocated(spherical_albedo_ice)) then
   allocate(spherical_albedo_ice(number_taus,number_wavelengths,number_iceradii))
   endif
   
   if(.not. allocated(escapefunction_ice_solar)) then
   allocate(escapefunction_ice_solar(number_fluxsolarzenith,number_wavelengths,number_iceradii))
   endif
   
   if(.not. allocated(flux_up_ice_solar)) then
   allocate(flux_up_ice_solar (number_fluxsolarzenith, number_taus,number_wavelengths,number_iceradii))
   endif
   
   if(.not. allocated(flux_down_ice_solar)) then
   allocate(flux_down_ice_solar(number_fluxsolarzenith, number_taus,number_wavelengths,number_iceradii)) 
   endif
   
   if(.not. allocated(escapefunction_ice_sensor)) then
   allocate(escapefunction_ice_sensor(number_fluxsensorzenith,number_wavelengths,number_iceradii))
   endif
   
   if(.not. allocated(flux_up_ice_sensor)) then
   allocate(flux_up_ice_sensor(number_fluxsensorzenith, number_taus,number_wavelengths,number_iceradii))
   endif
   
   if(.not. allocated(flux_down_ice_sensor)) then
   allocate(flux_down_ice_sensor(number_fluxsensorzenith, number_taus,number_wavelengths,number_iceradii)) 
   endif
   
   if(.not. allocated(library_reflectance_ice)) then
   allocate(library_reflectance_ice(number_sensorzenith, number_solarzenith, number_relazimuth, number_taus, &
                                    number_wavelengths,number_iceradii  ))
   endif
   
   
   
   
!  arrays for library interpolation
   if(allocated(int_reflectance_water)) then
     if( size(int_reflectance_water,1 ) /=  number_taus .or.  &
         size(int_reflectance_water,2 ) /=  number_wavelengths .or.  &
         size(int_reflectance_water,3 ) /=  number_waterradii) then
        deallocate(int_reflectance_water, int_fluxupwater_sensor, int_fluxdownwater_solar,int_fluxdownwater_sensor)
     endif
   endif
   
   if(allocated(int_reflectance_ice)) then
     if( size(int_reflectance_ice,1 ) /=  number_taus .or.  &
         size(int_reflectance_ice,2 ) /=  number_wavelengths .or.  &
         size(int_reflectance_ice,3 ) /=  number_iceradii) then
        deallocate(int_reflectance_ice, int_fluxupice_sensor, int_fluxdownice_solar,int_fluxdownice_sensor)
     endif
   endif   
   
   if(allocated(int_escapefuncwater_solar)) then
     if(size(int_escapefuncwater_solar,1) /= number_wavelengths  .or. &
        size(int_escapefuncwater_solar,2) /= number_waterradii ) then
       deallocate(int_escapefuncwater_solar, int_escapefuncwater_sensor, int_escapefuncice_solar, int_escapefuncice_sensor)
     endif
   endif
   
   
   if(.not. allocated(int_reflectance_water)) then
   allocate(int_reflectance_water(number_taus, number_wavelengths,number_waterradii))
   endif
   
   if(.not. allocated(int_reflectance_ice)) then
   allocate(int_reflectance_ice(number_taus, number_wavelengths,number_iceradii))
   endif
   
   if(.not. allocated(int_fluxupwater_sensor)) then
   allocate(int_fluxupwater_sensor(number_taus, number_wavelengths,number_waterradii))
   endif
   
   if(.not. allocated(int_fluxdownwater_solar)) then
   allocate(int_fluxdownwater_solar(number_taus, number_wavelengths,number_waterradii))
   endif
   
   if(.not. allocated(int_fluxdownwater_sensor)) then
   allocate(int_fluxdownwater_sensor(number_taus, number_wavelengths,number_waterradii))
   endif
   
   if(.not. allocated(int_fluxupice_sensor)) then
   allocate(int_fluxupice_sensor(number_taus, number_wavelengths,number_iceradii))
   endif
   
   if(.not. allocated(int_fluxdownice_solar)) then
      allocate(int_fluxdownice_solar(number_taus, number_wavelengths,number_iceradii))
   endif
   
   if(.not. allocated(int_fluxdownice_sensor)) then
     allocate(int_fluxdownice_sensor(number_taus, number_wavelengths,number_waterradii))
   endif
   
   if(.not. allocated(int_escapefuncwater_solar)) then
      allocate(int_escapefuncwater_solar( number_wavelengths,number_waterradii))
   endif
   
   if(.not. allocated(int_escapefuncwater_sensor)) then
      allocate(int_escapefuncwater_sensor( number_wavelengths,number_waterradii))
   endif
   
   if(.not. allocated(int_escapefuncice_solar)) then
      allocate(int_escapefuncice_solar( number_wavelengths,number_iceradii))
   endif
   
   if(.not. allocated(int_escapefuncice_sensor)) then
      allocate(int_escapefuncice_sensor( number_wavelengths,number_iceradii))
   endif
    
end subroutine allocate_libraryarrays
subroutine allocate_arrays ( start, stride, edge, debug,status ) 

   use GeneralAuxType
   use core_arrays
   use nonscience_parameters
   use libraryarrays
   use hdf
   
   implicit none
   
   integer, dimension(:), intent(in)     :: start, stride, edge
   logical,               intent(in)     :: debug
   integer,               intent (out)   :: status

   integer                               :: checkvariable, filelun, number, number_flux_solzen
   integer                               :: xdimension, ydimension, number_of_bands
   integer                               :: process_flags,cloudmask_bits,i,j,k
   
!  HDF library input
   integer                               :: file_id, var_id, var_index

   character(MAX_SDS_NAME_LEN)                         :: dummy_name
   integer                               :: dummy_type, dummy_num_attr, dummy_rank
   integer                               :: dim_sizes(6), model_layers
   
   character(MAX_FILE_NAME_LEN) :: hdf_ice_lib, hdf_water_lib, library_path
   logical       :: allocation_status, array_size_change
   
   status = 0
   model_layers = 16
   cloudmask_bits = 48
   number_of_bands = size(bands)
   xdimension =  edge(1)
   ydimension =  edge(2)
 
!  Use optical_thickness array as a size/allocation test to save time.
   allocation_status = allocated(optical_thickness) 

   array_size_change = .false.
   
   if (allocation_status) then
      if (size(optical_thickness,1) /= xdimension .or. &
          size(optical_thickness,2) /= ydimension) then
         array_size_change  = .true.
      endif
   endif


!  Core retrieval arrays   
   if(array_size_change .and. allocated(optical_thickness)) then
     deallocate(optical_thickness, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif

   if (.not. allocated(optical_thickness)) then
     allocate (optical_thickness(xdimension,ydimension), stat = checkvariable)
     if ( checkvariable /= 0 ) then
        status=1
     endif
   endif

   if(array_size_change .and. allocated(optical_thickness_1621)) then
     deallocate(optical_thickness_1621, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif

  if (.not. allocated(optical_thickness_1621)) then
   allocate (optical_thickness_1621(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   if(array_size_change .and. allocated(ozone_absorption)) then
     deallocate(ozone_absorption, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(ozone_absorption)) then
   allocate (ozone_absorption(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   if(array_size_change .and. allocated(effective_radius_16)) then
     deallocate(effective_radius_16, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(effective_radius_16)) then
   allocate (effective_radius_16(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   if(array_size_change .and. allocated(effective_radius_21)) then
     deallocate(effective_radius_21, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(effective_radius_21)) then
   allocate (effective_radius_21(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   if(array_size_change .and. allocated(effective_radius_37)) then
     deallocate(effective_radius_37, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(effective_radius_37)) then
   allocate (effective_radius_37(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif
   
   
   if(array_size_change .and. allocated(effective_radius_1621)) then
     deallocate(effective_radius_1621, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(effective_radius_1621)) then
   allocate (effective_radius_1621(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif
   
    
  if(array_size_change .and. allocated(liquid_water_path_1621)) then
     deallocate(liquid_water_path_1621, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif

  if (.not. allocated(liquid_water_path_1621)) then
   allocate (liquid_water_path_1621(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif
 
  if(array_size_change .and. allocated(liquid_water_path)) then
     deallocate(liquid_water_path, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
      
  if (.not. allocated(liquid_water_path)) then
   allocate (liquid_water_path(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

  if(array_size_change .and. allocated(optical_thickness_error)) then
     deallocate(optical_thickness_error, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif

  if (.not. allocated(optical_thickness_error)) then
   allocate (optical_thickness_error(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

  if(array_size_change .and. allocated(effective_radius_21_error)) then
     deallocate(effective_radius_21_error, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif

  if (.not. allocated(effective_radius_21_error)) then
   allocate (effective_radius_21_error(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

  if(array_size_change .and. allocated(liquid_water_path_error)) then
     deallocate(liquid_water_path_error, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif

  if (.not. allocated(liquid_water_path_error)) then
   allocate (liquid_water_path_error(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
  endif

  if(array_size_change .and. allocated(optical_thickness_1621_error)) then
     deallocate(optical_thickness_1621_error, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif

  if (.not. allocated(optical_thickness_1621_error)) then
   allocate (optical_thickness_1621_error(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

  if(array_size_change .and. allocated(effective_radius_1621_error)) then
     deallocate(effective_radius_1621_error, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif

  if (.not. allocated(effective_radius_1621_error)) then
   allocate (effective_radius_1621_error(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

  if(array_size_change .and. allocated(liquid_water_path_1621_error)) then
     deallocate(liquid_water_path_1621_error, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif

  if (.not. allocated(liquid_water_path_1621_error)) then
   allocate (liquid_water_path_1621_error(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
  endif

  if(array_size_change .and. allocated(cloud_layer_flag)) then
     deallocate(cloud_layer_flag, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(cloud_layer_flag)) then
   allocate (cloud_layer_flag(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

  if(array_size_change .and. allocated(CSR_flag_array)) then
     deallocate(CSR_flag_array, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(CSR_flag_array)) then
   allocate (CSR_flag_array(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
  endif


  if(array_size_change .and. allocated(ir_cloudphase)) then
     deallocate(ir_cloudphase, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
      
  if (.not. allocated(ir_cloudphase)) then
   allocate(ir_cloudphase(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

 
!  Core input data arrays
  if(array_size_change .and. allocated(latitude)) then
     deallocate(latitude, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(latitude)) then
   allocate (latitude(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

  if(array_size_change .and. allocated(longitude)) then
     deallocate(longitude, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(longitude)) then
   allocate (longitude(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif


  if(array_size_change .and. allocated(central_wavelength)) then
     deallocate(central_wavelength, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(central_wavelength)) then
   allocate (central_wavelength(number_of_bands), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

  if(array_size_change .and. allocated(sensor_zenith_angle)) then
     deallocate(sensor_zenith_angle, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
   
  if (.not. allocated(sensor_zenith_angle)) then
   allocate (sensor_zenith_angle(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif
   
  if(array_size_change .and. allocated(solar_zenith_angle)) then
     deallocate(solar_zenith_angle, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(solar_zenith_angle)) then
   allocate (solar_zenith_angle(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

  if(array_size_change .and. allocated(solar_azimuth_angle)) then
     deallocate(solar_azimuth_angle, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(solar_azimuth_angle)) then
   allocate (solar_azimuth_angle(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif
  
    if(array_size_change .and. allocated(sensor_azimuth_angle)) then
     deallocate(sensor_azimuth_angle, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(sensor_azimuth_angle)) then 
   allocate (sensor_azimuth_angle(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   if(array_size_change .and. allocated(relative_azimuth_angle)) then
     deallocate(relative_azimuth_angle, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(relative_azimuth_angle)) then
   allocate (relative_azimuth_angle(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif


   if(array_size_change .and. allocated(solar_spectral_irradiance)) then
     deallocate(solar_spectral_irradiance, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
   
  if (.not. allocated(solar_spectral_irradiance)) then
   allocate (solar_spectral_irradiance(number_of_bands), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   if(array_size_change .and. allocated(greenwichmeantime)) then
     deallocate(greenwichmeantime, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(greenwichmeantime)) then
   allocate (greenwichmeantime(ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   if(array_size_change .and. allocated(band_measurements)) then
     deallocate(band_measurements, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(band_measurements)) then
   allocate (band_measurements(xdimension,number_of_bands, ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

!  Ancillary data array

   if(array_size_change .and. allocated(surface_albedo)) then
     deallocate(surface_albedo, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
   
   if (.not. allocated(surface_albedo)) then
   allocate (surface_albedo(xdimension,ydimension,number_of_bands), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif
   
   if(array_size_change .and. allocated(surface_temperature)) then
     deallocate(surface_temperature, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
   if (.not. allocated(surface_temperature)) then
   allocate (surface_temperature(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   
   if(array_size_change .and. allocated(cloud_top_temperature)) then
     deallocate(cloud_top_temperature, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
   if (.not. allocated(cloud_top_temperature)) then
   allocate (cloud_top_temperature(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   if(array_size_change .and. allocated(above_cloud_temperature)) then
     deallocate(above_cloud_temperature, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(above_cloud_temperature)) then
   allocate (above_cloud_temperature(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   if(array_size_change .and. allocated(cloud_top_pressure)) then
     deallocate(cloud_top_pressure, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(cloud_top_pressure)) then
   allocate (cloud_top_pressure(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   if(array_size_change .and. allocated(abovecloud_watervapor)) then
     deallocate(abovecloud_watervapor, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(abovecloud_watervapor)) then
   allocate (abovecloud_watervapor(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif


   if(array_size_change .and. allocated(column_ozone)) then
     deallocate(column_ozone, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(column_ozone)) then
   allocate (column_ozone(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   if(array_size_change .and. allocated(cloudmask)) then
     deallocate(cloudmask, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(cloudmask)) then
   allocate (cloudmask(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif 

   if(array_size_change .and. allocated(model_temp_profile)) then
     deallocate(model_temp_profile, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
  
  if (.not. allocated(model_temp_profile)) then
   allocate (model_temp_profile(xdimension,ydimension,model_layers), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif 

   if(array_size_change .and. allocated(model_water_profile)) then
     deallocate(model_water_profile, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(model_water_profile)) then
   allocate (model_water_profile(xdimension,ydimension,model_layers), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif 

   if(array_size_change .and. allocated(model_pressure_levels)) then
     deallocate(model_pressure_levels, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(model_pressure_levels)) then
   allocate (model_pressure_levels(xdimension,ydimension,model_layers), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif 

!  QA and Processing arrays
   if(array_size_change .and. allocated(cloudsummary)) then
     deallocate(cloudsummary, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(cloudsummary)) then
   allocate (cloudsummary(xdimension,ydimension), stat = checkvariable)
    if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   if(array_size_change .and. allocated(processing_information)) then
     deallocate(processing_information, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(processing_information)) then
   allocate (processing_information(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif


!  atmospheric correction

   if(array_size_change .and. allocated(transmittance_stddev)) then
     deallocate(transmittance_stddev, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
  
   if (.not. allocated(transmittance_stddev)) then
   allocate (transmittance_stddev(xdimension,ydimension,7), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   if(array_size_change .and. allocated(transmittance_twoway)) then
     deallocate(transmittance_twoway, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1 
     endif
   endif
   
  if (.not. allocated(transmittance_twoway)) then
   allocate (transmittance_twoway(xdimension,ydimension,7), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   if(array_size_change .and. allocated(meandelta_trans)) then
     deallocate(meandelta_trans, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
  
  if (.not. allocated(meandelta_trans)) then
   allocate (meandelta_trans(xdimension,ydimension,7), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif


   if(array_size_change .and. allocated(thermal_correction_twoway)) then
     deallocate(thermal_correction_twoway, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(thermal_correction_twoway)) then
   allocate (thermal_correction_twoway(xdimension,ydimension,2), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   if(array_size_change .and. allocated(thermal_correction_oneway)) then
     deallocate(thermal_correction_oneway, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
  if (.not. allocated(thermal_correction_oneway)) then
   allocate (thermal_correction_oneway(xdimension,ydimension,2), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif


   if(array_size_change .and. allocated(thermal_correction_bands)) then
     deallocate(thermal_correction_bands, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   
   
  if (.not. allocated(thermal_correction_bands)) then
  allocate (thermal_correction_bands(2), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   if(array_size_change .and. allocated(nonabs_obs_corrected)) then
     deallocate(nonabs_obs_corrected, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   if (.not. allocated(nonabs_obs_corrected)) then 
   allocate (nonabs_obs_corrected(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   if(array_size_change .and. allocated(absorbing_obs_corrected)) then
     deallocate(absorbing_obs_corrected, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   if (.not. allocated(absorbing_obs_corrected)) then 
   allocate (absorbing_obs_corrected(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif

   if(array_size_change .and. allocated(nonabsorbing_band)) then
     deallocate(nonabsorbing_band, stat = checkvariable)
     if ( checkvariable /= 0 ) then
       status=1
     endif
   endif
   if (.not. allocated(nonabsorbing_band)) then
   allocate (nonabsorbing_band(xdimension,ydimension), stat = checkvariable)
   if ( checkvariable /= 0 ) then
      status=1
   endif
   endif


  
end subroutine allocate_arrays
subroutine optimumvector(file_id, sdsname, in_minvalue, in_maxvalue, vector, vectorsize,status)

   use hdf
   use GeneralAuxType
   use nonscience_parameters
   implicit none


   character(*),              intent (in)    :: sdsname
   integer,                   intent (in)    :: in_minvalue, in_maxvalue, file_id
   real(single),dimension(:), intent( inout) :: vector
   integer,                   intent(out)    :: vectorsize, status

   logical, dimension(:), allocatable    :: mask
   character(MAX_SDS_NAME_LEN)             :: dummy_name
   integer                   :: dummy_type, dummy_num_attr, dummy_rank, &
                                localmin, localmax,lowpoint,highpoint,temp(1),  &
                                start, edge, stride, var_id, var_index
   integer                   :: dim_sizes(6), minvalue, maxvalue

   real(single), dimension(:), allocatable    :: tempvector

!  this routine 

   maxvalue = in_maxvalue
   minvalue = in_minvalue

   var_index = sfn2index(file_id, sdsname)
   var_id = sfselect(file_id, var_index)

   status = sfginfo(var_id, dummy_name, dummy_rank, &
                      dim_sizes, dummy_type, dummy_num_attr)

   vectorsize= dim_sizes(1)

   allocate (tempvector(vectorsize))
   ! hdf data read setup
   start = 0
   edge = vectorsize
   stride = 1
   ! end setup

   status = sfrdata(var_id, start, stride, edge, tempvector)
   status = sfendacc(var_id)

   if (float(maxvalue) > maxval(tempvector)) maxvalue = aint( maxval(tempvector))

   allocate(mask(vectorsize))
     where(tempvector >= real(minvalue) )
       mask = .true.
     elsewhere
       mask = .false.
     end where

     temp = (/minloc(tempvector,mask)-1/)
     lowpoint = temp(1)

     where(tempvector >=  real(maxvalue) )
       mask = .true.
     elsewhere
       mask = .false.
     end where

     temp= (/minloc(tempvector,mask)+1/)
     highpoint = temp(1)

     if (lowpoint < 1)    lowpoint = 1
     if (highpoint > vectorsize) highpoint =vectorsize

     vectorsize= highpoint - lowpoint + 1
     vector(1:vectorsize) = tempvector(lowpoint:highpoint)

  deallocate(tempvector)
  deallocate(mask)
  if (vectorsize <= 1 ) vectorsize = 2
end subroutine optimumvector

integer function findpoint( vector, value)
   use GeneralAuxType

   implicit none

   real(single) ,intent(in) :: vector(:)
   real(single) ,intent(in) :: value
   integer                  ::temp(1)

   real(single) ,allocatable   :: localvector(:)

   allocate(localvector(size(vector)))

   localvector = vector
   where ( localvector == value)
     localvector = 1.
   elsewhere
     localvector = 0.
   endwhere

   temp = maxloc(localvector)
   findpoint = temp(1)
   deallocate(localvector)

end function findpoint
   subroutine finalcleanup(status)

   use core_arrays
   use nonscience_parameters

   implicit none

   integer,        intent (out) :: status

   integer                      :: checkvariable

   deallocate (bands, stat = checkvariable)
   if ( checkvariable /= 0 ) then
      call local_message_handler('Array: bands could not be deallocated. Check system resources.', &
                                  warning, &
                                  'finalcleanup')
      status=1
   endif


   end subroutine finalcleanup
subroutine init_qualitydata
   use GeneralAuxType
   use core_arrays, only: processing_information

    processing_information(:,:)%cloud_mask_usefulness = 0_integer_onebyte
    processing_information(:,:)%cloud_mask_unobs_fov = 0_integer_onebyte
    processing_information(:,:)%day_night = 0_integer_onebyte
    processing_information(:,:)%sun_glint = 0_integer_onebyte
    processing_information(:,:)%snow_ice = 0_integer_onebyte
    processing_information(:,:)%surface_type = 0_integer_onebyte
    processing_information(:,:)%heavy_aerosol = 0_integer_onebyte
    processing_information(:,:)%thin_cirrus = 0_integer_onebyte
    processing_information(:,:)%shadow = 0_integer_onebyte

!   product quality and retrieval processing QA flags
    processing_information(:,:)%optical_thickness_general = 0_integer_onebyte
    processing_information(:,:)%optical_thickness_confidence = 0_integer_onebyte
    processing_information(:,:)%optical_thickness_outofbounds = 0_integer_onebyte
    processing_information(:,:)%effective_radius_general = 0_integer_onebyte
    processing_information(:,:)%effective_radius_confidence = 0_integer_onebyte
    processing_information(:,:)%water_path_general = 0_integer_onebyte
    processing_information(:,:)%water_path_confidence = 0_integer_onebyte
    processing_information(:,:)%cloud_retrieval_processing_path = 0_integer_onebyte
    processing_information(:,:)%retrieval_outcome = 0_integer_onebyte
    processing_information(:,:)%rayleigh_correction = 0_integer_onebyte
    processing_information(:,:)%atmospheric_correction = 0_integer_onebyte
    processing_information(:,:)%band_used_for_optical_thickness = 0_integer_onebyte
    processing_information(:,:)%optical_thickness_1621_gen = 0_integer_onebyte
    processing_information(:,:)%optical_thickness_1621_conf = 0_integer_onebyte
    processing_information(:,:)%effective_radius_1621_gen = 0_integer_onebyte
    processing_information(:,:)%effective_radius_1621_conf = 0_integer_onebyte
    processing_information(:,:)%water_path_1621_gen= 0_integer_onebyte
    processing_information(:,:)%water_path_1621_conf= 0_integer_onebyte
    processing_information(:,:)%multi_layer_cloud = 0_integer_onebyte

end subroutine init_qualitydata
end module modis_frontend_module
