module modis_io_module


 implicit none
 private

 public :: output_retrieval, get_modis_data_cube, openclose_files,getintegerarray_toolkit

 contains


subroutine output_retrieval(mapi_filedata, &
                            filename,      &
                            currentscan,   &
                            start, edge, stride, &
                            status)
   use GeneralAuxType
   use core_arrays
   use nonscience_parameters
   implicit none

   include 'mapi.f90.inc'
   include 'PGS_MODIS_39500.f'
   include 'mod06.f90.inc'

   integer,      intent(in)          :: mapi_filedata(:), currentscan
   integer,      intent(in)          :: start(:), edge(:), stride(:)
   character(*), intent (in)         :: filename

   integer,      intent (out)        :: status

   integer                              :: buffer_xsize, buffer_ysize, i,j, count
   integer(integer_twobyte)             :: fillint_twobyte
   integer(integer_twobyte),allocatable :: outputbuffer(:)
   real(double)                         :: scale, add_offset 
   integer :: localstart(3), localedge(3), xsize, ysize
   integer(integer_onebyte),allocatable :: cloud_mask_1km(:,:,:), quality_assurance_1km(:,:,:)
   real, allocatable :: retrieval_diff(:,:,:)
   status = success
   xsize = size(optical_thickness,1)
   ysize = size(optical_thickness,2)

   allocate(cloud_mask_1km(xsize,ysize ,2))
   allocate(quality_assurance_1km(xsize,ysize , 5))

   call set_quality_data(xsize, ysize)

   call convert_binary_qa(cloud_mask_1km,         &
                          quality_assurance_1km,  &
                          status)
   call writeqaarray_toolkit(mapi_filedata,  &
                           filename,            &
                           currentscan,         &
                           cloud_mask_1km,   &
                           SDS_NAME06, &
                           status )
   call writeqaarray_toolkit(mapi_filedata,  &
                           filename,            &
                           currentscan,         &
                           quality_assurance_1km,   &
                           SDS_NAME07, &
                           status )
   deallocate(cloud_mask_1km, quality_assurance_1km)


   allocate(retrieval_diff(xsize,ysize,2))
   where(effective_radius_21 >0. .and. effective_radius_16 > 0.)
     retrieval_diff(:,:,1) = effective_radius_16 - effective_radius_21
   elsewhere
     retrieval_diff(:,:,1) = -999.
   endwhere

   where(effective_radius_21 >0. .and. effective_radius_37 > 0.)
     retrieval_diff(:,:,2) = effective_radius_37 - effective_radius_21
   elsewhere
     retrieval_diff(:,:,2) = -999.
   endwhere



   call writefloat3darray_toolkit(mapi_filedata,  &
                           filename,            &
                           currentscan,         &
                           retrieval_diff,   &
                           SDS_NAME05, &
                           status )
   deallocate(retrieval_diff)
 
   call writefloatarray_toolkit(mapi_filedata,  &
                           filename,            &
                           currentscan,         &
                           optical_thickness,   &
                           SDS_NAME02, &
                           status )

   call writefloatarray_toolkit(mapi_filedata,  &
                           filename,            &
                           currentscan,         &
                           optical_thickness_1621,   &
                           SDS_NAME04, &
                           status ) 

   call writefloatarray_toolkit(mapi_filedata,  &
                           filename,            &
                           currentscan,         &
                           effective_radius_21,   &
                           SDS_NAME01, &
                           status )

   call writefloatarray_toolkit(mapi_filedata,  &
                           filename,            &
                           currentscan,         &
                           effective_radius_1621,   &
                           SDS_NAME03, &
                           status )

   call writefloatarray_toolkit(mapi_filedata,  &
                           filename,            &
                           currentscan,         &
                           liquid_water_path,   &
                           SDS_NAME09, &
                           status )
   
   call writefloatarray_toolkit(mapi_filedata,  &
                           filename,            &
                           currentscan,         &
                           liquid_water_path_1621,   &
                           SDS_NAME10, &
                           status )

   call writefloatarray_toolkit(mapi_filedata,  &
                           filename,            &
                           currentscan,         &
                           optical_thickness_error,   &
                           SDS_NAME12, &
                           status )

   call writefloatarray_toolkit(mapi_filedata,  &
                           filename,            &
                           currentscan,         &
                           effective_radius_21_error,   &
                           SDS_NAME11, &
                           status )

   call writefloatarray_toolkit(mapi_filedata,  &
                           filename,            &
                           currentscan,         &
                           liquid_water_path_error,   &
                           SDS_NAME13, &
                           status )

   call writefloatarray_toolkit(mapi_filedata,  &
                           filename,            &
                           currentscan,         &
                           real(cloud_layer_flag),   &
                           SDS_NAME15, &
                           status )

   call writefloatarray_toolkit(mapi_filedata,  &
                           filename,            &
                           currentscan,         &
                           real(processing_information%cloud_retrieval_processing_path),   &
                           SDS_NAME14, &
                           status )

    call writefloatarray_toolkit(mapi_filedata,  &
                            filename,            &
                            currentscan,         &
                            optical_thickness_1621_error,   &
                            SDS_NAME16, &
                            status )
 
    call writefloatarray_toolkit(mapi_filedata,  &
                            filename,            &
                            currentscan,         &
                            effective_radius_1621_error,   &
                            SDS_NAME17, &
                            status )
 
    call writefloatarray_toolkit(mapi_filedata,  &
                            filename,            &
                            currentscan,         &
                            liquid_water_path_1621_error,   &
                            SDS_NAME18, &
                            status )

end subroutine output_retrieval
subroutine get_modis_data_cube ( level1b_filedata, level1b_filename,        &
                                 geolocation_filedata,geolocation_filename, &
                                 start, edge, stride, scan_number, debug, status)

!  Core sds of interest in the MODIS level 1B file read to fill the following:
!   central_wavelength
!   solar_spectral_irradiance (not available)
!   greenwichmeantime
!   latitude (MOD03)
!   longitude (MOD03)
!   band_measurements
!   solar_zenith_angle (MOD03)
!   sensor_zenith_angle (MOD03)
!   solar_azimuth_angle (MOD03)
!   sensor_azimuth_angle (MOD03)


   use GeneralAuxType
   use hdf
   use core_arrays
   use science_parameters
   use nonscience_parameters
   use mod06_run_settings

   implicit none

   include 'mapi.f90.inc'
   include 'PGS_SMF.f'
   include 'PGS_MODIS_39500.f'
   include 'L1B_Reader.f90.inc'

   character*(*), intent(in)                 :: geolocation_filename, level1b_filename
   integer, dimension (2), intent (in)       :: start, edge, stride
   integer, dimension(:), intent(in)         :: level1b_filedata, geolocation_filedata
   integer, intent(in)                       :: scan_number 
  
   logical,                intent (in)       :: debug
   integer,                intent (out)      :: status

   real(double)                              :: d_scale_factor  
   real(single), allocatable                 :: scale_factor(:), offset_factor(:)
   integer                                   :: numberofbands ,checkvariable,  i, j, k
   integer                                   :: scaleflag, offsetflag, index, value, count, array_index

   integer(integer_fourbyte)                 :: rank,  number_type, nattrs
   integer(integer_fourbyte)                 :: sds_id,sds_index,attr_index, hdfid

   integer(integer_fourbyte), dimension(2)   :: start2d, edge2d, stride2d, dimsizes_2d
   integer(integer_fourbyte), dimension(2)   :: central_start, central_edge, central_stride
   integer(integer_fourbyte), dimension(3)   :: start3d, edge3d, stride3d, dimsizes_3d
   integer(integer_twobyte),dimension(:,:), allocatable  :: temp_array

   integer                                   :: xdimension, ydimension, pixels_across_scan,temp_x, resol
   character(len=MAX_SDS_NAME_LEN)                         :: sds_name, local_sds_name, attr_name, &
                                                scale_factor_name, offset_factor_name
   real,dimension(:,:),allocatable           :: central_azimuth_angle,level1b_buffer
   integer(integer_onebyte),dimension(:,:), allocatable :: uncertainty_buffer, samples_buffer, v_flag
   logical                                   :: errorflag, useoffset
   character*1   Gain
   character*6   Cal_type
   integer Data_Size(2)


   status = success
   numberofbands = size(bands)

   start2d = start
   edge2d = edge
   stride2d = stride

!  get level 1b data
 

   xdimension =   edge2d(1)
   ydimension =   edge2d(2)
 
   Resol = Resol_Is_1km
   Gain = ' ' 
   allocate(level1b_buffer(xdimension, ydimension))
   allocate(uncertainty_buffer(xdimension, ydimension))
   allocate(samples_buffer(xdimension, ydimension))
   allocate(V_flag(xdimension, ydimension))

   do i = 1, numberofbands
     if(bands(i) < 20 .or. bands(i) == 26) then
       Cal_type = Cal_Is_Refl
     else
       Cal_Type = Cal_Is_Rad
     endif

     call Read_L1B(level1b_filedata,     &
                   level1b_filename,     &
                   scan_number,          &
                   bands(i), Gain,       &
                   Resol,Cal_type,       &
                   xdimension,ydimension,&
                   level1b_buffer,       &
                   uncertainty_buffer,   &
                   samples_buffer,       &
                   V_flag,               &
                   Data_size,            &
                   errorflag)
     band_measurements(:,i,:) = level1b_buffer(:,:)
     
     do j=1, xdimension
        do k=1, ydimension
           if (uncertainty_buffer(j,k) >= meas_uncertain_threshold) &
                band_measurements(j,i,k) = fillvalue_real
        enddo
     enddo


   enddo

   deallocate(level1b_buffer, uncertainty_buffer, samples_buffer, v_flag)

!  get full resolution Latitude and Longitude arrays
   useoffset = .false.
   call getfloatarray_toolkit(geolocation_filedata,   &
                              'Latitude',             &
                              start, stride, edge,    &
                              latitude,               &
                              status)
   call getfloatarray_toolkit(geolocation_filedata,   &
                              'Longitude',            &
                              start, stride, edge,    &
                              longitude,              &
                              status)

   call getintegerarray_toolkit(geolocation_filedata, &
                              'SensorZenith',         &
                              start, stride, edge,    &
                              useoffset, &
                              sensor_zenith_angle,    &
                              status)

   call getintegerarray_toolkit(geolocation_filedata, &
                              'SensorAzimuth',         &
                              start, stride, edge,    &
                              useoffset, &
                              sensor_azimuth_angle,    &
                              status)

   call getintegerarray_toolkit(geolocation_filedata, &
                              'SolarZenith',         &
                              start, stride, edge,    &
                              useoffset, &
                              solar_zenith_angle,    &
                              status)

   call getintegerarray_toolkit(geolocation_filedata, &
                              'SolarAzimuth',         &
                              start, stride, edge,    &
                              useoffset, &
                              solar_azimuth_angle,    &
                              status)
   do i = 1, numberofbands
     if(bands(i) < 20 .or. bands(i) == 26) then
        where (band_measurements(:,i,:) > 0.) 
           band_measurements(:,i,:) = band_measurements(:,i,:) / cos(d2r*solar_zenith_angle)
        endwhere
     endif
   enddo

!  calculate the relative azimuth
   relative_azimuth_angle =  solar_azimuth_angle + 180. - sensor_azimuth_angle

   do i = 1, size(relative_azimuth_angle,1)

     do j = 1,  size(relative_azimuth_angle,2)
       if (relative_azimuth_angle(i,j) <= 0.) relative_azimuth_angle(i,j) = -relative_azimuth_angle(i,j) 
       if (relative_azimuth_angle(i,j) > 180.) relative_azimuth_angle(i,j) = 360. - relative_azimuth_angle(i,j)  
     enddo
   enddo

   relative_azimuth_angle = abs(relative_azimuth_angle)
   where(relative_azimuth_angle > 180.) relative_azimuth_angle =360. - relative_azimuth_angle
end subroutine get_modis_data_cube
subroutine openclose_files (l1bname,             &
                            cloudmask_name,      &
                            geolocation_name,    &
                            mod06_name,          &
                            directive,           &
                            l1b_filedata,        &
                            cloudmask_filedata,  &
                            geolocation_filedata,&
                            mod06_filedata,      &
                            status)

  use nonscience_parameters
  implicit none
  include 'mapi.f90.inc'
  include 'PGS_SMF.f'
  include 'PGS_IO.f'
  include 'PGS_MODIS_39500.f'

  character*(*), intent(in) :: l1bname, cloudmask_name, geolocation_name, directive,mod06_name
  integer, intent(inout)    :: l1b_filedata(MODFILLEN), cloudmask_filedata(MODFILLEN),  &
                               geolocation_filedata(MODFILLEN),mod06_filedata(MODFILLEN)
  integer, intent(out)      :: status

  status = success 

  if (directive == 'open') then

    if(OPMFIL(l1bname,'r',l1b_filedata) /= MAPIOK) then
       CALL MODIS_SMF_SETDYNAMICMSG(MODIS_E_GENERIC,'MAPI open failed for L1B file, Check PCF.','openclose_files')
       status = failure
    else
       call MODIS_SMF_SETDYNAMICMSG(MODIS_S_GENERIC,'MAPI open for L1B file OK','openclose_files')
    endif    

    if(OPMFIL(cloudmask_name,'r',cloudmask_filedata) /= MAPIOK) then
       CALL MODIS_SMF_SETDYNAMICMSG(MODIS_E_GENERIC,'MAPI open failed for cloudmask file, Check PCF.','openclose_files')
       status = failure
    else    
       call MODIS_SMF_SETDYNAMICMSG(MODIS_S_GENERIC,'MAPI open for cloudmask file OK','openclose_files')
    endif

    if(OPMFIL(geolocation_name,'r',geolocation_filedata) /= MAPIOK) then
       CALL MODIS_SMF_SETDYNAMICMSG(MODIS_E_GENERIC,'MAPI open failed for geolocation, Check PCF.','openclose_files')
       status = failure 
    else    
       call MODIS_SMF_SETDYNAMICMSG(MODIS_S_GENERIC,'MAPI open for geolocation file OK','openclose_files')
    endif

    if(OPMFIL(mod06_name,'a',mod06_filedata) /= MAPIOK) then
       CALL MODIS_SMF_SETDYNAMICMSG(MODIS_E_GENERIC,'MAPI open failed for MOD06_L2, Check PCF.','openclose_files')
       status = failure
    else
       call MODIS_SMF_SETDYNAMICMSG(MODIS_S_GENERIC,'MAPI open for MOD06_L2 file OK','openclose_files')
    endif
  else
    if (CLMFIL(l1b_filedata) /= MAPIOK) then
      CALL MODIS_SMF_SETDYNAMICMSG(MODIS_E_GENERIC,'MAPI close failed for L1B file, Check PCF.','openclose_files')
    else
      call MODIS_SMF_SETDYNAMICMSG(MODIS_S_GENERIC,'MAPI close for L1B file OK','openclose_files')
    endif

    if (CLMFIL(cloudmask_filedata) /= MAPIOK) then
      CALL MODIS_SMF_SETDYNAMICMSG(MODIS_E_GENERIC,'MAPI close failed for cloudmask file, Check PCF.','openclose_files')
    else
      call MODIS_SMF_SETDYNAMICMSG(MODIS_S_GENERIC,'MAPI close for cloudmaskfile OK','openclose_files')
    endif

    if (CLMFIL(geolocation_filedata) /= MAPIOK) then
      CALL MODIS_SMF_SETDYNAMICMSG(MODIS_E_GENERIC,'MAPI close failed for geolocation file, Check PCF.','openclose_files')
    else
      call MODIS_SMF_SETDYNAMICMSG(MODIS_S_GENERIC,'MAPI close for geolocation file OK','openclose_files')
    endif
  
    if (CLMFIL(mod06_filedata) /= MAPIOK) then
      CALL MODIS_SMF_SETDYNAMICMSG(MODIS_E_GENERIC,'MAPI close failed for MOD06_L2, Check PCF.','openclose_files')
    else
      call MODIS_SMF_SETDYNAMICMSG(MODIS_S_GENERIC,'MAPI close for MOD06_L2 file OK','openclose_files')
    endif 
  endif

end subroutine openclose_files
subroutine set_quality_data(xsize, ysize)

   use GeneralAuxType
   use core_arrays
   use libraryarrays, only: water_radii, ice_radii

   integer , intent(in) :: xsize, ysize

   integer :: i, j

   integer, parameter :: marginal = 1, good = 2, very_good = 3, no_confidence = 0

   do i = 1, xsize
     do j = 1, ysize
       if (optical_thickness(i,j) > 0. .and. effective_radius_21(i,j) > 0.) then
         processing_information(i,j)%optical_thickness_general = 1
       else
         processing_information(i,j)%optical_thickness_general = 0
       endif
      
       if (effective_radius_21(i,j) > 4. ) then
         processing_information(i,j)%effective_radius_general = 1
       else
         processing_information(i,j)%effective_radius_general = 0
       endif 

          if ( debugPxlYcoord_1to20L0inclusiv /= 0 &
               .and. i == debugPxlXcoord_1to1354inclusiv &
               .and. ((iteration - 1) * 10 + j) == debugPxlYcoord_1to20L0inclusiv ) then
             debugPRN = .true.
          else
             debugPRN = .false.
          endif

!    Determine QA confidence level for tau, re, and water path - several changes
!        made in way logic constructed as well as updates to QA matrix for collection 5
!         - GTA 2/17/05

!     Set default to no confidence answer
      processing_information(i,j)%optical_thickness_confidence = no_confidence
      processing_information(i,j)%effective_radius_confidence = no_confidence
      processing_information(i,j)%water_path_confidence = no_confidence

!     Process according to QA matrix (12/30/04) for water cloud

       if(cloudsummary(i,j)%watercloud .or. cloudsummary(i,j)%unknowncloud) then
       
         if (optical_thickness(i,j) > 2. ) then
             if ((effective_radius_21(i,j) > 2. .and. &
                 effective_radius_21(i,j) <= 4.) .or. &
                 (effective_radius_21(i,j) > 25. .and. &
                 effective_radius_21(i,j) < 30.) ) then
               processing_information(i,j)%optical_thickness_confidence = marginal
               processing_information(i,j)%effective_radius_confidence = marginal
               processing_information(i,j)%water_path_confidence = marginal
             endif
             if (effective_radius_21(i,j) > 20. .and. &
                 effective_radius_21(i,j) <= 25. ) then
               processing_information(i,j)%optical_thickness_confidence = good
               processing_information(i,j)%effective_radius_confidence = good
               processing_information(i,j)%water_path_confidence = good
             endif
             if (effective_radius_21(i,j) > 4. .and. &
                 effective_radius_21(i,j) <= 20. ) then
               processing_information(i,j)%optical_thickness_confidence = very_good
               processing_information(i,j)%effective_radius_confidence = very_good
               processing_information(i,j)%water_path_confidence = very_good
             endif
             
         elseif(optical_thickness(i,j) <= 2. .and. optical_thickness(i,j) > 0.) then
             if (effective_radius_21(i,j) > 20. .and. &
                 effective_radius_21(i,j) <= 25. ) then
               processing_information(i,j)%optical_thickness_confidence = marginal
               processing_information(i,j)%effective_radius_confidence = marginal
               processing_information(i,j)%water_path_confidence = marginal
             endif
             if (effective_radius_21(i,j) > 4. .and. &
                 effective_radius_21(i,j) <= 20. ) then
               processing_information(i,j)%optical_thickness_confidence = good
               processing_information(i,j)%effective_radius_confidence = good
               processing_information(i,j)%water_path_confidence = good
             endif
         endif

       endif

!     Process according to QA matrix (12/30/04) for ice cloud

       if(cloudsummary(i,j)%icecloud) then
          if (effective_radius_21(i,j) < maxval(ice_radii) .and. &
              effective_radius_21(i,j) > minval(ice_radii) ) then
               processing_information(i,j)%optical_thickness_confidence = very_good
               processing_information(i,j)%effective_radius_confidence = very_good
               processing_information(i,j)%water_path_confidence = very_good
          endif
       endif


       if (optical_thickness_1621(i,j) > 0. .and. effective_radius_1621(i,j) > 0. )  then
         processing_information(i,j)%optical_thickness_1621_gen = 1
       else
         processing_information(i,j)%optical_thickness_1621_gen = 0
       endif
  
       if (effective_radius_1621(i,j) > 4.0 ) then
         processing_information(i,j)%effective_radius_1621_gen = 1
       else
         processing_information(i,j)%effective_radius_1621_gen = 0
       endif


!     Process confidence levels for 16-21 retrieval

!     Set default to no confidence answer
      processing_information(i,j)%optical_thickness_1621_conf = no_confidence
      processing_information(i,j)%effective_radius_1621_conf = no_confidence
      processing_information(i,j)%water_path_1621_conf = no_confidence


      if(cloudsummary(i,j)%watercloud .or. cloudsummary(i,j)%unknowncloud) then
         if (optical_thickness_1621(i,j) > 2. ) then
             if ((effective_radius_1621(i,j) > 2. .and. &
                 effective_radius_1621(i,j) <= 4.) .or. &
                 (effective_radius_1621(i,j) > 25. .and. &
                 effective_radius_1621(i,j) < 30.) ) then
               processing_information(i,j)%optical_thickness_1621_conf= marginal
               processing_information(i,j)%effective_radius_1621_conf= marginal
               processing_information(i,j)%water_path_1621_conf= marginal
             endif
             if (effective_radius_1621(i,j) > 20. .and. &
                 effective_radius_1621(i,j) <= 25. ) then
               processing_information(i,j)%optical_thickness_1621_conf= good
               processing_information(i,j)%effective_radius_1621_conf= good
               processing_information(i,j)%water_path_1621_conf= good
             endif
             if (effective_radius_1621(i,j) > 4. .and. &
                 effective_radius_1621(i,j) <= 20. ) then
               processing_information(i,j)%optical_thickness_1621_conf= very_good
               processing_information(i,j)%effective_radius_1621_conf= very_good
               processing_information(i,j)%water_path_1621_conf= very_good
             endif
         elseif (optical_thickness_1621(i,j) <= 2. .and. &
                 optical_thickness_1621(i,j) > 0. )then
             if (effective_radius_1621(i,j)> 20. .and. &
                    effective_radius_1621(i,j)<= 25.) then
               processing_information(i,j)%optical_thickness_1621_conf= marginal
               processing_information(i,j)%effective_radius_1621_conf= marginal
               processing_information(i,j)%water_path_1621_conf= marginal
             endif
             if (effective_radius_1621(i,j)> 4. .and. &
                    effective_radius_1621(i,j)<= 20.) then
               processing_information(i,j)%optical_thickness_1621_conf= good
               processing_information(i,j)%effective_radius_1621_conf= good
               processing_information(i,j)%water_path_1621_conf= good
             endif
         endif

      endif

       if(cloudsummary(i,j)%icecloud) then
         if (effective_radius_1621(i,j)< maxval(ice_radii) .and. &
                    effective_radius_1621(i,j)> minval(ice_radii)) then
             processing_information(i,j)%optical_thickness_1621_conf= very_good
             processing_information(i,j)%effective_radius_1621_conf= very_good
             processing_information(i,j)%water_path_1621_conf= very_good
         endif
       endif


! Added by Riedi Mon Apr 11 20:21:52 CEST 2005 
!     Process according to cloudsuspicious flag for all clouds and all retrievals
       if(cloudsummary(i,j)%cloudsuspicious) then
               processing_information(i,j)%optical_thickness_confidence = marginal
               processing_information(i,j)%effective_radius_confidence = marginal
               processing_information(i,j)%water_path_confidence = marginal

               processing_information(i,j)%optical_thickness_1621_conf= marginal
               processing_information(i,j)%effective_radius_1621_conf= marginal
               processing_information(i,j)%water_path_1621_conf= marginal
       endif 
! End of Riedi modifs


       if (liquid_water_path(i,j) > 0. ) then
         processing_information(i,j)%water_path_general = 1
       else
         processing_information(i,j)%water_path_general = 0
       endif

       if (liquid_water_path_1621(i,j) > 0. ) then
         processing_information(i,j)%water_path_1621_gen = 1
       else
          processing_information(i,j)%water_path_1621_gen = 0
       endif


       if (cloudsummary(i,j)%cloudmask_determined) then
       
          if(cloudsummary(i,j)%watercloud) then
            processing_information(i,j)%cloud_retrieval_processing_path = 2
          elseif(cloudsummary(i,j)%icecloud) then
            processing_information(i,j)%cloud_retrieval_processing_path = 3
          elseif(cloudsummary(i,j)%unknowncloud) then
            processing_information(i,j)%cloud_retrieval_processing_path = 4
          else
            processing_information(i,j)%cloud_retrieval_processing_path = 1
          endif
       else 
          processing_information(i,j)%cloud_retrieval_processing_path = 0
       endif 

       if (processing_information(i,j)%optical_thickness_general /= 0 .and. &
           processing_information(i,j)%effective_radius_general /= 0 ) then
         processing_information(i,j)%retrieval_outcome = 1 
       else
         processing_information(i,j)%retrieval_outcome = 0
       endif
        
       processing_information(i,j)%atmospheric_correction = 1
       if (processing_information(i,j)%optical_thickness_general /= 0 .and.  &
           processing_information(i,j)%band_used_for_optical_thickness ==1 )then
         processing_information(i,j)%rayleigh_correction = 1 
       else
         processing_information(i,j)%rayleigh_correction = 0
       endif 

       if (cloudsummary(i,j)%cloudmask_determined .and. &
            ( cloudsummary(i,j)%ocean_surface .or.      &
              cloudsummary(i,j)%snowice_surface)      )  then
          if(cloudsummary(i,j)%watercloud) then
            processing_information(i,j)%cloud_retrieval_proc_path_1621= 2
          elseif(cloudsummary(i,j)%icecloud) then
            processing_information(i,j)%cloud_retrieval_proc_path_1621= 3
          elseif(cloudsummary(i,j)%unknowncloud) then
            processing_information(i,j)%cloud_retrieval_proc_path_1621= 4
          else
            processing_information(i,j)%cloud_retrieval_proc_path_1621= 1
          endif 
       else
         processing_information(i,j)%cloud_retrieval_proc_path_1621= 0    
       endif

       if (processing_information(i,j)%optical_thickness_1621_gen /= 0 .and. &
           processing_information(i,j)%effective_radius_1621_gen /= 0) then
           processing_information(i,j)%retrieval_outcome_1621 = 1
       else
           processing_information(i,j)%retrieval_outcome_1621 = 0 
       endif
 


! initialize the variable first of all. -- GW 4.6.05
       processing_information(i,j)%multi_layer_cloud = 0

       if (cloudsummary(i,j)%cloudmask_determined) then

         if (processing_information(i,j)%cloud_retrieval_processing_path == 1)then ! dec.tree stop
            processing_information(i,j)%multi_layer_cloud = 1

         elseif (processing_information(i,j)%cloud_retrieval_processing_path == 2) then ! water cloud
            if (cloud_layer_flag(i,j) < 2) then 
               processing_information(i,j)%multi_layer_cloud = 2 ! SL water
            else 
               processing_information(i,j)%multi_layer_cloud = 3 ! ML water
            endif

         elseif(processing_information(i,j)%cloud_retrieval_processing_path == 3) then !ice cloud
            if (cloud_layer_flag(i,j) < 2) then 
               processing_information(i,j)%multi_layer_cloud = 4 ! SL ice
            else 
               processing_information(i,j)%multi_layer_cloud = 5 ! ML ice
            endif

         elseif (processing_information(i,j)%cloud_retrieval_processing_path == 4) then ! unknown cloud
            if (cloud_layer_flag(i,j) < 2) then 
               processing_information(i,j)%multi_layer_cloud = 6 ! SL unknown
            else 
               processing_information(i,j)%multi_layer_cloud = 7 ! ML unknown
            endif
         endif

       else
          processing_information(i,j)%multi_layer_cloud = 0 
       endif

! set the information for CSR QA -- GW. 4.7.05
       processing_information(i,j)%CSR_flag = 0
       processing_information(i,j)%CSR_flag = CSR_flag_array(i,j)



     enddo
   enddo
end subroutine set_quality_data
subroutine convert_binary_qa(cloud_mask_1km,         &
                             quality_assurance_1km, &
                             status)
use GeneralAuxType
use core_arrays, only: processing_information
use nonscience_parameters

implicit none
 
integer(integer_onebyte) , intent(out) :: cloud_mask_1km(:,:,:), quality_assurance_1km(:,:,:)
integer, intent(inout) :: status

integer   :: i,j, blah
cloud_mask_1km = 0
quality_assurance_1km = 0

do i = 1, size(cloud_mask_1km,1)
  do j= 1, size(cloud_mask_1km,2)  

!    Cloud Mask 1KM, Byte 1 -----------------------------------------------------------------------------------

     if(processing_information(i,j)%cloud_mask_usefulness == 1) cloud_mask_1km(i,j,1) = ibset(cloud_mask_1km(i,j,1),0)
    
     if (processing_information(i,j)%cloud_mask_unobs_fov ==1) then
         cloud_mask_1km(i,j,1) = ibset(cloud_mask_1km(i,j,1),1)
     elseif (processing_information(i,j)%cloud_mask_unobs_fov == 2) then
         cloud_mask_1km(i,j,1) = ibset(cloud_mask_1km(i,j,1),2)
     elseif  (processing_information(i,j)%cloud_mask_unobs_fov == 3) then
         cloud_mask_1km(i,j,1) = ibset(cloud_mask_1km(i,j,1),1)
         cloud_mask_1km(i,j,1) = ibset(cloud_mask_1km(i,j,1),2)
     endif

     if(processing_information(i,j)%day_night == 1) cloud_mask_1km(i,j,1) = ibset(cloud_mask_1km(i,j,1),3)
     if(processing_information(i,j)%sun_glint == 1) cloud_mask_1km(i,j,1) = ibset(cloud_mask_1km(i,j,1),4)
     if(processing_information(i,j)%snow_ice == 1)  cloud_mask_1km(i,j,1) = ibset(cloud_mask_1km(i,j,1),5)

     if(processing_information(i,j)%surface_type == 1) then
        cloud_mask_1km(i,j,1) = ibset(cloud_mask_1km(i,j,1),6)
     elseif(processing_information(i,j)%surface_type == 2) then
        cloud_mask_1km(i,j,1) = ibset(cloud_mask_1km(i,j,1),7)
     elseif(processing_information(i,j)%surface_type == 3) then
        cloud_mask_1km(i,j,1) = ibset(cloud_mask_1km(i,j,1),6)
        cloud_mask_1km(i,j,1) = ibset(cloud_mask_1km(i,j,1),7)
     endif

!    Cloud Mask 1KM, Byte 2 -----------------------------------------------------------------------------------

     if(processing_information(i,j)%heavy_aerosol == 1) cloud_mask_1km(i,j,2) = ibset(cloud_mask_1km(i,j,2),0)
     if(processing_information(i,j)%thin_cirrus == 1)  cloud_mask_1km(i,j,2) = ibset(cloud_mask_1km(i,j,2),1)
     if(processing_information(i,j)%shadow == 1) cloud_mask_1km(i,j,2) = ibset(cloud_mask_1km(i,j,2),2)

!    Quality Assurance 1KM, byte 1 ----------------------------------------------------------------------------

     if(processing_information(i,j)%optical_thickness_general == 1)  &
           quality_assurance_1km(i,j,1) =  ibset(quality_assurance_1km(i,j,1),0)
     if(processing_information(i,j)%optical_thickness_confidence == 1) then
           quality_assurance_1km(i,j,1) =  ibset(quality_assurance_1km(i,j,1),1)
     elseif(processing_information(i,j)%optical_thickness_confidence == 2) then
           quality_assurance_1km(i,j,1) =  ibset(quality_assurance_1km(i,j,1),2)
     elseif(processing_information(i,j)%optical_thickness_confidence == 3) then
           quality_assurance_1km(i,j,1) =  ibset(quality_assurance_1km(i,j,1),1)
           quality_assurance_1km(i,j,1) =  ibset(quality_assurance_1km(i,j,1),2)
     endif

     if(processing_information(i,j)%optical_thickness_outofbounds== 1) then
           quality_assurance_1km(i,j,1) =  ibset(quality_assurance_1km(i,j,1),3)
     elseif(processing_information(i,j)%optical_thickness_outofbounds== 2) then
           quality_assurance_1km(i,j,1) =  ibset(quality_assurance_1km(i,j,1),4)
     elseif(processing_information(i,j)%optical_thickness_outofbounds == 3) then 
           quality_assurance_1km(i,j,1) =  ibset(quality_assurance_1km(i,j,1),3)
           quality_assurance_1km(i,j,1) =  ibset(quality_assurance_1km(i,j,1),4)
     endif

     if(processing_information(i,j)%effective_radius_general == 1) &
           quality_assurance_1km(i,j,1) =  ibset(quality_assurance_1km(i,j,1),5)
 
     if(processing_information(i,j)%effective_radius_confidence== 1) then
           quality_assurance_1km(i,j,1) =  ibset(quality_assurance_1km(i,j,1),6)
     elseif(processing_information(i,j)%effective_radius_confidence== 2) then
           quality_assurance_1km(i,j,1) =  ibset(quality_assurance_1km(i,j,1),7)
     elseif(processing_information(i,j)%effective_radius_confidence==3) then
           quality_assurance_1km(i,j,1) =  ibset(quality_assurance_1km(i,j,1),6)
           quality_assurance_1km(i,j,1) =  ibset(quality_assurance_1km(i,j,1),7)
     endif

!    Quality Assurance 1KM, byte 2 ----------------------------------------------------------------------------

     if(processing_information(i,j)%water_path_general== 1) &
           quality_assurance_1km(i,j,2) =  ibset(quality_assurance_1km(i,j,2),0)

     if(processing_information(i,j)%water_path_confidence== 1) then
           quality_assurance_1km(i,j,2) =  ibset(quality_assurance_1km(i,j,2),1)
     elseif(processing_information(i,j)%water_path_confidence== 2) then
           quality_assurance_1km(i,j,2) =  ibset(quality_assurance_1km(i,j,2),2)
     elseif(processing_information(i,j)%water_path_confidence==3) then
           quality_assurance_1km(i,j,2) =  ibset(quality_assurance_1km(i,j,2),1)
           quality_assurance_1km(i,j,2) =  ibset(quality_assurance_1km(i,j,2),2)
     endif

     if(     processing_information(i,j)%cloud_retrieval_proc_path_1621 == 1 ) then
          quality_assurance_1km(i,j,2) = ibset(quality_assurance_1km(i,j,2),3)
     elseif (processing_information(i,j)%cloud_retrieval_proc_path_1621 == 2 ) then
          quality_assurance_1km(i,j,2) = ibset(quality_assurance_1km(i,j,2),4)
     elseif (processing_information(i,j)%cloud_retrieval_proc_path_1621 == 3 ) then
          quality_assurance_1km(i,j,2) = ibset(quality_assurance_1km(i,j,2),3)
          quality_assurance_1km(i,j,2) = ibset(quality_assurance_1km(i,j,2),4)
     elseif (processing_information(i,j)%cloud_retrieval_proc_path_1621 == 4 ) then
          quality_assurance_1km(i,j,2) = ibset(quality_assurance_1km(i,j,2),5)
     endif

     if(processing_information(i,j)%retrieval_outcome_1621 == 1)  &
          quality_assurance_1km(i,j,2) = ibset(quality_assurance_1km(i,j,2),6)

!    Quality Assurance 1KM, byte 3 ----------------------------------------------------------------------------

     if(processing_information(i,j)%cloud_retrieval_processing_path ==1 ) then
          quality_assurance_1km(i,j,3) = ibset(quality_assurance_1km(i,j,3),0)  
     elseif (processing_information(i,j)%cloud_retrieval_processing_path == 2) then
          quality_assurance_1km(i,j,3) = ibset(quality_assurance_1km(i,j,3),1)  
     elseif (processing_information(i,j)%cloud_retrieval_processing_path == 3) then
          quality_assurance_1km(i,j,3) = ibset(quality_assurance_1km(i,j,3),0)  
          quality_assurance_1km(i,j,3) = ibset(quality_assurance_1km(i,j,3),1)   
     elseif (processing_information(i,j)%cloud_retrieval_processing_path == 4) then
          quality_assurance_1km(i,j,3) = ibset(quality_assurance_1km(i,j,3),2)    
     endif

     if(processing_information(i,j)%retrieval_outcome == 1)  &
          quality_assurance_1km(i,j,3) = ibset(quality_assurance_1km(i,j,3),3) 

     if(processing_information(i,j)%rayleigh_correction == 1) &
          quality_assurance_1km(i,j,3) = ibset(quality_assurance_1km(i,j,3),4) 

     if(processing_information(i,j)%atmospheric_correction == 1)  &
          quality_assurance_1km(i,j,3) = ibset(quality_assurance_1km(i,j,3),5) 

     if(processing_information(i,j)%band_used_for_optical_thickness ==1 ) then
          quality_assurance_1km(i,j,3) = ibset(quality_assurance_1km(i,j,3),6) 
     elseif(processing_information(i,j)%band_used_for_optical_thickness == 2 ) then
          quality_assurance_1km(i,j,3) = ibset(quality_assurance_1km(i,j,3),7)
     elseif(processing_information(i,j)%band_used_for_optical_thickness == 3 ) then
          quality_assurance_1km(i,j,3) = ibset(quality_assurance_1km(i,j,3),6)
          quality_assurance_1km(i,j,3) = ibset(quality_assurance_1km(i,j,3),7) 
     endif

!    Quality Assurance 1KM, byte 4 ----------------------------------------------------------------------------

     if (processing_information(i,j)%optical_thickness_1621_gen == 1) &
          quality_assurance_1km(i,j,4) = ibset(quality_assurance_1km(i,j,4),0)

     if (processing_information(i,j)%optical_thickness_1621_conf ==1)  then
          quality_assurance_1km(i,j,4) = ibset(quality_assurance_1km(i,j,4),1)
     elseif (processing_information(i,j)%optical_thickness_1621_conf ==2)  then
          quality_assurance_1km(i,j,4) = ibset(quality_assurance_1km(i,j,4),2)
     elseif (processing_information(i,j)%optical_thickness_1621_conf ==3)  then
          quality_assurance_1km(i,j,4) = ibset(quality_assurance_1km(i,j,4),1)
          quality_assurance_1km(i,j,4) = ibset(quality_assurance_1km(i,j,4),2)
     endif

     if (processing_information(i,j)%effective_radius_1621_gen== 1) &
          quality_assurance_1km(i,j,4) = ibset(quality_assurance_1km(i,j,4),3)

     if (processing_information(i,j)%effective_radius_1621_conf==1)  then
          quality_assurance_1km(i,j,4) = ibset(quality_assurance_1km(i,j,4),4)
     elseif (processing_information(i,j)%effective_radius_1621_conf==2)  then
          quality_assurance_1km(i,j,4) = ibset(quality_assurance_1km(i,j,4),5)
     elseif (processing_information(i,j)%effective_radius_1621_conf==3)  then
          quality_assurance_1km(i,j,4) = ibset(quality_assurance_1km(i,j,4),4)
          quality_assurance_1km(i,j,4) = ibset(quality_assurance_1km(i,j,4),5)
     endif

     ! CSR QA added by G.Wind 4.7.05
     if (processing_information(i,j)%CSR_flag == 1) &
          quality_assurance_1km(i,j,4) = ibset(quality_assurance_1km(i,j,4),6)
     if (processing_information(i,j)%CSR_flag == 2) &
          quality_assurance_1km(i,j,4) = ibset(quality_assurance_1km(i,j,4),7)
     if (processing_information(i,j)%CSR_flag == 3) then
          quality_assurance_1km(i,j,4) = ibset(quality_assurance_1km(i,j,4),6)
          quality_assurance_1km(i,j,4) = ibset(quality_assurance_1km(i,j,4),7)
     endif


!    Quality Assurance 1KM, byte 5 ----------------------------------------------------------------------------

     if (processing_information(i,j)%water_path_1621_gen == 1) &
          quality_assurance_1km(i,j,5) = ibset(quality_assurance_1km(i,j,5),0)

     if (processing_information(i,j)%water_path_1621_conf==1)  then
          quality_assurance_1km(i,j,5) = ibset(quality_assurance_1km(i,j,5),1)
     elseif (processing_information(i,j)%water_path_1621_conf==2)  then
          quality_assurance_1km(i,j,5) = ibset(quality_assurance_1km(i,j,5),2)
     elseif (processing_information(i,j)%water_path_1621_conf==3)  then
          quality_assurance_1km(i,j,5) = ibset(quality_assurance_1km(i,j,5),1)
          quality_assurance_1km(i,j,5) = ibset(quality_assurance_1km(i,j,5),2)
     endif

     if (processing_information(i,j)%multi_layer_cloud == 1) then
       quality_assurance_1km(i,j,5) = ibset(quality_assurance_1km(i,j,5),3)
     elseif(processing_information(i,j)%multi_layer_cloud == 2) then
       quality_assurance_1km(i,j,5) = ibset(quality_assurance_1km(i,j,5),4)
     elseif(processing_information(i,j)%multi_layer_cloud == 3) then
       quality_assurance_1km(i,j,5) = ibset(quality_assurance_1km(i,j,5),3)
       quality_assurance_1km(i,j,5) = ibset(quality_assurance_1km(i,j,5),4)
     elseif(processing_information(i,j)%multi_layer_cloud == 4) then
       quality_assurance_1km(i,j,5) = ibset(quality_assurance_1km(i,j,5),5)
     elseif(processing_information(i,j)%multi_layer_cloud == 5) then
       quality_assurance_1km(i,j,5) = ibset(quality_assurance_1km(i,j,5),3)
       quality_assurance_1km(i,j,5) = ibset(quality_assurance_1km(i,j,5),5)
     elseif(processing_information(i,j)%multi_layer_cloud == 6) then
       quality_assurance_1km(i,j,5) = ibset(quality_assurance_1km(i,j,5),4)
       quality_assurance_1km(i,j,5) = ibset(quality_assurance_1km(i,j,5),5)
     elseif(processing_information(i,j)%multi_layer_cloud == 7) then
       quality_assurance_1km(i,j,5) = ibset(quality_assurance_1km(i,j,5),3)
       quality_assurance_1km(i,j,5) = ibset(quality_assurance_1km(i,j,5),4)
       quality_assurance_1km(i,j,5) = ibset(quality_assurance_1km(i,j,5),5)
     endif 

     !This is a COPY of the retrieval_outcome bit for Level 3 compatibility
     if(processing_information(i,j)%retrieval_outcome == 1)  &
          quality_assurance_1km(i,j,5) = ibset(quality_assurance_1km(i,j,5),6)
  enddo
enddo   
  
end subroutine convert_binary_qa
subroutine writeqaarray_toolkit(mapi_filedata, &
                                   filename,      &
                                   currentscan,  &
                                   array,         &
                                   sdsname,       &
                                   status)

  use GeneralAuxType
  use nonscience_parameters

  implicit none
  include 'mapi.f90.inc'
  include 'PGS_MODIS_39500.f'

  integer(integer_onebyte), intent(in)           :: array(:,:,:)
  integer, intent(in)        :: mapi_filedata(:), currentscan
  character(*), intent (in)  :: filename, sdsname

  integer,      intent (out)        :: status 

  real(double)     :: scale_factor, add_offset
  integer          :: buffer_xsize, buffer_ysize, i,j, count, k, buffer_zsize
  integer          :: rank, dimsize(3), nms
  integer          :: localstart(3), localedge(3)
  character(len=MAX_SDS_NAME_LEN)     :: groupname, data_type, attribute
  integer(integer_onebyte),allocatable :: outputbuffer_onebyte(:)

  buffer_xsize = size(array,1)
  buffer_ysize = size(array,2)
  buffer_zsize = size(array,3)

  groupname =  ' '
  rank = 3
  status = gmardm(mapi_filedata, sdsname, groupname, data_type, rank, dimsize)
  localstart(1) = 0
  localstart(2) = 0
  localstart(3) = (currentscan-1)*buffer_ysize
  localedge(1) = buffer_zsize
  localedge(2) = buffer_xsize
  localedge(3) = buffer_ysize

  status = failure

  if (data_type(1:9) == 'INTEGER*1') then
  allocate(outputbuffer_onebyte(buffer_xsize*buffer_ysize*buffer_zsize))
  status = success
  count = 0
  do j = 1, buffer_ysize
     do i = 1, buffer_xsize
       do k = 1, buffer_zsize
         count = count + 1
         outputbuffer_onebyte(count) = array(i,j,k)
       enddo
     enddo
  enddo

  groupname = ' '
  status = pmar (mapi_filedata, &
                 sdsname,       &
                 groupname,     &
                 localstart,    &
                 localedge,     &
                 outputbuffer_onebyte)
  deallocate(outputbuffer_onebyte)
  else
   status = failure
  endif

end subroutine writeqaarray_toolkit
subroutine writefloatarray_toolkit(mapi_filedata, &
                                   filename,      &
                                   currentscan,  &
                                   array,         &
                                   sdsname,       &
                                   status)

  use GeneralAuxType
  use nonscience_parameters

  implicit none
  include 'mapi.f90.inc'
  include 'PGS_MODIS_39500.f'

  real, intent(in)           :: array(:,:)
  integer, intent(in)        :: mapi_filedata(:), currentscan
  character(*), intent (in)  :: filename, sdsname

  integer,      intent (out)        :: status 

  real(double)     :: scale_factor, add_offset
  integer          :: buffer_xsize, buffer_ysize, i,j, count
  integer          :: rank, dimsize(2), nms
  integer          :: localstart(3), localedge(3)
  character(len=MAX_SDS_NAME_LEN)     :: groupname, data_type, attribute
  integer(integer_twobyte),allocatable :: outputbuffer_twobyte(:)
  integer(integer_onebyte),allocatable :: outputbuffer_onebyte(:)

  buffer_xsize = size(array,1)
  buffer_ysize = size(array,2)
  rank = 2

  data_type = 'REAL*8'
  nms = 1
  attribute = 'scale_factor'
  groupname =  ' '
  status = gmarin( mapi_filedata, sdsname, groupname, attribute, data_type ,nms, scale_factor )

  data_type = 'REAL*8'
  nms = 1
  attribute = 'add_offset'
  groupname =  ' '
  status = gmarin( mapi_filedata, sdsname, groupname, attribute, data_type ,nms, add_offset)

  status = gmardm(mapi_filedata, sdsname, groupname , data_type, rank, dimsize)
  localstart(1) = 0
  localstart(2) = (currentscan-1)*buffer_ysize
  localstart(3) = 0
  localedge(1) = buffer_xsize
  localedge(2) = buffer_ysize
  localedge(3) = 1

  status = failure


  if (data_type(1:9) == 'INTEGER*2') then
  allocate(outputbuffer_twobyte(buffer_xsize*buffer_ysize))
  status = success
  count = 0
  do j = 1, buffer_ysize
     do i = 1, buffer_xsize
       count = count + 1
       if (array(i,j) < 0. ) then
          outputbuffer_twobyte(count) = -9999
       else
          outputbuffer_twobyte(count) = nint(array(i,j)/scale_factor+add_offset)
       endif
     enddo
  enddo

  localstart(1) = 0
  localstart(2) = (currentscan-1)*buffer_ysize
  localstart(3) = 0
  localedge(1) = buffer_xsize
  localedge(2) = buffer_ysize
  localedge(3) = 1

  groupname = ' '
  status = pmar (mapi_filedata, &
                  sdsname,       &
                  groupname,     &
                  localstart,    &
                  localedge,     &
                  outputbuffer_twobyte)
  deallocate(outputbuffer_twobyte)
  endif

  if (data_type(1:9) == 'INTEGER*1') then
  allocate(outputbuffer_onebyte(buffer_xsize*buffer_ysize))
  status = success
  count = 0
  do j = 1, buffer_ysize
     do i = 1, buffer_xsize
       count = count + 1
       if (array(i,j) < 0. ) then
          outputbuffer_onebyte(count) = -99
       else
          outputbuffer_onebyte(count) = nint(array(i,j)/scale_factor+add_offset)
       endif
     enddo
  enddo

  localstart(1) = 0
  localstart(2) = (currentscan-1)*buffer_ysize
  localstart(3) = 0
  localedge(1) = buffer_xsize
  localedge(2) = buffer_ysize
  localedge(3) = 1

  groupname = ' '
  status = pmar (mapi_filedata, &
                  sdsname,       &
                  groupname,     &
                  localstart,    &
                  localedge,     &
                  outputbuffer_onebyte)
  deallocate(outputbuffer_onebyte)
  endif
 
end subroutine writefloatarray_toolkit
subroutine writefloat3darray_toolkit(mapi_filedata, &
                                   filename,      &
                                   currentscan,  &
                                   array,         &
                                   sdsname,       &
                                   status)

  use GeneralAuxType
  use nonscience_parameters

  implicit none
  include 'mapi.f90.inc'
  include 'PGS_MODIS_39500.f'

  real, intent(in)           :: array(:,:,:)
  integer, intent(in)        :: mapi_filedata(:), currentscan
  character(*), intent (in)  :: filename, sdsname

  integer,      intent (out)        :: status 

  real(double)     :: scale_factor, add_offset
  integer          :: buffer_xsize, buffer_ysize, i,j, count, buffer_zsize,k
  integer          :: rank, dimsize(20), nms
  integer          :: localstart(3), localedge(3)
  character(len=MAX_SDS_NAME_LEN)     :: groupname, data_type, attribute
  integer(integer_twobyte),allocatable :: outputbuffer_twobyte(:)

  
  buffer_xsize = size(array,1)
  buffer_ysize = size(array,2)
  buffer_zsize = size(array,3)
  rank =  3

  data_type = 'REAL*8'
  nms = 1
  attribute = 'scale_factor'
  groupname =  ' '
  status = gmarin( mapi_filedata, sdsname, groupname, attribute, data_type ,nms, scale_factor )
  data_type = 'REAL*8'
  nms = 1
  attribute = 'add_offset'
  groupname =  ' '
  status = gmarin( mapi_filedata, sdsname, groupname, attribute, data_type ,nms, add_offset)

  !data_type = ' '
  !status = gmardm(mapi_filedata, sdsname, groupname , data_type, rank, dimsize)
  localstart(1) = 0
  localstart(2) =  (currentscan-1)*buffer_ysize
  localstart(3) = 0
  localedge(1) = buffer_xsize
  localedge(2) = buffer_ysize
  localedge(3) = buffer_zsize

  status = failure


  allocate(outputbuffer_twobyte(buffer_xsize*buffer_ysize*buffer_zsize))
  status = success
  count = 0
  do k = 1, buffer_zsize
  do j = 1, buffer_ysize
     do i = 1, buffer_xsize
       count = count + 1
       if (array(i,j,k) == -999. ) then
          outputbuffer_twobyte(count) = -9999
       else
          outputbuffer_twobyte(count) = nint(array(i,j,k)/scale_factor+add_offset)
       endif
     enddo
  enddo
  enddo

  groupname = ' '
  status = pmar (mapi_filedata, &
                  sdsname,       &
                  groupname,     &
                  localstart,    &
                  localedge,     &
                  outputbuffer_twobyte)
  deallocate(outputbuffer_twobyte)

end subroutine writefloat3darray_toolkit
subroutine getfloatarray_toolkit(mapi_fileindex, &
                                 sds_name,       &
                                 start, stride, edge, &
                                 floatarray,     &
                                 status)
   use nonscience_parameters
   use hdf

   implicit none

   include 'mapi.f90.inc'
   include 'PGS_PC.f'
   include 'PGS_MODIS_39500.f'

   integer, intent(in)       :: mapi_fileindex(:)
   character*(*), intent(in) :: sds_name
   integer, intent(in)       :: start(2), stride(2), edge(2)

   real, intent(out)     :: floatarray(:,:)
   integer, intent(out)  :: status

   integer               :: rank, dim_size(2), return
   character(len=MAX_SDS_NAME_LEN)          :: groupname, data_type

   rank = 2
   groupname = ' '
   status = success

   if( size(floatarray,1) /= edge(1) .or. &
       size(floatarray,2) /= edge(2)) then
       status = failure
       call local_message_handler('SDS read, array size mismatch. Contact SDST', &
                                  status, &
                                  'getfloatarray_toolkit')
   endif

!   return = GMARDM(mapi_fileindex, sds_name, groupname , data_type, Rank, Dim_Size)
!   if (return /= mapiok) then
!     status = failure
!     call local_message_handler('sds info read (gdmardm) failed for array = '//sds_name// &
!                                '[OPERATOR ACTION: Check system resources]', &
!                                status, &
!                                'getfloatarray_toolkit')
!   end if

   return = GMAR(mapi_fileindex,sds_name,groupname,start,edge,floatarray)
   if (return /= mapiok) then
     status = failure
     call local_message_handler('sds read (gmar) failed for array '//sds_name// &
                                '[OPERATOR ACTION: Check system resources]', &
                                status, &
                                'getfloatarray_toolkit')
     floatarray(:,:)  = fillvalue_real
   endif
   
end subroutine getfloatarray_toolkit 

subroutine getintegerarray_toolkit(mapi_fileindex, &
                                 sds_name,       &
                                 start, stride, edge, &
                                 useoffset, & 
                                 floatarray,     &
                                 status)
   use nonscience_parameters
   use hdf
   use GeneralAuxType

   implicit none

   include 'mapi.f90.inc'
   include 'PGS_PC.f'
   include 'PGS_MODIS_39500.f'

   integer, intent(in)       :: mapi_fileindex(:)
   character*(*), intent(in) :: sds_name
   integer, intent(in)       :: start(2), stride(2), edge(2)
   logical, intent(in)       :: useoffset

   real, intent(out)     :: floatarray(:,:)
   integer, intent(out)  :: status

   real(double)          ::  scale_factor, add_offset
   integer(integer_twobyte), allocatable :: integerarray(:,:)
   integer               :: rank, dim_size(2), return, nms
   character(len=MAX_SDS_NAME_LEN)          :: groupname, data_type,attribute
   integer(integer_twobyte) :: fillvalue

   rank = 2
   groupname = ' '
   status = success

   if( size(floatarray,1) /= edge(1) .or. &
       size(floatarray,2) /= edge(2)) then
       status = failure
       call local_message_handler('SDS read, array size mismatch. Contact SDST', &
                                  status, &
                                  'getfloatarray_toolkit')
   endif

   allocate(integerarray(size(floatarray,1), size(floatarray,2)))

   return = gmar(mapi_fileindex,sds_name,groupname,start,edge,integerarray)

   if (return /= mapiok) then
     status = failure
     call local_message_handler('sds read (gmar) failed for array '//sds_name// &
                                '[OPERATOR ACTION: Check system resources]', &
                                status, &
                                'getfloatarray_toolkit')
     floatarray(:,:)  = fillvalue_real
  else

!  we have an array read in, grab the scale factor, offset and fillvalue 
   data_type = 'REAL*8'
   nms = 1
   attribute = 'scale_factor'
   groupname =  ' ' 
   return = gmarin( mapi_fileindex, sds_name, groupname, attribute, data_type ,nms, scale_factor )
   
   if(useoffset)then
     data_type = 'REAL*8'
     nms = 1
     attribute = 'add_offset'
     groupname =  ' '
     return = gmarin( mapi_fileindex, sds_name, groupname, attribute, data_type ,nms, add_offset)
   else
     add_offset = 0.
   endif

   data_type = 'INTEGER*2'
   nms = 1
   attribute = '_FillValue'
   groupname =  ' '
   return = gmarin( mapi_fileindex, sds_name, groupname, attribute, data_type ,nms, fillvalue)

   where(integerarray /= fillvalue)
     floatarray = (real(integerarray)-add_offset)*scale_factor
   elsewhere
     floatarray = fillvalue_real
   endwhere 
  endif 
  deallocate(integerarray)   
end subroutine getintegerarray_toolkit

end module modis_io_module
