module modis_numerical_module

implicit none

private

public :: bisectionsearch, linearinterpolation, linearinterpolation_3d, asymptotic_calcu

contains


subroutine bisectionsearch(xx,x,jlo,jhi)
 
 use GeneralAuxType

IMPLICIT NONE
INTEGER, INTENT(INOUT) :: jlo, jhi
REAL(single), INTENT(IN) :: x
REAL(single), DIMENSION(:), INTENT(IN) :: xx
INTEGER :: n,inc,jm
LOGICAL :: ascnd
n=size(xx)
ascnd = (xx(n) >= xx(1))
if (jlo <= 0 .or. jlo > n) then
   jlo=0
   jhi=n+1
else
   inc=1
   if (x >= xx(jlo) .eqv. ascnd) then
      do
         jhi=jlo+inc
         if (jhi > n) then
            jhi=n+1
            exit
         else
            if (x < xx(jhi) .eqv. ascnd) exit
            jlo=jhi
            inc=inc+inc
         end if
      end do
   else
      jhi=jlo
      do
         jlo=jhi-inc
         if (jlo < 1) then
            jlo=0
            exit
         else
            if (x >= xx(jlo) .eqv. ascnd) exit
            jhi=jlo
            inc=inc+inc
         end if
      end do
   end if
end if
do
   if (jhi-jlo <= 1) then
      if (x == xx(n)) jlo=n-1
      if (x == xx(1)) jlo=1
      exit
   else
      jm=(jhi+jlo)/2
      if (x >= xx(jm) .eqv. ascnd) then
         jlo=jm
      else
         jhi=jm
      end if
   end if
end do

! G.Wind 12.16.05 inserted logic to make sure that the values can not go outside
! the valid array bounds
if (jlo < 1) jlo = 1
if (jlo > n) jlo = n
if (jhi < 1) jhi = 1
if (jhi > n) jhi = n


end subroutine bisectionsearch


real function linearinterpolation(X,Y,XX)


!       XX    R     Interpolation POINT
!       X   R(NN)   INDEPENDENT VARIABLE
!       Y   R(NN)   DEPENDENT   VARIABLE
!
!       Written by      
!        Mark A Gray
!        SM&A SSG .
!        Code 913, NASA/GSFC
!        Greenbelt, MD 20771

   use GeneralAuxType
   
   implicit none

   real(single), intent(in)  ::   X(2), xx 
   real(single), intent(in)  ::   y(2)

   if (x(1) == x(2)) then
     LinearInterpolation = y(1)
     return
   elseif(x(1) == xx) then 
     LinearInterpolation = y(1)
     return
   elseif (x(2) == xx) then 
     LinearInterpolation = y(2)
     return
   else 
     LinearInterpolation = y(1) + ( ( xx - x(1) ) / ( x(2) - x(1)) ) * ( y(2) -y(1) )
   endif

end function linearinterpolation
real function LinearInterpolation_3d(index_theta01,index_theta02,  &
                                       index_theta1, index_theta2,  &
                                       index_phi1,   index_phi2,&
                                       theta,theta0,phi,&
                                       theta_grid,  &
                                       theta0_grid,&
                                       phi_grid,&
                                       debug, &
                                       reflectance)
       
       
!Description: 
!       Performa a linear interpolation within a 3d grid for a defined
!       point bounded by defined points.
! 
!Input parameters:
!
!Output parameters:
!
!Revision history: 
!
!Team-unique header: 
!
!References and credits:
!    written by; Mark Gray
!                  
   use GeneralAuxType
   
   implicit none

   integer, intent(in)  :: index_theta1, index_theta2, index_phi1,index_phi2, &
                           index_theta01, index_theta02
       
   real(single), intent(in)     :: reflectance(:,:,:)
   real(single), intent(in)     :: theta_grid(:), &
                             theta0_grid(:),phi_grid(:),theta,theta0,phi
   
   logical, intent(in)    :: debug
      
   real(single)                 :: corner1, corner2, corner3, corner4,  &
                                   corner5, corner6, corner7, corner8
   real(single)                 :: volume1, volume2, volume3, volume4,  &
                                   volume5, volume6, volume7, volume8
   real(single)                 :: num_1, num_2    
   real(single)                 :: theta0_val_1, theta0_val_2, theta_val_1, theta_val_2,phi_val_1, phi_val_2


   theta0_val_1 = theta0_grid(index_theta01)-theta0
   theta0_val_2 = theta0_grid(index_theta02)-theta0

   theta_val_1 = theta_grid(index_theta1)-theta
   theta_val_2 = theta_grid(index_theta2)-theta
 
   phi_val_1 = phi_grid(index_phi1)-phi
   phi_val_2 = phi_grid(index_phi2)-phi

   volume2= abs(theta0_val_1 * theta_val_1 * phi_val_2)
   volume3= abs(theta0_val_1 * theta_val_2 * phi_val_2)
   volume4= abs(theta0_val_1 * theta_val_2 * phi_val_1)
   volume5= abs(theta0_val_2 * theta_val_1 * phi_val_1)
   volume6= abs(theta0_val_2 * theta_val_1 * phi_val_2)
   volume7= abs(theta0_val_2 * theta_val_2 * phi_val_2)
   volume8= abs(theta0_val_2 * theta_val_2 * phi_val_1)
   volume1= abs(theta0_val_1 * theta_val_1 * phi_val_1)

   num_1 =     reflectance(index_theta01,index_theta1,index_phi1) * volume7 + &
               reflectance(index_theta01,index_theta1,index_phi2) * volume8 +  &
               reflectance(index_theta02,index_theta1,index_phi2) * volume4 +  &
               reflectance(index_theta02,index_theta1,index_phi1) * volume3 +  &
               reflectance(index_theta01,index_theta2,index_phi1) * volume6 +  &
               reflectance(index_theta01,index_theta2,index_phi2) * volume5 +  &
               reflectance(index_theta02,index_theta2,index_phi2) * volume1 +  &
               reflectance(index_theta02,index_theta2,index_phi1) * volume2 

   num_2 =     volume1 + volume2 +  &
               volume3 + volume4 +  &
               volume5 + volume6 +  &
               volume7 + volume8   

   LinearInterpolation_3d = num_1/num_2

end function LinearInterpolation_3d

subroutine asymptotic_calcu(tc,g,w,ak,el,em,en, &
                            sfr, rfi, fk1,fk0,ag,rfg)
  use GeneralAuxType
  implicit none

  real(single), intent(in)    :: tc,ag, rfi
  real(single), intent(in)    :: g,w,ak,el,em,en,sfr, fk1, fk0
  real(single), intent(out)   :: rfg

  real(single)                :: q1, rf0, ft1, sfr1, x, fkk, exc, ft0
  parameter (q1=0.712)

  fkk = fk1*fk0

  if (w > 0.999991) then

!    conservative case
     x = 4.0/3.0/ ((1.-g)*tc+2.0*q1)
     ft1 = fk1*x                          !this is eq.(17)
     ft0 = fk0*x                          !this is eq.(17)
     sfr1 = 1.0- x                        !this is eq.(19)
     rf0 = rfi - fkk*x                    !this is eq.(4)
  else

!    non-conservative case
     exc = ak*tc
     if (exc <= 50.0) then
        exc = exp(-exc)
     else
        exc = 0.
     end if

     x = em*exc/ (1.0 - (el*exc)**2)
     ft1 = en*fk1*x                       !this is eq.(20)
     ft0 = en*fk0*x                       !this is eq.(20)
     sfr1 = sfr - el*en**2*exc*x          !this is eq.(22)
     rf0 = rfi - el*fkk*exc*x             !this is eq.(6)
  end if

  rfg = rf0 + ft1*ft0*ag/ (1.-ag*sfr1)    !this is eq.(11)
end subroutine asymptotic_calcu
end module modis_numerical_module
