module modis_science_module

implicit none

private

public:: scienceinterface

contains

subroutine scienceinterface(threshold_solar_zenith,      &
                            threshold_sensor_zenith,     &
                            threshold_relative_azimuth,  &
                            debug,                       &
                            status)

   use modis_cloudstructure, only:cloudphase
   use core_arrays
   use science_parameters
   use corescience_module
   use multi_layer_clouds
   use clear_sky_restoral
   use mod06_run_settings
   use nonscience_parameters

   implicit none

   logical, intent(in) :: debug
   real,    intent(in) :: threshold_solar_zenith,      &
                          threshold_sensor_zenith,     &
                          threshold_relative_azimuth

   integer, intent(out) :: status


   integer             :: xdimension, ydimension,i,j, count_interpolations, &
                          retrievalcount
   integer             :: retrieval_failcount, library_failcount, cloud_failcount, atmoscorr_failcount
   integer             :: cloudstatus, retrievalstatus, librarystatus, atmoscorrstatus,cloudiness_degree_250m
   real                :: lastinterp_solar_zenith,     &
                          lastinterp_sensor_zenith,    &
                          lastinterp_relative_azimuth, &
                          diff_solar_zenith,           &
                          diff_sensor_zenith,          &
                          diff_relative_azimuth
  
   logical :: ldebug, sunglint_dust_test, lowvariability_confidence_test
   status = 0
 
   xdimension  = size(band_measurements,1)
   ydimension  = size(band_measurements,3)
   
   lastinterp_solar_zenith     = 0.
   lastinterp_sensor_zenith    = 0.
   lastinterp_relative_azimuth = 0. 
   count_interpolations   = 0
   retrievalcount = 0
   retrieval_failcount = 0
   library_failcount = 0
   cloud_failcount  = 0
   atmoscorr_failcount = 0
 
   call init_science_arrays


   
   do i = 1, xdimension
     do j = 1, ydimension
     
!  initialize each cloud/phase cloudsummary variable to ensure correct values get
!    fed into Cloud_Phase_Optical_Properties SDS (particularly when solar zenith
!    angle of pixel greater that processing threshold    GTA 5-25-2005

   cloudsummary(i,j)%cloudmask_determined = .false.
   cloudsummary(i,j)%cloudobserved = .false.
   cloudsummary(i,j)%watercloud = .false.
   cloudsummary(i,j)%icecloud = .false.
   cloudsummary(i,j)%unknowncloud = .false.
   cloudsummary(i,j)%mixedcloud = .false.


! we shouldn't run CSR if the retrieval will not execute due to low sun angles. 
       if ( solar_zenith_angle(i,j) <= solar_zenith_threshold ) then 

          if ( debugPxlYcoord_1to20L0inclusiv /= 0 &
               .and. i == debugPxlXcoord_1to1354inclusiv &
               .and. ((iteration - 1) * 10 + j) == debugPxlYcoord_1to20L0inclusiv ) then
             debugPRN = .true.
          else
             debugPRN = .false.
          endif
!         added surface albedo to subroutine arguments - GTA 01/26/05

!     write(*,*) i,j



         call clouddecision(platform_name,              &
                            cloudmask(i,j),             &
                            band_measurements(i,:,j),   &
                            cloud_top_temperature(i,j), &
                            surface_albedo(i,j,:),     &
                            ir_cloudphase(i,j),         &
                            cloudsummary(i,j),          &
                            ldebug,                      &
                            cloudstatus)  

         if (cloudstatus /= 0) cloud_failcount = cloud_failcount+1


!         if (i>=3 .and. i<=(xdimension-2) .and. j>=1 .and. j<=(ydimension-1)) then 
!
!           call cloudiness_test (cloudmask(i,j),           &
!                 cloudsummary(i,j),        &
!                 band_measurements(i,:,j), &
!                 band_measurements(i-2:i+2,band_0086,j:j+1), &
!                 ir_cloudphase(i,j),       &
!                 sunglint_dust_test,       &
!                 cloudiness_degree_250m, &
!                 lowvariability_confidence_test, &
!                 CSR_flag_array(i,j))

! G. Wind 2.8.05: fixed a typo in band measurement selection instead of i:i+1, the original text
! said i:1+1, which is obviously wrong. Another problem fixed is that band_measurements array itself
! was being passed in as incorrectly as it could possibly be. I can't even describe the original code

!  Riedi clear sky restoral algorithm test modfied by SEP and JR 5/17/05 (for alternate two-day 
!   science test.  Further modified 5/19/05 corrected run of alternate two-day scince test
!  After much testing will be employed as primary CSR algorithm 6/7/05 GTA

         if (i>=3 .and. i<=(xdimension-2) .and. j>=1 .and. j<=(ydimension-1)) then 
            call cloudiness_test (cloudmask(i,j),           &
                                           cloudsummary(i,j),        &
                                           band_measurements(i,:,j),  &
                                           band_measurements(i-2:i+2,:,j:j+1), &
                                           ir_cloudphase(i,j),       &
                                           sunglint_dust_test,       &
                                           lowvariability_confidence_test, &
                                           CSR_flag_array(i,j))
 

!        sunglint_dust_test tells us where 
!          - we've run a scene as cloud
!          - we now think the scene was actually dust or sunglint
!
!        cloudiness_degree_250m tells us how many 250m resolution scenes under the 1km scene
!        were identified as cloudy
         endif

        if (i>=3 .and. i<=(xdimension-2) .and. j==ydimension) then 
            call cloudiness_test (cloudmask(i,j),           &
                                           cloudsummary(i,j),        &
                                           band_measurements(i,:,j),  &
                                           band_measurements(i-2:i+2,:,j-1:j), &
                                           ir_cloudphase(i,j),       &
                                           sunglint_dust_test,       &
                                           lowvariability_confidence_test, &
                                           CSR_flag_array(i,j))
         endif


! Riedi modifications : Mon Apr 11 20:26:19 CEST 2005
!        lowvariability_confidence_test, tells us where 
!          - we've run a scene as cloud
!          - we now think the scene might not be a cloud but we're not certain
!            so we decide to process anyway and only lower the quality index 
!            -> this is done by setting the cloudsuspicious flag in cloudsummary
!               This flag will be then used within modis_io_module in the set_quality_data
         if( lowvariability_confidence_test ) then
           cloudsummary(i,j)%cloudsuspicious = .TRUE.
         endif 
! End of Riedi Modifs.

! example of using the debug variable
!         if (debugPRN) then 
!            print*, optical_thickness(i,j)
!         endif

       endif
     enddo
   enddo


! Now that we've done the clear sky restoral, we remove the edges of the clouds (ED)
   call remove_edge_scenes(cloudsummary, &
                           CSR_flag_array, &
                           xdimension, ydimension,  &
                           status)


!   reset cloudsummary variables for pixels "cleared" by CSR or ED, this step necessary
!     so that "cleared" pixels in cloud optical properties SDS get set to clear, and  
!     so that pertainent QA can be properly identified when set   GTA 6/7/05


     where(CSR_flag_array > 0) 
       cloudsummary%cloudobserved = .false.
       cloudsummary%watercloud = .false.
       cloudsummary%icecloud = .false.
       cloudsummary%unknowncloud = .false.
       cloudsummary%mixedcloud = .false.
       endwhere
                                  



! now we need to compute the inventory metadata that relates to the cloudiness percentage
! water cloud percentage and ice cloud percentage. We need to aggregate the little suckers

   do i=1, xdimension
      do j = 1, ydimension
! first of all we need to make sure that we have a cloud
         if (solar_zenith_angle(i,j) <= solar_zenith_threshold .and. &
              cloudsummary(i,j)%cloudobserved) then 
            IM_cloudy_count = IM_cloudy_count + 1
            
! now that we're sure, we can count the ice and water cloud pixels
            if (cloudsummary(i,j)%watercloud) then 
               IM_water_cloud_count = IM_water_cloud_count + 1
            endif
            if (cloudsummary(i,j)%icecloud) then 
               IM_ice_cloud_count = IM_ice_cloud_count + 1
            endif

          endif

      end do
   end do



   do i = 1, xdimension

     do j = 1, ydimension

       if (solar_zenith_angle(i,j) <= solar_zenith_threshold .and. cloudsummary(i,j)%cloudobserved ) then


          if ( debugPxlYcoord_1to20L0inclusiv /= 0 &
               .and. i == debugPxlXcoord_1to1354inclusiv &
               .and. ((iteration - 1) * 10 + j) == debugPxlYcoord_1to20L0inclusiv ) then
             debugPRN = .true.
          else
             debugPRN = .false.
          endif


         diff_solar_zenith = abs(solar_zenith_angle(i,j) - lastinterp_solar_zenith)
         diff_sensor_zenith= abs(sensor_zenith_angle(i,j)- lastinterp_sensor_zenith)
         diff_relative_azimuth= abs(relative_azimuth_angle(i,j)-lastinterp_relative_azimuth)

         if (diff_solar_zenith     > threshold_solar_zenith  .or. &
            diff_sensor_zenith    > threshold_sensor_zenith .or. &
            diff_relative_azimuth > threshold_relative_azimuth .and. &
            cloudsummary(i,j)%cloudobserved ) then 
            call libraryinterpolate(solar_zenith_angle(i,j),    &
                                  sensor_zenith_angle(i,j),   &
                                  relative_azimuth_angle(i,j),&
                                  debug, &
                                  librarystatus)
            if (librarystatus == 0 ) count_interpolations = count_interpolations+1
            if (librarystatus /= 0)  library_failcount = library_failcount+1

            lastinterp_solar_zenith = solar_zenith_angle(i,j)
            lastinterp_sensor_zenith = sensor_zenith_angle(i,j)
            lastinterp_relative_azimuth = relative_azimuth_angle(i,j)
         endif

         if (cloudsummary(i,j)%cloudobserved) then
           call atmospheric_correction(i,j,debug,atmoscorrstatus)
           if (atmoscorrstatus /= 0 ) atmoscorr_failcount = atmoscorr_failcount+1

!          We want a clearly successful calculation from
!                1. Atmospheric correction
!                2. Cloud Decision
!                3. library interpolation before we permit a retrieval

           if (atmoscorrstatus == 0 .and. cloudstatus == 0 .and. librarystatus == 0 ) then 
             call corescience (i, j, cloudsummary(i,j), debug, retrievalstatus)
           else
             retrievalstatus = 1
           endif
         else
           retrievalstatus = 1
         endif

         if (retrievalstatus == 0 ) retrievalcount = retrievalcount+1
         if (retrieval_failcount /= 0) retrieval_failcount = retrieval_failcount +1


         if (optical_thickness(i,j) > 0.) then
            IM_successful_retrieval_count = IM_successful_retrieval_count + 1
         endif


       else
         ! failure before retrieval 
         retrievalstatus = 1
         call assign_retrieval_error(i,j)
         retrieval_failcount = retrieval_failcount + 1
       endif


     enddo
   enddo




end subroutine scienceinterface
subroutine clouddecision(platform_name,  &
                         cloudmask,      &
                         measurements,         &
                         cloud_top_temp,       &
                         surfaceAlbedo,        &
                         ir_phase,cloudsummary,&
                         debug,status)


   ! input argument is the full cloudmask and level 1B reflectances
   ! output argument is the cloudsummary.  This reports on the underlying
   ! ecosystem and the cloud phase type

   use modis_cloudstructure
   use mod06_run_settings

   implicit none
   
   character*(*), intent(in)         :: platform_name
   logical, intent(in)               :: debug  
   type(cloudmask_type),intent(in)   :: cloudmask
   type(processflag)  , intent(out)  :: cloudsummary 
   type(cloudphase), intent(out)     :: ir_phase
   real, intent(in)                  :: measurements(:), cloud_top_temp,  surfaceAlbedo(:)
   integer, intent(out)              :: status

   real, external                    :: modis_bright
   real                              :: band_8_11_difference, xdimension,  &
                                        threshold_1, threshold_2, threshold_3, &
                                        threshold_4, bright_temp_11, bright_temp_85, &
                                        water_particle_threshold, ice_particle_threshold
   logical                           :: decision_made
   integer                           :: nonabsorbing_band
   integer                           :: cntr1,cntr2
   cntr1 = 0
   cntr2 = 0


   decision_made = .false. 
   status = 0 

   !do the decision tree
   !initialize outputs, just to be sure 
   cloudsummary%cloudmask_determined = .true.
   cloudsummary%cloudobserved = .false.
   cloudsummary%watercloud = .false.
   cloudsummary%icecloud = .false.
   cloudsummary%unknowncloud = .false.
   cloudsummary%mixedcloud = .false.
   cloudsummary%cloudsuspicious = .false.


   !check if we have a cloud


   if (.not. cloudmask%cloudmask_determined) then
      cloudsummary%cloudmask_determined = .false.
      decision_made = .true.
   endif
   
   if (cloudmask%night) then
      decision_made = .true.
   endif

! commented out since no longer part of decision tree  GTA 2/5/05 
!   if (cloudmask%shadow) then
!      decision_made = .true.
!   endif


!  In the following series of logical tests there is much repetition of logic
!  across the 5 different ecosystem types.  While the extra code is redundant 
!  it is better to keep the ecosystem logic seperate.  This will facilitate 
!  adding ecosystem dependant tests at some later point. - mag
!  note for future developers: choose readability over clever coding here,
!  other people have to be able to *read* this.

!  SNOW ICE SURFACE

   if (cloudmask%snowice_surface .and. .not. decision_made) then

      if (cloudmask%land_surface) cloudsummary%land_surface = .true.
      if (cloudmask%water_surface) cloudsummary%ocean_surface = .true.
      if (cloudmask%desert_surface) cloudsummary%desert_surface = .true.
      if (cloudmask%coastal_surface) cloudsummary%coastal_surface = .true.


      if (cloudmask%confident_cloudy .or. cloudmask%probablyclear_66) then
         cloudsummary%cloudobserved = .true.

!        Old version of MODIS decision tree did a Heavy aerosol test here, but deleted in later versions

!        Following test commented out - not in collection 4 or decision tree flowchart - GTA 01/28/05
!         if  (.not. cloudmask%test_high_138 .and. cloudmask%test_irdiff .and.  &
!              cloudmask%applied_highcloud138 .and. cloudmask%applied_irtempdifference .and. &
!              .not. decision_made) then
!            cloudsummary%watercloud = .true.
!            decision_made = .true.
!         endif

         if (cloudmask%test_39_11 .and. .not. cloudmask%test_high_138  .and.  &
             cloudmask%applied_39_11 .and. cloudmask%applied_highcloud138 .and. & 
             .not. decision_made) then
            cloudsummary%watercloud = .true.
            decision_made = .true.
         endif

         if (cloudmask%test_high_67 .and. cloudmask%applied_highcloud67 .and.  &
             .not. decision_made) then
            cloudsummary%icecloud = .true.
            decision_made = .true.
         endif

!         Drop 1.38 test over ice surface from decision tree - GTA 4/08/05
!         if (cloudmask%test_high_138 .and. cloudmask%applied_highcloud138 .and. &
!             .not. decision_made) then
!            cloudsummary%icecloud = .true.
!            decision_made = .true.
!         endif

         if( .not. decision_made) then
            cloudsummary%unknowncloud= .true.
            decision_made = .true.
         endif

      else
            cloudsummary%cloudobserved = .false.
            decision_made = .true.
!         endif
      endif
   endif


!  LAND SURFACE

   if (cloudmask%land_surface .and.  .not. decision_made) then
     if (cloudsummary%ocean_surface) then
       cloudsummary%coastal_surface= .true.
       cloudsummary%ocean_surface = .false.
       endif

!      cloudsummary%land_surface= .true.


      if (cloudmask%confident_cloudy .or. cloudmask%probablyclear_66) then
         cloudsummary%cloudobserved = .true.

!        MODIS decision tree does a Heavy aerosol test here - test deleted from this code

!        Following test commented out - not in collection 4 or decision tree flowchart - GTA 01/28/05
!         if  (.not. cloudmask%test_high_138 .and. cloudmask%test_irdiff .and.  &
!              cloudmask%applied_highcloud138 .and. cloudmask%applied_irtempdifference .and. &
!              .not. decision_made) then
!            cloudsummary%watercloud = .true.
!            decision_made = .true.
!         endif

         if (cloudmask%test_39_11 .and. .not. cloudmask%test_high_138  .and.  &
             cloudmask%applied_39_11 .and. cloudmask%applied_highcloud138 .and. & 
             .not. decision_made) then
            cloudsummary%watercloud = .true.
            decision_made = .true.
         endif

         if (cloudmask%test_high_67 .and. cloudmask%applied_highcloud67 .and.  &
             .not. decision_made) then
            cloudsummary%icecloud = .true.
            decision_made = .true.
         endif

!         Drop 1.38 test over land surface from decision tree - GTA 4/08/05
!         if (cloudmask%test_high_138 .and. cloudmask%applied_highcloud138 .and. &
!             .not. decision_made) then
!            cloudsummary%icecloud = .true.
!            decision_made = .true.
!         endif

         if( .not. decision_made) then
            cloudsummary%unknowncloud= .true.
            decision_made = .true.
         endif

      else
            cloudsummary%cloudobserved = .false.
            decision_made = .true.
!         endif
      endif
   endif


!  WATER(OCEAN) SURFACE
!  published documentation on this path refers to it as the 'Ocean Ecosystem' path

   if (cloudmask%water_surface .and.  .not. decision_made) then

!      cloudsummary%ocean_surface= .true.
      if (cloudmask%confident_cloudy .or. cloudmask%probablyclear_66) then
             cloudsummary%cloudobserved = .true. 
!         if (cloudmask%probablyclear_66 .and. cloudmask%sunglint) then
!            decision_made = .true.
!            cloudsummary%cloudobserved = .false.
!         endif 

!        MODIS decision tree does a Heavy aerosol test here - test deleted from this code

!        Following test commented out - not in collection 4 or decision tree flowchart - GTA 01/28/05
!         if  (.not. cloudmask%test_high_138 .and. cloudmask%test_irdiff .and.  &
!              cloudmask%applied_highcloud138 .and. cloudmask%applied_irtempdifference .and. &
!              .not. decision_made) then
!            cloudsummary%watercloud = .true.
!            decision_made = .true.
!         endif

         if (cloudmask%test_39_11 .and. .not. cloudmask%test_high_138  .and.  &
             cloudmask%applied_39_11 .and. cloudmask%applied_highcloud138 .and. & 
             .not. decision_made) then
            cloudsummary%watercloud = .true.
            decision_made = .true.
         endif

         if (cloudmask%test_high_67 .and. cloudmask%applied_highcloud67 .and.  &
             .not. decision_made) then
            cloudsummary%icecloud = .true.
            decision_made = .true.
         endif

!         Drop 1.38 test over water surface from decision tree - GTA 4/08/05
!         if (cloudmask%test_high_138 .and. cloudmask%applied_highcloud138 .and. &
!             .not. decision_made) then
!            cloudsummary%icecloud = .true.
!            decision_made = .true.
!         endif

         if( .not. decision_made) then
            cloudsummary%unknowncloud= .true.
            decision_made = .true.
         endif

      else
            cloudsummary%cloudobserved = .false.
            decision_made = .true.
!         endif
      endif
   endif


!  COASTAL SURFACE

   if (cloudmask%coastal_surface .and.  .not. decision_made) then
     if (cloudsummary%ocean_surface) then
       cloudsummary%coastal_surface= .true.
       cloudsummary%ocean_surface = .false.
       endif


     if (cloudmask%confident_cloudy .or. cloudmask%probablyclear_66) then
         cloudsummary%cloudobserved = .true.

!        MODIS decision tree does a Heavy aerosol test here - test deleted from this code

!        Following test commented out - not in collection 4 or decision tree flowchart - GTA 01/28/05
!         if  (.not. cloudmask%test_high_138 .and. cloudmask%test_irdiff .and.  &
!              cloudmask%applied_highcloud138 .and. cloudmask%applied_irtempdifference .and. &
!              .not. decision_made) then
!            cloudsummary%watercloud = .true.
!            decision_made = .true.
!         endif

       if (cloudmask%test_39_11 .and. .not. cloudmask%test_high_138  .and.  &
         cloudmask%applied_39_11 .and. cloudmask%applied_highcloud138 .and. & 
           .not. decision_made) then
         cloudsummary%watercloud = .true.
         decision_made = .true.
       endif

       if (cloudmask%test_high_67 .and. cloudmask%applied_highcloud67 .and.  &
           .not. decision_made) then
         cloudsummary%icecloud = .true.
         decision_made = .true.
       endif

!         Drop 1.38 test over coastal surface from decision tree - GTA 4/08/05
!         if (cloudmask%test_high_138 .and. cloudmask%applied_highcloud138 .and. &
!             .not. decision_made) then
!            cloudsummary%icecloud = .true.
!            decision_made = .true.
!         endif

       if( .not. decision_made) then
          cloudsummary%unknowncloud= .true.
          decision_made = .true.
       endif

     else
            cloudsummary%cloudobserved = .false.
            decision_made = .true.
!         endif
      endif
   endif

!  DESERT SURFACE

   if (cloudmask%desert_surface .and.  .not. decision_made) then
     if (cloudsummary%ocean_surface) then
       cloudsummary%coastal_surface= .true.
       cloudsummary%ocean_surface = .false.
       endif

!      cloudsummary%desert_surface= .true.
      if (cloudmask%confident_cloudy .or. cloudmask%probablyclear_66) then
         cloudsummary%cloudobserved = .true.

!        MODIS decision tree does a Heavy aerosol test here - test deleted from this code

!        Following test commented out - not in collection 4 or decision tree flowchart - GTA 01/28/05
!         if  (.not. cloudmask%test_high_138 .and. cloudmask%test_irdiff .and.  &
!              cloudmask%applied_highcloud138 .and. cloudmask%applied_irtempdifference .and. &
!              .not. decision_made) then
!            cloudsummary%watercloud = .true.
!            decision_made = .true.
!         endif

         if (cloudmask%test_39_11 .and. .not. cloudmask%test_high_138  .and.  &
             cloudmask%applied_39_11 .and. cloudmask%applied_highcloud138 .and. & 
             .not. decision_made) then
            cloudsummary%watercloud = .true.
            decision_made = .true.
         endif

         if (cloudmask%test_high_67 .and. cloudmask%applied_highcloud67 .and.  &
             .not. decision_made) then
            cloudsummary%icecloud = .true.
            decision_made = .true.
         endif


!        added additional ice phase test for desert ecosystem as required by dec. tree flowchart - GTA 02/07/05

         if (cloudmask%test_irdiff .and. cloudmask%test_visiblereflectance .and.  &
             cloudmask%applied_irtempdifference .and. cloudmask%applied_visiblereflectance .and. & 
             .not. decision_made) then
            cloudsummary%icecloud = .true.
            decision_made = .true.
         endif

!         Drop 1.38 test over desert surface from decision tree - GTA 4/08/05
!         if (cloudmask%test_high_138 .and. cloudmask%applied_highcloud138 .and. &
!             .not. decision_made) then
!            cloudsummary%icecloud = .true.
!            decision_made = .true.
!         endif

         if( .not. decision_made) then
            cloudsummary%unknowncloud= .true.
            decision_made = .true.
         endif

      else

            cloudsummary%cloudobserved = .false.
            decision_made = .true.
!         endif
      endif
   endif

!  If we have a cloud run Baum phase test (needed to try to set a phase for
!     for unknown phase and also used later multi cloud detection algorithm)  GTA - 03/04/05

   if (cloudsummary%cloudobserved .and. decision_made) then

      bright_temp_85 = modis_bright(platform_name,measurements(band_0850),29,1)
      bright_temp_11 = modis_bright(platform_name,measurements(band_1100),31,1)

!     ir bi spectral technique
      band_8_11_difference = bright_temp_85 - bright_temp_11 

!     ir bi spectral defaults to unknown
      ir_phase%icecloud       = .false.
      ir_phase%watercloud     = .false.
      ir_phase%unknowncloud   = .true.
      ir_phase%mixedcloud     = .false.
      ir_phase%cloudobserved  = .true.
      
      if (bright_temp_11 > 285. .and. band_8_11_difference <= -0.5) then
        ir_phase%watercloud     = .true.
        ir_phase%unknowncloud   = .false.
      endif

!      "<" in water phase test written in original Baum algorithm as "<=" 
!        modification necessary to avoid ambiguity with the unknowncloud test below 4/12/05

      if (bright_temp_11  > 238. .and. band_8_11_difference < -1.0) then
        ir_phase%watercloud     = .true.
        ir_phase%unknowncloud   = .false.
      endif

      if (bright_temp_11       > 238.  .and. &
          bright_temp_11       < 268.  .and. &
          band_8_11_difference < -.25  .and. &
          band_8_11_difference >= -1.0 ) then
        ir_phase%unknowncloud   = .true.
      endif

      if (bright_temp_11       > 238.  .and. &
          bright_temp_11       < 268.  .and. &
          band_8_11_difference >= -.25 .and. &
          band_8_11_difference < .5 ) then
        ir_phase%mixedcloud     = .true.
        ir_phase%unknowncloud   = .false.
      endif
      if (bright_temp_11 < 238. .or. &
         band_8_11_difference  > .5) then
        ir_phase%icecloud       = .true.
        ir_phase%unknowncloud   = .false.
      endif

!  if decision tree phase is "unknown", modify "cloudsummary" according to phase 
!      result from Baum test GTA - 03/04/05

      if (cloudsummary%unknowncloud) then
        cloudsummary%icecloud     = ir_phase%icecloud
        cloudsummary%watercloud   = ir_phase%watercloud
        cloudsummary%unknowncloud = ir_phase%unknowncloud
        cloudsummary%mixedcloud   = ir_phase%mixedcloud
     endif

!   set "mixed" phase cloud to "ice" phase

      if( cloudsummary%mixedcloud ) then
        cloudsummary%icecloud       = .true.
        cloudsummary%watercloud     = .false.
        cloudsummary%unknowncloud   = .false.
        cloudsummary%mixedcloud     = .false.
      endif

!  if  phase is still "unknown", set to ice cloud if 1.38 test bit on - 5/5/05 GTA 

   if (cloudsummary%unknowncloud) then      
      if (cloudmask%test_high_138 .and. cloudmask%applied_highcloud138) then
         cloudsummary%icecloud = .true.
         cloudsummary%unknowncloud = .false.
         endif
      endif
   endif


   nonabsorbing_band = band_0065
   if (cloudmask%land_surface) nonabsorbing_band = band_0065
   if (cloudmask%desert_surface) nonabsorbing_band = band_0065
   if (cloudmask%coastal_surface) nonabsorbing_band = band_0086
   if (cloudmask%water_surface) nonabsorbing_band = band_0086
   if (cloudmask%snowice_surface) nonabsorbing_band = band_0124


   if (cloudsummary%cloudobserved) then
!  Riedi Threshold tests
!  These threshold were modified based on updated work
!  from Dr Jerome Riedi [11/30/04 mag]

!     check reflectance of non-absorbing band to see if Riedi tests should be applied - GTA 10/28/05

     if(measurements(nonabsorbing_band) > (surfaceAlbedo(nonabsorbing_band) + 0.15)) then

!     Set SWIR test thresholds (based on surface type and reflectance magnitude)

!     Threshold values modified to comply with Riedi 12/04 values - GTA 1/28/05
!     Expanded control of thresholds for surface type and reflecatance magnitude  - 4/12/05
!     threshold_1 and threshold_3 for non-ice surface modified - GTA 4/12/05

       if (cloudmask%snowice_surface) then

         if (measurements(nonabsorbing_band) <= .5) then
           threshold_1 = .20    
           threshold_2 = .45
           threshold_3 = .45   
           threshold_4 = .70
         else
           threshold_1 = .20    
           threshold_2 = .45
           threshold_3 = .45   
           threshold_4 = .70
         endif

       elseif (cloudmask%water_surface .or. cloudmask%coastal_surface) then

         if (measurements(nonabsorbing_band) <= .5) then
!           threshold_1 = .30          ! baseline threshold - established 4-12-05 GTA 5-19-05
           threshold_1 = .25           ! alt science test threshold GTA 5-19-05
           threshold_2 = .65
!           threshold_3 = .50          ! baseline threshold - established 4-12-05 GTA 5-19-05
           threshold_3 = .42           ! alt science test threshold GTA 5-19-05 - corrected 5-23-05
           threshold_4 = .90
         else
!           threshold_1 = .30          ! baseline threshold - established 4-12-05 GTA 5-19-05
           threshold_1 = .25           ! alt science test threshold GTA 5-19-05 
           threshold_2 = .55
!           threshold_3 = .50          ! baseline threshold - established 4-12-05 GTA 5-19-05
           threshold_3 = .42           ! alt science test threshold GTA 5-19-05 - corrected 5-23-05
           threshold_4 = .80
         endif

       elseif (cloudmask%land_surface .or. cloudmask%desert_surface) then

         if (measurements(nonabsorbing_band) <= .5) then
!           threshold_1 = .30          ! baseline threshold - established 4-12-05 GTA 5-19-05
           threshold_1 = .25           ! alt science test threshold GTA 5-19-05
           threshold_2 = .65
!           threshold_3 = .50          ! baseline threshold - established 3-12-05 GTA 5-19-05
           threshold_3 = .42           ! alt science test threshold GTA 5-19-05 - corrected 5-23-05
           threshold_4 = .90
         else
!           threshold_1 = .30          ! baseline threshold - established 4-12-05 GTA 5-19-05
           threshold_1 = .25           ! alt science test threshold GTA 5-19-05
           threshold_2 = .55
!           threshold_3 = .50          ! baseline threshold - established 4-12-05 GTA 5-19-05
           threshold_3 = .42           ! alt science test threshold GTA 5-19-05 - corrected 5-23-05
           threshold_4 = .80
         endif

       endif


       if(measurements(band_0213) > 0. .and. measurements(nonabsorbing_band) > 0.) then
         if (decision_made .and. measurements(band_0213)/measurements(nonabsorbing_band) <=  threshold_1 ) then 
           cloudsummary%icecloud       = .true.
           cloudsummary%watercloud     = .false.
           cloudsummary%unknowncloud   = .false.
           cloudsummary%mixedcloud     = .false.
         elseif  (decision_made .and. measurements(band_0213)/measurements(nonabsorbing_band) >= threshold_2) then 
           cloudsummary%icecloud       = .false.
           cloudsummary%watercloud     = .true.
           cloudsummary%unknowncloud   = .false.
           cloudsummary%mixedcloud     = .false.
         endif
       endif


!  1.6um tests are not to be run for Aqua
       if (platform_name(1:5) == 'Terra') then 
         if (measurements(band_0163) > 0. .and. measurements(nonabsorbing_band) > 0.) then
           if (decision_made .and. measurements(band_0163)/measurements(nonabsorbing_band) <= threshold_3 ) then
              cloudsummary%icecloud       = .true.
              cloudsummary%watercloud     = .false.
              cloudsummary%unknowncloud   = .false.
              cloudsummary%mixedcloud     = .false.
           elseif  (decision_made .and. measurements(band_0163)/measurements(nonabsorbing_band) >= threshold_4) then
             cloudsummary%icecloud       = .false.
             cloudsummary%watercloud     = .true.
             cloudsummary%unknowncloud   = .false.
             cloudsummary%mixedcloud     = .false.
           endif
         endif

       endif
     
!  Cloud Top Temperature based warm sanity test applied here GTA 5/06/05
     
     
       water_particle_threshold = 273.
 
       if (cloud_top_temp>= water_particle_threshold) then
         cloudsummary%icecloud       = .false.
         cloudsummary%watercloud     = .true.
         cloudsummary%unknowncloud   = .false.
         cloudsummary%mixedcloud     = .false.
       endif

     endif
   endif

!  Cloud Top Temperature based sanity tests
!  We are not confident that these tests are really helping the phase
!  determination.  For now both tests (the cold and warm ) are
!  commented out. This needs to be seriously studied [11/30/04 mag] 
 
!  ice_particle_threshold = 233.
!  water_particle_threshold = 275.
!
!  if (cloudsummary%cloudobserved) then
!     if (cloud_top_temp<= ice_particle_threshold .and. &
!         cloud_top_temp> 150.) then
!       cloudsummary%icecloud       = .true.
!       cloudsummary%watercloud     = .false.
!       cloudsummary%unknowncloud   = .false.
!       cloudsummary%mixedcloud     = .false.
!     else
!       if (cloud_top_temp>= water_particle_threshold) then
!           cloudsummary%icecloud       = .false.
!           cloudsummary%watercloud     = .true.
!           cloudsummary%unknowncloud   = .false.
!           cloudsummary%mixedcloud     = .false.
!       warm test left out for now [mag]
!
!       endif
!     endif
!  endif

   !debug and status checking
   if  (.not. decision_made) then
      status = -1
   endif

   if (.not. cloudsummary%land_surface .and. &
       .not. cloudsummary%ocean_surface .and. &
       .not. cloudsummary%desert_surface .and. &
       .not. cloudsummary%snowice_surface .and. &
       .not. cloudsummary%coastal_surface) then
      status = -1
   endif


end subroutine clouddecision
subroutine libraryinterpolate(local_solarzenith, &
                              local_sensorzenith, &
                              local_relativeazimuth, &
                              debug, &
                              status)

   use GeneralAuxType
   use core_arrays
   use libraryarrays
   use libraryinterpolates
   
   implicit none

   real(single), intent(in)   :: local_solarzenith, local_sensorzenith, local_relativeazimuth
   logical,intent(in)   :: debug
   integer,intent(out)  :: status

   integer         :: iw,ir,i,ii,iii,iiii

   status = 0

!  loop over radii, wavelength, phase
!  inputs to interp_lib_flux
!     thermalband
!     number_taus
!     number_fluxsolarzenith
!     local_solarzenith
!     local_sensorzenith
!     library_fluxsolarzenith
!     escapefunction_water
!     flux_down_water
!     flux_up_water


! sep, 5 May: why is Mark implying that some of these are for tau=inf?
!  outputs to interp_lib_flux
!     int_escapewater_inf
!     int_fluxdownwater_inf
!     int_fluxupwater_inf
!     int_escapewater
!     int_fluxdownwater


!  inputs to interp_lib_reflectance

!  outputs to interp_lib_reflectance 
   call interp_lib_reflectance(local_sensorzenith,        &
                               local_solarzenith,       &
                               local_relativeazimuth,    &
                               library_sensor_zenith,     &
                               library_solar_zenith,    &
                               library_relative_azimuth, &
                               library_reflectance_water,&
                               int_reflectance_water,    &
                               status)

   call interp_lib_reflectance(local_sensorzenith,        &
                               local_solarzenith,       &
                               local_relativeazimuth,    &
                               library_sensor_zenith,     &
                               library_solar_zenith,    &
                               library_relative_azimuth, &
                               library_reflectance_ice,&
                               int_reflectance_ice,    &
                               status)

   call interp_lib_flux(local_solarzenith, &
                        local_sensorzenith, &
                        library_fluxsolarzenith, &
                        library_fluxsensorzenith, &
                        escapefunction_water_solar,     &
                        flux_down_water_solar,          &
                        flux_up_water_solar,            &
                        escapefunction_water_sensor,     &
                        flux_down_water_sensor,          &
                        flux_up_water_sensor,            &
                        int_escapefuncwater_solar,&
                        int_fluxdownwater_solar,       &
                        int_fluxupwater_sensor,         &
                        int_escapefuncwater_sensor,&
                        int_fluxdownwater_sensor)
   call interp_lib_flux(local_solarzenith, &
                        local_sensorzenith, &
                        library_fluxsolarzenith, &
                        library_fluxsensorzenith, &
                        escapefunction_ice_solar,     &
                        flux_down_ice_solar,          &
                        flux_up_ice_solar,            &
                        escapefunction_ice_sensor,     &
                        flux_down_ice_sensor,          &
                        flux_up_ice_sensor,            &
                        int_escapefuncice_solar,&
                        int_fluxdownice_solar,       &
                        int_fluxupice_sensor,         &
                        int_escapefuncice_sensor,&
                        int_fluxdownice_sensor)
end subroutine libraryinterpolate
subroutine interp_lib_reflectance(theta0,  &
                                  theta,   &
                                  phi,     &
                                  theta0g, &
                                  thetag,  &
                                  phig,    &
                                  reflectance,      &
                                  interpolated_refl,     &
                                  status)

!input parameters:   
!
!output parameters: 
!
!
!revision history:  
!
!team-unique header: 
!
!       cloud retrieval group, nasa/gsfc, greenbelt, maryland, usa
!
!references and credits:
!
!design notes:
   use GeneralAuxType
   use modis_numerical_module
   implicit none

   real(single),  intent(in) :: theta0g(:),thetag(:),phig(:)
   real(single),  intent(in) :: theta0,theta,phi
   real(single),  intent(in) :: reflectance(:,:,:,:,:,:)

   real(single),  intent(out):: interpolated_refl(:,:,:)
   integer, intent(out):: status

   logical             :: debug
   integer             :: i,i1,iim1,iim2,its,j,j1,jjm1, &
                          jjm2,k,k1,kkf1,kkf2,nts,iw,ir,&
                          number_iw,number_ir, ii

   integer             :: ii3, jj3, i33 

   real (single)             :: phi33

   status = 0
   

   number_iw = size(reflectance,5)
   number_ir = size(reflectance,6)
   nts       = size(reflectance,4)

   iim1=0
   iim2=0
   jjm1=0
   jjm2=0
   kkf1=0
   kkf2=0

 
   if (theta0 == maxval(theta0g) )then 
     iim2 = size(theta0g)
     iim1 = iim2 - 1
   else
     call BisectionSearch(theta0g, theta0,iim1,iim2)
   endif 
  
   if (theta == maxval(thetag) ) then
     jjm2 = size(thetag)
     jjm1 = jjm2 - 1
   else
     call BisectionSearch(thetag, theta,jjm1, jjm2)
   endif

   if (phi == maxval(phig) ) then
     kkf2  = size(phig)
     kkf1 = kkf2-1
   else  
    call BisectionSearch(phig,phi,kkf1,kkf2)
   endif

   do iw = 1,number_iw 
      do ir = 1,number_ir 

!        linear interpolation of reflection function

         debug = .false.
         do its = 1, nts
 
            interpolated_refl(its,iw,ir) = LinearInterpolation_3d(iim1,iim2, &
                                                                  jjm1,jjm2, &
                                                                  kkf1,kkf2, &
                                                                  theta,theta0,phi,&
                                                                  thetag,theta0g,phig,&
                                                                  debug, &
                                                                  reflectance(:,:,:,its,iw,ir))
         enddo
      enddo
   enddo
end subroutine interp_lib_reflectance
subroutine interp_lib_flux(solarzenith,sensorzenith,solarzenith_angles,sensorzenith_angles,&
                           escapefunction_solar,fluxdown_solar,fluxup_solar, &
                           escapefunction_sensor, fluxdown_sensor, fluxup_sensor, &
                           interp_escapefunction_solar,&
                           interp_fluxdown_solar,&
                           interp_fluxup_sensor,&
                           interp_escapefunction_sensor,&
                           interp_fluxdown_sensor)

   use GeneralAuxType
   use modis_numerical_module
   implicit none


   real(single) , intent(in)   :: solarzenith_angles(:),sensorzenith_angles(:)
   real(single) , intent(in)   :: escapefunction_solar(:,:,:), fluxdown_solar(:,:,:,:),  fluxup_solar(:,:,:,:)
   real(single) , intent(in)   :: escapefunction_sensor(:,:,:), fluxdown_sensor(:,:,:,:),  fluxup_sensor(:,:,:,:)
   real(single) , intent(in)   :: sensorzenith, solarzenith
   real(single) , intent(out)  :: interp_escapefunction_solar(:,:),interp_fluxdown_solar(:,:,:), &
                                  interp_fluxup_sensor(:,:,:),interp_escapefunction_sensor(:,:),  &
                                  interp_fluxdown_sensor(:,:,:)

   integer                     :: lowbound_solar,highbound_solar,lowbound_sensor,highbound_sensor,&
                                  its,nts,iw,ir,number_iw,number_ir, blah
   real(single)                :: angles(3),functionvalues(3)
   logical                     :: wthermal

   nts = size(fluxdown_solar,2)
   number_iw = size(fluxdown_solar,3)
   number_ir = size(fluxdown_solar,4)
!  interpolation for the solar zenith angle (ind < 0)

!  find interpolation region for solar zenith angle

   interp_escapefunction_solar = 0.
   interp_fluxdown_solar = 0.
   interp_fluxup_sensor = 0.
   interp_escapefunction_sensor = 0.
   interp_fluxdown_sensor = 0.

!sep, 5 May: found that nts=4 as expected


   call bisectionsearch(solarzenith_angles,       &
                        solarzenith,              &
                        lowbound_solar,           &
                        highbound_solar)
                        
   call bisectionsearch(sensorzenith_angles,      &
                        sensorzenith,             &
                        lowbound_sensor,          &
                        highbound_sensor)

   do  iw = 1, number_iw
      do ir = 1, number_ir
         angles(1)=sensorzenith_angles(lowbound_sensor)
         angles(2)=sensorzenith_angles(highbound_sensor)
         functionvalues(1)=escapefunction_sensor(lowbound_sensor,iw,ir)
         functionvalues(2)=escapefunction_sensor(highbound_sensor,iw,ir)
         interp_escapefunction_sensor(iw,ir) = linearinterpolation(angles,functionvalues,sensorzenith)

!         do its = 1,nts - 1
         do its = 1,nts
            functionvalues(1) = fluxdown_sensor(lowbound_sensor,its,iw,ir)
            functionvalues(2) = fluxdown_sensor(highbound_sensor,its,iw,ir)
            interp_fluxdown_sensor(its,iw,ir) = linearinterpolation(angles,functionvalues,sensorzenith)
         enddo

!        interpolation for emergent zenith angles (ind = 0)
         angles(1) = solarzenith_angles(lowbound_solar)
         angles(2) = solarzenith_angles(highbound_solar)

         functionvalues(1) = escapefunction_solar(lowbound_solar,iw,ir)
         functionvalues(2) = escapefunction_solar(highbound_solar,iw,ir)
         interp_escapefunction_solar(iw,ir) = linearinterpolation(angles,functionvalues,solarzenith)

!         do  its = 1,nts - 1
         do its = 1,nts
            functionvalues(1) = fluxdown_solar(lowbound_solar,its,iw,ir)
            functionvalues(2) = fluxdown_solar(highbound_solar,its,iw,ir)
            interp_fluxdown_solar(its,iw,ir) = linearinterpolation(angles,functionvalues,solarzenith)
         enddo

!        interpolation of the cloud albedo for 3.7 micron band
         if(iw == 6)then
!            do its=1,nts-1
             do its = 1,nts
! sep, 5 May: reflected flux for the 3.7 um band, used for emissvity calculations, needs to be done for 
!             the sensor zenith angle, not the solar angle!
               functionvalues(1)=fluxup_sensor(lowbound_sensor,its,iw,ir)
               functionvalues(2)=fluxup_sensor(highbound_sensor,its,iw,ir)
               interp_fluxup_sensor(its,iw,ir)=linearinterpolation(angles,functionvalues,sensorzenith)
            enddo
         endif
      enddo
   enddo
end subroutine interp_lib_flux
subroutine atmospheric_correction(xpoint,ypoint,debug,status)

   use GeneralAuxType
   use science_parameters
   use core_arrays
   use libraryarrays
   use libraryinterpolates
   use mod06_run_settings
   use nonscience_parameters
   implicit none

   logical, intent(in)  :: debug
   integer, intent(in)  :: xpoint,ypoint
   integer, intent(out) :: status

   real                 :: oneway_correction(2), twoway_correction(7), &
                           oneway_standarddev(2),twoway_standarddev(7), &
                           twoway_correction_lower(7), twoway_correction_upper(7)
   real                 :: cos_solarzenith, cos_viewingangle
   integer              :: errorlevel
   integer, parameter   :: oneway_correction_index = 2, twoway_correction_index =7
   real                 :: ozone_absorp_coeff, angle, ozone_transmittance

   ozone_absorp_coeff = 5.56209e-5  !   2.07e-21 * 2.687e16

   status = 0



   cos_solarzenith = cos(d2r*solar_zenith_angle(xpoint,ypoint))
  
   cos_viewingangle = cos(d2r*sensor_zenith_angle(xpoint,ypoint))

   angle = 1./cos_solarzenith + 1./cos_viewingangle

!  currently the transmittance table breaks the MODIS band ordering by
!  choosing index the wavelength without regard to the SDS ordering implicit
!  in the data
!  bottom line; in the transmittance data .94 site between .86 and 1.2
!               in the measurements .94 sits betweem 2.1 and 3.7 (because .94 is in the 1km dataset)

   if ( cloud_top_pressure(xpoint,ypoint)     >= 0. .and.  &
        cloud_top_temperature(xpoint, ypoint) >= 0. ) then
     call gettransmittancedata(oneway_correction_index, &
                           twoway_correction_index, &
                           transmit_correction_table, &
                           transmit_stddev_table, &
                           cloud_top_pressure(xpoint,ypoint), &
                           abovecloud_watervapor(xpoint,ypoint), &
                           cos_solarzenith, &
                           cos_viewingangle, &
                           oneway_correction, &
                           twoway_correction, &
                           oneway_standarddev, &
                           twoway_standarddev, &
                           errorlevel)
     call gettransmittance_simple( &
                           transmit_correction_table, &
                           cloud_top_pressure(xpoint,ypoint), &
                           abovecloud_watervapor(xpoint,ypoint)*(1. - watervapor_error), &
                           cos_solarzenith, &
                           cos_viewingangle, &
                           twoway_correction_lower, &
                           errorlevel)
     call gettransmittance_simple( &
                           transmit_correction_table, &
                           cloud_top_pressure(xpoint,ypoint), &
                           abovecloud_watervapor(xpoint,ypoint)*(1. + watervapor_error), &
                           cos_solarzenith, &
                           cos_viewingangle, &
                           twoway_correction_upper, &
                           errorlevel)

! the values will never be fill, because there has been a change in the gettransmittance routines 
! used above. G.Wind 2.2.05


! we need to re-order the mean

     meandelta_trans(xpoint, ypoint, :) = ( abs(twoway_correction_lower - twoway_correction) + &
                                            abs(twoway_correction_upper - twoway_correction) ) /2.
     transmittance_twoway(xpoint, ypoint, :) = twoway_correction
     transmittance_stddev(xpoint, ypoint, :) = twoway_standarddev
   else

!    these values are set to have a benign effect on subsequent computations
!    status is set an an early return performed.  It is expected that subsequest
!    retrieval computation on this scene will be terminated  

     oneway_correction = 1.
     twoway_correction = 1.
     transmittance_twoway(xpoint, ypoint, :) = 1.
     transmittance_stddev(xpoint, ypoint, :) = 0.
     meandelta_trans(xpoint, ypoint, :) = 0.
     status = 1
     return
   endif
!  Beer's law for ozone transmittance
   ozone_transmittance = exp (-1.0 * column_ozone(xpoint, ypoint)* ozone_absorp_coeff * angle)

!  the case of fill values of transmittance is
!  dealt with in the gettransmittancedata subroutine G. Wind 2.2.05

if(debugPRN) then
	write (*,'(/a/)')                '### IN SUBROUTINE atmospheric_correction'
	write (*,'(/a/)')                'Rmeas: 0.65, 0.86, .94, 1.2, 1.6, 2.1 micons:'
	write (*,'(6(3x,f5.3)/)') band_measurements(xpoint,band_0065,ypoint), band_measurements(xpoint,band_0086,ypoint), &
	band_measurements(xpoint,band_0935,ypoint), band_measurements(xpoint,band_0124,ypoint), &
	band_measurements(xpoint,band_0163,ypoint), band_measurements(xpoint,band_0213,ypoint)
endif


   if (band_measurements(xpoint, band_0065, ypoint) /= fillvalue_real) &
        band_measurements(xpoint,band_0065,ypoint) = band_measurements(xpoint,band_0065,ypoint)/ &
                                                (ozone_transmittance*twoway_correction(band_0065)) ! .68

   if (band_measurements(xpoint, band_0086, ypoint) /= fillvalue_real) &
        band_measurements(xpoint,band_0086,ypoint) = band_measurements(xpoint,band_0086,ypoint)/&
                            twoway_correction(band_0086) ! .86

   if (band_measurements(xpoint, band_0124, ypoint) /= fillvalue_real) &
        band_measurements(xpoint,band_0124,ypoint) = band_measurements(xpoint,band_0124,ypoint)/&
                      twoway_correction(band_0124) ! 1.2

   if (band_measurements(xpoint, band_0163, ypoint) /= fillvalue_real) &
        band_measurements(xpoint,band_0163,ypoint) = band_measurements(xpoint,band_0163,ypoint)/&
                                   twoway_correction(band_0163) ! 1.6

   if (band_measurements(xpoint, band_0213, ypoint) /= fillvalue_real) &
        band_measurements(xpoint,band_0213,ypoint) = band_measurements(xpoint,band_0213,ypoint)/&
                                   twoway_correction(band_0213) ! 2.1

   if (band_measurements(xpoint, band_0935, ypoint) /= fillvalue_real) &
        band_measurements(xpoint,band_0935,ypoint) = band_measurements(xpoint,band_0935,ypoint)/&
                                 twoway_correction(band_0935) ! .94



   thermal_correction_twoway(xpoint, ypoint,1:2) = 1.
   thermal_correction_oneway(xpoint, ypoint,1:2) = 1.

   meandelta_trans(xpoint, ypoint, band_0065) = meandelta_trans(xpoint, ypoint, band_0065) * ozone_transmittance
   transmittance_twoway(xpoint, ypoint, band_0065) = transmittance_twoway(xpoint, ypoint, band_0065) * ozone_transmittance
   transmittance_stddev(xpoint, ypoint,band_0065 ) = transmittance_stddev(xpoint, ypoint, band_0065) * ozone_transmittance


!  3.7 transmittance corrections cannot be applied until we have seperated the visible/thermal contribution to 
!  the total radiance

   thermal_correction_twoway(xpoint,ypoint,1) = twoway_correction(band_0370) ! 2 way correction for 3.7um, to be used later
   thermal_correction_oneway(xpoint,ypoint,1) = oneway_correction (1)  ! 1 way correction for 3.7um, to be user later
   thermal_correction_bands = (/3.7, 11.0/)
   ozone_absorption(xpoint,ypoint) = ozone_transmittance

if(debugPRN) then
	write (*,'(a,2(3x,I5))')         'xpoint, ypoint:           ', xpoint, ypoint
	write (*,'(a,3x,f5.2)')          'cos_solarzenith:          ', cos_solarzenith
	write (*,'(a,3x,f5.2)')          'cos_viewingangle:         ', cos_viewingangle
	write (*,'(a,3x,f5.0)')          'cloud_top_pressure:       ', cloud_top_pressure(xpoint,ypoint)
	write (*,'(a,3x,f5.0)')          'cloud_top_temperat:       ', cloud_top_temperature(xpoint,ypoint)
	write (*,'(a,3x,f5.2)')          'abovecloud_watervapor:    ', abovecloud_watervapor(xpoint,ypoint)
	write (*,'(a,3x,f5.2)')          'watervapor_error:         ', watervapor_error
	write (*,'(a,3x,f5.2)')          'ozone_transmittance:      ', ozone_transmittance
	write (*,'(/a/)')                'twoway_* vector = 0.65, 0.86, .94, 1.2, 1.6, 2.1, 3.7 micons:'
	write (*,'(a,7(3x,f5.3))')       'twoway_standarddev:       ', &
	  twoway_standarddev(band_0065),twoway_standarddev(band_0086), &
	  twoway_standarddev(band_0935),twoway_standarddev(band_0124), &
	  twoway_standarddev(band_0163),twoway_standarddev(band_0213),twoway_standarddev(band_0370)
	write (*,'(a,7(3x,f5.3))')       'twoway_correct (no O3):   ', &
	  twoway_correction(band_0065),twoway_correction(band_0086), &
	  twoway_correction(band_0935),twoway_correction(band_0124), &
	  twoway_correction(band_0163),twoway_correction(band_0213),twoway_correction(band_0370)
	write (*,'(a,7(3x,f5.3))')       'twoway_correct_up (no O3):', &
	  twoway_correction_upper(band_0065),twoway_correction_upper(band_0086), &
	  twoway_correction_upper(band_0935),twoway_correction_upper(band_0124), &
	  twoway_correction_upper(band_0163),twoway_correction_upper(band_0213),twoway_correction_upper(band_0370)
	write (*,'(a,7(3x,f5.3))')       'twoway_correct_low(no O3):', &
	  twoway_correction_lower(band_0065),twoway_correction_lower(band_0086), &
	  twoway_correction_lower(band_0935),twoway_correction_lower(band_0124), &
	  twoway_correction_lower(band_0163),twoway_correction_lower(band_0213),twoway_correction_lower(band_0370)
	write (*,'(/a/)')                'Rcorrected: 0.65, 0.86, .94, 1.2, 1.6, 2.1 micons:'
	write (*,'(6(3x,f5.3)/)') band_measurements(xpoint,band_0065,ypoint), band_measurements(xpoint,band_0086,ypoint), &
	band_measurements(xpoint,band_0935,ypoint), band_measurements(xpoint,band_0124,ypoint), &
	band_measurements(xpoint,band_0163,ypoint), band_measurements(xpoint,band_0213,ypoint)
endif

end subroutine atmospheric_correction
SUBROUTINE GetTransmittanceData(KDIM_1WAY,   &
                                KDIM_2WAY,   &
                                BigTauTable, &
                                BigSdevTable,&
                                p,           &
                                tpw,         &
                                miu0,        &
                                miu1,        &
                                tau1way,     &
                                tau2way,     &
                                sdev1way,    &
                                sdev2way,    &
                                errorLevel)
!................................................................................
!
!................................................................................
! !F90
!
! !DESCRIPTION:     
!     given pressure, integrated precipitable water amount, miu0 and miu1, this
!     program returns: 
!     (1) one way transmittance for 3.7 and 11 um bands.
!     (2) two way transmittance for 0.67, 0.86, 1.2, 1.6, 2.1 and 3.7 um bands.
! 
!
! !INPUT PARAMETERS:
!
!       KDIM_1WAY = delcared band-dimension for 1-way tau array; >= 2
!       KDIM_2WAY = declared band-dimension for 2-way tau array; >= 6
!
!       p(XDIM, YDIM) = pressure height in mb.
!     tpw(XDIM, YDIM) = precipitable water (g/cm2) from level p to TOA.
!    miu0(XDIM, YDIM) = cosine of solar zenith angle.
!    miu1(XDIM, YDIM) = cosine of viewing zenith at level p.
!
!
! !OUTPUT PARAMETERS:
!
!      tau1way(XDIM, YDIM, 2) = contains 1-way spectral transmittance information.
!      tau2way(XDIM, YDIM, 6) = contains 2-way spectral transmittance information.
!      sdev1way(XDIM, YDIM, 2) = contains 1-way standard deviations.
!      sdev2way(XDIM, YDIM, 6) = contains 2-way standard deviations.
! 
!      *** Note ***: Fill value for tau1way and tau2way is -1.
!
!      errorLevel = return error status
!                   0 : none. 
!                   1 : second nearest neighbour maybe used (warning).
!                   2 : cannot find required files (fatal)
! 
!
!
!
! !REQUIRED EXTERNAL PROGRAMS: 
!                 netcdf.f90 and ModisAuxType.f90
!................................................................................
! !Revision History:
!
!
! Revision 2.5 2004/06/30    MAG
! fixed confusion between transmittance table and standard deviation table
!
! Revision 2.5 2004/05/06    wind
!   added standard deviation processing in order to facilitate uncertainty calculations
! Revision 2.0  2002/10/25           wind,mag
! add bilinear smoothing to fields
!
! Revision 1.5  2001/05/01 18:50:08  jyli
! read entire transmittance table once to speed up process.
!
! Revision 1.4  2001/04/18 21:56:35  jyli
! return both 1-way (for longwave) and 2-way (for shortwave) tau.
!
! Revision 1.3  2001/04/06 19:11:57  jyli
! first delivery version which includs MODIS run environment stuff.
!
! Revision 1.1  2001/04/06 18:46:11  jyli
! Initial revision
!
!................................................................................
! !Team-unique Header:
!             Cloud Retrieval Group, NASA/GSFC
!
! !PROGRAMMER:
!
!             Mark Gray (L3 Com)
!             Climate and Radiation Branch
!             NASA Goddard Space Flight Center
!             Greenbelt, Maryland 20771, U.S.A.
!
!             Gala Wind (L3 Com)
!             Climate and Radiation Branch
!             NASA Goddard Space Flight Center
!             Greenbelt, Maryland 20771, U.S.A.
!
!             Jason Li (Emergent-IT)
!             Climate and Radiation Branch
!             NASA Goddard Space Flight Center
!             Greenbelt, Maryland 20771, U.S.A.
!*******************************************************************************
! !END
use modis_numerical_module, only: linearinterpolation
use mod06_run_settings

IMPLICIT NONE


!... i/o variable delcarations:

INTEGER, intent(in) :: KDIM_1WAY, KDIM_2WAY
REAL , intent(in)            :: p, tpw, miu0, miu1
REAL, intent(out),DIMENSION(KDIM_1WAY) :: tau1way, sdev1way
REAL, intent(out),DIMENSION(KDIM_2WAY) :: tau2way, sdev2way
INTEGER, intent(out)                               :: errorLevel
REAL, intent(in), DIMENSION(:,:,:,:)   :: bigTauTable, bigSdevTable

!... scale in p, tpw and miu dimension:

INTEGER, PARAMETER :: NumberOfPressureHeights = 10
REAL, DIMENSION(NumberOfPressureHeights), PARAMETER :: &
heightScale = (/ 100., 200., 300., 400., 500., 600., 700., 800., 900., 1000. /)


! changed by G. Wind 5.6.04 to match the new axis of transmittance table
INTEGER, PARAMETER :: NumberOfPrecipitableWater = 53
REAL, DIMENSION(NumberOfPrecipitableWater), PARAMETER :: &
 pwScale = (/ 0.0, 0.02, 0.04, 0.06, 0.08, 0.1, 0.12, 0.14, 0.16, 0.18, 0.2, &
                    0.4, 0.6, 0.8, 1.0, 1.2, 1.4, 1.6,1.8, &
                   2.0, 2.2, 2.4, 2.6, 2.8, 3.0, 3.2, 3.4, 3.6, 3.8, 4.0, &
                   4.2, 4.4, 4.6, 4.8, 5.0, 5.2, 5.4, 5.6, 5.8, 6.0, 6.2, &
                   6.4, 6.6, 6.8, 7.0, 7.2, 7.4, 7.6, 7.8, 8.0, 8.2, 8.4, &
                   8.6 /)

INTEGER, PARAMETER :: NumberOfMiu = 20 
REAL, DIMENSION(NumberOfMiu), PARAMETER :: &
miuScale = (/ 0.05, 0.10, 0.15, 0.20, 0.25, 0.30, 0.35, 0.40, 0.45, &
              0.50, 0.55, 0.60, 0.65, 0.70, 0.75, 0.80, 0.85, 0.90, &
              0.95, 1.00 /) 


INTEGER :: i, pIndex, miu1Index, miu2Index, pwIndex, set_fill
INTEGER, DIMENSION(4) :: start, counts, mindex

INTEGER           :: NumberOfPixels_1km, NumberOfLines_1km
INTEGER, PARAMETER :: NumberOfShortWavelengths = 7  
INTEGER, PARAMETER :: NumberOfLongWavelengths  = 2  
INTEGER, DIMENSION(NumberOfShortWavelengths) :: bandIndexMapSW=(/ 1,2,3,4,5,6,7 /)
INTEGER, DIMENSION(NumberOfLongWavelengths)  :: bandIndexMapLW=(/ 7,8 /)

REAL            :: miu, tauValue
real            :: tau11, tau12, tau21, tau22, tau_mid1, tau_mid2
real            :: sdev11, sdev12, sdev21, sdev22, sdev_mid1, sdev_mid2
Integer         :: first_pwIndex, first_miuindex, second_pwIndex, second_miuindex
real, dimension(numberofshortwavelengths) :: temp_trans, temp_sdev
!................................................................................


! initialize values
temp_trans = -999.
temp_sdev = -999.
 

errorLevel = 0

   miu = (miu0*miu1)/(miu0+miu1)

!
!... find array indicies along p, pw and miu dimension:
!
   pIndex = NINT( (p - heightScale(1)) / 100.0 ) + 1
   set_fill = 0 
   if (p< 0. .and. tpw < 0.) set_fill = 1

   if(pIndex < 1) then
      pIndex = 1
      errorLevel = 1
   elseif(pIndex > NumberOfPressureHeights) then
      pIndex = NumberOfPressureHeights
      errorLevel = 1
   endif

   miu1Index = NINT( (miu1 - miuScale(1)) / 0.05 ) + 1
   if (miu1Index < 1) then
       miu1Index = 1
       errorLevel = 1
   elseif(miu1Index > NumberOfMiu) then
       miu1Index = NumberOfMiu
       errorLevel = 1
   endif

   miu2Index = NINT( (miu - miuScale(1)) / 0.05 ) + 1
   if (miu2Index < 1) then
       miu2Index = 1
       errorLevel = 1
   elseif(miu2Index > NumberOfMiu) then
       miu2Index = NumberOfMiu
       errorLevel = 1
   endif


! changed by G. Wind 5.6.04 to match the new bins in transmittance table
   if(tpw < 0.0) then
       pwIndex = 1
       errorLevel = 1
   elseif(tpw < 0.2) then
       pwIndex = NINT( (tpw- pwScale(1)) / 0.02 ) + 1
   else
       pwIndex = NINT( (tpw - pwScale(11)) / 0.20 ) + 11
       if(pwIndex > NumberOfPrecipitableWater) then
          pwIndex = NumberOfPrecipitableWater
          errorLevel = 1
       endif
   endif

   !... 
   !... 1-way case (for longwave 3.7 and 11 micron bands):
   !... 

   mindex(1) = pIndex
   mindex(2) = pwIndex
   mindex(3) = miu1Index
   mindex(4) = 1

   tauValue = bigTauTable(mindex(1),mindex(2),mindex(3),mindex(4))

! we need to step back until we find a value that is non-fill, the second neighbor will not be
! enough in many cases

   if (tauValue < 0.0) then  ! do I need 2nd nearest neighbour?
!       errorLevel = 1
!       mindex(2) = pwIndex - 1  
      
       do i=mindex(2), 1, -1
          tauValue =  bigTauTable(mindex(1),i,mindex(3),mindex(4))
          if (tauValue >= 0.) exit
       end do

! now we have a good value of trans, non-fill, so we set the index accordingly
       mindex(2) = i

   endif

!  grab the 4 points for bilinear interpolation: pw1,u1 pw2,u1 pw1,u2, pw2,u2

   if (pwscale(mindex(2)) >= tpw ) then
      second_pwIndex = mindex(2)
      if (mindex(2) == 1) then 
         first_pwIndex = mindex(2)
      else
         first_pwIndex = mindex(2) - 1
         if (bigtautable(mindex(1), first_pwIndex, mindex(3), mindex(4)) < 0.) &
            first_pwIndex = mindex(2)
      endif

   else
      first_pwIndex = mindex(2)
      if (mindex(2) == numberofprecipitablewater) then 
         second_pwIndex = mindex(2) 
      else 
         second_pwIndex = mindex(2) + 1
         if (bigTauTable(mindex(1),second_pwIndex,mindex(3),mindex(4)) < 0.0) &
            second_pwIndex = mindex(2)
      endif
   endif

   if (miuScale(miu1index) >= miu1 ) then
      first_miuindex = miu1index -1
      second_miuindex = miu1index
   else
      first_miuindex = miu1index
      second_miuindex = miu1index + 1
   endif

   do i = 1, NumberOfLongWavelengths
      mindex(4) = bandIndexMapLW(i)

      tau11 = BigTauTable(pindex, first_pwIndex, first_miuindex, mindex(4))
      tau12 = BigTauTable(pindex, first_pwIndex, second_miuindex, mindex(4))

      tau21 = BigTauTable(pindex, second_pwIndex, first_miuindex, mindex(4))
      tau22 = BigTauTable(pindex, second_pwIndex, second_miuindex, mindex(4))

      sdev11 = BigSdevTable(pindex, first_pwIndex, first_miuindex, mindex(4))
      sdev12 = BigSdevTable(pindex, first_pwIndex, second_miuindex, mindex(4))

      sdev21 = BigSdevTable(pindex, second_pwIndex, first_miuindex, mindex(4))
      sdev22 = BigSdevTable(pindex, second_pwIndex, second_miuindex, mindex(4))


!     first we interpolate along the PW lines
      tau_mid1 = linearinterpolation( (/pwscale(first_pwIndex), pwscale(second_pwIndex) /), &
                                      (/tau11, tau21 /), &
                                      tpw)
      tau_mid2 = linearinterpolation( (/pwscale(first_pwIndex), pwscale(second_pwIndex) /), &
                                      (/tau12, tau22 /), &
                                      tpw)

      sdev_mid1 =linearinterpolation( (/pwscale(first_pwIndex),  pwscale(second_pwIndex)/), &
                                      (/sdev11, sdev21/), &
                                      tpw)
      sdev_mid2 =linearinterpolation( (/pwscale(first_pwIndex),  pwscale(second_pwIndex)/), &
                                      (/sdev12, sdev22/), &
                                      tpw)


!     now we interpolate along the MIU line through tau_mid1 and tau_mid2
!     (bilinear interpolation)
      tau1way(i) = linearinterpolation( (/miuscale(first_miuindex), miuscale(second_miuindex)/), &
                                         (/tau_mid1, tau_mid2/), &
                                         miu1)
      sdev1way(i) = linearinterpolation( (/miuscale(first_miuindex), miuscale(second_miuindex)/), &
                                         (/sdev_mid1, sdev_mid2/), &
                                         miu1) 
      if (set_fill == 1 )then
         tau1way(i) = -999.
         sdev1way(i) = -999.
      endif
   enddo

   !
   !... 2-way case (for shortwave channels upto 3.7 micron band):
   !

   mindex(2) = pwIndex
   mindex(3) = miu2Index
   mindex(4) = 1

! the pw_index values had already been determined and they don't change with changing miu, 
! so pray tell why should we recompute them. This code had been deleted. G.Wind 2.2.05

   if (miuScale(miu2index) >= miu ) then
      first_miuindex = miu2index -1
      second_miuindex = miu2index
   else
      first_miuindex = miu2index
      second_miuindex = miu2index + 1
   endif

   do i = 1, NumberOfShortWavelengths
      mindex(4) = bandIndexMapSW(i)

      tau11 = BigTauTable(pindex, first_pwIndex, first_miuindex, mindex(4))
      tau12 = BigTauTable(pindex, first_pwIndex, second_miuindex, mindex(4))

      tau21 = BigTauTable(pindex, second_pwIndex, first_miuindex, mindex(4))
      tau22 = BigTauTable(pindex, second_pwIndex, second_miuindex, mindex(4))

      sdev11 = BigSdevTable(pindex, first_pwIndex, first_miuindex, mindex(4))
      sdev12 = BigSdevTable(pindex, first_pwIndex, second_miuindex, mindex(4))

      sdev21 = BigSdevTable(pindex, second_pwIndex, first_miuindex, mindex(4))
      sdev22 = BigSdevTable(pindex, second_pwIndex, second_miuindex, mindex(4))

!     first we interpolate along the PW lines
      tau_mid1 = linearinterpolation( (/pwscale(first_pwIndex), pwscale(second_pwIndex) /), &
                                      (/tau11, tau21 /), &
                                      tpw)
      tau_mid2 = linearinterpolation( (/pwscale(first_pwIndex), pwscale(second_pwIndex) /), &
                                      (/tau12, tau22 /), &
                                      tpw)
!     then we repeat for the standard deviations
      sdev_mid1 =linearinterpolation( (/pwscale(first_pwIndex),  pwscale(second_pwIndex)/), &
                                      (/sdev11, sdev21/), &
                                      tpw)
      sdev_mid2 =linearinterpolation( (/pwscale(first_pwIndex),  pwscale(second_pwIndex)/), &
                                      (/sdev12, sdev22/), &
                                      tpw)
!     now we interpolate along the MIU line through tau_mid1 and tau_mid2
!     nothing but rather standard bilinear interpolation
!     and that is our answer.
      temp_trans(i) = linearinterpolation( (/miuscale(first_miuindex),  miuscale(second_miuindex)/), &
                                        (/tau_mid1, tau_mid2/), &
                                        miu)      

!     and these are the final standard deviations
      temp_sdev(i) = linearinterpolation( (/miuscale(first_miuindex), miuscale(second_miuindex)/), &
                                         (/sdev_mid1, sdev_mid2/), &
                                         miu )
      if (set_fill == 1 )then
         temp_trans(i) = -999.
         temp_sdev(i) = -999.
      endif
         
   end do

! remap the bands, so they can be indexed by the main MODIS band index
   tau2way(band_0065) = temp_trans(1)
   tau2way(band_0086) = temp_trans(2)
   tau2way(band_0124) = temp_trans(4)
   tau2way(band_0163) = temp_trans(5)
   tau2way(band_0213) = temp_trans(6)
   tau2way(band_0935) = temp_trans(3)
   tau2way(band_0370) = temp_trans(7)

   sdev2way(band_0065) = temp_sdev(1)
   sdev2way(band_0086) = temp_sdev(2)
   sdev2way(band_0124) = temp_sdev(4)
   sdev2way(band_0163) = temp_sdev(5)
   sdev2way(band_0213) = temp_sdev(6)
   sdev2way(band_0935) = temp_sdev(3)
   sdev2way(band_0370) = temp_sdev(7)






9999 RETURN

END SUBROUTINE GetTransmittanceData
SUBROUTINE gettransmittance_simple( &
                                   BigTauTable, &
                                   p,           &
                                   tpw,         &
                                   miu0,        &
                                   miu1,        &
                                   tau2way,     &
                                   errorLevel)
!................................................................................
!
!................................................................................
! !F90
!
! !DESCRIPTION:     
!     given pressure, integrated precipitable water amount, miu0 and miu1, this
!     program returns: 
!     (1) one way transmittance for 3.7 and 11 um bands.
!     (2) two way transmittance for 0.67, 0.86, 1.2, 1.6, 2.1 and 3.7 um bands.
! 
!
! !INPUT PARAMETERS:
!
!       KDIM_2WAY(xdim, ydim, n) = declared band-dimension for 2-way tau array; >= n bands
!
!       p(XDIM, YDIM) = pressure height in mb.
!     tpw(XDIM, YDIM) = precipitable water (g/cm2) from level p to TOA.
!    miu0(XDIM, YDIM) = cosine of solar zenith angle.
!    miu1(XDIM, YDIM) = cosine of viewing zenith at level p.
!
!
! !OUTPUT PARAMETERS:
!
!      tau2way(XDIM, YDIM, n) = contains 2-way spectral transmittance information.
! 
!      *** Note ***: Fill value for tau1way and tau2way is -1.
!
!      errorLevel = return error status
!                   0 : none. 
!                   1 : second nearest neighbour maybe used (warning).
!                   2 : cannot find required files (fatal)
! 
!
!
!
! !REQUIRED EXTERNAL PROGRAMS: 
!                 netcdf.f90 and ModisAuxType.f90
!................................................................................
! !Revision History:
!................................................................................
! !Team-unique Header:
!             Cloud Retrieval Group, NASA/GSFC
!
! !PROGRAMMER:
!
!             Mark Gray (L3 Com)
!             Climate and Radiation Branch
!             NASA Goddard Space Flight Center
!             Greenbelt, Maryland 20771, U.S.A.
!
!             Gala Wind (L3 Com)
!             Climate and Radiation Branch
!             NASA Goddard Space Flight Center
!             Greenbelt, Maryland 20771, U.S.A.
!
!             Jason Li (Emergent-IT)
!             Climate and Radiation Branch
!             NASA Goddard Space Flight Center
!             Greenbelt, Maryland 20771, U.S.A.
!*******************************************************************************
! !END

use modis_numerical_module, only: linearinterpolation
use mod06_run_settings

IMPLICIT NONE



!... i/o variable delcarations:

REAL , intent(in)            :: p, tpw, miu0, miu1
REAL, intent(out),DIMENSION(:) :: tau2way 
INTEGER, intent(out)                               :: errorLevel
REAL, intent(in), DIMENSION(:,:,:,:)   :: bigTauTable

!... scale in p, tpw and miu dimension:

INTEGER, PARAMETER :: NumberOfPressureHeights = 10
REAL, DIMENSION(NumberOfPressureHeights), PARAMETER :: &
heightScale = (/ 100., 200., 300., 400., 500., 600., 700., 800., 900., 1000. /)


! changed by G. Wind 5.6.04 to match the new axis of transmittance table
INTEGER, PARAMETER :: NumberOfPrecipitableWater = 53
REAL, DIMENSION(NumberOfPrecipitableWater), PARAMETER :: &
 pwScale = (/ 0.0, 0.02, 0.04, 0.06, 0.08, 0.1, 0.12, 0.14, 0.16, 0.18, 0.2, &
                    0.4, 0.6, 0.8, 1.0, 1.2, 1.4, 1.6,1.8, &
                   2.0, 2.2, 2.4, 2.6, 2.8, 3.0, 3.2, 3.4, 3.6, 3.8, 4.0, &
                   4.2, 4.4, 4.6, 4.8, 5.0, 5.2, 5.4, 5.6, 5.8, 6.0, 6.2, &
                   6.4, 6.6, 6.8, 7.0, 7.2, 7.4, 7.6, 7.8, 8.0, 8.2, 8.4, &
                   8.6 /)

INTEGER, PARAMETER :: NumberOfMiu = 20 
REAL, DIMENSION(NumberOfMiu), PARAMETER :: &
miuScale = (/ 0.05, 0.10, 0.15, 0.20, 0.25, 0.30, 0.35, 0.40, 0.45, &
              0.50, 0.55, 0.60, 0.65, 0.70, 0.75, 0.80, 0.85, 0.90, &
              0.95, 1.00 /) 


INTEGER :: i, pIndex, miu1Index, miu2Index, pwIndex, set_fill
INTEGER, DIMENSION(4) :: start, counts, mindex

INTEGER           :: NumberOfPixels_1km, NumberOfLines_1km
INTEGER, PARAMETER :: NumberOfShortWavelengths = 7  
INTEGER, PARAMETER :: NumberOfLongWavelengths  = 2  
INTEGER, DIMENSION(NumberOfShortWavelengths) :: bandIndexMapSW=(/ 1,2,3,4,5,6,7 /)
INTEGER, DIMENSION(NumberOfLongWavelengths)  :: bandIndexMapLW=(/ 7,8 /)

REAL            :: miu, tauValue
real            :: tau11, tau12, tau21, tau22, tau_mid1, tau_mid2
real            :: sdev11, sdev12, sdev21, sdev22, sdev_mid1, sdev_mid2
Integer         :: first_pwIndex, first_miuindex, second_pwIndex, second_miuindex
real, dimension(numberofshortwavelengths) :: temp_trans
!................................................................................


! initialize values
temp_trans = -999.



errorLevel = 0

   miu = (miu0*miu1)/(miu0+miu1)

!
!... find array indicies along p, pw and miu dimension:
!
   pIndex = NINT( (p - heightScale(1)) / 100.0 ) + 1
   set_fill = 0 
   if (p< 0. .and. tpw < 0.) set_fill = 1

   if(pIndex < 1) then
      pIndex = 1
      errorLevel = 1
   elseif(pIndex > NumberOfPressureHeights) then
      pIndex = NumberOfPressureHeights
      errorLevel = 1
   endif

   miu1Index = NINT( (miu1 - miuScale(1)) / 0.05 ) + 1
   if (miu1Index < 1) then
       miu1Index = 1
       errorLevel = 1
   elseif(miu1Index > NumberOfMiu) then
       miu1Index = NumberOfMiu
       errorLevel = 1
   endif

   miu2Index = NINT( (miu - miuScale(1)) / 0.05 ) + 1
   if (miu2Index < 1) then
       miu2Index = 1
       errorLevel = 1
   elseif(miu2Index > NumberOfMiu) then
       miu2Index = NumberOfMiu
       errorLevel = 1
   endif


! changed by G. Wind 5.6.04 to match the new bins in transmittance table\
   pwIndex = 1
   if(tpw < 0.0) then
       errorLevel = 1
   elseif(tpw < 0.2) then
       pwIndex = NINT( (tpw- pwScale(1)) / 0.02 ) + 1
   else
       pwIndex = NINT( (tpw - pwScale(11)) / 0.20 ) + 11
       if(pwIndex > NumberOfPrecipitableWater) then
          pwIndex = NumberOfPrecipitableWater
          errorLevel = 1
       endif
   endif

   !
   !... 2-way case (for shortwave channels upto 3.7 micron band):
   !

   mindex(1) = pIndex
   mindex(2) = pwIndex
   mindex(3) = miu2Index
   mindex(4) = 1
   tauValue =  bigTauTable(mindex(1),mindex(2),mindex(3),mindex(4))
  
! we need to step back until we find a value that is non-fill, the second neighbor will not be
! enough in many cases

   if (tauValue < 0.0) then  ! do I need 2nd nearest neighbour?
!       errorLevel = 1
!       mindex(2) = pwIndex - 1  
      
       do i=mindex(2), 1, -1
          tauValue =  bigTauTable(mindex(1),i,mindex(3),mindex(4))
          if (tauValue >= 0.) exit
       end do

! now we have a good value of trans, non-fill, so we set the index accordingly
       mindex(2) = i

   endif


!  grab the 4 points for bilinear interpolation: pw1,u1 pw2,u1 pw1,u2, pw2,u2
   if (pwscale(mindex(2)) >= tpw ) then
      second_pwIndex = mindex(2)
      if (mindex(2) == 1) then 
         first_pwIndex = mindex(2)
      else
         first_pwIndex = mindex(2) - 1
         if (bigtautable(mindex(1), first_pwIndex, mindex(3), mindex(4)) < 0.) &
            first_pwIndex = mindex(2)
      endif

   else
      first_pwIndex = mindex(2)
      if (mindex(2) == numberofprecipitablewater) then 
         second_pwIndex = mindex(2) 
      else 
         second_pwIndex = mindex(2) + 1
         if (bigTauTable(mindex(1),second_pwIndex,mindex(3),mindex(4)) < 0.0) &
            second_pwIndex = mindex(2)
      endif
   endif


   if (miuScale(miu2index) >= miu ) then
      first_miuindex = miu2index -1
      second_miuindex = miu2index
   else
      first_miuindex = miu2index
      second_miuindex = miu2index + 1
   endif

   do i = 1, NumberOfShortWavelengths
      mindex(4) = bandIndexMapSW(i)

      tau11 = BigTauTable(pindex, first_pwIndex, first_miuindex, mindex(4))
      tau12 = BigTauTable(pindex, first_pwIndex, second_miuindex, mindex(4))

      tau21 = BigTauTable(pindex, second_pwIndex, first_miuindex, mindex(4))
      tau22 = BigTauTable(pindex, second_pwIndex, second_miuindex, mindex(4))

!     first we interpolate along the PW lines
      tau_mid1 = linearinterpolation( (/pwscale(first_pwIndex), pwscale(second_pwIndex) /), &
                                      (/tau11, tau21 /), &
                                      tpw)
      tau_mid2 = linearinterpolation( (/pwscale(first_pwIndex), pwscale(second_pwIndex) /), &
                                      (/tau12, tau22 /), &
                                      tpw)

!     now we interpolate along the MIU line through tau_mid1 and tau_mid2
!     nothing but rather standard bilinear interpolation
!     and that is our answer.
      temp_trans(i) = linearinterpolation( (/miuscale(first_miuindex),  miuscale(second_miuindex)/), &
                                        (/tau_mid1, tau_mid2/), &
                                        miu)

      if (set_fill == 1 )then
         temp_trans(i) = -999.
      endif

   end do


! remap the bands, so they can be indexed by the main MODIS band index
   tau2way(band_0065) = temp_trans(1)
   tau2way(band_0086) = temp_trans(2)
   tau2way(band_0124) = temp_trans(4)
   tau2way(band_0163) = temp_trans(5)
   tau2way(band_0213) = temp_trans(6)
   tau2way(band_0935) = temp_trans(3)
   tau2way(band_0370) = temp_trans(7)



9999 RETURN

END SUBROUTINE gettransmittance_simple
subroutine assign_retrieval_error(xpoint, ypoint)
   use GeneralAuxType
   use nonscience_parameters
   use core_arrays
   integer,           intent(in)       :: xpoint,ypoint

   optical_thickness(xpoint, ypoint)     = fillvalue_real
   optical_thickness_1621(xpoint, ypoint) = fillvalue_real
   effective_radius_16(xpoint, ypoint)   = fillvalue_real
   effective_radius_21(xpoint, ypoint)   = fillvalue_real
   effective_radius_1621(xpoint, ypoint) = fillvalue_real
   effective_radius_37(xpoint, ypoint)   = fillvalue_real


end subroutine assign_retrieval_error

subroutine init_science_arrays
  use GeneralAuxType
  use nonscience_parameters
  use core_arrays

  optical_thickness = fillvalue_real
  optical_thickness_1621 = fillvalue_real
  effective_radius_16 = fillvalue_real
  effective_radius_21 = fillvalue_real
  effective_radius_37 = fillvalue_real
  effective_radius_1621 = fillvalue_real
  liquid_water_path = fillvalue_real
  liquid_water_path_1621 = fillvalue_real
  optical_thickness_error = fillvalue_real
  effective_radius_21_error = fillvalue_real
  liquid_water_path_error = fillvalue_real
  cloud_layer_flag = 0
  CSR_flag_array = 0
  optical_thickness_1621_error = fillvalue_real
  effective_radius_1621_error = fillvalue_real
  liquid_water_path_1621_error = fillvalue_real


end subroutine init_science_arrays

end module modis_science_module

