###########################################################################
# !MAKE
#
# !MAKEFILE NAME:
#       MOD_PR07.mk
#
# !DESCRIPTION:
#       Makefile for the MODIS Atmospheric Profiles Algorithm (MOD_PR07).
#
# !VARIABLES:
#       Internal
#        Macros            Definition 
#       ~~~~~~~~           ~~~~~~~~~~
#       ADD_CFLAGS         Process specific C compiler options
#       ADD_F77FLAGS       Process specific FORTRAN 77 compiler options
#       F90                FORTRAN 90 compiler definition in n32 mode
#       HDF5OPT_LIB        Path name of HDF5 dynamically shared library
#       INC                Accumulated include file search path
#       INC_FILE_SCIENCE   Include files developed by science team
#       INC_FILE_SHARED    Atmosphere shared include files 
#       LIB                Accumulated library search path
#       OBJ                Accumulated set of object files
#       PATH_SRC_SDST      Directory path to SDST L2 atmosphere shared source
#       PATH_SRC_UW        Directory path to UW-Madison shared source
#       TARGET             Executable file name
#
# !ENV VARIABLES
#         System
#       Environment 
#        Variables         Definition
#       ~~~~~~~~~~~        ~~~~~~~~~~
#       API_INC            Directory path to M-API include files
#       API_LIB            Directory path to M-API library
#       BRAND              Compiler brand. Current valids are "sgi64/sgi32/linux".
#                          "linux" denotes PGI compilers for FORTRAN code and GNU
#                          compilers for C.
#       CC                 C compiler definition
#       CFLAGS             C compiler options
#       F77                FORTRAN 77 compiler definition
#       F77FLAGS           FORTRAN 77 compiler options
#       HDFEOS_INC         Directory path to HDFEOS include files
#       HDFEOS_LIB         Directory path to HDFEOS library
#       HDFINC             Directory path to HDF include files
#       HDFLIB             Directory path to HDF library
#       HDF5LIB            Directory path to HDF5 libraries
#       PGSINC             Directory path to SDPTK include files
#       PGSLIB             Directory path to SDPTK library
#
#          User
#       Environment
#        Variables         Definition
#       ~~~~~~~~~~~        ~~~~~~~~~~
#       MODIS_STORE        Home directory of MODIS shared source code
#
# !TEAM-UNIQUE HEADER:
#       This software was developed by MODIS Science Data Support Team
#       for the National Aeronautics and Space Administration,
#       Goddard Space Flight Center, under contract NAS5-32373.
#
# !REVISION HISTORY:
#
# !NOTE:
#       1) Must follow MAPI, SDP TK, HDF, and other header/libs order
#       2) HDFLIB must be explicit and in order of -lmfhdf, -ldf, -ljpeg, -lz.
#       3) system environments, such as API_INC, PGSINC, HDFINC, PGSLIB,
#          API_LIB etc. should be defined before executing this makefile.
#       4) Compiler option "-byteswapio" must add to rountine modis_ges101.f
#          for the Linux PGI compiler.
# !END
###########################################################################

PATH_SRC_SDST = $(MODIS_STORE)/shared_src/atmos_src/src_L2
PATH_SRC_UW   = $(MODIS_STORE)/shared_src/atmos_src/src_UW

##---------------------------------------------------------------------------
##                  UW Development Environment
#
#PATH_SRC_SDST = $(HOME)/OPS/MODIS/shared/src_L2
#PATH_SRC_UW   = $(HOME)/OPS/MODIS/shared/src_UW
#
## Define development compiler options (this is the default mode)
## NOTE: THESE COMPILER FLAGS WERE CAREFULLY BY CHOSEN BY UW TO ASSIST IN
## DEVELOPMENT AND TESTING. UW IS NOT RESPONSIBLE FOR ANY ERRORS
## INTRODUCED IN THE CODE IF THESE FLAGS ARE DISABLED OR MODIFIED.
#CC=cc
#CFLAGS=-c -n32 -fullwarn -xansi -D_POSIX_SOURCE
#ADD_CFLAGS=-g -C
#
#F77=f77
#F77FLAGS=-c -n32 -fullwarn -u -bytereclen -extend_source -woff 2271,2272,2274
#ADD_F77FLAGS=-g -C
#
#F90=f90
#F90FLAGS=-c -n32 -fullwarn -u -bytereclen -extend_source -woff 2271,2272,2274
#ADD_F90FLAGS=-g -C
#
## Define optimized compiler options
## (To optimize the code uncomment these lines only: do nothing else!)
## NOTE: UW HAS TESTED THIS CODE ON SGI SYSTEMS USING '-O2' OPTIMZATION ONLY.
## UW IS NOT RESPONSIBLE FOR RUN-TIME ERRORS WHICH OCCUR AT ANY HIGHER
## LEVEL OF OPTIMIZATION.
#ADD_CFLAGS=-O2
#ADD_F77FLAGS=-O2
#ADD_F90FLAGS=-O2
#
##---------------------------------------------------------------------------
#
### Libraries for development/debug
## (Note addition of floating point library -lfpe at end of list) 
#LIB = -L$(API_LIB) -lmapi\
#      -L$(PGSLIB) -lPGSTK -lm\
#      -L$(HDFEOS_LIB) -lhdfeos -lGctp\
#      -L$(HDFLIB) -lmfhdf -ldf -ljpeg -lz\
#      -lfpe 
##---------------------------------------------------------------------------

# Objects
OBJ = \
CCSDS_FullTimeFormat_A.o \
CONCATENATE.o \
Copy_cldmask_meta.o \
Delete_CommonTempFiles_Atmos_V2.1.o \
Delete_PCF_TempFile.o \
Gen_Modis_FileName.o \
Get_Core_Metadata.o \
Get_Global_Metadata.o \
Get_date_time.o \
MODIS_IO_GEN_CLOSE.o \
MODIS_IO_GEN_OPEN.o \
MODIS_SMF_SETDYNAMICMSG_V2.1.o \
MOD_PR07.o \
Met_Common_Atmos.o \
Parse_ECS_DateTime.o \
Read_CldMsk_V2.o \
Read_GEO_V2.o \
Read_L1B_V2.2.o \
STRING_LOC.o \
Set_ArchiveMetadata_V2.1.o \
Set_CoreMetadata_V2.1.o \
Write_ECSMetadata_nonHDF.o \
bright_m.o \
brite_m.o \
ccsds_to_date.o \
ceil.o \
chk_input_L2.o \
compare_times.o \
copy_info.o \
create_hdf.o \
date_to_ccsds.o \
dewtem.o \
eswat.o \
ezlh_convert.o \
floor.o \
get_ancillary_1.o \
get_coeff.o \
get_debug.o \
get_platform_name.o \
get_sfcpres_pwat.o \
get_spec_info.o \
gphite.o \
indices.o \
interp.o \
message_1.o \
mod07_Prepare_ECS_Metadata.o \
mod07_check_box.o \
mod07_close_files.o \
mod07_compute_products.o \
mod07_get_data.o \
mod07_initialize_output.o \
mod07_open_files.o \
mod07_set_qa.o \
mod07_write_products.o \
modis_bright.o \
modis_grib_driver_1.o \
modis_grib_read.o \
modis_ges101.o \
openr_temp.o \
precwv.o \
read_nise.o \
satmix.o \
set_qa_bit.o \
setup_create_hdf.o \
strcompress.o \
strlen.o \
strpos.o \
svpwat.o \
total_ozone.o \
tovlif.o \
tprecw_new2.o

# Include file search path
INC = -I$(API_INC) \
      -I$(HDFINC) \
      -I$(PGSINC) \
      -I$(HDFEOS_INC) \
      -I$(PATH_SRC_SDST) \
      -I$(PATH_SRC_UW)

# Include files developed by ST
INC_FILES_SCIENCE = mod07_data.inc \
                    mod07_meta.inc \
                    mod07_pcfnum.inc

# Include files from MODIS shared source
## 18 March 2004: SWS changed L1B_Reader_V2.1.inc to L1B_Reader_V2.2.inc
## Also changed path from $(PATH_SRC_SDST) to $(PATH_SRC_UW)
INC_FILES_SHARED  = $(PATH_SRC_SDST)/Atmos_AncData.inc \
                    $(PATH_SRC_SDST)/Atmos_ECSMET.inc \
                    $(PATH_SRC_UW)/L1B_Reader_V2.2.inc \
                    $(PATH_SRC_SDST)/atmos_hdf_eos.inc \
                    $(PATH_SRC_UW)/modis_grib_read.h \
                    $(PATH_SRC_UW)/debug.inc \
                    $(PATH_SRC_UW)/platform_name.inc

# Library search path includes libhdf5.so for SDPTK 5.2.7v3 and above.
# Note the use of make substitution operator: $(variable:pattern=replacement)
#
# Addendum 7/2004:  MOD_PR07 is linked with the HDF5 static libraries
# instead of dynamic libraries as described above.  This ensures no 
# conflicting usage of the HDF5 shared libraries when switching toolkit 
# environments (64_f90 or 64_f77) among the processes of PGE03.
#
# 2005-01-18: Reference to HDF5LIB changed to format below in order to compile
# under PGFTN/x86 Linux/x86 5.2-4 F77.  However, this points to the shared dynamic
# library at SDST.  It should point to the static library at the GDAAC.
#
LIB = -L$(API_LIB) -lmapi \
      -L$(PGSLIB) -lPGSTK -lm \
      -L$(HDFEOS_LIB) -lhdfeos -lGctp \
      -L$(HDF5LIB) -lhdf5 \
      -L$(HDFLIB) -lmfhdf -ldf -ljpeg -lz

# Compiler options are set based on IRIX/LINUX environment. On Linux,
# FORTRAN codes are compiled using LINUX PGI compilers; C codes use the
# GNU compilers
target :
	case "$(BRAND)" in \
          linux ) \
            $(MAKE) $(TARGET) -f MOD_PR07.mk \
                    ADD_CFLAGS=" -O2 -c -D_POSIX_SOURCE -funsigned-char" \
                    ADD_F77FLAGS=" -O2 -c" \
                    ADD_F90FLAGS=" -O2 -c" \
            ;; \
          * ) \
            $(MAKE) $(TARGET) -f MOD_PR07.mk \
                    ADD_CFLAGS=" -O2 -c -D_POSIX_SOURCE" \
                    ADD_F77FLAGS=" -O2 -c -u -bytereclen -extend_source -woff 2271,2272,2274" \
                    ADD_F90FLAGS=" -O2 -c -u -bytereclen -extend_source -woff 2271,2272,2274" \
            ;; \
        esac

# Define internal macros
TARGET = MOD_PR07.exe

# Make executable
$(TARGET): $(OBJ)
	$(F90) $(OBJ) $(LIB) -o $@


#---------------------------------------------------------
# Build objects derived from mod07 science code
#---------------------------------------------------------

MOD_PR07.o: MOD_PR07.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) MOD_PR07.f

ceil.o: ceil.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) ceil.f

dewtem.o: dewtem.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) dewtem.f

eswat.o: eswat.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) eswat.f

floor.o: floor.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) floor.f

get_coeff.o: get_coeff.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) get_coeff.f

get_sfcpres_pwat.o: get_sfcpres_pwat.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) get_sfcpres_pwat.f

gphite.o: gphite.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) gphite.f

indices.o: indices.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) indices.f

mod07_Prepare_ECS_Metadata.o: mod07_Prepare_ECS_Metadata.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod07_Prepare_ECS_Metadata.f

mod07_check_box.o: mod07_check_box.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod07_check_box.f

mod07_close_files.o: mod07_close_files.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod07_close_files.f

mod07_compute_products.o: mod07_compute_products.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod07_compute_products.f

mod07_get_data.o: mod07_get_data.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod07_get_data.f

mod07_initialize_output.o: mod07_initialize_output.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod07_initialize_output.f

mod07_open_files.o: mod07_open_files.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod07_open_files.f

mod07_set_qa.o: mod07_set_qa.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod07_set_qa.f

mod07_write_products.o: mod07_write_products.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) mod07_write_products.f

modis_ges101.o: modis_ges101.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	case "$(BRAND)" in \
          sgi32 | sgi64 ) \
             $(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) modis_ges101.f ;; \
          linux ) \
             $(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) -byteswapio modis_ges101.f ;; \
          * ) \
          echo "  ERROR: Environment variable BRAND (set to \"$(BRAND)\") not recognized." >&2 ; \
          exit 1 ;; \
        esac

precwv.o: precwv.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) precwv.f

satmix.o: satmix.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) satmix.f

total_ozone.o: total_ozone.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) total_ozone.f

tovlif.o: tovlif.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) tovlif.f

tprecw_new2.o: tprecw_new2.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) tprecw_new2.f

#----------------------------------------------------------------------------------
# Build object modules based on UW-Madison shared source library files 
#----------------------------------------------------------------------------------

Copy_cldmask_meta.o: $(PATH_SRC_UW)/Copy_cldmask_meta.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/Copy_cldmask_meta.f

Get_Core_Metadata.o: $(PATH_SRC_UW)/Get_Core_Metadata.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/Get_Core_Metadata.f

Get_Global_Metadata.o: $(PATH_SRC_UW)/Get_Global_Metadata.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/Get_Global_Metadata.f

Read_GEO_V2.o: $(PATH_SRC_UW)/Read_GEO_V2.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/Read_GEO_V2.f

## 18 March 2004: SWS moved Read_L1B_V2.1_mod07.f from science code section to
## here and changed name to L1B_Reader_V2.2.f and added path $(PATH_SRC_UW)

Read_L1B_V2.2.o: $(PATH_SRC_UW)/Read_L1B_V2.2.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/Read_L1B_V2.2.f

bright_m.o: $(PATH_SRC_UW)/bright_m.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/bright_m.f

brite_m.o: $(PATH_SRC_UW)/brite_m.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/brite_m.f

ccsds_to_date.o: $(PATH_SRC_UW)/ccsds_to_date.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/ccsds_to_date.f

compare_times.o: $(PATH_SRC_UW)/compare_times.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/compare_times.f

copy_info.o: $(PATH_SRC_UW)/copy_info.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/copy_info.f

create_hdf.o: $(PATH_SRC_UW)/create_hdf.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/create_hdf.f

date_to_ccsds.o: $(PATH_SRC_UW)/date_to_ccsds.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/date_to_ccsds.f

ezlh_convert.o: $(PATH_SRC_UW)/ezlh_convert.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/ezlh_convert.f

get_debug.o: $(PATH_SRC_UW)/get_debug.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/get_debug.f

get_platform_name.o: $(PATH_SRC_UW)/get_platform_name.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/get_platform_name.f

get_spec_info.o: $(PATH_SRC_UW)/get_spec_info.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/get_spec_info.f

interp.o: $(PATH_SRC_UW)/interp.f $(INC_FILES_SCIENCE) $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/interp.f

modis_bright.o: $(PATH_SRC_UW)/modis_bright.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/modis_bright.f

modis_grib_read.o: $(PATH_SRC_UW)/modis_grib_read.c $(PATH_SRC_UW)/modis_grib_read.h
	$(CC) $(CFLAGS) $(ADD_CFLAGS) $(INC) $(PATH_SRC_UW)/modis_grib_read.c

openr_temp.o: $(PATH_SRC_UW)/openr_temp.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/openr_temp.f

read_nise.o: $(PATH_SRC_UW)/read_nise.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/read_nise.f

set_qa_bit.o: $(PATH_SRC_UW)/set_qa_bit.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/set_qa_bit.f

setup_create_hdf.o: $(PATH_SRC_UW)/setup_create_hdf.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/setup_create_hdf.f

strpos.o: $(PATH_SRC_UW)/strpos.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/strpos.f

strcompress.o: $(PATH_SRC_UW)/strcompress.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/strcompress.f

svpwat.o: $(PATH_SRC_UW)/svpwat.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_UW)/svpwat.f

#----------------------------------------------------------------------------
# Build object modules based on SDST shared source library files 
#----------------------------------------------------------------------------

CCSDS_FullTimeFormat_A.o: $(PATH_SRC_SDST)/CCSDS_FullTimeFormat_A.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/CCSDS_FullTimeFormat_A.f

CONCATENATE.o: $(PATH_SRC_SDST)/CONCATENATE.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/CONCATENATE.f

Delete_CommonTempFiles_Atmos_V2.1.o: $(PATH_SRC_SDST)/Delete_CommonTempFiles_Atmos_V2.1.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Delete_CommonTempFiles_Atmos_V2.1.f

Delete_PCF_TempFile.o: $(PATH_SRC_SDST)/Delete_PCF_TempFile.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Delete_PCF_TempFile.f

Gen_Modis_FileName.o: $(PATH_SRC_SDST)/Gen_Modis_FileName.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Gen_Modis_FileName.f

get_ancillary_1.o: $(PATH_SRC_SDST)/get_ancillary_1.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/get_ancillary_1.f

Get_date_time.o: $(PATH_SRC_SDST)/Get_date_time.f $(INC_FILES_SHARED)
	$(F90) $(F90FLAGS) $(ADD_F90FLAGS) $(PATH_SRC_SDST)/Get_date_time.f

MODIS_IO_GEN_OPEN.o: $(PATH_SRC_SDST)/MODIS_IO_GEN_OPEN.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/MODIS_IO_GEN_OPEN.f

MODIS_IO_GEN_CLOSE.o: $(PATH_SRC_SDST)/MODIS_IO_GEN_CLOSE.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/MODIS_IO_GEN_CLOSE.f

MODIS_SMF_SETDYNAMICMSG_V2.1.o: $(PATH_SRC_SDST)/MODIS_SMF_SETDYNAMICMSG_V2.1.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/MODIS_SMF_SETDYNAMICMSG_V2.1.f

Met_Common_Atmos.o: $(PATH_SRC_SDST)/Met_Common_Atmos.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Met_Common_Atmos.f

Parse_ECS_DateTime.o: $(PATH_SRC_SDST)/Parse_ECS_DateTime.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Parse_ECS_DateTime.f

Read_CldMsk_V2.o: $(PATH_SRC_SDST)/Read_CldMsk_V2.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Read_CldMsk_V2.f

STRING_LOC.o: $(PATH_SRC_SDST)/STRING_LOC.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/STRING_LOC.f

Set_ArchiveMetadata_V2.1.o: $(PATH_SRC_SDST)/Set_ArchiveMetadata_V2.1.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Set_ArchiveMetadata_V2.1.f

Set_CoreMetadata_V2.1.o: $(PATH_SRC_SDST)/Set_CoreMetadata_V2.1.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Set_CoreMetadata_V2.1.f

Write_ECSMetadata_nonHDF.o: $(PATH_SRC_SDST)/Write_ECSMetadata_nonHDF.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/Write_ECSMetadata_nonHDF.f

chk_input_L2.o: $(PATH_SRC_SDST)/chk_input_L2.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/chk_input_L2.f

message_1.o: $(PATH_SRC_SDST)/message_1.f
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/message_1.f

modis_grib_driver_1.o: $(PATH_SRC_SDST)/modis_grib_driver_1.c $(PATH_SRC_UW)/modis_grib_read.h
	$(CC) $(CFLAGS) $(ADD_CFLAGS) $(INC) $(PATH_SRC_SDST)/modis_grib_driver_1.c

strlen.o: $(PATH_SRC_SDST)/strlen.f $(INC_FILES_SHARED)
	$(F77) $(F77FLAGS) $(ADD_F77FLAGS) $(INC) $(PATH_SRC_SDST)/strlen.f
#********************** END OF MAKEFILE ***************************
