c-----------------------------------------------------------------------
c !F77-INC
c
c !DESCRIPTION:
c     Include file for UW-MODIS Version2 MOD_PR07
c
c !INPUT PARAMETERS:
c     None
c
c !OUTPUT PARAMETERS:
c     None
c
c !REVISION HISTORY:
c    26 February 2001: SWS included water_vapor_direct
c    March 2003: SWS added pwat1, GDAS precipitable water for TPW dependent biases
c
c !TEAM-UNIQUE HEADER:
c     Developed by the MODIS Group, CIMSS/SSEC, UW-Madison.
c
c !END
c-----------------------------------------------------------------------

c-----------------------------------------------------------------------
c ... Global Parameters
c-----------------------------------------------------------------------

c ... Maximum number of pixels and lines in a 1km scan

      integer max_pixel, max_line
      parameter( max_pixel = 1500, max_line = 10 )

c ... Sampling factor (for 5x5 sampling, isamp = 5)

      integer isamp
      parameter ( isamp = 5 )

c ... Maximum number of sampled pixels and lines in a 1km scan

      integer max_samp_pixel, max_samp_line
      parameter ( max_samp_pixel = max_pixel / isamp,
     &  max_samp_line = max_line / isamp )

c ... Number of 1km IR bands

      integer nbands
      parameter( nbands = 12 )

c ... SWS added for call to new Read_L1B_V2.2.f 
      integer LAST_MODIS_BAND
      parameter ( LAST_MODIS_BAND = 36 )

c ... Bad data value

      real bad_value
      parameter( bad_value = -999.0 )

c ... Minimum number of good pixels allowed in each pixel sampling box

      integer min_ngood
      parameter ( min_ngood = 5 )

c ... Number of bytes per 1km pixel for cloud mask result and QA arrays

      integer nmask, nmask_qa
      parameter( nmask = 6, nmask_qa = 10 )

c ... Number of atmospheric levels in output retrieval profiles

      integer outlevels
      parameter ( outlevels = 20 )

c ... Number of product QA bytes per 5x5 pixel box

      integer nproduct_qa
      parameter( nproduct_qa = 10 )

c ... Number of infrared water vapor QA bytes per 5x5 pixel box 
      integer nwater_vapor_qa
      parameter( nwater_vapor_qa = 5 )
      
c-----------------------------------------------------------------------
c ... REAL global variables stored in COMMON
c-----------------------------------------------------------------------

c ... MODIS L1B input radiance data for all 1km IR bands

      real radiance1( max_pixel, max_line, nbands )

c ... MODIS geolocation latitude, longitude, surface elevation, and
c ... sensor zenith angle data for 1km bands

      real lat1( max_pixel, max_line ),
     &     lon1( max_pixel, max_line ),
     &     elv1( max_pixel, max_line ),
     &     zen1( max_pixel, max_line )

c ... NCEP ancillary input surface temperature, water vapor mixing
c ... ratio, surface pressure, and precipitable water for 1km bands

      real tem1( max_pixel, max_line ), wat1( max_pixel, max_line ),
     &  pre1( max_pixel, max_line ), pwat1( max_pixel, max_line )

c ... MODIS L2 output products at 5x5 sampling resolution

      real brightness_temp( max_samp_pixel, max_samp_line, nbands ),
     &  sfc_temp( max_samp_pixel, max_samp_line ),
     &  sfc_pres( max_samp_pixel, max_samp_line ),
     &  sfc_elev( max_samp_pixel, max_samp_line ),
     &  height_tropopause( max_samp_pixel, max_samp_line ),
     &  guess_temp_profile( max_samp_pixel, max_samp_line, outlevels ),
     &  guess_dewp_profile( max_samp_pixel, max_samp_line, outlevels ),
     &  retr_temp_profile( max_samp_pixel, max_samp_line, outlevels ),
     &  retr_dewp_profile( max_samp_pixel, max_samp_line, outlevels ),
     &  retr_hite_profile( max_samp_pixel, max_samp_line, outlevels ),
     &  water_vapor( max_samp_pixel, max_samp_line ),
     &  water_vapor_direct( max_samp_pixel, max_samp_line ),
     &  water_vapor_low( max_samp_pixel, max_samp_line ),
     &  water_vapor_high( max_samp_pixel, max_samp_line ),
     &  total_ozone( max_samp_pixel, max_samp_line ),
     &  total_totals( max_samp_pixel, max_samp_line ),
     &  lifted_index( max_samp_pixel, max_samp_line ),
     &  k_index( max_samp_pixel, max_samp_line )

c-----------------------------------------------------------------------
c ... INTEGER global variables stored in COMMON
c-----------------------------------------------------------------------

c ... Good data flag array for each pixel box

      integer box_flag( isamp, isamp )

c ... 1km IR band numbers

      integer bands( nbands )      
      
c-----------------------------------------------------------------------
c ... BYTE global variables stored in COMMON
c-----------------------------------------------------------------------

c ... Land/water mask for 1km bands

      byte land1( max_pixel, max_line )

c ... Cloud mask result and QA arrays for 1km bands

      byte mask1( nmask, max_pixel, max_line ), mask1_qa( nmask_qa,
     &  max_pixel, max_line)

c ... Product QA array at 5x5 sampling resolution

      byte product_qa( nproduct_qa, max_samp_pixel, max_samp_line )

c ... IR water vapor QA array at 5x5 sampling resolution

      byte water_vapor_qa( nwater_vapor_qa, max_samp_pixel,
     &  max_samp_line )

c-----------------------------------------------------------------------
c ... COMMON block to hold global variables
c ... (Arrays are listed in the order REALs, INTEGERs, BYTEs)
c-----------------------------------------------------------------------

      common / mod07_data /
     &  radiance1, lat1, lon1, elv1, zen1, tem1, wat1, pre1, pwat1,
     &  brightness_temp, sfc_temp, sfc_pres, sfc_elev,
     &  height_tropopause, guess_temp_profile, guess_dewp_profile,
     &  retr_temp_profile, retr_dewp_profile, retr_hite_profile,
     &  water_vapor, water_vapor_direct, water_vapor_low, 
     &  water_vapor_high, total_ozone, total_totals, 
     &  lifted_index, k_index, box_flag, bands, 
     &  land1, mask1, mask1_qa, product_qa, water_vapor_qa
