      SUBROUTINE MODIS_IO_GEN_OPENF(FILE_PCF_INDEX,FILE_ACCESS, 
     &           RECORD_LENGTH, FILE_HANDLE, FILE_VERSION)

      IMPLICIT NONE
      INCLUDE 'PGS_SMF.f'
      INCLUDE 'PGS_MODIS_39500.f'

C---------------------------------------------------------------------
C !F77
C
C !DESCRIPTION:
C
C      SUBROUTINE TO OPEN A FILE USING THE SDP TK GNERIC FILE
C      OPEN FUNCTION.
C
C !INPUT PARAMETERS:
C
C      FILE_PCF_INDEX  LOGICAL IDENTIFIER OF THE FILE IN THE PROCESS
C                      CONTROL FILE (PCF).
C      FILE_ACCESS     SDPTK FILE ACCESS TYPE
C      RECORD_LENGTH   LENGTH OF DATA RECORD (IGNORED, EXCEPT FOR
C                      DIRECT ACCESS FILES)
C      FILE_VERSION    VERSION NUMBER OF FILE 
C
C !OUTPUT PARAMETERS:
C
C      FILE_HANDLE    FORTRAN LOGICAL FILE UNIT NUMBER.  THIS IS THE 
C                     REFERENCE TO THE FILE THAT IS USED FOR NATIVE 
C                     FORTRAN READ AND WRITE STATEMENTS.
C
C !REVISION HISTORY:
c Revision 1.8  1997/10/14  12:31:46  rhucek
c Remove PGS_IO_* and PGS_PC_* include file statements.
c
c Revision 1.7  1996/04/24  17:04:58  vlin
c put in "!' sign & replaced "_F_" with "_W_"
c
c Revision 1.6  1995/11/20  15:55:33  vlin
c updated E-Mail address
c
c Revision 1.5  1995/10/16  13:39:45  vlin
c removed 'PRINT" statement
c
c Revision 1.4  1995/10/16  12:56:06  rhucek
c change routine from FORTRAN FUNCTION to SUBROUTINE
c
c Revision 1.3  1995/08/22  14:35:37  vlin
c updated prolog
c
C !TEAM-UNIQUE HEADER:
C
C      THIS SOFTWARE WAS DEVELOPED BY THE MODIS SCIENCE DATA SUPPORT
C      TEAM FOR THE NATIONAL AERONAUTICS AND SPACE ADMINISTRATION, 
C      GODDARD SPACE FLIGHT CENTER, UNDER CONTRACT NAS5-32373
C
C !REFERENCES AND CREDITS:
C
C      WRITTEN BY RICHARD HUCEK
C      RESEARCH AND DATA SYSTEMS CORPORATION
C      SAIC/GSC MODIS SCIENCE DATA SUPPORT OFFICE
C      7501 FORBES BLVD, SEABROOK MD 20706
C
C      rhucek@ltpmail.gsfc.nasa.gov
C
C !DESIGN NOTES:
C 
C  EXTERNALS: 
C
C      NAMED CONSTANTS:
C               MODIS_W_GENERIC         (PGS_MODIS_39500.f)
C               MODIS_S_SUCCESS         (PGS_MODIS_39500.f)
C               PGS_S_SUCCESS           (PGS_SMF.f)
C
C      FUNCTIONS AND SUBROUTINES:
C               MODIS_SMF_SETDYNAMICMSG             
C               PGS_IO_GEN_OPENF        (libPGSTK.a)
C
C !END
C---------------------------------------------------------------------

C DECLARATION OF VARIABLES

      CHARACTER*1024 INFO_STRING
      INTEGER RETURNSTATUS
      INTEGER FILE_PCF_INDEX 
      INTEGER FILE_ACCESS
      INTEGER RECORD_LENGTH
      INTEGER FILE_HANDLE
      INTEGER FILE_VERSION

C DECLARATION OF FUNCTIONS 
      INTEGER PGS_IO_GEN_OPENF

C INITIALIZE VARIABLES 
      INFO_STRING = ' '
      RETURNSTATUS = 0

C PUT FILE INDENTIFIER IN A STRING
      WRITE(INFO_STRING, '(I10)') FILE_PCF_INDEX

C OPEN FILE USING SDPTK GENERIC FILE OPEN FUNCTION 

      RETURNSTATUS = PGS_IO_GEN_OPENF(FILE_PCF_INDEX, FILE_ACCESS,
     &               RECORD_LENGTH, FILE_HANDLE, FILE_VERSION)

C CHECK RETURNSTATUS OF SDPTK FILE OPEN CALL AND REPORT SUCCESS/ERROR 
C MESSAGES TO LOG FILES.

      IF (RETURNSTATUS .NE. PGS_S_SUCCESS) THEN
         CALL MODIS_SMF_SETDYNAMICMSG(MODIS_W_GENERIC,
     &        INFO_STRING,'MODIS_IO_GEN_OPENF')
      ELSE
         CALL MODIS_SMF_SETDYNAMICMSG(MODIS_S_SUCCESS,
     &        INFO_STRING,'MODIS_IO_GEN_OPENF')
      END IF

      RETURN
      END

