       SUBROUTINE MODIS_SMF_SETDYNAMICMSG(CODE,STRING_USER,FUNC_NAME)
       IMPLICIT NONE

C-----------------------------------------------------------------------
C !F77
C
C !DESCRIPTION: 
C
C       WRITES A MESSAGE STRING TO THE LOGSTATUS FILE.  THE
C       COMPLETE MESSAGE CONSISTS OF THREE PIECES:  A PREDEFINED MODIS
C       OR SDPTK STATUS MESSAGE; A DYNAMIC MESSAGE SEGMENT (STRING_USER)
C       PASSED BY THE CALLING PROGRAM; AND THE NAME OF THE FUNCTION OR 
C       SUBROUTINE THAT CALLS MODIS_SMF_SETDYNAMICMSG. 
C
C !INPUT PARAMETERS:  
C
C   INTEGER    CODE - A NAMED CONSTANT RELATED TO A PREDEFINED MODIS 
C                     OR SDP TK MESSAGE STRING
C   CHARACTER  STRING_USER - A DYNAMIC MESSAGE STRING DEFINED BY 
C                            CALLING PROGRAM
C   CHARACTER  FUNC_NAME - NAME OF MODULE REPORTING MESSAGE
C
C !OUTPUT PARAMETERS:     NONE
C
C !REVISION HISTORY:
C
C !TEAM-UNIQUE HEADER: 
C
C      THIS SOFTWARE WAS DEVELOPED BY THE MODIS SCIENCE DATA SUPPORT
C      TEAM FOR THE NATIONAL AERONAUTICS AND SPACE ADMINISTRATION,
C      GODDARD SPACE FLIGHT CENTER, UNDER CONTRACT NAS5-32373.
C
C !REFERENCES AND CREDITS:
C
C      WRITTEN BY VICKY LIN 
C
C      RESEARCH AND DATA SYSTEMS CORPORATION
C      SAIC/GSC MODIS SCIENCE DATA SUPPORT OFFICE
C      7501 FORBES BLVD, SEABROOK MD 20706
C
C      vlin@ltpmail.gsfc.nasa.gov
C
C !DESIGN NOTES:
C
C      PREDEFINED MESSAGES WITH A FATAL STATUS LEVEL (i.e _F_) 
C      CAUSE PROGRAM TO EXIT VIA A FORTRAN EXIT STATEMENT.  
C
C      NONSUCCESSFUL COMPLETION OF EMBEDDED SUBROUTINE CALL ALSO 
C      CAUSE PROGRAM TO ABORT VIA A FORTRAN EXIT STATEMENT.  
C
C  EXTERNALS: 
C
C      NAMED CONSTANTS:
C            MODIS_F_GENERIC                (PGS_MODIS_39500.f)
C            PGS_S_SUCCESS                  (PGS_SMF.f)
C            PGS_TRUE                       (PGS_SMF.f)
C
C      FUNCTIONS AND SUBROUTINES:
C            STRING_LOC
C            CONCATENATE
C            PGS_SMF_SETDYNAMICMSG          (libPGSTK.a)
C            PGS_SMF_GETMSGBYCODE           (libPGSTK.a)
C            PGS_SMF_TESTFATALLEVEL         (libPGSTK.a)
C
C !END
C-----------------------------------------------------------------------

C SDP TOOLKIT HEADER FILE

      INCLUDE 'PGS_SMF.f'
      INCLUDE 'PGS_MODIS_39500.f'

C PARAMETER DECLARATIONS
      CHARACTER*(*)  FUNCNAME
      PARAMETER     (FUNCNAME = 'MODIS_SMF_SETDYNAMICMSG') 

      INTEGER        USER_MAX_BUF_LEN
      PARAMETER     (USER_MAX_BUF_LEN = 4096)
      

C DECLARATION OF VARIABLES
      CHARACTER*(*)  STRING_USER
      CHARACTER*(*)  FUNC_NAME
      CHARACTER*25   MSG25
      CHARACTER*512  MSG
      CHARACTER*(USER_MAX_BUF_LEN) BUF

      INTEGER RTN,RETURNSTATUS, RETURNSTATUS2
      INTEGER FBYTE,LBYTE,CODE

C  FUNCTION DECLARATIONS
      INTEGER STRING_LOC
      INTEGER PGS_SMF_SETDYNAMICMSG
      INTEGER PGS_SMF_GETMSGBYCODE
      INTEGER PGS_SMF_TESTFATALLEVEL

C  INITIALIZE VARIABLES
      BUF = ' '
      MSG = ' '
      RETURNSTATUS = 0
      RETURNSTATUS2 = 0

C  GET PREDEFINED MESSAGE FROM SEED FILE BASED ON ASSOCIATED NAMED
C  CONSTANT.  IF UNABLE TO GET PREDEFINED MESSAGE, REPORT PROBLEM
C  TO LOGSTATUS FILE AND EXIT. 

      RETURNSTATUS = PGS_SMF_GETMSGBYCODE(CODE,MSG)

      IF (RETURNSTATUS .NE. PGS_S_SUCCESS) THEN
         WRITE(MSG25,'(I25)') CODE
         RTN = STRING_LOC(MSG25,FBYTE,LBYTE)

         BUF = 'PGS_SMF_GETMSGBYCODE failed to retrieve message text for ' 
     1         // 'message ID number ' // MSG25(FBYTE:LBYTE) 
     2         // char(10) // 'Operator Action:  Check system environment and SDPTK configuration. ' 
     3         // char(10) // 'If a fault is identified, correct and rerun PGE.  Otherwise, notify '
     4         // char(10) // 'SDST.'

         RETURNSTATUS2 = PGS_SMF_SETDYNAMICMSG(MODIS_F_GENERIC,BUF,
     2                   FUNCNAME) 
         CALL EXIT (1) 
      END IF
          

C  CONCATENATE USER MESSAGE WITH NMEMONIC MESSAGE.

      CALL CONCATENATE(MSG,STRING_USER,BUF)


C  WRITE MESSAGE TO LOGSTATUS FILE.

      RETURNSTATUS = PGS_SMF_SETDYNAMICMSG(CODE,BUF(1:PGS_SMF_MAX_MSGBUF_SIZE),FUNC_NAME)
      IF (RETURNSTATUS .NE. PGS_S_SUCCESS) CALL EXIT (1) 


C  TEST FOR FATAL ERROR.

      RETURNSTATUS = PGS_SMF_TESTFATALLEVEL(CODE)
      IF (RETURNSTATUS .EQ. PGS_TRUE) CALL EXIT (1) 

      RETURN
      END

