#!/usr/local/bin/perl -w
#
# Description:
#     
#   HREPACK.pl uses the National Center for Supercomputing Applications (NCSA) 
#   hrepack utility to compresses a single HDF file given its pathname.  See
#   Design Notes below.
# 
#     
# Input Parameters:
#   HDF_FileName        Path name of HDF file to compress (or file name only if
#                       it resides in current directory)
#
# Output Parameters:
#   None   
#
# Externals:
#   /usr/local/bin/hrepack
#
# Revision History:
#
#   gfireman 20041012   Initial inline design by Gwyn Fireman 
#
#   rhucek   20041108   Repackaged as a separate script with minor 
#                       modifications to error reporting  
#
#   rhucek   20060118   Replaced "!defined $HDF_FileName" condition with 
#                       "if (!-e $HDF_FileName)" and restructured system of
#                       if blocks. 
#
#
# Team-unique header:
#   This software is developed by the MODIS Science Data Support Team 
#   for the National Aeronautics and Space Administration, Goddard Space 
#   Flight Center, under contract NAS5-32373.
#
# Design Notes:
#   A single call to NCSA's hrepack utility is employed.  It is configured to 
#   compresses all HDF data objects in the file using the same 'GZIP 4' 
#   deflation algorithm on each. 
#
# 
# Example :
#   HREPACK.pl MOD35_L2.A2003094.1415.005.2004313210148.hdf 
#
################################################################################

use strict;
use MTVS::PGE;

my $HDF_FileName = $ARGV[0]; 
my $exe_hrepack  = "/usr/local/bin/hrepack";

# return if file does not exist
if ( !-e $HDF_FileName ) {
   MTVS::Print('I',"HDF file $HDF_FileName does not exist.  Compression step not attempted.");

} else {
   my $tempfile = $HDF_FileName . ".temp"; 

   if ( system "$exe_hrepack -i $HDF_FileName -o $tempfile -t '*:GZIP 4'" )
      { MTVS::PGE::VdcExitPGEScript($MTVS::FAILURE, "Unable to compress hdf file $HDF_FileName") }

   if ( !rename ($tempfile, $HDF_FileName) )
      { MTVS::PGE::VdcExitPGEScript($MTVS::FAILURE, "Unable to rename $tempfile to $HDF_FileName") }
}
