      subroutine blint_met(grid, i, j, lat, lon, t, ret)

      IMPLICIT NONE
      SAVE

c-----------------------------------------------------------------------
c
c!F77
c
c!Description:
c   Bi-linear interpolation scheme used with GDAS gridded products.
c
c!Input Parameters:
c    grid   Real array containg SST grid values (360x180)
c    i      Grid index in the x direction
c    j      Grid index in the y direction
c    lat    Pixel latitude value
c    lon    Pixel longitude value
c
c!Output Parameters:
c    t      Interpolated value
c    ret    Return flag (0 = successful)
c
c!Revision History:
c 06/04 Collection 5  R. Frey    Original version.
c
c!Team-unique Header:
c
c!End
c
c-----------------------------------------------------------------------

c ... Input arguments

      REAL grid, lat, lon
      INTEGER i, j

c ... Output arguments

      REAL t
      INTEGER ret

c ... Parameters

      INTEGER npoints_x, npoints_y
      PARAMETER ( npoints_x = 360 )
      PARAMETER ( npoints_y = 180 )
     
      REAL dx, dy
      PARAMETER ( dx = 1.0)
      PARAMETER ( dy = 1.0)

      DIMENSION grid(0:npoints_x-1, 0:npoints_y)

c ... Local variables

      REAL p, ip0, ip1, jp0, jp1, t00, t01, t0,
     +     t10, t11, t1, ldi, ldj, colat
      INTEGER i1, j1 
      LOGICAL y_interp

      ret = 0

      if( j .lt. 0 .or. j .gt. (npoints_y) ) then
        ret = -1
      end if
      if( i .lt. 0 .or. i .gt. (npoints_x - 1) ) then
        ret = -1
      end if

      if( ret .eq. 0 ) then

        y_interp = .true.

        ldi = lon - float(i)
        if(ldi .ge. 0.0) then
          i1 = i + 1
          if( i .eq. (npoints_x - 1) ) i1 = 0
        else
          i1 = i - 1
          if( i .eq. 0 ) i1 = npoints_x - 1
        end if

        colat = 90.0 - lat
        ldj = colat - float(j)
        if(ldj .ge. 0.0) then
          j1 = j + 1
          if( j .eq. npoints_y ) y_interp = .false.
        else
          j1 = j - 1
          if( j .eq. 0 ) y_interp = .false.
        end if
      
        p = lon - float(i1)
        if( p .lt. -1.0) p = p + 360.0
        if( p .gt. 1.0) p = p - 360.0
        ip0 = abs( p / dx)
        ip1 = dx - ip0

        t00 = grid( i, j)
        t01 = grid( i1, j)
        t0 = (ip0 * t00) + (ip1 * t01)

        if( y_interp ) then
 
          p = colat - float(j1)
          jp0 = abs( p / dy )
          jp1 = dy - jp0
        
          t10 = grid(i, j1)
          t11 = grid(i1, j1)
          t1 = (ip0 * t10) + (ip1 * t11)

          t = (jp0 * t0) + (jp1 * t1)

        else

          t = t0

        end if

      end if

      return 
      end
